import bpy

from bpy.props import (FloatProperty)



#================================================================
#            Animated Bevel - Closed Curve
#================================================================



class OBJECT_OT_anim_curve_bevel_closed_anbx(bpy.types.Operator):
    '''Animated Bevel - Selected Object(s)
- if Curve is Closed'''

    bl_idname = "object.anim_curve_bevel_closed_anbx"
    bl_label = "Closed Curve"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):

        ob_types = ['CURVE', 'MESH']


        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) != 0 and
                context.object.type in ob_types 
                )



    def execute(self, context):

        bpy.context.scene.tool_settings.transform_pivot_point = 'INDIVIDUAL_ORIGINS'

        act_ob = bpy.context.active_object
        sel_ob = bpy.context.selected_objects
        ob = bpy.context.object


        if ob.type == 'MESH': 
            for o in sel_ob:
                bpy.context.view_layer.objects.active = o
                bpy.ops.object.convert(target='CURVE') 
                bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='MEDIAN')
                bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)
               
 

        # Object Data Name == Object Name
        for obj in bpy.context.selected_objects:
            if obj.type == 'CURVE':
                obj.name = obj.name
                obj.data.name = obj.name 
                bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)
         

 
        # Check if Curve is closed - Deselect Open Curve
        def deselect_open_curve(): 
            for o in sel_ob:
                bpy.context.view_layer.objects.active = o

                if o.type == 'CURVE':       
                    for c in o.data.splines:                
                        if not c.use_cyclic_u:
                            o.select_set(0) 
           
        deselect_open_curve()  
 
#--------------------------------------------------

        # Curve Material
        def add_new_mat_to_active():
            
            ob = bpy.context.active_object
            mat = bpy.data.materials.get("anim_curve_mat")
            if mat is None:
                mat = bpy.data.materials.new(name="anim_curve_mat")    
               

            if ob.data.materials:
                ob.data.materials[0] = mat
            else:
                ob.data.materials.append(mat)       
                   
            bpy.context.object.active_material.use_nodes = True
                    
            for n in bpy.data.materials:

                num = 17
                # Check Blender Version
                blender_version = bpy.app.version[0]

                if blender_version == 2:
                    num_emission_color = 17
                    num_emission_strength = 18

                if blender_version == 3: 
                    num_emission_color = 19
                    num_emission_strength = 20

                if blender_version == 4: 
                    num_emission_color = 26
                    num_emission_strength = 27


                if n.name.startswith("anim_curve_mat"):
                    n.node_tree.nodes["Principled BSDF"].inputs[num_emission_color].default_value = (0.88, 0.84, 0.57, 1)
                    n.node_tree.nodes["Principled BSDF"].inputs[num_emission_strength].default_value = 5


        # Material to Selected
        for o in bpy.context.selected_objects:
            bpy.context.view_layer.objects.active = o
            add_new_mat_to_active()

                 

#==============================================
#               Closed Curve Animation
#==============================================

        def anim_closed_curve():
            
            cur_frame = bpy.context.scene.frame_current   
             
            # Insert Keyframe --> Scale / Bevel_Depth
            def insert_keys_closed_curve(frame):
                for o in bpy.context.selected_objects:
                    bpy.context.view_layer.objects.active = o
                    o.keyframe_insert(data_path = 'scale', frame= frame)
                    o.data.keyframe_insert(data_path = 'bevel_depth', frame= frame)


            # Curve Bevel Depth
            def curve_bevel_depth(depth_num):
                for o in bpy.context.selected_objects:
                    bpy.context.view_layer.objects.active = o
                    bpy.context.object.data.bevel_mode = 'ROUND'
                    o.data.bevel_depth = depth_num


            # Curve Scale
            def curve_scale(num):
                for o in bpy.context.selected_objects:
                    bpy.context.view_layer.objects.active = o
                    o.scale.x = num
                    o.scale.y = num
                    o.scale.z = num


            for o in bpy.context.selected_objects:
                bpy.context.view_layer.objects.active = o


                # First Frame - Current Frame
                curve_scale(0)
                curve_bevel_depth(0.3)
                insert_keys_closed_curve(cur_frame)

                # Second Frame += 12
                curve_scale(1)
                curve_bevel_depth(0.0)
                insert_keys_closed_curve(cur_frame + 15)


#==============================================
#            Closed Curve Animation
#==============================================


        for obj in bpy.context.selected_objects:
            if obj.type == 'CURVE':
                anim_closed_curve()
 
         # Scene Update
        bpy.context.view_layer.update()
    


        return {'FINISHED'}


#================================================================
#             End Animated Bevel - Closed Curve
#================================================================







#================================================================
#             Animated Bevel - Open Curve
#================================================================



class OBJECT_OT_anim_curve_bevel_open_anbx(bpy.types.Operator):
    '''Animated Bevel - Selected Object(s) 
- if Curve is Open'''

    bl_idname = "object.anim_curve_bevel_open_anbx"
    bl_label = "Open Curve"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):

        ob_types = ['CURVE', 'MESH']

        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) != 0 and
                context.object.type in ob_types 
                )



    def execute(self, context):


        act_ob = bpy.context.active_object
        sel_ob = bpy.context.selected_objects
        ob = bpy.context.object


        if ob.type == 'MESH': 
            for o in sel_ob:
                bpy.context.view_layer.objects.active = o
                bpy.ops.object.convert(target='CURVE') 
                bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='MEDIAN')
                bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)
               
 


        # Object Data Name == Object Name
        for obj in bpy.context.selected_objects:
            if obj.type == 'CURVE':
                obj.name = obj.name
                obj.data.name = obj.name 
                bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)
         


        # Check if Curve is closed - Deselect Closed Curve
        def deselect_closed_curve(): 
            for o in sel_ob:
                bpy.context.view_layer.objects.active = o

                if o.type == 'CURVE':       
                    for c in o.data.splines:                
                        if c.use_cyclic_u:
                            o.select_set(0) 
           
        deselect_closed_curve()  
 
#--------------------------------------------------

        # Curve Material
        def add_new_mat_to_active():
            
            ob = bpy.context.active_object
            mat = bpy.data.materials.get("anim_curve_mat")
            if mat is None:
                mat = bpy.data.materials.new(name="anim_curve_mat")    
               

            if ob.data.materials:
                ob.data.materials[0] = mat
            else:
                ob.data.materials.append(mat)       
                   
            bpy.context.object.active_material.use_nodes = True
                    
            for n in bpy.data.materials:

                num = 17
                # Check Blender Version
                blender_version = bpy.app.version[0]

                if blender_version == 2:
                    num_emission_color = 17
                    num_emission_strength = 18

                if blender_version == 3: 
                    num_emission_color = 19
                    num_emission_strength = 20

                if blender_version == 4: 
                    num_emission_color = 26
                    num_emission_strength = 27


                if n.name.startswith("anim_curve_mat"):
                    n.node_tree.nodes["Principled BSDF"].inputs[num_emission_color].default_value = (0.88, 0.84, 0.57, 1)
                    n.node_tree.nodes["Principled BSDF"].inputs[num_emission_strength].default_value = 5


        # Material to Selected
        for o in bpy.context.selected_objects:
            bpy.context.view_layer.objects.active = o
            add_new_mat_to_active()



#==============================================
#               Open Curve Animation
#==============================================


        def anim_open_curve():

            cur_frame = bpy.context.scene.frame_current

            ob = bpy.context.object
            # Insert Keyframe --> Scale / Bevel_Depth
            def insert_keys_curve_bevel(frame):
                for o in bpy.context.selected_objects:
                    bpy.context.view_layer.objects.active = o
                    bpy.context.object.data.bevel_mode = 'ROUND'
                    o.data.keyframe_insert(data_path = 'bevel_factor_start', frame= frame)
                    o.data.keyframe_insert(data_path = 'bevel_factor_end', frame= frame)
                    o.data.keyframe_insert(data_path = 'bevel_depth', frame= frame)


            # Curve Bevel - Start / End
            def curve_bevel(depth_num, start_num, end_num):
                for o in bpy.context.selected_objects:
                    bpy.context.view_layer.objects.active = o
                    o.data.bevel_depth = depth_num
                    o.data.bevel_factor_start = start_num
                    o.data.bevel_factor_end = end_num



            # First Frame - Current Frame
            curve_bevel(0.05, 0.0, 0.0)
            insert_keys_curve_bevel(cur_frame)


            cur_frame += 10


            # Second Frame
            curve_bevel(0.05, 0.1, 0.8)
            insert_keys_curve_bevel(cur_frame)


            cur_frame += 10


            # Tird Frame
            curve_bevel(0.05, 1.0, 1.0)
            insert_keys_curve_bevel(cur_frame)

#==============================================
#             Open Curve Animation
#==============================================


        for obj in bpy.context.selected_objects:
            if obj.type == 'CURVE':
                anim_open_curve()
 
 
        
        # Scene Update
        bpy.context.view_layer.update()
           
           

        return {'FINISHED'}


#================================================================
#             End Animated Bevel - Open Curve
#================================================================




# Bevel Depth Slider - set Keyframe(s) --------------------------


def bevel_depth(self, context):
     
    ob = bpy.context.object
    mode = ob.mode

    sel_ob = bpy.context.selected_objects
    
    if bpy.context.mode == 'OBJECT':
        for o in sel_ob:             
            if o.type == 'MESH' and not o.data.polygons:
                bpy.ops.object.convert(target='CURVE')
                

    def selected_edges():
        for o in bpy.context.selected_objects: 
            for e in o.data.edges:
                if e.select:
                    return True
    
     
    def convert_mesh_to_curve():     
            
        if bpy.context.mode == 'EDIT_MESH':
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
        
             
        for e in bpy.context.object.data.edges:
            if e.select:
                bpy.ops.object.mode_set(mode='EDIT', toggle=False)

        if selected_edges():
            bpy.ops.object.mode_set(mode='EDIT', toggle=False)
             
    
              
        if bpy.context.mode == 'EDIT_MESH': 
            bpy.ops.mesh.duplicate()
            bpy.ops.mesh.delete(type='ONLY_FACE')        
            bpy.ops.mesh.separate(type='LOOSE')

            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
             
              
        for o in bpy.context.selected_objects:
            if o.type == 'MESH' and o.data.polygons:  
                o.select_set(0) 
            else:
                o.select_set(1) 
                bpy.context.view_layer.objects.active = o      
              
 
  
                # Make Single User
                bpy.ops.object.make_single_user(object=False, obdata=True, material=True, animation=True)  
                bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='MEDIAN')            

                

    if bpy.context.object is not None:
        if bpy.context.object.type != 'CURVE' and bpy.context.mode == 'EDIT_MESH':
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            if selected_edges():
                convert_mesh_to_curve()
                bpy.ops.object.convert(target='CURVE')
     

            else:
                bpy.ops.object.mode_set(mode=mode)
                

                   
  
    if bpy.context.object is not None:
        if bpy.context.object.type != 'CURVE' and bpy.context.mode == 'OBJECT':
            if selected_edges():
                convert_mesh_to_curve()
                bpy.ops.object.convert(target='CURVE')
              
              
    if bpy.context.object is not None:            
        if bpy.context.object.type == 'CURVE':
            for o in bpy.context.selected_objects:
                o.data.bevel_depth =  self.change_bevel_depth_value_anbx
            
                o.data.keyframe_insert(data_path='bevel_depth')



# Bevel Depth Slider - set Keyframe(s) --------------------------





classes = [OBJECT_OT_anim_curve_bevel_closed_anbx,
           OBJECT_OT_anim_curve_bevel_open_anbx,]




def register():
    for cls in classes:
        bpy.utils.register_class(cls)



    bpy.types.Scene.change_bevel_depth_value_anbx = bpy.props.FloatProperty(
            name="Curve Bevel Depth",
            default=0.0,
            min=0.0,
            description="Curve Bevel Depth\n-Convert Mesh to Curve (Selected Edges)\n-Auto Keys",
            update = bevel_depth
            )




def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)


    del bpy.types.Scene.change_bevel_depth_value_anbx


if __name__ == "__main__":
    register()





