import bpy

from bpy.props import (BoolProperty)



#================================================================
#             Circle Explode
#================================================================



class OBJECT_OT_circle_explode_anbx(bpy.types.Operator):
    '''If nothing is selected:
    'Circle Explode' will be placed at the 3D Cursor position.
If selected: 
    at the location of the Selected Object'''

    bl_idname = "object.circle_explode_anbx"
    bl_label = "Circle Explode"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.mode == 'OBJECT')



    def execute(self, context):


        # Ref -> Align to View Bool
        align_to_view = context.scene.align_to_view_bool_mdh
        

        sel_ob = bpy.context.selected_objects
        act_ob = bpy.context.active_object
        cur_frame = bpy.context.scene.frame_current

        cursor = bpy.context.scene.cursor
        cursor_location = bpy.context.scene.cursor.location
        cursor_previous_location = cursor.location.copy()



        def exploding_circle_main():

            def create_exploding_circle():


                reg_d = bpy.context.region_data 

                if align_to_view:
                    # Align to View
                    bpy.ops.curve.primitive_bezier_circle_add(radius=1, enter_editmode=False, align='WORLD', 
                                                              location=(cursor.location), scale=(1, 1, 1), 
                                                              rotation=(reg_d.view_rotation.to_euler()))



                else:
                    # Align to Front View    
                    bpy.ops.curve.primitive_bezier_circle_add(radius=1, enter_editmode=False, align='WORLD', 
                                                              location=(cursor.location), scale=(1, 1, 1), 
                                                              rotation=(1.5708, 0.0, 0.0))



                # Rename
                for o in bpy.context.selected_objects:
                    bpy.context.view_layer.objects.active = o
                    o.name = 'circle_explode'
                    o.data.name = 'circle_explode'

            create_exploding_circle()



            def add_new_mat_to_active():
                
                ob = bpy.context.active_object
                mat = bpy.data.materials.get("circle_explode_mat")
                if mat is None:
                    mat = bpy.data.materials.new(name="circle_explode_mat")    
                   

                if ob.data.materials:
                    ob.data.materials[0] = mat
                else:
                    ob.data.materials.append(mat)       
                       
                bpy.context.object.active_material.use_nodes = True
                        
                for n in bpy.data.materials:

                    num = 17
                    # Check Blender Version
                    blender_version = bpy.app.version[0]

                    if blender_version == 2:
                        num_emission_color = 17
                        num_emission_strength = 18
            
                    if blender_version == 3: 
                        num_emission_color = 19
                        num_emission_strength = 20

                    if blender_version == 4: 
                        num_emission_color = 26
                        num_emission_strength = 27                        


                    if n.name.startswith("circle_explode_mat"):
                        n.node_tree.nodes["Principled BSDF"].inputs[num_emission_color].default_value = (0.88, 0.84, 0.57, 1)
                        n.node_tree.nodes["Principled BSDF"].inputs[num_emission_strength].default_value = 5


            add_new_mat_to_active()



            # Insert Keyframe --> Scale / Bevel_Depth
            def insert_keys(frame):
                o.keyframe_insert(data_path = 'scale', frame= frame)
                o.data.keyframe_insert(data_path = 'bevel_depth', frame= frame)

            # Curve Scale
            def curve_scale(num):
                o.scale.x = num
                o.scale.y = num
                o.scale.z = num

            # Curve Bevel_Depth
            def curve_bevel_depth(num):
                for c in bpy.data.curves:
                    if c.name.startswith('circle_explode'):
                        curve_name = c.name
                        bpy.data.curves[curve_name].bevel_depth = num



            for o in bpy.context.selected_objects:
                bpy.context.view_layer.objects.active = o
                if o.type == 'CURVE':

                    # First Frame - Current Frame
                    curve_scale(0)
                    curve_name = o.name
                    bpy.context.object.data.bevel_mode = 'ROUND'
                    curve_bevel_depth(0.3)
                    insert_keys(cur_frame)

                    # Second Frame += 12
                    curve_scale(1)
                    curve_bevel_depth(0.0)
                    insert_keys(cur_frame + 15)



            for o in bpy.context.selected_objects:
                if o.name.startswith('circle_explode'):
                    o.select_set(0)


        #exploding_circle_main()


        def place_at_cursor():

        # If some Object(s) - selected -
                for o in sel_ob:
                    bpy.context.view_layer.objects.active = o
                    cursor.location = o.location

                    exploding_circle_main()



        if sel_ob:
            place_at_cursor()
            

        if not sel_ob:
            exploding_circle_main()


        # Scene Update
        bpy.context.view_layer.update()
 

        
        # Cursor back
        cursor.location = cursor_previous_location
           


        return {'FINISHED'}





#================================================================
#          End Circle Explode
#================================================================





classes = [OBJECT_OT_circle_explode_anbx]



def register():
    for cls in classes:
        bpy.utils.register_class(cls)


    bpy.types.Scene.align_to_view_bool_mdh = bpy.props.BoolProperty(
        name="",
        description="ALIGH TO VIEW",
        default = False
    )




def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)

    del bpy.types.Scene.align_to_view_bool_mdh


if __name__ == "__main__":
    register()



