import bpy
import bmesh 
import string
import random
import math

from bpy.props import (BoolProperty, FloatProperty, IntProperty)
from bpy.types import (Panel)

 
class OBJECT_OT_spin_wobble_rig_anbx(bpy.types.Operator):
    '''Spin Wobble Rig -
- Set up a rig to spin/wobble cylindrical objects, like a bottle or coin...'''
    bl_idname = "object.spin_wobble_rig_anbx"
    bl_label = "Spin Wobble Rig"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' 
                and len(context.selected_objects) == 1
                and context.object.type == 'MESH')


    def execute(self, context):
        

        
        # Ref
        num_of_full_rotations = context.scene.number_of_full_rotations_anbx
        spin_frames = context.scene.number_of_spin_frames_anbx      
        
        
        # Set active Scene Collection
        scene_collection = bpy.context.view_layer.layer_collection
        bpy.context.view_layer.active_layer_collection = scene_collection        
                
        
        ob  = bpy.context.object
        ob_name = ob.name

  
        # Cursor -----------------------------------------
        cursor = bpy.context.scene.cursor
        cursor_location = bpy.context.scene.cursor.location
        cursor_previous_location = cursor.location.copy()   
        


        # Insert Keyframe 
        def insert_y_rotation_keys(obj, axis, first_frame_val, sec_frame_val):
            
            cur_frame = bpy.context.scene.frame_current  
            obj.rotation_euler[axis] = first_frame_val
            obj.keyframe_insert(data_path = 'rotation_euler', frame= cur_frame)

            next_frame = cur_frame + spin_frames

            bpy.context.scene.frame_set(next_frame)
            obj.rotation_euler[axis] = sec_frame_val
            obj.keyframe_insert(data_path = 'rotation_euler', frame= next_frame)
            
            bpy.context.scene.frame_set(cur_frame)
            
            
        
        # Random Name ---------------------------------------------
        
        def random_name(obj, length):

            source = string.ascii_lowercase + string.digits
            result_str = ''.join((random.choice(source) for i in range(length)))

            name_spl = obj.name[:3]
            rand_name = ( '_' + result_str + '_')

            obj_name = name_spl + rand_name
            if obj is not None:
                obj.name = obj_name
                if obj.data:
                    obj.data.name = obj_name 

        #-----------------------------------------------------------

                 
 
        # Negate Rotation Empty ------------------------------------
        
        def create_negate_rotation_empty():
            
    
            for empty_ob in bpy.data.objects:
                if empty_ob.name.startswith('Negate_Rot_Empty'):
                    random_name(empty_ob, 6)
       
                    

            for o in bpy.context.selected_objects:    

                empty = bpy.data.objects.new("Negate_Rot_Empty" + '.' + o.name, None )
                o.empty_display_type = 'PLAIN_AXES'

                o.users_collection[-1].objects.link(empty)
                
                empty.location = o.location
                
                bpy.context.evaluated_depsgraph_get().update()
                
                display_size = o.dimensions.z
                if display_size < 1:
                    empty.empty_display_size = 1
                
                elif display_size == 1:
                    empty.empty_display_size = 1.2
                    
                elif display_size > 1:
                    empty.empty_display_size = 1 + (display_size - 1)
                    
                o.parent = empty

                o.matrix_parent_inverse = empty.matrix_world.inverted() 

                empty.select_set(0)
                o.select_set(1)

                bpy.context.view_layer.objects.active = o


        create_negate_rotation_empty()


        
        bpy.ops.object.mode_set(mode='EDIT', toggle=False)
        bpy.ops.mesh.select_mode(use_extend=False, use_expand=False, type='VERT')
        bpy.ops.mesh.select_all(action='DESELECT')
        bpy.ops.object.mode_set(mode='OBJECT', toggle=False)        
        


        # Edit Mode
        def select_lowest_verts():
            o  = bpy.context.object  
            mw = o.matrix_world   

            glob_vert_co = [ mw @ v.co for v in o.data.vertices ]  

            minZ = min( [ co.z for co in glob_vert_co ] ) 

            for v in o.data.vertices:
                if (mw @ v.co).z == minZ: 
                    v.select = True
                    
                else:
                    v.select = False                    


        select_lowest_verts()
        
        # ------------------------------------------------------
        
        
        bpy.ops.object.mode_set(mode='EDIT', toggle=False)        
        

        bpy.ops.mesh.duplicate_move()
        bpy.ops.mesh.separate(type='SELECTED')
        bpy.ops.object.mode_set(mode='OBJECT', toggle=False)


        sel_ob = bpy.context.selected_objects

        bpy.context.view_layer.objects.active = sel_ob[1]

        sel_ob[0].select_set(0)


        obj_anbx = bpy.context.object 
        o_tem_dx = obj_anbx.dimensions.x
        rightmost_pt = min([(obj_anbx.matrix_world @ v.co).x for v in obj_anbx.data.vertices]) 
        lowest_pt_tmp = min([(obj_anbx.matrix_world @ v.co).z for v in obj_anbx.data.vertices]) 
        cursor.location = obj_anbx.location
        cursor.location.z = lowest_pt_tmp

        cursor.location.x += o_tem_dx / 2 
        
        bpy.data.objects.remove(sel_ob[1], do_unlink=True)


 
        # Select Main Object ----------------------
        
        main_obj = bpy.data.objects[ob_name]
        main_obj.select_set(True)
        bpy.context.view_layer.objects.active = main_obj


        #---------------------------- Tilt Ctrl -----------------------------------
        #--------------------------------------------------------------------------
        #--------------------------------------------------------------------------
        
        def create_tilt_ctrl():
            
            for tilt_ob in bpy.data.objects:
                if tilt_ob.name.startswith('Tilt_Ctrl'):
                    random_name(tilt_ob, 6)
            
            ob = bpy.context.object
            ob_name = ob.name
            
            
            odx = ob.dimensions.x
            ody = ob.dimensions.y
            odz = ob.dimensions.z
            
            rad = odx - (odx / 10)
            
            highest_pt = max([(ob.matrix_world @ v.co).z for v in ob.data.vertices])
            
            olx = ob.location.x
            oly = ob.location.y
            olz = ob.location.z
         
            bpy.ops.mesh.primitive_circle_add(vertices=24, radius=rad, enter_editmode=False, align='WORLD', location=(olx, oly, highest_pt), scale=(1, 1, 1))
            
            bpy.context.evaluated_depsgraph_get().update()            
            
            bpy.ops.object.editmode_toggle()
            
            bpy.ops.mesh.select_all(action='DESELECT')
            
            indices = [0, 6, 12, 18] 

            obj = bpy.context.object
            obj.name = 'Tilt_Ctrl' + '.' + ob_name 
            mesh = obj.data
            bm = bmesh.from_edit_mesh(mesh)

            vertices= [i for i in bm.verts]

            for vert in vertices:
                if vert.index in indices:
                    vert.select = True
                else:
                    vert.select = False

            bmesh.update_edit_mesh(mesh)  
             
            bpy.ops.transform.translate(value=(-0, -0, -rad / 4))
            
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            
            bpy.ops.object.origin_set(type='ORIGIN_CURSOR', center='MEDIAN')
            
            tilt_ob =  bpy.context.object
      
            for obj in bpy.data.objects:
                if obj.name.startswith('Negate_Rot_Empty'):
 
                    obj.parent = tilt_ob  
                    obj.matrix_parent_inverse = tilt_ob.matrix_world.inverted() 


            # insert keys
 
            insert_y_rotation_keys(tilt_ob, 1, 0.6, 0.0)
                          
            # Select Main Object ----------------------
            
            main_obj = bpy.data.objects[ob_name]
            main_obj.select_set(True)
            bpy.context.view_layer.objects.active = main_obj   
            tilt_ob.select_set(False)       
                
         
        create_tilt_ctrl()
        
        


        #-------------------------------- Spin Ctrl -------------------------------
        #--------------------------------------------------------------------------
        #--------------------------------------------------------------------------
        
        def create_spin_ctrl():
                
            
            for spin_ob in bpy.data.objects:
                if spin_ob.name.startswith('Spin_Ctrl'):
                    random_name(spin_ob, 6)
            
            ob = bpy.context.object
            ob_name = ob.name
            
            
            odx = ob.dimensions.x
            ody = ob.dimensions.y
            odz = ob.dimensions.z
            
            rad = odx - (odx / 10)
            
            lowest_pt = min([(ob.matrix_world @ v.co).z for v in ob.data.vertices])
            
            olx = ob.location.x
            oly = ob.location.y
            olz = ob.location.z
         
            bpy.ops.mesh.primitive_circle_add(vertices=32, radius=rad, enter_editmode=False, align='WORLD', location=(olx, oly, lowest_pt), scale=(1, 1, 1))
            
            bpy.context.evaluated_depsgraph_get().update() 
            
            bpy.ops.object.editmode_toggle()
            
            bpy.ops.mesh.select_all(action='DESELECT')
            
            indices = [0, 8, 16, 24]

            obj = bpy.context.object
            obj.name = 'Spin_Ctrl' + '.' + ob_name 
            mesh = obj.data
            bm = bmesh.from_edit_mesh(mesh)

            vertices= [i for i in bm.verts]

            for vert in vertices:
                if vert.index in indices:
                    vert.select = True
                else:
                    vert.select = False

            bmesh.update_edit_mesh(mesh)  

            # Transform
            bpy.ops.transform.resize(value=(1.2, 1.2, 1.2))
            bpy.ops.transform.rotate(value= 0.3, orient_axis='Z')    
          


            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            
            bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='MEDIAN')

            
            spin_ob = bpy.context.object
      
            for obj in bpy.data.objects:
                if obj.name.startswith('Tilt_Ctrl'):
 
                    obj.parent = spin_ob  
                    obj.matrix_parent_inverse = spin_ob.matrix_world.inverted() 
                  
                    
            revolt_num = 360 * num_of_full_rotations
            
            spin_num = math.radians(revolt_num)
            # insert keys
            insert_y_rotation_keys(spin_ob, 2, 0.0, spin_num) 
                  
                  
                
            spin_ob = bpy.context.object
 
            # Adjust F-Curve for 'realistic' ending
            def rotate_curve_handle(name):
                
                ob_fcurve = spin_ob.animation_data.action.fcurves
                    
                for fc in ob_fcurve:
                    if fc.data_path.endswith(('rotation_euler')):

                        if fc.array_index == 2:

                            fc.keyframe_points[1].handle_right_type = 'FREE'
                            fc.keyframe_points[1].handle_left_type = 'FREE'             
                            fc.keyframe_points[1].handle_left.y -= (spin_frames / (math.pi))
                            fc.keyframe_points[1].handle_left.x += (num_of_full_rotations)

                            fc.update()
                            
            rotate_curve_handle(spin_ob.name)          
          
          
        create_spin_ctrl() 



        # Negate Driver -------------------------------------
        
        def create_negate_rot_driver():
            
            for spin_ob in bpy.data.objects:
                if spin_ob.name.startswith('Spin_Ctrl'): 
                    spin_ob_name = spin_ob.name           
            
            for empty_ob in bpy.data.objects:
                if empty_ob.name.startswith('Negate_Rot_Empty'):          
                
                    n_dr = empty_ob.driver_add('rotation_euler', 2)
                    
                    var = n_dr.driver.variables.new()
                    var.name = 'var'
                    var.type =  'TRANSFORMS'
                    
                    n_dr.driver.type = 'SCRIPTED'
                    target = var.targets[0]
                    target.id = bpy.data.objects.get(spin_ob_name)
                    target.transform_type = 'ROT_Z'

                    n_dr.driver.expression = "var * -1"
                     
        create_negate_rot_driver()




        #-------------------------------- Main Ctrl -------------------------------
        def create_main_ctrl():
                
            
            for main_ob in bpy.data.objects:
                if main_ob.name.startswith('Main_Spin_Wobble_Ctrl'):
                    random_name(main_ob, 6)         
            
            ob = bpy.context.object
            ob_name = ob.name
            
            
            odx = ob.dimensions.x
            ody = ob.dimensions.y
            odz = ob.dimensions.z
            
            rad = odx - (odx / 4)
            
            lowest_pt = min([(ob.matrix_world @ v.co).z for v in ob.data.vertices])
            
            olx = ob.location.x
            oly = ob.location.y
            olz = ob.location.z
         
            bpy.ops.mesh.primitive_circle_add(vertices=32, radius=rad, enter_editmode=False, align='WORLD', location=(olx, oly, lowest_pt), scale=(1, 1, 1))
            
            bpy.context.evaluated_depsgraph_get().update() 
            

            obj = bpy.context.object
            obj.name = 'Main_Spin_Wobble_Ctrl'

            bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='MEDIAN')

            
            main_ob = bpy.context.object
          
            for obj in bpy.data.objects:
                if obj.name.startswith('Spin_Ctrl'):
         
                    obj.parent = main_ob  
                    obj.matrix_parent_inverse = main_ob.matrix_world.inverted() 
                    
          
        create_main_ctrl() 


        # Cursor back ----------------------------
        cursor.location = cursor_previous_location 
 

        return {'FINISHED'}







# =====================================================
#              SPIN PANEL
# =====================================================


class PANEL_PT_spin_rig_animbox_ax(bpy.types.Panel):
    bl_label = "Spin Rig"
    bl_idname = 'PANEL_PT_spin_rig_animbox_ax'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Animbox"
    bl_parent_id = 'PANEL_PT_animbox_ax_04'
    bl_options = {'DEFAULT_CLOSED'}   
    bl_order = 0

 

    def draw(self, context):
        layout = self.layout


        box = layout.box()  
        
        row = box.row()
        row.label(text=" Full Rotations")
        row.scale_x = 0.6
        row.prop(bpy.context.scene, "number_of_full_rotations_anbx")
        row = box.row()
        row.label(text=" Frames", icon='KEYFRAME')
        row.scale_x = 0.6
        row.prop(bpy.context.scene, "number_of_spin_frames_anbx")           
        row = box.row()
        row.scale_y = 1.2
        row.operator("object.spin_wobble_rig_anbx", icon='MESH_CYLINDER')

#---------------------------------------------------------------------------------


classes = [ OBJECT_OT_spin_wobble_rig_anbx,
            PANEL_PT_spin_rig_animbox_ax,
            
          ]
 
 
 
def register():
    for cls in classes:
        bpy.utils.register_class(cls)
        


    #--------------- Number of Full Rotations ---------
    
    bpy.types.Scene.number_of_full_rotations_anbx = bpy.props.IntProperty(
        name="",
        default=8,
        description="Number of Full Rotations",
        min=1,
        
    )
 
    #-------------------------------------------------------------
 
    #----------------------- Number of Spin Frames ---------------
    
    bpy.types.Scene.number_of_spin_frames_anbx = bpy.props.IntProperty(
        name="",
        default=80,
        description="Number of Frames",
        min=0,
    )



def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)

 
    del bpy.types.Scene.number_of_full_rotations_anbx
    del bpy.types.Scene.number_of_spin_frames_anbx
 
 
 
if __name__ == "__main__":
    register() 