import bpy
import bmesh


#------------------------------------------------------------------------------

class OBJECT_OT_trim_circle_anbx(bpy.types.Operator):
    '''"Trim" Circle
- will be placed at the 3D Cursor position 
- Use "G" to move one or multiple Empties'''
    bl_idname = "object.trim_circle_anbx"
    bl_label = 'Circle Flow'
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.mode == 'OBJECT')


    def execute(self, context):


        ob = bpy.context.object
        sel_ob = bpy.context.selected_objects
        act_ob = bpy.context.active_object


        cursor = bpy.context.scene.cursor
        cursor_location = cursor.location

 
        reg_d = bpy.context.region_data

        def create_trim_obj():

            bpy.ops.curve.primitive_bezier_circle_add(radius=1,
                                                      enter_editmode=False,
                                                      align='WORLD',
                                                      location=(cursor.location),
                                                      scale=(1, 1, 1), rotation=(1.5708, 0.0, 0.0))



            for o in bpy.context.selected_objects:

                o.name = 'trim_circle'
                o.data.name = 'trim_circle'

                bpy.ops.object.transform_apply(location=False, rotation=True, scale=False)


        create_trim_obj()

        act_ob_name = bpy.context.active_object.name



        #########====== Vert ====== #########

        def create_vert():

            
            bpy.ops.mesh.primitive_plane_add(size=2, enter_editmode=False)

            ob = bpy.context.object
            ob.name = 'trim_plane'
            ob.data.name = ob.name

            bpy.ops.object.mode_set(mode='EDIT', toggle=False)
            bpy.ops.mesh.merge(type='CENTER')    


            bpy.ops.transform.translate(value=(-1, 0, 0))
            bpy.ops.mesh.extrude_region_move(TRANSFORM_OT_translate={"value":(2, 0, 0)})


            bpy.context.object.modifiers.new('Skin', 'SKIN')
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)


            ob = bpy.context.object
            for v in ob.data.skin_vertices[0].data:
                v.radius = 0.1, 0.1

            bpy.context.object.modifiers.new('Curve', 'CURVE')
            ob.modifiers["Curve"].object = bpy.data.objects[act_ob_name]
            ob.modifiers["Skin"].use_smooth_shade = True

            bpy.context.object.modifiers["Curve"].deform_axis = 'NEG_X'


            bpy.ops.object.subdivision_set(level=2, relative=False)

            bpy.ops.object.mode_set(mode='EDIT', toggle=False)

            ob = bpy.context.object
            bm = bmesh.from_edit_mesh( ob.data )

            for i in range( len( bm.verts ) ):
                bpy.ops.mesh.select_all( action = 'DESELECT' ) # Deselect all vertices
                bm = bmesh.from_edit_mesh( ob.data )
                bm.verts.ensure_lookup_table()
                v  = bm.verts[i]
                v.select = True
                bm.select_flush( True )
                bpy.ops.object.hook_add_newob()

            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)


        create_vert()


        def add_new_mat_to_active():

            ob = bpy.context.active_object
            if ob.name.startswith('trim_plane'):
                mat_name = ob.name

            mat = bpy.data.materials.get(mat_name)
            if mat is None:
                mat = bpy.data.materials.new(name=mat_name + '_mat')


            if ob.data.materials:
                ob.data.materials[0] = mat
            else:
                ob.data.materials.append(mat)

            bpy.context.object.active_material.use_nodes = True

            for n in bpy.data.materials:

                num = 17
                # Check Blender Version
                blender_version = bpy.app.version[0]

                if blender_version == 2:
                    num_emission_color = 17
                    num_emission_strength = 18

                if blender_version == 3:
                    num_emission_color = 19
                    num_emission_strength = 20

                if blender_version == 4:
                    num_emission_color = 26
                    num_emission_strength = 27




                if n.name.startswith(mat_name):
                    n.node_tree.nodes["Principled BSDF"].inputs[num_emission_color].default_value = (0.88, 0.84, 0.57, 1)
                    n.node_tree.nodes["Principled BSDF"].inputs[num_emission_strength].default_value = 5


        add_new_mat_to_active()


        def lock_transform():

            bpy.context.object.lock_location[1] = True
            bpy.context.object.lock_location[2] = True

            bpy.context.object.lock_rotation[0] = True
            bpy.context.object.lock_rotation[1] = True
            bpy.context.object.lock_rotation[2] = True

            bpy.context.object.lock_scale[0] = True
            bpy.context.object.lock_scale[1] = True
            bpy.context.object.lock_scale[2] = True


        hook_name = []

        d_ob = bpy.data.objects

        e_type = bpy.types.Object.bl_rna.properties['empty_display_type'].enum_items.keys()


        for d in d_ob:
            if d.name.startswith('trim_plane'):
                for m in d.modifiers:
                    if m.name.startswith('Hook'):
                        hook_name.append(m.object.name)


        for o in d_ob:
            if o.name in hook_name:
                o.select_set(1)
                bpy.context.view_layer.objects.active = o
                bpy.context.object.empty_display_size = 0.5
                bpy.context.object.empty_display_type = e_type[-3]
                lock_transform()
                bpy.ops.object.transforms_to_deltas(mode='LOC')

                o.select_set(0)


        return {'FINISHED'}

#---------------------------------------------------------------------------

classes = [ OBJECT_OT_trim_circle_anbx ]



def register():
    for cls in classes:
        bpy.utils.register_class(cls)



def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)

 

if __name__ == "__main__":
    register()
