import bpy
from mathutils import Matrix
from math import radians

import random
import string
 
from bpy.props import (IntProperty)


#----------------------------------------------------------------
# Selected Bone - circular array - Z Axis

class OBJECT_OT_circular_array_bone_z_axis_anbx(bpy.types.Operator):
    '''Selected Bone - circular array - Z Axis'''
    bl_idname = "object.circular_array_bone_z_axis_anbx"
    bl_label = "Circular Array - Z Axis"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.type == 'ARMATURE' and
                context.mode == 'EDIT_ARMATURE' and
                len(context.selected_editable_bones) == 1)



    def execute(self, context):
        
        bpy.context.scene.tool_settings.transform_pivot_point = 'MEDIAN_POINT'
        
        cur = bpy.context.scene.cursor
        cur_loc_x = cur.location.x
        cur_loc_y = cur.location.y
        cur_loc_z = cur.location.z
  
        # Ref
        array_bone = context.scene.circular_array_bone_count_anbx

          
        ob = bpy.context.object

        act_bn = ob.data.edit_bones.active
        
        cur.location = act_bn.head
        
        b_length = act_bn.length
        
        cur.location.x += b_length
        act_bn.tail = cur.location
        
        
        def random_name(length):


            source = string.ascii_lowercase + string.digits
            result_str = ''.join((random.choice(source) for i in range(length)))

            name_spl = act_bn.name[:3]
            rand_name = ( '_' + result_str + '_')

            act_bn.name = name_spl + rand_name


        random_name(6)         
        

        name = act_bn.name
        
        for i in range(array_bone):
            bpy.ops.armature.duplicate()
            
        for bn in ob.data.edit_bones:
            if name in bn.name:
                bn.select = 1            
                bn.select_head = 1            
                bn.select_tail = 1    


        sel_bones = bpy.context.selected_bones


        num = 0

        bone_num = len(sel_bones)

        num_main = 360 / bone_num
        

        for b in sel_bones:

            old_head = b.head.copy()
            R = Matrix.Rotation(radians(30+ num), 4, b.z_axis.normalized())  
            b.transform(R, roll=True)
            offset_vec = -(b.head - old_head)
            b.head += offset_vec
            b.tail += offset_vec
            
            num += num_main

            # Move all selected bone away for 0.5
            b.head.x +=  b.vector.x * 0.5
            b.head.y +=  b.vector.y * 0.5
            b.head.z +=  b.vector.z * 0.5

            b.tail.x +=  b.vector.x  
            b.tail.y +=  b.vector.y  
            b.tail.z +=  b.vector.z  
         



        return {'FINISHED'}

#----------------------------------------------------------------


#----------------------------------------------------------------
# Selected Bone - circular array - Y Axis

class OBJECT_OT_circular_array_bone_y_axis_anbx(bpy.types.Operator):
    '''Selected Bone - circular array - Y Axis'''
    bl_idname = "object.circular_array_bone_y_axis_anbx"
    bl_label = "Circular Array - Y Axis"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.type == 'ARMATURE' and
                context.mode == 'EDIT_ARMATURE' and
                len(context.selected_editable_bones) == 1)



    def execute(self, context):
        
        bpy.context.scene.tool_settings.transform_pivot_point = 'MEDIAN_POINT'
        

        # Ref
        array_bone = context.scene.circular_array_bone_count_anbx

        ob = bpy.context.object

        act_bn = ob.data.edit_bones.active
        

        
        def random_name(length):


            source = string.ascii_lowercase + string.digits
            result_str = ''.join((random.choice(source) for i in range(length)))

            name_spl = act_bn.name[:3]
            rand_name = ( '_' + result_str + '_')

            act_bn.name = name_spl + rand_name


        random_name(6)         
        

        name = act_bn.name
        
        for i in range(array_bone):
            bpy.ops.armature.duplicate()
            
        for bn in ob.data.edit_bones:
            if name in bn.name:
                bn.select = 1            
                bn.select_head = 1            
                bn.select_tail = 1    


        sel_bones = bpy.context.selected_bones

        num = 0

        bone_num = len(sel_bones)

        num_main = 360 / bone_num
        

        for b in sel_bones:

            old_head = b.head.copy()
            R = Matrix.Rotation(radians(30+ num), 4, b.z_axis.normalized())  
            b.transform(R, roll=True)
            offset_vec = -(b.head - old_head)
            b.head += offset_vec
            b.tail += offset_vec
            
            num += num_main

            # Move all selected bone away for 0.5
            b.head.x +=  b.vector.x * 0.5
            b.head.y +=  b.vector.y * 0.5
            b.head.z +=  b.vector.z * 0.5

            b.tail.x +=  b.vector.x  
            b.tail.y +=  b.vector.y  
            b.tail.z +=  b.vector.z  
         


        return {'FINISHED'}

#----------------------------------------------------------------

#----------------------------------------------------------------
# Selected Bone - circular array - X Axis

class OBJECT_OT_circular_array_bone_x_axis_anbx(bpy.types.Operator):
    '''Selected Bone - circular array - X Axis'''
    bl_idname = "object.circular_array_bone_x_axis_anbx"
    bl_label = "Circular Array - X Axis"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.type == 'ARMATURE' and
                context.mode == 'EDIT_ARMATURE' and
                len(context.selected_editable_bones) == 1)



    def execute(self, context):
        
        bpy.context.scene.tool_settings.transform_pivot_point = 'MEDIAN_POINT'
        

        # Ref
        array_bone = context.scene.circular_array_bone_count_anbx

        ob = bpy.context.object

        act_bn = ob.data.edit_bones.active
        

        
        def random_name(length):

            source = string.ascii_lowercase + string.digits
            result_str = ''.join((random.choice(source) for i in range(length)))

            name_spl = act_bn.name[:3]
            rand_name = ( '_' + result_str + '_')

            act_bn.name = name_spl + rand_name


        random_name(6)         
        

        name = act_bn.name
        
        for i in range(array_bone):
            bpy.ops.armature.duplicate()
            
        for bn in ob.data.edit_bones:
            if name in bn.name:
                bn.select = 1            
                bn.select_head = 1            
                bn.select_tail = 1    


        sel_bones = bpy.context.selected_bones

        num = 0

        bone_num = len(sel_bones)

        num_main = 360 / bone_num
        

        for b in sel_bones:

            old_head = b.head.copy()
            R = Matrix.Rotation(radians(30+ num), 4, b.x_axis.normalized())  
            b.transform(R, roll=True)
            offset_vec = -(b.head - old_head)
            b.head += offset_vec
            b.tail += offset_vec
            
            num += num_main

            # Move all selected bone away for 0.5
            b.head.x +=  b.vector.x * 0.5
            b.head.y +=  b.vector.y * 0.5
            b.head.z +=  b.vector.z * 0.5

            b.tail.x +=  b.vector.x  
            b.tail.y +=  b.vector.y  
            b.tail.z +=  b.vector.z  
         


        return {'FINISHED'}

#----------------------END SELECTED Bone Circular Array-----------------------------------
#-----------------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------------




#--------------------------AUTO Circular Array---------------------------
#------------------------------------------------------------------------
#------------------------------------------------------------------------

# Bone -  Auto circular array - Z Axis

class OBJECT_OT_auto_circular_array_bone_z_axis_anbx(bpy.types.Operator):
    '''Auto circular array - Bone - Z Axis'''
    bl_idname = "object.auto_circular_array_bone_z_axis_anbx"
    bl_label = "Auto Circular Array - Z Axis"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.type == 'ARMATURE' and
                context.mode == 'EDIT_ARMATURE')


    def execute(self, context):
        
        bpy.ops.armature.select_all(action='DESELECT')

        # Ref
        array_bone = context.scene.circular_array_bone_count_anbx

        cur = bpy.context.scene.cursor
        cur_loc_x = cur.location.x
        cur_loc_y = cur.location.y
        cur_loc_z = cur.location.z
        
        ob = bpy.context.object
        
        def random_name(length):
            
            name = 'bone_circ_arr_bn'

            source = string.ascii_lowercase + string.digits
            result_str = ''.join((random.choice(source) for i in range(length)))

            name_spl = name[:3]
            rand_name = ( '_' + result_str + '_')

            name = name_spl + rand_name     
            
            return name
        
        rn = random_name(6)   
        
        
        def new_bone():

            new_bone = ob.data.edit_bones.new(rn)
            new_bone.head = cur.location
            
            new_bone.tail = (cur_loc_x, cur_loc_y + 1, cur_loc_z)
            
            new_bone.select = 1
            ob.data.edit_bones.active = new_bone
 
                
            
        for i in range(array_bone):  
            new_bone()


        sel_bones = bpy.context.selected_bones


        num = 0

        bone_num = len(sel_bones)

        num_main = 360 / bone_num
        

        for b in sel_bones:

            old_head = b.head.copy()
            R = Matrix.Rotation(radians(30+ num), 4, b.z_axis.normalized())  
            b.transform(R, roll=True)
            offset_vec = -(b.head - old_head)
            b.head += offset_vec
            b.tail += offset_vec
            
            num += num_main

            # Move all selected bone away for 0.5
            b.head.x +=  b.vector.x * 0.5
            b.head.y +=  b.vector.y * 0.5
            b.head.z +=  b.vector.z * 0.5

            b.tail.x +=  b.vector.x  
            b.tail.y +=  b.vector.y  
            b.tail.z +=  b.vector.z  
         


        return {'FINISHED'}

#----------------------------------------------------------------



#----------------------------------------------------------------
# Bone -  Auto circular array - Y Axis

class OBJECT_OT_auto_circular_array_bone_y_axis_anbx(bpy.types.Operator):
    '''Auto circular array - Bone - Y Axis'''
    bl_idname = "object.auto_circular_array_bone_y_axis_anbx"
    bl_label = "Auto Circular Array - Y Axis"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.type == 'ARMATURE' and
                context.mode == 'EDIT_ARMATURE')


    def execute(self, context):
        
        bpy.ops.armature.select_all(action='DESELECT')

        # Ref
        array_bone = context.scene.circular_array_bone_count_anbx


        cur = bpy.context.scene.cursor
        cur_loc_x = cur.location.x
        cur_loc_y = cur.location.y
        cur_loc_z = cur.location.z
        
        ob = bpy.context.object
        
        def random_name(length):
            
            name = 'bone_circ_arr_bn'


            source = string.ascii_lowercase + string.digits
            result_str = ''.join((random.choice(source) for i in range(length)))

            name_spl = name[:3]
            rand_name = ( '_' + result_str + '_')

            name = name_spl + rand_name     
            
            return name
        
        rn = random_name(6)   
        
 
        def new_bone():

            new_bone = ob.data.edit_bones.new(rn)
            new_bone.head = cur.location
            
            new_bone.tail = (cur_loc_x, cur_loc_y, cur_loc_z + 1)
            
            new_bone.select = 1
            ob.data.edit_bones.active = new_bone
 
                   
        for i in range(array_bone):  
            new_bone()

        sel_bones = bpy.context.selected_bones

        num = 0

        bone_num = len(sel_bones)

        num_main = 360 / bone_num
        
        for b in sel_bones:

            old_head = b.head.copy()
            R = Matrix.Rotation(radians(30+ num), 4, b.z_axis.normalized())  
            b.transform(R, roll=True)
            offset_vec = -(b.head - old_head)
            b.head += offset_vec
            b.tail += offset_vec
            
            num += num_main

            # Move all selected bone away for 0.5
            b.head.x +=  b.vector.x * 0.5
            b.head.y +=  b.vector.y * 0.5
            b.head.z +=  b.vector.z * 0.5

            b.tail.x +=  b.vector.x  
            b.tail.y +=  b.vector.y  
            b.tail.z +=  b.vector.z  
         


        return {'FINISHED'}

#----------------------------------------------------------------



#----------------------------------------------------------------
# Bone -  Auto circular array - X Axis

class OBJECT_OT_auto_circular_array_bone_x_axis_anbx(bpy.types.Operator):
    '''Auto circular array - Bone - X Axis'''
    bl_idname = "object.auto_circular_array_bone_x_axis_anbx"
    bl_label = "Auto Circular Array - X Axis"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.type == 'ARMATURE' and
                context.mode == 'EDIT_ARMATURE')


    def execute(self, context):
        
        bpy.ops.armature.select_all(action='DESELECT')

        # Ref
        array_bone = context.scene.circular_array_bone_count_anbx

        cur = bpy.context.scene.cursor
        cur_loc_x = cur.location.x
        cur_loc_y = cur.location.y
        cur_loc_z = cur.location.z
        
        ob = bpy.context.object
        
        def random_name(length):
            
            name = 'bone_circ_arr_bn'


            source = string.ascii_lowercase + string.digits
            result_str = ''.join((random.choice(source) for i in range(length)))

            name_spl = name[:3]
            rand_name = ( '_' + result_str + '_')

            name = name_spl + rand_name     
            
            return name
        
        rn = random_name(6)   
        
 
        def new_bone():

            new_bone = ob.data.edit_bones.new(rn)
            new_bone.head = cur.location
            
            new_bone.tail = (cur_loc_x, cur_loc_y, cur_loc_z + 1)
            
            new_bone.select = 1
            ob.data.edit_bones.active = new_bone
 
                
        for i in range(array_bone):  
            new_bone()

        
        sel_bones = bpy.context.selected_bones

        num = 0

        bone_num = len(sel_bones)

        num_main = 360 / bone_num
        

        for b in sel_bones:

            old_head = b.head.copy()
            R = Matrix.Rotation(radians(30+ num), 4, b.x_axis.normalized())  
            b.transform(R, roll=True)
            offset_vec = -(b.head - old_head)
            b.head += offset_vec
            b.tail += offset_vec
            
            num += num_main

            # Move all selected bone away for 0.5
            b.head.x +=  b.vector.x * 0.5
            b.head.y +=  b.vector.y * 0.5
            b.head.z +=  b.vector.z * 0.5

            b.tail.x +=  b.vector.x  
            b.tail.y +=  b.vector.y  
            b.tail.z +=  b.vector.z  
         


        return {'FINISHED'}

#------------------------------------------------------------------------------------------


#-----------------------END AUTO Circular Array--------------------------------------------
#------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------











#------------------------------------------------------------------------------------------
#---------------------------MOVE IN OUT----------------------------------------------------
#------------------------------------------------------------------------------------------


# Move Selected Bones Out

class OBJECT_OT_move_selected_bone_out_anbx(bpy.types.Operator):
    '''Move Selected Bones Out'''
    bl_idname = "object.move_selected_bone_out_anbx"
    bl_label = "Move Out"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.type == 'ARMATURE' and
                context.mode == 'EDIT_ARMATURE' and
                len(context.selected_editable_bones) != 0)



    def execute(self, context):
        
        sel_bones = bpy.context.selected_bones

        for s in sel_bones:
            s.head.x +=  s.vector.x * 0.5
            s.head.y +=  s.vector.y * 0.5
            s.head.z +=  s.vector.z * 0.5

            s.tail.x +=  s.vector.x  
            s.tail.y +=  s.vector.y  
            s.tail.z +=  s.vector.z  
 

        return {'FINISHED'}


#----------------------------------------------------------------
#----------------------------------------------------------------
#----------------------------------------------------------------


# Move Selected Bones IN

class OBJECT_OT_move_selected_bone_in_anbx(bpy.types.Operator):
    '''Move Selected Bones In'''
    bl_idname = "object.move_selected_bone_in_anbx"
    bl_label = "Move In"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.type == 'ARMATURE' and
                context.mode == 'EDIT_ARMATURE' and
                len(context.selected_editable_bones) != 0)



    def execute(self, context):
        
        sel_bones = bpy.context.selected_bones

        for s in sel_bones:

            s.tail.x -=  s.vector.x  * 0.5
            s.tail.y -=  s.vector.y  * 0.5
            s.tail.z -=  s.vector.z  * 0.5 

            s.head.x -=  s.vector.x 
            s.head.y -=  s.vector.y  
            s.head.z -=  s.vector.z              
 

        return {'FINISHED'}


#----------------------------------------------------------------
#----------------------------------------------------------------
#----------------------------------------------------------------


    

        
        
        

classes = [ OBJECT_OT_circular_array_bone_z_axis_anbx,
            OBJECT_OT_circular_array_bone_y_axis_anbx,
            OBJECT_OT_circular_array_bone_x_axis_anbx,
            OBJECT_OT_auto_circular_array_bone_z_axis_anbx,
            OBJECT_OT_auto_circular_array_bone_y_axis_anbx,
            OBJECT_OT_auto_circular_array_bone_x_axis_anbx,
            OBJECT_OT_move_selected_bone_out_anbx,
            OBJECT_OT_move_selected_bone_in_anbx,

          ]
 
 
 
def register():
    for cls in classes:
        bpy.utils.register_class(cls)
        
        
        
    #----------------------- Circular Array Bone Count ------------------------------
    
    bpy.types.Scene.circular_array_bone_count_anbx = bpy.props.IntProperty(
        name="",
        default=3,
        description="Circular Array - number of Bones",
        min=3,
    )
    

    
  

 
def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)

 
    del bpy.types.Scene.circular_array_bone_count_anbx

    
     
 
if __name__ == "__main__":
    register() 