import bpy




#--------------------------------------------------------------------------
#-----------------Select Bone - children recursive------------------------- 
#--------------------------------------------------------------------------


class OBJECT_OT_select_bone_children_anbx(bpy.types.Operator):
    '''Select Bone Children'''
    bl_idname = "object.select_bone_children_anbx"
    bl_label = "Select Children"
    bl_options = {'REGISTER', 'UNDO'}

    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.type == 'ARMATURE' and
                context.mode == 'EDIT_ARMATURE' or 
                context.mode == 'POSE')


    def execute(self, context):


        def select_bone_children_pose_mode():
            
            sel_bones = bpy.context.selected_pose_bones
            act_bone = bpy.context.active_pose_bone
            
         
            for s in sel_bones:   
                act_bone = s  
                children = act_bone.children_recursive
         
                for c  in children:
                    c.bone.select = 1
                    


        def select_bone_children_edit_mode():
            
            sel_bones = bpy.context.selected_bones
            act_bone = bpy.context.active_bone 
            
            for s in sel_bones:
                act_bone = s
                children = act_bone.children_recursive

                for c  in children:
                    c.select = 1



        mode = bpy.context.mode
        
        if mode == 'POSE':
            select_bone_children_pose_mode()
        
        if mode == 'EDIT_ARMATURE':
            select_bone_children_edit_mode()        
        

        return {'FINISHED'}


#--------------------------------------------------------------------------
#-----------------Select Bone - children recursive------------------------- 
#------------------------------End-----------------------------------------



#--------------------------------------------------------------------------
#--------------- Select Similar - Name of Active Bone ---------------------
#--------------------------------------------------------------------------


# Select Similar - Name of Active Bone
class OBJECT_OT_select_similar_by_name_active_bone_anbx(bpy.types.Operator):
    '''Select similar - by name of Active Bone'''
    bl_idname = "object.select_similar_by_name_active_bone_anbx"
    bl_label = "Select Similar"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.type == 'ARMATURE' and
                context.mode == 'EDIT_ARMATURE' or 
                context.mode == 'POSE')

    def execute(self, context):
        
        mode = bpy.context.mode

        if mode == 'POSE':
            act_bone = bpy.context.active_pose_bone
        
        if mode == 'EDIT_ARMATURE':
            act_bone = bpy.context.active_bone
  
        
        name = act_bone.name.split('.')


        if len(name) == 1:
            get_name = name[0] + '*'  
            bpy.ops.object.select_pattern(pattern=get_name, case_sensitive=False)  
            
        if len(name) > 1 and len(name) < 3:
            if name[1] != 'L' and name[1] != 'R':
                get_name = name[0] + '*'
                bpy.ops.object.select_pattern(pattern=get_name, case_sensitive=False)      

         
        # If Name has 'L' of 'R' after first '.'  
        if len(name) > 1:
            if name[1] == 'L':
                name_l = name[0] + '.' + name[1]
                get_name_l = name_l + '*'
                bpy.ops.object.select_pattern(pattern=get_name_l, case_sensitive=False)
            
            if name[1] == 'R':
                name_r = name[0] + '.' + name[1]
                get_name_r = name_r + '*'
                bpy.ops.object.select_pattern(pattern=get_name_r, case_sensitive=False)    
            

        # If Name ends with 'L' or 'R' 
        if len(name) > 1:
            if name[-1] == 'L':
                name_l = name[0] + '*' + name[-1]
                bpy.ops.object.select_pattern(pattern=name_l, case_sensitive=False) 
                
                      
        if len(name) > 1:
            if name[-1] == 'R':
                name_r = name[0] +  '*' + name[-1]
                bpy.ops.object.select_pattern(pattern=name_r, case_sensitive=False)  



        return {'FINISHED'}

#--------------------------------------------------------------------------
#--------------- Select Similar - Name of Active Bone ---------------------
#------------------------------ End ---------------------------------------





# Select Bone Head ----------------------------------------------------------- 

class OBJECT_OT_select_bone_head_anbx(bpy.types.Operator):
    '''Select Bone Head'''
    bl_idname = "object.select_bone_head_anbx"
    bl_label = "Select Bone Head"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.type == 'ARMATURE' and
                context.mode == 'EDIT_ARMATURE' and
                len(context.selected_editable_bones) != 0)

    def execute(self, context):

        sel_bones = bpy.context.selected_bones
        
        for bn in sel_bones:
            bn.select = 0          
            bn.select_head = 1            
            bn.select_tail = 0              


        return {'FINISHED'}
    
# Select Bone Tail -----------------------------------------------------------    
    
class OBJECT_OT_select_bone_tail_anbx(bpy.types.Operator):
    '''Select Bone Tail'''
    bl_idname = "object.select_bone_tail_anbx"
    bl_label = "Select Bone Tail"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.type == 'ARMATURE' and
                context.mode == 'EDIT_ARMATURE' and
                len(context.selected_editable_bones) != 0)

    def execute(self, context):
        
        sel_bones = bpy.context.selected_bones
        for bn in sel_bones:
            bn.select = 0          
            bn.select_head = 0           
            bn.select_tail = 1 


        return {'FINISHED'}
    
        
#-----------------------------------------------------------------------------







classes = [ 
            OBJECT_OT_select_bone_children_anbx,
            OBJECT_OT_select_similar_by_name_active_bone_anbx,
            OBJECT_OT_select_bone_head_anbx,
            OBJECT_OT_select_bone_tail_anbx,

 
          ]



def register():
    for cls in classes:
        bpy.utils.register_class(cls)




def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)



if __name__ == "__main__":
    register() 


