import bpy
from bpy.props import (BoolProperty, FloatProperty, IntProperty)




#-----------------------------------------------------------
#------------- Create Mesh on selected Bones ---------------
#-----------------------------------------------------------


class OBJECT_OT_create_mesh_on_sel_bones_anbx(bpy.types.Operator):
    '''Create Mesh on Selected Bones'''
    bl_idname = "object.create_mesh_on_sel_bones_anbx"
    bl_label = "Add Mesh"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.type == 'ARMATURE' and
                context.mode == 'EDIT_ARMATURE' or 
                context.mode == 'POSE')


    def execute(self, context):
        
        # Cursor -----------------------------------------
        cursor = bpy.context.scene.cursor
        cursor_previous_location = cursor.location.copy()   
        #-------------------------------------------------         
        
        
        mode = bpy.context.mode
        
        mode_list = []
        mode_list.append(mode)
        
        # Ref --------------------------------------------------------
        parent_to_act_bone = context.scene.mesh_parent_to_bone_anbx      
        box_to_bone = context.scene.mesh_box_to_bone_anbx       
        cylinder_to_bone = context.scene.mesh_cylinder_to_bone_anbx     
        sphere_to_bone = context.scene.mesh_sphere_to_bone_anbx    
        cone_to_bone = context.scene.mesh_cone_to_bone_anbx  
        
        bone_tip = context.scene.align_mesh_to_bone_tip_anbx 
        bone_body = context.scene.align_mesh_to_bone_body_anbx 
        bone_root = context.scene.align_mesh_to_bone_root_anbx

        add_size = context.scene.size_radius_of_mesh_to_bone_anbx
        #-------------------------------------------------------------
        

        bpy.ops.object.mode_set(mode='OBJECT', toggle=False)  
        
        ob = bpy.context.object    
        arm_name = ob.name 

        arm = bpy.data.objects[arm_name] 
        arm_data_name = arm.data.name
        

        arm_bones = bpy.data.armatures[arm_data_name].bones 

        for b in arm_bones:
            if b.select:
                bone_length = b.length
                b_name = b.name

                bpy.context.scene.cursor.location = b.head_local
                
                clx = bpy.context.scene.cursor.location.x
                cly = bpy.context.scene.cursor.location.y
                clz = bpy.context.scene.cursor.location.z
                
                # By Default = Cube
                if not box_to_bone and not cylinder_to_bone and not sphere_to_bone and not cone_to_bone:
                    bpy.ops.mesh.primitive_cube_add(enter_editmode=False, align='WORLD', 
                    location=(clx, cly, clz), 
                    scale=((bone_length / 6.283)+add_size, (bone_length / 6.283)+add_size, (bone_length / 6.283) + add_size))       
 
                 
                # BOX
                if box_to_bone:
                    bpy.ops.mesh.primitive_cube_add(enter_editmode=False, align='WORLD', 
                    location=(clx, cly, clz), 
                    scale=((bone_length / 6.283)+add_size, bone_length / 2.2, (bone_length / 6.283)+add_size))       

                # CYLINDER
                if cylinder_to_bone:                
                    bpy.ops.mesh.primitive_cylinder_add(vertices=8, radius=(bone_length / 6.283)+add_size, depth=(bone_length - (bone_length / 10)), 
                    end_fill_type='NOTHING', enter_editmode=False, align='WORLD', 
                    location=(clx, cly, clz), rotation=(-1.5708, 0, 0), scale=(1, 1, 1)) 
                     
                    bpy.ops.object.transform_apply(location=False, rotation=True, scale=False)
                
                # SPHERE
                if sphere_to_bone: 
                    bpy.ops.mesh.primitive_uv_sphere_add(segments=12, ring_count=8, radius= (bone_length / 2.1)+add_size, 
                    enter_editmode=False, align='WORLD', location=(0, 0, 0), 
                    rotation=(-1.5708, 0, 0), scale=(1, 1, 1))   
                                      
                    bpy.ops.object.transform_apply(location=False, rotation=True, scale=False)
                                    
                    
                # CONE
                if cone_to_bone:             
                    bpy.ops.mesh.primitive_cone_add(vertices=8, radius1=(bone_length / 6.283)+add_size, radius2=0, depth=(bone_length - (bone_length / 10)), 
                    end_fill_type='NOTHING', enter_editmode=False, align='WORLD', location=(0, 0, 0), 
                    rotation=(-1.5708, 0, 0), scale=(1, 1, 1))                
                    
                    bpy.ops.object.transform_apply(location=False, rotation=True, scale=False)
                                              
                
                # Object Name
                ob = bpy.context.object  
                ob.name = 'Mesh2Bone' 
                ob.data.name = 'Mesh2Bone'       
                       
                # Constraint ------------------------------------------
                
                const = ob.constraints.new(type='COPY_TRANSFORMS')
                const.name = "CT"   
                     
                const.target = bpy.data.objects[arm_name] 
                const.subtarget = b_name  
                
                if bone_body:
                    const.head_tail = 0.5  
                     
                if bone_tip:
                    const.head_tail = 1.0 
                                     
                if bone_root:
                    const.head_tail = 0.0               
                
                if not bone_body and not bone_tip and not bone_root:
                    const.head_tail = 0.5 
                
                 
         
                if parent_to_act_bone:
                    ob.parent = arm
                    ob.parent_bone = b_name
                    ob.parent_type = 'BONE'
                
                bpy.ops.constraint.apply(constraint="CT", owner='OBJECT')
            
                bpy.ops.object.transforms_to_deltas(mode='ALL')

                bpy.ops.object.select_all(action='DESELECT')
                
 
                
        arm.select_set(1)
        bpy.context.view_layer.objects.active = arm   
        bpy.ops.object.select_all(action='DESELECT')
        
        if mode_list[0] == 'EDIT_ARMATURE':
            bpy.ops.object.mode_set(mode='EDIT', toggle=False)        
                 
        if mode_list[0] == 'POSE':
            bpy.ops.object.posemode_toggle()        
                 
        # Cursor back
        cursor.location = cursor_previous_location           

        return {'FINISHED'}

#-----------------------------------------------------------
#------------- Create Mesh on selected Bones ---------------
#--------------------------- End ---------------------------




#----------------------------------------------------------------------------       
#----------------------------------------------------------------------------       
#----------------------------------------------------------------------------       
 
def add_mesh_box_anbx(self, context):
    sc = bpy.context.scene

    sc.property_unset('mesh_cylinder_to_bone_anbx')
    sc.property_unset('mesh_sphere_to_bone_anbx')        
    sc.property_unset('mesh_cone_to_bone_anbx')
 
        
def add_mesh_cylinder_anbx(self, context):
    sc = bpy.context.scene

    sc.property_unset('mesh_box_to_bone_anbx')
    sc.property_unset('mesh_sphere_to_bone_anbx')        
    sc.property_unset('mesh_cone_to_bone_anbx')        


def add_mesh_cone_anbx(self, context):
    sc = bpy.context.scene

    sc.property_unset('mesh_box_to_bone_anbx')
    sc.property_unset('mesh_cylinder_to_bone_anbx')
    sc.property_unset('mesh_sphere_to_bone_anbx')        
      

def add_mesh_sphere_anbx(self, context):
    sc = bpy.context.scene

    sc.property_unset('mesh_box_to_bone_anbx')
    sc.property_unset('mesh_cylinder_to_bone_anbx')       
    sc.property_unset('mesh_cone_to_bone_anbx')       
 
 
 
 
def align_mesh_to_bone_anbx(self, context):
    sc = bpy.context.scene

    sc.property_unset('align_mesh_to_bone_root_anbx')       
    sc.property_unset('align_mesh_to_bone_tip_anbx')      
 
 
def align_mesh_to_bone_head_anbx(self, context):
    sc = bpy.context.scene

    sc.property_unset('align_mesh_to_bone_body_anbx')      
    sc.property_unset('align_mesh_to_bone_tip_anbx')       
  
 
def align_mesh_to_bone_tail_anbx(self, context):
    sc = bpy.context.scene

    sc.property_unset('align_mesh_to_bone_body_anbx')
    sc.property_unset('align_mesh_to_bone_root_anbx')       
  


#----------------------------------------------------------------------------        
        

classes = [ 
            OBJECT_OT_create_mesh_on_sel_bones_anbx,
 
          ]
 
 
 
def register():
    for cls in classes:
        bpy.utils.register_class(cls)
        


    #------------------- Mesh to Bone - Size / Radius ----------------

    bpy.types.Scene.size_radius_of_mesh_to_bone_anbx = bpy.props.FloatProperty(
        name="Adjust Size-Radius",
        default=0.01,
        description="Size / Radius of the Mesh",
        step=1,
        precision=2,

    )

    #------------------- Parent 'Mesh2Bone' to Bone -------------------
    
    bpy.types.Scene.mesh_parent_to_bone_anbx = bpy.props.BoolProperty(
        name=" Parent to Bone",
        description="Parent to Bone",
        default = False,
    )

    #-------------------------------------------------------------


    #------------------- Box Bool -------------------
    
    bpy.types.Scene.mesh_box_to_bone_anbx = bpy.props.BoolProperty(
        name="",
        description="Add Mesh - Box to Selected Bones",
        default = False,
        update = add_mesh_box_anbx,
    )

    #-------------------------------------------------------------
    #------------------- Cylinder Bool -------------------
    
    bpy.types.Scene.mesh_cylinder_to_bone_anbx = bpy.props.BoolProperty(
        name="",
        description="Add Mesh - Cylinder to Selected Bones",
        default = False,
        update = add_mesh_cylinder_anbx,
    )

    #-------------------------------------------------------------

    #------------------- Cone Bool -------------------
    

    bpy.types.Scene.mesh_cone_to_bone_anbx = bpy.props.BoolProperty(
        name="",
        description="Add Mesh - Cone to Selected Bones",
        default = False,
        update = add_mesh_cone_anbx,
    )

    #-------------------------------------------------------------

    #------------------- Sphere Bool -------------------
    

    bpy.types.Scene.mesh_sphere_to_bone_anbx = bpy.props.BoolProperty(
        name="",
        description="Add Mesh - Sphere to Selected Bones",
        default = False,
        update = add_mesh_sphere_anbx,
    )

    #-------------------------------------------------------------
    

    #----------------------- Align to to Bone Body - Bool -------------

    bpy.types.Scene.align_mesh_to_bone_body_anbx = bpy.props.BoolProperty(
        name="",
        description="Align Mesh to Bone Body",
        default = False,
        update = align_mesh_to_bone_anbx ,
    )   

    #-------------------------------------------------------------
    
    #----------------------- Align to to Bone Root (Head) - Bool 
    
    bpy.types.Scene.align_mesh_to_bone_root_anbx = bpy.props.BoolProperty(
        name="",
        description="Align Mesh to Bone Root (Head)",
        default = False,
        update = align_mesh_to_bone_head_anbx,
    )   


    #-------------------------------------------------------------
    
    #----------------------- Align to to Bone Tip (Tail) - Bool 
    
    bpy.types.Scene.align_mesh_to_bone_tip_anbx = bpy.props.BoolProperty(
        name="",
        description="Align Mesh to Bone Tip (Tail)",
        default = False,
        update = align_mesh_to_bone_tail_anbx,
    )   
    


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)

    del bpy.types.Scene.size_radius_of_mesh_to_bone_anbx
    del bpy.types.Scene.mesh_parent_to_bone_anbx
    del bpy.types.Scene.mesh_box_to_bone_anbx
    del bpy.types.Scene.mesh_cylinder_to_bone_anbx
    del bpy.types.Scene.mesh_cone_to_bone_anbx
    del bpy.types.Scene.mesh_sphere_to_bone_anbx

 
    del bpy.types.Scene.align_mesh_to_bone_body_anbx    
    del bpy.types.Scene.align_mesh_to_bone_root_anbx
    del bpy.types.Scene.align_mesh_to_bone_tip_anbx
 
 
if __name__ == "__main__":
    register() 