import bpy
from bpy.props import (BoolProperty, IntProperty)





############################## OFFSET KEYs ################################



#-----------------------------------------------------------------------------------
#------------------ Offset Bone Keyframes - Selected -------------------------------
#-----------------------------------------------------------------------------------

class OBJECT_OT_offset_keyframes_selected_bones_anbx(bpy.types.Operator):
    '''Offset Keyframes on Selected Bones'''
    bl_idname = "object.offset_keyframes_selected_bones_anbx"
    bl_label = "Offset Keyframes - Selected Bones"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
        context.mode == 'POSE' and len(context.selected_pose_bones) != 0)


    def execute(self, context):
        
        # Ref
        rev_bool = context.scene.revers_anim_bool_anbx
        frames_to_offset = context.scene.offset_frames_number_bone_anbx

        sel_bones = bpy.context.selected_pose_bones
        pose_bones = bpy.context.object.pose.bones

        def armature_name():
            ob = bpy.context.object
            return ob.name
      
        def bone_keyframes_offset():
            
            sel_bones = bpy.context.selected_pose_bones

            arm_name = armature_name()
            arm = bpy.data.objects[arm_name]
            act_bone = bpy.context.active_pose_bone

            offset = 0
            
            if rev_bool:
                sel_bones = reversed(sel_bones) 

            for b in sel_bones:
                act_bone = b
                act_bone_name = b.name

                if arm.animation_data:
                    if arm.animation_data.action:

                        fc = arm.animation_data.action.fcurves

                        for c in fc:
                            if act_bone_name in c.data_path:

                                keyframePoints = c.keyframe_points

                                for kf in keyframePoints:
                                    kf.co[0] += offset
                                    kf.handle_left[0] += offset
                                    kf.handle_right[0] += offset

                        offset += frames_to_offset

        bone_keyframes_offset()


        return {'FINISHED'}
    
#-----------------------------------------------------------------------------------
#------------------ Offset Bone Keyframes - Selected -------------------------------
#---------------------------------End-----------------------------------------------





#-----------------------------------------------------------------------------------
#------------------ Offset Bone Keyframes - Similar --------------------------------
#-----------------------------------------------------------------------------------

class OBJECT_OT_offset_keyframes_similar_bones_anbx(bpy.types.Operator):
    '''Offset Keyframes on Similar - by name of active Bone'''
    bl_idname = "object.offset_keyframes_similar_bones_anbx"
    bl_label = "Offset Keyframes - Similar Bones"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
        context.mode == 'POSE' and len(context.selected_pose_bones) != 0)


    def execute(self, context):
        
        def select_similar_by_act_bone_name():
        
            act_bone = bpy.context.active_pose_bone

            name = act_bone.name.split('.')


            if len(name) == 1:
                get_name = name[0] + '*'  
                bpy.ops.object.select_pattern(pattern=get_name, case_sensitive=False)  
                
            if len(name) > 1 and len(name) < 3:
                if name[1] != 'L' and name[1] != 'R':
                    get_name = name[0] + '*'
                    bpy.ops.object.select_pattern(pattern=get_name, case_sensitive=False)      

             
            # If Name has 'L' of 'R' after first '.'  
            if len(name) > 1:
                if name[1] == 'L':
                    name_l = name[0] + '.' + name[1]
                    get_name_l = name_l + '*'
                    bpy.ops.object.select_pattern(pattern=get_name_l, case_sensitive=False)
                
                if name[1] == 'R':
                    name_r = name[0] + '.' + name[1]
                    get_name_r = name_r + '*'
                    bpy.ops.object.select_pattern(pattern=get_name_r, case_sensitive=False)    
                

            # If Name ends with 'L' or 'R' 
            if len(name) > 1:
                if name[-1] == 'L':
                    name_l = name[0] + '*' + name[-1]
                    bpy.ops.object.select_pattern(pattern=name_l, case_sensitive=False) 
                    
                          
            if len(name) > 1:
                if name[-1] == 'R':
                    name_r = name[0] +  '*' + name[-1]
                    bpy.ops.object.select_pattern(pattern=name_r, case_sensitive=False)         
        
        
        
        
        # Ref -----------------------------------------------------
        rev_bool = context.scene.revers_anim_bool_anbx
        frames_to_offset = context.scene.offset_frames_number_bone_anbx

        sel_bones = bpy.context.selected_pose_bones
        pose_bones = bpy.context.object.pose.bones

        def armature_name():
            ob = bpy.context.object
            return ob.name
      
        def bone_keyframes_offset():
            
            sel_bones = bpy.context.selected_pose_bones

            arm_name = armature_name()
            arm = bpy.data.objects[arm_name]
            act_bone = bpy.context.active_pose_bone

            offset = 0
            
            if rev_bool:
                sel_bones = reversed(sel_bones) 

            for b in sel_bones:
                act_bone = b
                act_bone_name = b.name

                if arm.animation_data:
                    if arm.animation_data.action:

                        fc = arm.animation_data.action.fcurves

                        for c in fc:
                            if act_bone_name in c.data_path:

                                keyframePoints = c.keyframe_points

                                for kf in keyframePoints:
                                    kf.co[0] += offset
                                    kf.handle_left[0] += offset
                                    kf.handle_right[0] += offset

                        offset += frames_to_offset


        select_similar_by_act_bone_name()
        bone_keyframes_offset()


        return {'FINISHED'}
    
#-----------------------------------------------------------------------------------
#------------------ Offset Bone Keyframes - Similar --------------------------------
#---------------------------------End-----------------------------------------------
    






#--------------------------------------------------------------------------
#--------------------------------Add Cycles Modifier-----------------------
#--------------------------------------------------------------------------


#==========================================================================
#  Add Cycles Modifier to Selected Bones
#==========================================================================

 
class OBJECT_OT_add_cycles_modifier_to_sel_bones_anbx(bpy.types.Operator):
    '''Add Cycles Modifier to Selected Bones -
Location / Rotation / Scale

Alt + Click to Remove'''
    bl_idname = "object.add_cycles_modifier_to_sel_bones_anbx"
    bl_label = "Add Cycles Modifier to Selected"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        
        return (context.object is not None and
                 context.mode == 'POSE' 
                 and context.object.animation_data is not None
                 and len(context.selected_pose_bones) != 0)
                 
        
    def invoke(self, context, event):
        
        anim_data = bpy.context.object.animation_data 
        ob = bpy.context.object

        sel_bones = bpy.context.selected_pose_bones
 
            
        def add_pre_post_infinity_sel():
               
            for bone in sel_bones:
                bone_name = bone.name  

                if anim_data:
                    if anim_data.action:
                        ob_fcurve = ob.animation_data.action.fcurves
                       
                        for fc in ob_fcurve:
                            d_path = ('location', 'rotation_quaternion', 'rotation_euler', 'scale')
                            if fc.data_path.endswith((d_path)) and bone_name in fc.data_path:
                                for m in fc.modifiers:
                                   if (m.type == 'CYCLES'): 
                                       fc.modifiers.remove(m)
                                fc.modifiers.new(type='CYCLES')
                         
                            fc.update()     


        def remove_pre_post_infinity_sel():

            for bone in sel_bones:
                bone_name = bone.name  
                
                if anim_data:
                    if anim_data.action:
                        ob_fcurve = ob.animation_data.action.fcurves
                       
                        for fc in ob_fcurve:
                            d_path = ('location', 'rotation_quaternion', 'rotation_euler', 'scale')
                            if fc.data_path.endswith((d_path)) and bone_name in fc.data_path:
                                for m in fc.modifiers:
                                   if (m.type == 'CYCLES'): 
                                       fc.modifiers.remove(m)

                         
                            fc.update()     

 
        if event: 
            add_pre_post_infinity_sel()  
                     
 
        if event.alt:
            remove_pre_post_infinity_sel()
            
            
        return {'FINISHED'}


#==========================================================================
# END - ADD CYCLES MODIFIER - PRE POST INFINITY
#==========================================================================




classes = [ 

            OBJECT_OT_offset_keyframes_selected_bones_anbx,
            OBJECT_OT_offset_keyframes_similar_bones_anbx,
            OBJECT_OT_add_cycles_modifier_to_sel_bones_anbx,
 
          ]



def register():
    for cls in classes:
        bpy.utils.register_class(cls)




    #-----------------------Reverse Offset Bool-------------------


    bpy.types.Scene.revers_anim_bool_anbx = bpy.props.BoolProperty(
        name="",
        description="REVERSED OFFSET",
        default = False
    )

    #-------------------------------------------------------------
    
    #----------------------- Frames to Offset Bool ---------------
    
    bpy.types.Scene.offset_frames_number_bone_anbx = bpy.props.IntProperty(
        name="Offset Frames",
        default=1,
        description="Frames to Offset",
        min=1,
    )


    #-------------------------------------------------------------


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)



    del bpy.types.Scene.revers_anim_bool_anbx
    del bpy.types.Scene.offset_frames_number_bone_anbx



if __name__ == "__main__":
    register() 


 