import bpy
import math
from math import radians, degrees
import random
import string

from bpy.props import (BoolProperty, IntProperty)



#===========================================================
#            Circular Array -around the 3D cursor
#===========================================================

class OBJECT_OT_circular_array_around_cursor_anbx(bpy.types.Operator):
    '''Circular Array around the 3D cursor

-Alt Click = Insert Keys (Loc / Rot / Scale)'''

    bl_idname = "object.circular_array_around_cursor_anbx"
    bl_label = "Around 3D Cursor"
    bl_options = {'REGISTER', 'UNDO'}

    @classmethod
    def poll(cls, context):

        ob_types = ['MESH', 'CURVE', 'SURFACE', 'META', 
                    'FONT',  'ARMATURE', 'LATTICE', ]

        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) == 1 and
                context.object.type in ob_types)

    def invoke(self, context, event):
  
        # Refs
        x_axis = context.scene.circular_array_x_bool_anbx
        y_axis = context.scene.circular_array_y_bool_anbx
        z_axis = context.scene.circular_array_z_bool_anbx  
        
        array_oc = context.scene.array_object_count_anbx  
  
        cursor = bpy.context.scene.cursor
        sel_ob = bpy.context.selected_objects
        ob = bpy.context.object

        if x_axis or y_axis or z_axis:
            
            # FONTS - Text to Mesh
            def text_to_mesh():
                bpy.ops.object.convert(target='MESH')
                bpy.ops.object.mode_set(mode='EDIT')
                bpy.ops.mesh.select_all(action='SELECT')
                bpy.ops.mesh.remove_doubles()

                bpy.ops.object.mode_set(mode='OBJECT', toggle=False)

                bpy.ops.object.transform_apply(location=False, rotation=True, scale=False)
                bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='MEDIAN')

                ob = bpy.context.object
                mod = ob.modifiers.get("Font_Split")

                if mod is None:
                    ob.modifiers.new('Font_Split', 'EDGE_SPLIT')

            if ob.type == 'FONT':
                text_to_mesh()
     
            # Active Collection
            act_col = bpy.context.view_layer.active_layer_collection
            
            # Change Name
     
            def random_name(length):

                ob = bpy.context.object

                source = string.ascii_lowercase + string.digits
                result_str = ''.join((random.choice(source) for i in range(length)))

                name_spl = ob.name[:3]
                rand_name = ( '_' + result_str + '_')

                ob.name = name_spl + rand_name
                if ob.data:
                    ob.data.name = ob.name

            random_name(6)

            # Apply Rotation / Scale
            bpy.ops.object.transform_apply(location=False, rotation=True, scale=True)

            # Origin to Cursor
            bpy.ops.object.origin_set(type='ORIGIN_CURSOR', center='MEDIAN')

            def create_circular_array(num):

                # Number of Copies
                degr_num = 360 / num
                # degrees to radians
                pi_val = math.radians(degr_num)
             
                rot = pi_val
                for i in range(num - 1):
                    orig_ob = bpy.data.objects[ob.name]
                    dupl_ob = bpy.data.objects.new(ob.name, orig_ob.data)
                    act_col.collection.objects.link(dupl_ob)
                    dupl_ob.location = orig_ob.location
                    
     
                    if x_axis:
                        dupl_ob.rotation_euler.x += rot
             
                    if y_axis:
                        dupl_ob.rotation_euler.y += rot         
             
                    if z_axis:
                        dupl_ob.rotation_euler.z += rot       
     
                    rot += pi_val
             
            # Int Property => Object Count

            create_circular_array(array_oc)


            ##==================== Select by Type/Name =============

            # Select Objects by Name / Type
            def select_by_name_type():

                act_name = bpy.context.view_layer.objects.active.name
                act_type = bpy.context.view_layer.objects.active.type

             
                name = act_name.split('.')
                name_spl = name[0]

                for obj in bpy.context.scene.objects:
                    if obj.name.startswith(name_spl) and obj.type == act_type:
                        obj.select_set(1)


            select_by_name_type()

            ##==================== Select by Type/Name =============


            # Copy Modifier
            bpy.ops.object.make_links_data(type='MODIFIERS')


            ## Make Single User
            bpy.ops.object.make_single_user(object=True, obdata=True, material=True, animation=True)
            
            # Name
            for o in bpy.context.selected_objects:
                o.name = o.name 
                o.data.name = o.name 
                

            # Set Origing to Geometry, Scene Pivot Point to 'INDIVIDUAL_ORIGINS'
            # Add Parent Empty to Selected

            bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='MEDIAN')

            bpy.context.scene.tool_settings.transform_pivot_point = 'INDIVIDUAL_ORIGINS'

            # Parent Empty

            ca_empty = bpy.data.objects.new( "Circular Array", None )    
            act_col.collection.objects.link(ca_empty)
            ca_empty.empty_display_size = 1
            ca_empty.empty_display_type = 'SPHERE'    
            ca_empty.location = cursor.location

            ca_empty.select_set(1)
            bpy.context.view_layer.objects.active = ca_empty
            
            # Show Empty in Viewport
            bpy.context.object.show_name = True

            bpy.ops.object.parent_set(type='OBJECT')
            
            bpy.context.scene.transform_orientation_slots[0].type = 'LOCAL'

            ca_empty.select_set(0)


            for o in bpy.context.selected_objects:
                bpy.context.view_layer.objects.active = o            


        def insert_keys():
            for o in bpy.context.selected_objects:
                o.keyframe_insert(data_path='location')
                o.keyframe_insert(data_path='scale')
                o.keyframe_insert(data_path='rotation_euler') 
                

        if event.alt:
            insert_keys()

 
        return {'FINISHED'}

#===========================================================
#       End Circular Array around the 3D cursor 
#===========================================================



#===========================================================
#              Circular Array around self Origin
#===========================================================

class OBJECT_OT_circular_array_around_self_origin_anbx(bpy.types.Operator):
    '''Circular Array around self Origin

-Alt Click = Insert Keys (Loc / Rot / Scale)'''

    bl_idname = "object.circular_array_around_self_origin_anbx"
    bl_label = "Around Self Origin"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):

        ob_types = ['MESH', 'CURVE', 'SURFACE', 'META', 
                    'FONT',  'ARMATURE', 'LATTICE', ]

        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) == 1 and
                context.object.type in ob_types)


    def invoke(self, context, event):
  
        # Refs
        x_axis = context.scene.circular_array_x_bool_anbx
        y_axis = context.scene.circular_array_y_bool_anbx
        z_axis = context.scene.circular_array_z_bool_anbx
        
        array_oc = context.scene.array_object_count_anbx         
 
        sel_ob = bpy.context.selected_objects
        ob = bpy.context.object

 
        # Active Collection
        act_col = bpy.context.view_layer.active_layer_collection

        if x_axis or y_axis or z_axis:


            #----------------Fonts--------------------------

            act_ob = bpy.context.active_object

            # FONTS - Text to Mesh
            def text_to_mesh():
                bpy.ops.object.convert(target='MESH')
                bpy.ops.object.mode_set(mode='EDIT')
                bpy.ops.mesh.select_all(action='SELECT')
                bpy.ops.mesh.remove_doubles()

                bpy.ops.object.mode_set(mode='OBJECT', toggle=False)

                bpy.ops.object.transform_apply(location=False, rotation=True, scale=False)
                bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='MEDIAN') 
                        
                ob = bpy.context.object
                mod = ob.modifiers.get("Font_Split")

                if mod is None:
                    ob.modifiers.new('Font_Split', 'EDGE_SPLIT')


            if act_ob.type == 'FONT':
                text_to_mesh()
             
            #----------------Fonts--------------------------  

            def random_name(length):

                ob = bpy.context.object

                source = string.ascii_lowercase + string.digits
                result_str = ''.join((random.choice(source) for i in range(length)))


                name_spl = ob.name[:3]
                rand_name = ( '_' + result_str + '_')

                ob.name = name_spl + rand_name
                if ob.data:
                    ob.data.name = ob.name

            random_name(6) 

            # Apply Rotation / Scale
            if bpy.context.object.type == 'LIGHT':
                bpy.ops.object.transform_apply(location=False, rotation=False, scale=False)
                ob.rotation_euler.x = 0
                ob.rotation_euler.y = 0
                ob.rotation_euler.z = 0

            else:
                bpy.ops.object.transform_apply(location=False, rotation=True, scale=True)


            def create_circular_array(num):

                # Number of Copies
                degr_num = 360 / num
                # degrees to radians
                pi_val = math.radians(degr_num)
             
                rot = pi_val
                for i in range(num - 1):
                    orig_ob = bpy.data.objects[ob.name]
                    dupl_ob = bpy.data.objects.new(ob.name, orig_ob.data)
                    act_col.collection.objects.link(dupl_ob)
                    dupl_ob.location = orig_ob.location
                    
                    if x_axis:
                        dupl_ob.rotation_euler.x += rot
                        
                    if y_axis:
                        dupl_ob.rotation_euler.y += rot
                        
                    if z_axis:
                        dupl_ob.rotation_euler.z += rot                                        
             
                    rot += pi_val
             
            # Int Property => Object Count
            create_circular_array(array_oc)
                 
      
            #==================== Select by Type/Name =============

               # Select Objects by Name / Type
            def select_by_name_type():

                act_name = bpy.context.view_layer.objects.active.name
                act_type = bpy.context.view_layer.objects.active.type

             
                name = act_name.split('.')
                name_spl = name[0]

                for obj in bpy.context.scene.objects:
                    if obj.name.startswith(name_spl) and obj.type == act_type:
                        obj.select_set(1)
                        
                    # Deselect other
                    else:
                        obj.select_set(0)
                        

            select_by_name_type()


            #==================== Select by Type/Name =============

            # Copy Modifier
            bpy.ops.object.make_links_data(type='MODIFIERS')

            # Make Single User
            bpy.ops.object.make_single_user(object=False, obdata=True, material=True, animation=True)

            # Name
            for o in bpy.context.selected_objects:
                if o.data:
                    o.name = o.name 
                    o.data.name = o.name 
                    
            bpy.context.scene.tool_settings.transform_pivot_point = 'INDIVIDUAL_ORIGINS'
     
            # Parent Empty
            # Add Parent Empty to Selected
            def parent_empty():

                ca_empty = bpy.data.objects.new( "Circular Array", None )    
                act_col.collection.objects.link(ca_empty)
                ca_empty.empty_display_size = 1
                ca_empty.empty_display_type = 'SPHERE'    
                ca_empty.location = ob.location

                ca_empty.select_set(1)
                bpy.context.view_layer.objects.active = ca_empty

                # Show Empty in Viewport
                bpy.context.object.show_name = True

                bpy.ops.object.parent_set(type='OBJECT')

                bpy.context.scene.transform_orientation_slots[0].type = 'LOCAL'

                ca_empty.select_set(0)

                for o in bpy.context.selected_objects:
                    bpy.context.view_layer.objects.active = o                   


            parent_empty()
        

        def insert_keys():
            for o in bpy.context.selected_objects:
                if o.type in not_act_ob_types: 

                    o.keyframe_insert(data_path='location')
                    o.keyframe_insert(data_path='scale')
                    o.keyframe_insert(data_path='rotation_euler') 
              
 
        if event.alt:
            insert_keys()


        return {'FINISHED'}

#===========================================================
#       End Circular Array around self Origin
#===========================================================


#===========================================================
#      Circular Array around Active Object
#===========================================================

 
class OBJECT_OT_circular_array_around_active_anbx(bpy.types.Operator):
    '''Circular Array around Active Object

-Alt Click = Insert Keys (Loc / Rot / Scale)'''
    bl_idname = "object.circular_array_around_active_anbx"
    bl_label = "Around Active Object"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):

        ob_types = ['MESH', 'CURVE', 'SURFACE', 'META', 
                    'FONT',  'ARMATURE', 'LATTICE', ]

        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) == 2 and
                context.object.type in ob_types)


    def invoke(self, context, event):
  
        # Refs
        x_axis = context.scene.circular_array_x_bool_anbx
        y_axis = context.scene.circular_array_y_bool_anbx
        z_axis = context.scene.circular_array_z_bool_anbx
         
        array_oc = context.scene.array_object_count_anbx

        act_ob = bpy.context.active_object
          
        cursor = bpy.context.scene.cursor
        cursor_previous_location = cursor.location.copy()

        # Active Collection
        act_col = bpy.context.view_layer.active_layer_collection

        if x_axis or y_axis or z_axis:

            # Origin to Geo
            bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='MEDIAN')

            # ----------------------------------------
            sel_obj = bpy.context.selected_objects
            act_obj = bpy.context.view_layer.objects

            cursor.location = act_ob.location
                
            # Swap Object Selection
            if act_obj.active == sel_obj[0]:
                act_obj.active = sel_obj[-1]
                
                
            elif act_obj.active == sel_obj[-1]:
                act_obj.active = sel_obj[0]
                
                   
            #----------------Fonts--------------------------

            act_ob = bpy.context.active_object

            def text_to_mesh():
                bpy.ops.object.convert(target='MESH')
                bpy.ops.object.mode_set(mode='EDIT')  
                bpy.ops.mesh.select_all(action='SELECT')
                bpy.ops.mesh.remove_doubles()

                bpy.ops.object.mode_set(mode='OBJECT', toggle=False)

                bpy.ops.object.transform_apply(location=False, rotation=True, scale=False)
                bpy.ops.object.origin_set(type='ORIGIN_CENTER_OF_VOLUME', center='MEDIAN')
                
                ob = bpy.context.object
                mod = ob.modifiers.get("Font_Split")

                if mod is None:
                    ob.modifiers.new('Font_Split', 'EDGE_SPLIT')

            if act_ob.type == 'FONT':
                text_to_mesh()
             
                
            #----------------Fonts End --------------------------    
                
                
                
            # Random Name ---------------------------------------
            def random_name(length):

                ob = bpy.context.object

                source = string.ascii_lowercase + string.digits
                result_str = ''.join((random.choice(source) for i in range(length)))

                name_spl = ob.name[:3]
                rand_name = ( '_' + result_str + '_')

                ob.name = name_spl + rand_name
                if ob.data:
                    ob.data.name = ob.name

            random_name(6)
                  

            # Origin to Cursor
            bpy.ops.object.origin_set(type='ORIGIN_CURSOR', center='MEDIAN')

             
            def create_circular_array(num):

                act_ob = bpy.context.active_object
                # Number of Copies
                degr_num = 360 / num
                # degrees to radians
                pi_val = math.radians(degr_num)
             
                rot = pi_val
                for i in range(num - 1):
                    orig_ob = bpy.data.objects[act_ob.name]
                    dupl_ob = bpy.data.objects.new(act_ob.name, orig_ob.data)
                    act_col.collection.objects.link(dupl_ob)
                    dupl_ob.location = orig_ob.location
                    
                    if x_axis:
                        dupl_ob.rotation_euler.x += rot
                    if y_axis:
                        dupl_ob.rotation_euler.y += rot
                    if z_axis:
                        dupl_ob.rotation_euler.z += rot                    
                        
             
                    rot += pi_val
             

            bpy.ops.object.transform_apply(location=False, rotation=True, scale=True)
            create_circular_array(array_oc)
                 

            #==================== Select by Type/Name =============

               # Select Objects by Name / Type
            def select_by_name_type():

                act_name = bpy.context.view_layer.objects.active.name
                act_type = bpy.context.view_layer.objects.active.type

                name = act_name.split('.')
                name_spl = name[0]

                for obj in bpy.context.scene.objects:
                    if obj.name.startswith(name_spl) and obj.type == act_type:
                        obj.select_set(1)
                        
                    # Deselect other
                    else:
                        obj.select_set(0)
                        
            select_by_name_type()

            #==================== Select by Type/Name =============

            # Make Single User
            bpy.ops.object.make_single_user(object=False, obdata=True, material=True, animation=True)

            # Name
            for o in bpy.context.selected_objects:
                if o.data:
                    o.name = o.name 
                    o.data.name = o.name 
                    
                    bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='MEDIAN')


            # Parent Empty
            # Add Parent Empty to Selected
            def parent_empty():

                ca_empty = bpy.data.objects.new( "Circular Array", None )    
                act_col.collection.objects.link(ca_empty)
                ca_empty.empty_display_size = 1
                ca_empty.empty_display_type = 'SPHERE'    
                ca_empty.location = cursor.location

                ca_empty.select_set(1)
                bpy.context.view_layer.objects.active = ca_empty

                # Show Empty in Viewport
                bpy.context.object.show_name = True

                bpy.ops.object.parent_set(type='OBJECT')

                bpy.context.scene.transform_orientation_slots[0].type = 'LOCAL'

                ca_empty.select_set(0)

                for o in bpy.context.selected_objects:
                    bpy.context.view_layer.objects.active = o   

            parent_empty()


            # Cursor back
            cursor.location = cursor_previous_location

             
            def insert_keys():
                for o in bpy.context.selected_objects:
                    if o.type in not_act_ob_types: 

                        o.keyframe_insert(data_path='location')
                        o.keyframe_insert(data_path='scale')
                        o.keyframe_insert(data_path='rotation_euler') 
                  
     
            if event.alt:
                insert_keys()         
             
            bpy.context.scene.tool_settings.transform_pivot_point = 'INDIVIDUAL_ORIGINS'

          
        return {'FINISHED'}

 
#===========================================================
#       End Circular Array around Active Object
#===========================================================



#-------------------------- Fun ----------------------------

def reset_circular_array_x_fun_anbx(self, context):
    sc = bpy.context.scene

    sc.property_unset('circular_array_y_bool_anbx')
    sc.property_unset('circular_array_z_bool_anbx')
 

def reset_circular_array_y_fun_anbx(self, context):
    sc = bpy.context.scene

    sc.property_unset('circular_array_x_bool_anbx')
    sc.property_unset('circular_array_z_bool_anbx')
        

def reset_circular_array_z_fun_anbx(self, context):
    sc = bpy.context.scene

    sc.property_unset('circular_array_x_bool_anbx')
    sc.property_unset('circular_array_y_bool_anbx')
 


#-----------------------------------------------------------


classes = [ OBJECT_OT_circular_array_around_self_origin_anbx,
            OBJECT_OT_circular_array_around_cursor_anbx,
            OBJECT_OT_circular_array_around_active_anbx,

            
          ]
 
 
 
def register():
    for cls in classes:
        bpy.utils.register_class(cls)
        



    #----------------------- Object Count ------------------------------
    
    bpy.types.Scene.array_object_count_anbx = bpy.props.IntProperty(
        name="",
        default=3,
        description="Array Object Count",
        min=0,
    )
    
    
    #----------------------- Axis --------------------------------------

    bpy.types.Scene.circular_array_x_bool_anbx = bpy.props.BoolProperty(
        name="",
        description="X Axis",
        default = False,
        update = reset_circular_array_x_fun_anbx, 
    )


    bpy.types.Scene.circular_array_y_bool_anbx = bpy.props.BoolProperty(
        name="",
        description="Y Axis",
        default = False,
        update = reset_circular_array_y_fun_anbx, 
    )


    bpy.types.Scene.circular_array_z_bool_anbx = bpy.props.BoolProperty(
        name="",
        description="Z Axis",
        default = False,
        update = reset_circular_array_z_fun_anbx, 
    )


 
def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)

    del bpy.types.Scene.array_object_count_anbx
    del bpy.types.Scene.circular_array_x_bool_anbx
    del bpy.types.Scene.circular_array_y_bool_anbx
    del bpy.types.Scene.circular_array_z_bool_anbx
 
 
 
if __name__ == "__main__":
    register() 