import bpy
from bpy.props import (FloatProperty, IntProperty)



#--------------------------------------------------------------------------    
#--------------------------------Capsule-----------------------------------   
#--------------------------------------------------------------------------    
 


class OBJECT_OT_create_capsule_cbox(bpy.types.Operator):
    '''Create Capsule'''
    bl_idname = "object.create_capsule_cbox"
    bl_label = "Capsule"
    bl_options = {'REGISTER', 'UNDO'}


    init_mouse_x: IntProperty()
    init_value: FloatProperty()

    def modal(self, context, event):

        kc = bpy.context.window_manager.keyconfigs
        km = kc.active.keymaps['3D View'].keymap_items['view3d.cursor3d']


        # temp. change
        km.type = 'LEFTMOUSE'
        km.shift = 0  


        # Ref
        stay_in_sculpt_mode = context.scene.stay_in_sculpt_mode_cx  
        create_and_center = context.scene.create_and_center_x_axis_cx
        create_and_mirror = context.scene.create_and_mirror_cx


        def main_capsule():

            mode = bpy.context.mode
            if mode != 'OBJECT':
                bpy.ops.object.mode_set(mode='OBJECT', toggle=False)




            cursor = bpy.context.scene.cursor
            cursor_location = bpy.context.scene.cursor.location
            cursor_previous_location = cursor.location.copy()


            # Cube --------------------------------------------------------
            def add_cube():
                
                bpy.ops.mesh.primitive_cube_add(size=2, 
                                                enter_editmode=True, 
                                                align='WORLD', 
                                                location=(0, 0, 0), 
                                                scale=(0.5, 0.5, 0.5))
                
            #---------------------------------------------------------------
                                 
            add_cube()
            
            bpy.ops.mesh.select_all(action='DESELECT')
            bpy.ops.object.mode_set(mode='OBJECT')        
            

            ob = bpy.context.object
            ob.name = 'capsule_block_out' 
            ob.data.name = 'capsule_block_out' 
          
          
            for i in range(3):
                # sub mod
                ob.modifiers.new('Subdivision', type='SUBSURF')
                ob.modifiers["Subdivision"].levels = 1
                bpy.ops.object.modifier_apply(modifier="Subdivision")


                # cast mod
                ob.modifiers.new('Cast', type='CAST')
                ob.modifiers["Cast"].factor = 1
                bpy.ops.object.modifier_apply(modifier="Cast")



            # cast mod additional
            
            for i in range(8):
                ob.modifiers.new('Cast', type='CAST')
                ob.modifiers["Cast"].factor = 1
                bpy.ops.object.modifier_apply(modifier="Cast")


            
            bpy.ops.object.editmode_toggle()

            #----------------------------------------------------------------------
            
            # Cut in the middle
            bpy.ops.mesh.loopcut_slide(MESH_OT_loopcut={"number_cuts":1, "smoothness":0, 
                                                        "object_index":0, 
                                                        "edge_index":302, 
                                                        "mesh_select_mode_init":(False, True, False)}, 
                                                        TRANSFORM_OT_edge_slide={"value":0.8, })
                                                        
         
            # Scale 0 in Z Axis
            bpy.ops.transform.resize(value=(1, 1, 0))
            
            bpy.ops.mesh.select_all(action='SELECT')
          
            # Move all down 
            bpy.ops.transform.translate(value=(-0, -0, -0.005))


            
            def hemisphere():
                
                bpy.ops.mesh.select_all(action='DESELECT')
                bpy.ops.object.mode_set(mode='OBJECT')
                
                # Find and delete verts -------------------
                for v in bpy.context.object.data.vertices:
                    if v.co[2] < -0.02:
                        v.select = True
                bpy.ops.object.mode_set(mode='EDIT')
                bpy.ops.mesh.delete(type='VERT')
                
                #------------------------------------------    
                
                bpy.ops.mesh.select_all(action='DESELECT')
                bpy.ops.object.mode_set(mode='OBJECT')
                              
                # Find and Select verts -------------------     
                for v in bpy.context.object.data.vertices:
                    if v.co[2] < -0.0:
                        v.select = True     
                bpy.ops.object.mode_set(mode='EDIT')    
                
                bpy.ops.mesh.select_mode(use_extend=False, use_expand=False, type='VERT')
                bpy.ops.mesh.select_mode(use_extend=False, use_expand=False, type='EDGE')
      
      
                # Scale 0 in Z Axis
                bpy.ops.transform.resize(value=(1, 1, 0))
                
                # Move selected down
                bpy.ops.transform.translate(value=(-0, -0, -0.4))
                
                
                bpy.ops.view3d.snap_cursor_to_selected()
                
                bpy.ops.object.mode_set(mode='OBJECT')
                
                bpy.ops.object.origin_set(type='ORIGIN_CURSOR', center='MEDIAN')
                
                bpy.ops.object.editmode_toggle()

                
                # Cut in the middle
                bpy.ops.mesh.loopcut_slide(MESH_OT_loopcut={"number_cuts":5, 
                                                            "object_index":0, 
                                                            "edge_index":414, 
                                                            "mesh_select_mode_init":(False, True, False)})
                                                            
                    
                
                bpy.ops.mesh.select_all(action='DESELECT')        
                bpy.ops.object.mode_set(mode='OBJECT')
                
                bpy.ops.view3d.snap_cursor_to_center()
                bpy.ops.view3d.snap_selected_to_cursor(use_offset=False)

            
            hemisphere()
            
            ob = bpy.context.object
            ob.name = 'capsule_block_out' 
            ob.data.name = 'capsule_block_out' 
            
            mod_name = 'mirror_capsule'
            modifier = 'MIRROR'
            ob.modifiers.new(mod_name, type='MIRROR')        
          
            # Active mirror axis
            ob.modifiers["mirror_capsule"].use_axis[0] = False
            ob.modifiers["mirror_capsule"].use_axis[1] = False
            ob.modifiers["mirror_capsule"].use_axis[2] = True

            bpy.ops.object.modifier_apply(modifier="mirror_capsule")


            # Cursor back
            bpy.context.scene.cursor.location = cursor_previous_location

            # Selected to Cursor
            bpy.ops.view3d.snap_selected_to_cursor(use_offset=False)


            ob = bpy.context.object
            ob.scale.x /= 10
            ob.scale.y /= 10
            ob.scale.z /= 10


        #------------------------------------------------------------------------

        # Place 3D Cursor
        if event.type == 'LEFTMOUSE' and event.value == 'PRESS':
            return {'PASS_THROUGH'}  


        if event.type == 'LEFTMOUSE' and event.value == 'RELEASE':
            main_capsule()
           
                
            return {'RUNNING_MODAL'}
        
 
        sel_ob = bpy.context.selected_objects
        
        if sel_ob:
            obj = sel_ob[0]
            
            if event.type == 'MOUSEMOVE':
                delta = self.init_mouse_x - (event.mouse_x - event.mouse_prev_press_x)
                obj.scale.x = self.init_value + delta * -0.05
                obj.scale.y = self.init_value + delta * -0.05
                obj.scale.z = self.init_value + delta * -0.05


                # change mouse cursor
                bpy.context.window.cursor_modal_set("MOVE_X")

                # header info
                bpy.context.area.header_text_set('Right Click or press  ESC | SHIFT | CTRL | ALT | SPACEBAR | ENTER  -  to Confirm')


        if event.type in {'RIGHTMOUSE', 'SPACE', 'RET', 'LEFT_CTRL', 'LEFT_ALT', 'LEFT_SHIFT', 'ESC'} and event.value == 'PRESS':

            # restore keymaps
            km.type = 'RIGHTMOUSE'
            km.shift = 1           
            

            # restore mouse cursor
            bpy.context.window.cursor_modal_restore()

            # restore info header
            bpy.context.area.header_text_set(None)

            # apply scale
            bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)


            act_ob = bpy.context.active_object
            if stay_in_sculpt_mode:
                if act_ob is not None and act_ob.type == 'MESH':
                    bpy.ops.object.mode_set(mode='SCULPT', toggle=False)

            if create_and_center:
                bpy.ops.object.center_obj_cbox()


            if create_and_mirror:
                bpy.ops.object.mirror_mod_cbox()

            return {'FINISHED'}

        
        return {'RUNNING_MODAL'}


    def invoke(self, context, event):

        mode = bpy.context.mode
        if mode != 'OBJECT':
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)


        bpy.ops.object.select_all(action='DESELECT')

        context.window_manager.modal_handler_add(self)

        return {'RUNNING_MODAL'}



#--------------------------------------------------------------------------
#----------------------------Capsule End--------------------------------
#--------------------------------------------------------------------------



#---------------------------Tapered Capsule-------------------------------------
#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------

# Tapered Capsule

class OBJECT_OT_create_tapered_capsule_cbox(bpy.types.Operator):
    '''Create Tapered Capsule'''
    bl_idname = "object.create_tapered_capsule_cbox"
    bl_label = "Tapered Capsule"
    bl_options = {'REGISTER', 'UNDO'}


    init_mouse_x: IntProperty()
    init_value: FloatProperty()

    def modal(self, context, event):


        kc = bpy.context.window_manager.keyconfigs
        km = kc.active.keymaps['3D View'].keymap_items['view3d.cursor3d']


        # temp. change
        km.type = 'LEFTMOUSE'
        km.shift = 0  


        # Ref
        stay_in_sculpt_mode = context.scene.stay_in_sculpt_mode_cx  
        create_and_center = context.scene.create_and_center_x_axis_cx
        create_and_mirror = context.scene.create_and_mirror_cx


        def main_capsule():

            
            mode = bpy.context.mode
            if mode != 'OBJECT':
                bpy.ops.object.mode_set(mode='OBJECT', toggle=False)


            cursor = bpy.context.scene.cursor
            cursor_location = bpy.context.scene.cursor.location
            cursor_previous_location = cursor.location.copy()


            # Cube --------------------------------------------------------
            def add_cube():
                
                bpy.ops.mesh.primitive_cube_add(size=2, 
                                                enter_editmode=True, 
                                                align='WORLD', 
                                                location=(0, 0, 0), 
                                                scale=(0.5, 0.5, 0.5))
                
            #---------------------------------------------------------------
                                 
            add_cube()
            
            bpy.ops.mesh.select_all(action='DESELECT')
            bpy.ops.object.mode_set(mode='OBJECT')        
            

            ob = bpy.context.object
            ob.name = 'taper_capsule' 
            ob.data.name = 'taper_capsule' 
          
          
            for i in range(3):
                # sub mod
                ob.modifiers.new('Subdivision', type='SUBSURF')
                ob.modifiers["Subdivision"].levels = 1
                bpy.ops.object.modifier_apply(modifier="Subdivision")


                # cast mod
                ob.modifiers.new('Cast', type='CAST')
                ob.modifiers["Cast"].factor = 1
                bpy.ops.object.modifier_apply(modifier="Cast")


            # cast mod additional
            
            for i in range(10):
                ob.modifiers.new('Cast', type='CAST')
                ob.modifiers["Cast"].factor = 1
                bpy.ops.object.modifier_apply(modifier="Cast")


            
            bpy.ops.object.editmode_toggle()

            #----------------------------------------------------------------------
            
            # Cut in the middle
            bpy.ops.mesh.loopcut_slide(MESH_OT_loopcut={"number_cuts":1, "smoothness":0, 
                                                        "object_index":0, 
                                                        "edge_index":302, 
                                                        "mesh_select_mode_init":(False, True, False)}, 
                                                        TRANSFORM_OT_edge_slide={"value":0.8, })
                                                        
         
            # Scale 0 in Z Axis
            bpy.ops.transform.resize(value=(1, 1, 0))
            
            bpy.ops.mesh.select_all(action='SELECT')
          
            # Move all down 
            bpy.ops.transform.translate(value=(-0, -0, -0.005))


            def hemisphere():
                
                bpy.ops.mesh.select_all(action='DESELECT')
                bpy.ops.object.mode_set(mode='OBJECT')
                
                # Find and delete verts -------------------
                for v in bpy.context.object.data.vertices:
                    if v.co[2] < -0.02:
                        v.select = True
                bpy.ops.object.mode_set(mode='EDIT')
                bpy.ops.mesh.delete(type='VERT')
                
                #------------------------------------------    
                
                bpy.ops.mesh.select_all(action='DESELECT')
                bpy.ops.object.mode_set(mode='OBJECT')
                              
                # Find and Select verts -------------------     
                for v in bpy.context.object.data.vertices:
                    if v.co[2] < -0.0:
                        v.select = True     
                bpy.ops.object.mode_set(mode='EDIT')    
                
                bpy.ops.mesh.select_mode(use_extend=False, use_expand=False, type='VERT')
                bpy.ops.mesh.select_mode(use_extend=False, use_expand=False, type='EDGE')
      
      
                # Scale 0 in Z Axis
                bpy.ops.transform.resize(value=(1, 1, 0))
                
                # Move selected down
                bpy.ops.transform.translate(value=(-0, -0, -0.4))
                
                
                bpy.ops.view3d.snap_cursor_to_selected()
                
                bpy.ops.object.mode_set(mode='OBJECT')
                
                bpy.ops.object.origin_set(type='ORIGIN_CURSOR', center='MEDIAN')
                
                bpy.ops.object.editmode_toggle()

                     
                bpy.ops.mesh.select_all(action='DESELECT')        
                bpy.ops.object.mode_set(mode='OBJECT')
                
                bpy.ops.view3d.snap_cursor_to_center()
                bpy.ops.view3d.snap_selected_to_cursor(use_offset=False)

            
            hemisphere()
            

            ob = bpy.context.object

            mod_name = 'mirror_capsule'
            modifier = 'MIRROR'
            ob.modifiers.new(mod_name, type='MIRROR')        
          
            # Active mirror axis
            ob.modifiers["mirror_capsule"].use_axis[0] = False
            ob.modifiers["mirror_capsule"].use_axis[1] = False
            ob.modifiers["mirror_capsule"].use_axis[2] = True

            bpy.ops.object.modifier_apply(modifier="mirror_capsule")


            for v in bpy.context.object.data.vertices:
                if v.co[2] == 0.0:
                    v.select = True
            bpy.ops.object.mode_set(mode='EDIT')
            bpy.ops.mesh.select_mode(use_extend=False, use_expand=False, type='VERT')

            bpy.ops.mesh.delete_edgeloop()

            bpy.ops.mesh.select_all(action='DESELECT')
            bpy.ops.object.mode_set(mode='OBJECT')


            for v in bpy.context.object.data.vertices:
                if v.co[2] > 0.0:
                    v.select = True
            bpy.ops.object.mode_set(mode='EDIT')
            bpy.ops.mesh.select_mode(use_extend=False, use_expand=False, type='VERT')


            bpy.ops.transform.resize(value=(0.2, 0.2, 0.2))
            
            bpy.ops.mesh.vertices_smooth(factor=0.5)

            bpy.ops.mesh.loopcut_slide(MESH_OT_loopcut={"number_cuts":8, 
                                                        "object_index":0, 
                                                        "edge_index":789, 
                                                        "mesh_select_mode_init":(True, False, False)})
                                                        

            bpy.ops.mesh.select_all(action='SELECT')
            bpy.ops.mesh.vertices_smooth(factor=0.5, repeat=3)


            bpy.ops.mesh.select_all(action='DESELECT')
            bpy.ops.object.mode_set(mode='OBJECT')
            
            # Cursor back
            bpy.context.scene.cursor.location = cursor_previous_location

            # Selected to Cursor
            bpy.ops.view3d.snap_selected_to_cursor(use_offset=False)


            ob = bpy.context.object
            ob.scale.x /= 10
            ob.scale.y /= 10
            ob.scale.z /= 10


        #------------------------------------------------------------------------

        # Place 3D Cursor
        if event.type == 'LEFTMOUSE' and event.value == 'PRESS':
            return {'PASS_THROUGH'}  


        if event.type == 'LEFTMOUSE' and event.value == 'RELEASE':
            main_capsule()
           
                
            return {'RUNNING_MODAL'}
        
 
        sel_ob = bpy.context.selected_objects
        
        if sel_ob:
            obj = sel_ob[0]
            
            if event.type == 'MOUSEMOVE':
                delta = self.init_mouse_x - (event.mouse_x - event.mouse_prev_press_x)
                obj.scale.x = self.init_value + delta * -0.05
                obj.scale.y = self.init_value + delta * -0.05
                obj.scale.z = self.init_value + delta * -0.05


                # change mouse cursor
                bpy.context.window.cursor_modal_set("MOVE_X")

                # header info
                bpy.context.area.header_text_set('Right Click or press  ESC | SHIFT | CTRL | ALT | SPACEBAR | ENTER  -  to Confirm')


        if event.type in {'RIGHTMOUSE', 'SPACE', 'RET', 'LEFT_CTRL', 'LEFT_ALT', 'LEFT_SHIFT', 'ESC'} and event.value == 'PRESS':

            # restore keymaps
            km.type = 'RIGHTMOUSE'
            km.shift = 1           
            

            # restore mouse cursor
            bpy.context.window.cursor_modal_restore()

            # restore info header
            bpy.context.area.header_text_set(None)

            # apply scale
            bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)

            act_ob = bpy.context.active_object
            if stay_in_sculpt_mode:
                if act_ob is not None and act_ob.type == 'MESH':
                    bpy.ops.object.mode_set(mode='SCULPT', toggle=False)

            if create_and_center:
                bpy.ops.object.center_obj_cbox()

            if create_and_mirror:
                bpy.ops.object.mirror_mod_cbox()                



            return {'FINISHED'}

        
        return {'RUNNING_MODAL'}



    def invoke(self, context, event):

        mode = bpy.context.mode
        if mode != 'OBJECT':
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)


        bpy.ops.object.select_all(action='DESELECT')

        context.window_manager.modal_handler_add(self)

        return {'RUNNING_MODAL'}



#--------------------------------------------------------------------------

#----------------------------Tapered Capsule End--------------------------------

#--------------------------------------------------------------------------





classes = [ 

            OBJECT_OT_create_capsule_cbox,
            OBJECT_OT_create_tapered_capsule_cbox,

          ]
 
 
 
def register():
    for cls in classes:
        bpy.utils.register_class(cls)
        

 
def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)
