import bpy
from bpy.props import (FloatProperty, IntProperty)


#--------------------------------------------------------------------------

#---------------------------Cylinder---------------------------------------

#--------------------------------------------------------------------------


# Low Res
 
class OBJECT_OT_create_cylinder_low_cbox(bpy.types.Operator):
    '''Create cylinder with 1 subdivision levels'''
    bl_idname = "object.create_cylinder_low_cbox"
    bl_label = "Cylinder x1"
    bl_options = {'REGISTER', 'UNDO'}


    init_mouse_x: IntProperty()
    init_value: FloatProperty()

    def modal(self, context, event):


        kc = bpy.context.window_manager.keyconfigs
        km = kc.active.keymaps['3D View'].keymap_items['view3d.cursor3d']


        # temp. change
        km.type = 'LEFTMOUSE'
        km.shift = 0  


        # Ref
        stay_in_sculpt_mode = context.scene.stay_in_sculpt_mode_cx 
        create_and_center = context.scene.create_and_center_x_axis_cx
        create_and_mirror = context.scene.create_and_mirror_cx
        

        def main_cylinder():

            mode = bpy.context.mode
            if mode != 'OBJECT':
                bpy.ops.object.mode_set(mode='OBJECT', toggle=False)


            cursor = bpy.context.scene.cursor

            c_x = cursor.location[0]
            c_y = cursor.location[1]
            c_z = cursor.location[2]        
            
            # Cube --------------------------------------------------------
            
            def add_cylinder():
                
                bpy.ops.mesh.primitive_cube_add(size=2, 
                                                enter_editmode=True, 
                                                align='WORLD', 
                                                location=(c_x, c_y, c_z), 
                                                scale=(0.2, 0.2, 2))
                
            #---------------------------------------------------------------
                                 
            add_cylinder()
            
            #---------------------------------------------------------------        
            def add_loop():
                
                bpy.ops.mesh.loopcut_slide(MESH_OT_loopcut={"number_cuts":1,                                                     
                                                            "object_index":0, 
                                                            "edge_index":9, 
                                                            "mesh_select_mode_init":(False, True, False)},
                                            TRANSFORM_OT_edge_slide={"value":-0.95})
                                                    
            
            
                bpy.ops.mesh.loopcut_slide(MESH_OT_loopcut={"number_cuts":1,                                                     
                                                            "object_index":0, 
                                                            "edge_index":9, 
                                                            "mesh_select_mode_init":(False, True, False)},
                                            TRANSFORM_OT_edge_slide={"value":0.95})
                                                    
                    
                
            
            
                bpy.ops.mesh.loopcut_slide(MESH_OT_loopcut={"number_cuts":10,                                                     
                                                            "object_index":0, 
                                                            "edge_index":22, 
                                                            "mesh_select_mode_init":(False, True, False)},
                                            TRANSFORM_OT_edge_slide={"value":0.0})
                                                    

                bpy.ops.mesh.select_all(action='DESELECT')               
            #---------------------------------------------------------------               
            
            
            add_loop()
            
            
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            
            ob = bpy.context.object
            ob.name = 'cylinder_block_out' 
            ob.data.name = 'cylinder_block_out'

            ob.scale.x /= 10
            ob.scale.y /= 10
            ob.scale.z /= 10             
            
            # add and apply sub mod
            ob.modifiers.new('Subdivision', type='SUBSURF')
            ob.modifiers["Subdivision"].levels = 1
            bpy.ops.object.modifier_apply(modifier="Subdivision")




        # Place 3D Cursor
        if event.type == 'LEFTMOUSE' and event.value == 'PRESS':
            return {'PASS_THROUGH'}  


        if event.type == 'LEFTMOUSE' and event.value == 'RELEASE':
            main_cylinder()
           
                
            return {'RUNNING_MODAL'}



                    
        sel_ob = bpy.context.selected_objects
        
        if sel_ob:
            obj = sel_ob[0]
            
            if event.type == 'MOUSEMOVE':
                delta = self.init_mouse_x - (event.mouse_x - event.mouse_prev_press_x)
                obj.scale.x = self.init_value + delta * -0.01
                obj.scale.y = self.init_value + delta * -0.01
                obj.scale.z = self.init_value + delta * -0.01


                # change mouse cursor
                bpy.context.window.cursor_modal_set("MOVE_X")

                # header info
                bpy.context.area.header_text_set('Right Click or press  ESC | SHIFT | CTRL | ALT | SPACEBAR | ENTER  -  to Confirm')


        if event.type in {'RIGHTMOUSE', 'SPACE', 'RET', 'LEFT_CTRL', 'LEFT_ALT', 'LEFT_SHIFT', 'ESC'} and event.value == 'PRESS':

            # restore keymaps
            km.type = 'RIGHTMOUSE'
            km.shift = 1           
            

            # restore mouse cursor
            bpy.context.window.cursor_modal_restore()

            # restore info header
            bpy.context.area.header_text_set(None)

            # apply scale
            bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)


            act_ob = bpy.context.active_object
            if stay_in_sculpt_mode:
                if act_ob is not None and act_ob.type == 'MESH':
                    bpy.ops.object.mode_set(mode='SCULPT', toggle=False)
                    

            if create_and_center:
                bpy.ops.object.center_obj_cbox()

            if create_and_mirror:
                bpy.ops.object.mirror_mod_cbox()                


            return {'FINISHED'}


       
        return {'RUNNING_MODAL'}



    def invoke(self, context, event):

        mode = bpy.context.mode
        if mode != 'OBJECT':
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)



        bpy.ops.object.select_all(action='DESELECT')

        context.window_manager.modal_handler_add(self)

        return {'RUNNING_MODAL'}



#--------------------------------------------------------------------------
#--------------------------------------------------------------------------
#--------------------------------------------------------------------------

# Mid Res
 
class OBJECT_OT_create_cylinder_mid_cbox(bpy.types.Operator):
    '''Create cylinder with 2 subdivision levels'''
    bl_idname = "object.create_cylinder_mid_cbox"
    bl_label = "Cylinder x2"
    bl_options = {'REGISTER', 'UNDO'}


    init_mouse_x: IntProperty()
    init_value: FloatProperty()

    def modal(self, context, event):


        kc = bpy.context.window_manager.keyconfigs
        km = kc.active.keymaps['3D View'].keymap_items['view3d.cursor3d']


        # temp. change
        km.type = 'LEFTMOUSE'
        km.shift = 0  


        # Ref
        stay_in_sculpt_mode = context.scene.stay_in_sculpt_mode_cx 
        create_and_center = context.scene.create_and_center_x_axis_cx
        create_and_mirror = context.scene.create_and_mirror_cx


        def main_cylinder():

            mode = bpy.context.mode
            if mode != 'OBJECT':
                bpy.ops.object.mode_set(mode='OBJECT', toggle=False)


            cursor = bpy.context.scene.cursor

            c_x = cursor.location[0]
            c_y = cursor.location[1]
            c_z = cursor.location[2]        
            
            # Cube --------------------------------------------------------
            
            def add_cylinder():
                
                bpy.ops.mesh.primitive_cube_add(size=2, 
                                                enter_editmode=True, 
                                                align='WORLD', 
                                                location=(c_x, c_y, c_z), 
                                                scale=(0.2, 0.2, 2))
                
            #---------------------------------------------------------------
                                 
            add_cylinder()
            
            #---------------------------------------------------------------        
            def add_loop():
                
                bpy.ops.mesh.loopcut_slide(MESH_OT_loopcut={"number_cuts":1,                                                     
                                                            "object_index":0, 
                                                            "edge_index":9, 
                                                            "mesh_select_mode_init":(False, True, False)},
                                            TRANSFORM_OT_edge_slide={"value":-0.95})
                                                    
            
            
                bpy.ops.mesh.loopcut_slide(MESH_OT_loopcut={"number_cuts":1,                                                     
                                                            "object_index":0, 
                                                            "edge_index":9, 
                                                            "mesh_select_mode_init":(False, True, False)},
                                            TRANSFORM_OT_edge_slide={"value":0.95})
                                                    
                    
                
            
            
                bpy.ops.mesh.loopcut_slide(MESH_OT_loopcut={"number_cuts":10,                                                     
                                                            "object_index":0, 
                                                            "edge_index":22, 
                                                            "mesh_select_mode_init":(False, True, False)},
                                            TRANSFORM_OT_edge_slide={"value":0.0})
                                                    

                bpy.ops.mesh.select_all(action='DESELECT')               
            #---------------------------------------------------------------               
            
            
            add_loop()
            
            
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            
            ob = bpy.context.object
            ob.name = 'cylinder_block_out' 
            ob.data.name = 'cylinder_block_out'

            ob.scale.x /= 10
            ob.scale.y /= 10
            ob.scale.z /= 10             
            
            # add and apply sub mod
            ob.modifiers.new('Subdivision', type='SUBSURF')
            ob.modifiers["Subdivision"].levels = 2
            bpy.ops.object.modifier_apply(modifier="Subdivision")




        # Place 3D Cursor
        if event.type == 'LEFTMOUSE' and event.value == 'PRESS':
            return {'PASS_THROUGH'}  


        if event.type == 'LEFTMOUSE' and event.value == 'RELEASE':
            main_cylinder()
           
                
            return {'RUNNING_MODAL'}



                    
        sel_ob = bpy.context.selected_objects
        
        if sel_ob:
            obj = sel_ob[0]
            
            if event.type == 'MOUSEMOVE':
                delta = self.init_mouse_x - (event.mouse_x - event.mouse_prev_press_x)
                obj.scale.x = self.init_value + delta * -0.01
                obj.scale.y = self.init_value + delta * -0.01
                obj.scale.z = self.init_value + delta * -0.01


                # change mouse cursor
                bpy.context.window.cursor_modal_set("MOVE_X")

                # header info
                bpy.context.area.header_text_set('Right Click or press  ESC | SHIFT | CTRL | ALT | SPACEBAR | ENTER  -  to Confirm')


        if event.type in {'RIGHTMOUSE', 'SPACE', 'RET', 'LEFT_CTRL', 'LEFT_ALT', 'LEFT_SHIFT', 'ESC'} and event.value == 'PRESS':

            # restore keymaps
            km.type = 'RIGHTMOUSE'
            km.shift = 1           
            

            # restore mouse cursor
            bpy.context.window.cursor_modal_restore()

            # restore info header
            bpy.context.area.header_text_set(None)

            # apply scale
            bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)


            act_ob = bpy.context.active_object
            if stay_in_sculpt_mode:
                if act_ob is not None and act_ob.type == 'MESH':
                    bpy.ops.object.mode_set(mode='SCULPT', toggle=False)
                    

            if create_and_center:
                bpy.ops.object.center_obj_cbox()

            if create_and_mirror:
                bpy.ops.object.mirror_mod_cbox()                


            return {'FINISHED'}


       
        return {'RUNNING_MODAL'}



    def invoke(self, context, event):

        mode = bpy.context.mode
        if mode != 'OBJECT':
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)



        bpy.ops.object.select_all(action='DESELECT')

        context.window_manager.modal_handler_add(self)

        return {'RUNNING_MODAL'}



#--------------------------------------------------------------------------
#--------------------------------------------------------------------------
#--------------------------------------------------------------------------

# High Res
 
class OBJECT_OT_create_cylinder_high_cbox(bpy.types.Operator):
    '''Create cylinder with 3 subdivision levels'''
    bl_idname = "object.create_cylinder_high_cbox"
    bl_label = "Cylinder x3"
    bl_options = {'REGISTER', 'UNDO'}


    init_mouse_x: IntProperty()
    init_value: FloatProperty()

    def modal(self, context, event):



        kc = bpy.context.window_manager.keyconfigs
        km = kc.active.keymaps['3D View'].keymap_items['view3d.cursor3d']


        # temp. change
        km.type = 'LEFTMOUSE'
        km.shift = 0  


        # Ref
        stay_in_sculpt_mode = context.scene.stay_in_sculpt_mode_cx 
        create_and_center = context.scene.create_and_center_x_axis_cx
        create_and_mirror = context.scene.create_and_mirror_cx


        def main_cylinder():

            mode = bpy.context.mode
            if mode != 'OBJECT':
                bpy.ops.object.mode_set(mode='OBJECT', toggle=False)


            cursor = bpy.context.scene.cursor

            c_x = cursor.location[0]
            c_y = cursor.location[1]
            c_z = cursor.location[2]        
            
            # Cube --------------------------------------------------------
            
            def add_cylinder():
                
                bpy.ops.mesh.primitive_cube_add(size=2, 
                                                enter_editmode=True, 
                                                align='WORLD', 
                                                location=(c_x, c_y, c_z), 
                                                scale=(0.2, 0.2, 2))
                
            #---------------------------------------------------------------
                                 
            add_cylinder()
            
            #---------------------------------------------------------------        
            def add_loop():
                
                bpy.ops.mesh.loopcut_slide(MESH_OT_loopcut={"number_cuts":1,                                                     
                                                            "object_index":0, 
                                                            "edge_index":9, 
                                                            "mesh_select_mode_init":(False, True, False)},
                                            TRANSFORM_OT_edge_slide={"value":-0.95})
                                                    
            
            
                bpy.ops.mesh.loopcut_slide(MESH_OT_loopcut={"number_cuts":1,                                                     
                                                            "object_index":0, 
                                                            "edge_index":9, 
                                                            "mesh_select_mode_init":(False, True, False)},
                                            TRANSFORM_OT_edge_slide={"value":0.95})
                                                    
                    
                
            
            
                bpy.ops.mesh.loopcut_slide(MESH_OT_loopcut={"number_cuts":10,                                                     
                                                            "object_index":0, 
                                                            "edge_index":22, 
                                                            "mesh_select_mode_init":(False, True, False)},
                                            TRANSFORM_OT_edge_slide={"value":0.0})
                                                    

                bpy.ops.mesh.select_all(action='DESELECT')               
            #---------------------------------------------------------------               
            
            
            add_loop()
            
            
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            
            ob = bpy.context.object
            ob.name = 'cylinder_block_out' 
            ob.data.name = 'cylinder_block_out'

            ob.scale.x /= 10
            ob.scale.y /= 10
            ob.scale.z /= 10             
            
            # add and apply sub mod
            ob.modifiers.new('Subdivision', type='SUBSURF')
            ob.modifiers["Subdivision"].levels = 3
            bpy.ops.object.modifier_apply(modifier="Subdivision")




        # Place 3D Cursor
        if event.type == 'LEFTMOUSE' and event.value == 'PRESS':
            return {'PASS_THROUGH'}  


        if event.type == 'LEFTMOUSE' and event.value == 'RELEASE':
            main_cylinder()
           
                
            return {'RUNNING_MODAL'}



                    
        sel_ob = bpy.context.selected_objects
        
        if sel_ob:
            obj = sel_ob[0]
            
            if event.type == 'MOUSEMOVE':
                delta = self.init_mouse_x - (event.mouse_x - event.mouse_prev_press_x)
                obj.scale.x = self.init_value + delta * -0.01
                obj.scale.y = self.init_value + delta * -0.01
                obj.scale.z = self.init_value + delta * -0.01


                # change mouse cursor
                bpy.context.window.cursor_modal_set("MOVE_X")

                # header info
                bpy.context.area.header_text_set('Right Click or press  ESC | SHIFT | CTRL | ALT | SPACEBAR | ENTER  -  to Confirm')


        if event.type in {'RIGHTMOUSE', 'SPACE', 'RET', 'LEFT_CTRL', 'LEFT_ALT', 'LEFT_SHIFT', 'ESC'} and event.value == 'PRESS':

            # restore keymaps
            km.type = 'RIGHTMOUSE'
            km.shift = 1           
            

            # restore mouse cursor
            bpy.context.window.cursor_modal_restore()

            # restore info header
            bpy.context.area.header_text_set(None)

            # apply scale
            bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)


            act_ob = bpy.context.active_object
            if stay_in_sculpt_mode:
                if act_ob is not None and act_ob.type == 'MESH':
                    bpy.ops.object.mode_set(mode='SCULPT', toggle=False)
                    

            if create_and_center:
                bpy.ops.object.center_obj_cbox()

            if create_and_mirror:
                bpy.ops.object.mirror_mod_cbox()                


            return {'FINISHED'}


       
        return {'RUNNING_MODAL'}



    def invoke(self, context, event):

        mode = bpy.context.mode
        if mode != 'OBJECT':
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)



        bpy.ops.object.select_all(action='DESELECT')

        context.window_manager.modal_handler_add(self)

        return {'RUNNING_MODAL'}




#--------------------------------------------------------------------------
#---------------------------Cylinder End-----------------------------------
#--------------------------------------------------------------------------



#---------------------------Short Cylinder------------------------------------
#-----------------------------------------------------------------------------
#-----------------------------------------------------------------------------

class OBJECT_OT_create_short_cylinder_cbox(bpy.types.Operator):
    '''Create Short Cylinder'''
    bl_idname = "object.create_short_cylinder_cbox"
    bl_label = "Short Cylinde"
    bl_options = {'REGISTER', 'UNDO'}


 
    init_mouse_x: IntProperty()
    init_value: FloatProperty()

    def modal(self, context, event):


        kc = bpy.context.window_manager.keyconfigs
        km = kc.active.keymaps['3D View'].keymap_items['view3d.cursor3d']


        # temp. change
        km.type = 'LEFTMOUSE'
        km.shift = 0  


        # Ref
        stay_in_sculpt_mode = context.scene.stay_in_sculpt_mode_cx 
        create_and_center = context.scene.create_and_center_x_axis_cx
        create_and_mirror = context.scene.create_and_mirror_cx
        

        def main_cylinder():

            mode = bpy.context.mode
            if mode != 'OBJECT':
                bpy.ops.object.mode_set(mode='OBJECT', toggle=False)


            cursor = bpy.context.scene.cursor

            c_x = cursor.location[0]
            c_y = cursor.location[1]
            c_z = cursor.location[2]        
            
            # Cube --------------------------------------------------------
            
            def add_cylinder():
                
                bpy.ops.mesh.primitive_cube_add(size=2, 
                                                enter_editmode=True, 
                                                align='WORLD', 
                                                location=(c_x, c_y, c_z), 
                                                scale=(0.6, 0.6, 0.2))
                
            #---------------------------------------------------------------
                                 
            add_cylinder()
            
            #---------------------------------------------------------------        
            def add_loop():
                
                bpy.ops.mesh.loopcut_slide(MESH_OT_loopcut={"number_cuts":1,                                                     
                                                            "object_index":0, 
                                                            "edge_index":9, 
                                                            "mesh_select_mode_init":(False, True, False)},
                                            TRANSFORM_OT_edge_slide={"value":-0.9})
                                                    
            
            
                bpy.ops.mesh.loopcut_slide(MESH_OT_loopcut={"number_cuts":1,                                                     
                                                            "object_index":0, 
                                                            "edge_index":9, 
                                                            "mesh_select_mode_init":(False, True, False)},
                                            TRANSFORM_OT_edge_slide={"value":0.9})
                                                    
                                            
                bpy.ops.mesh.select_mode(use_extend=False, use_expand=False, type='EDGE')

                bpy.ops.mesh.select_all(action='DESELECT')               
            #---------------------------------------------------------------               
            
            
            add_loop()
            
            
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            
            ob = bpy.context.object
            ob.name = 'short_cylinder' 
            ob.data.name = 'short_cylinder'

            ob.scale.x /= 10
            ob.scale.y /= 10
            ob.scale.z /= 10             
            
            # add and apply sub mod
            ob.modifiers.new('Subdivision', type='SUBSURF')
            ob.modifiers["Subdivision"].levels = 2
            bpy.ops.object.modifier_apply(modifier="Subdivision")



        # Place 3D Cursor
        if event.type == 'LEFTMOUSE' and event.value == 'PRESS':
            return {'PASS_THROUGH'}  


        if event.type == 'LEFTMOUSE' and event.value == 'RELEASE':
            main_cylinder()
           
                
            return {'RUNNING_MODAL'}


                    
        sel_ob = bpy.context.selected_objects
        
        if sel_ob:
            obj = sel_ob[0]
            
            if event.type == 'MOUSEMOVE':
                delta = self.init_mouse_x - (event.mouse_x - event.mouse_prev_press_x)
                obj.scale.x = self.init_value + delta * -0.01
                obj.scale.y = self.init_value + delta * -0.01
                obj.scale.z = self.init_value + delta * -0.01


                # change mouse cursor
                bpy.context.window.cursor_modal_set("MOVE_X")

                # header info
                bpy.context.area.header_text_set('Right Click or press  ESC | SHIFT | CTRL | ALT | SPACEBAR | ENTER  -  to Confirm')


        if event.type in {'RIGHTMOUSE', 'SPACE', 'RET', 'LEFT_CTRL', 'LEFT_ALT', 'LEFT_SHIFT', 'ESC'} and event.value == 'PRESS':

            # restore keymaps
            km.type = 'RIGHTMOUSE'
            km.shift = 1           
            

            # restore mouse cursor
            bpy.context.window.cursor_modal_restore()

            # restore info header
            bpy.context.area.header_text_set(None)

            # apply scale
            bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)


            act_ob = bpy.context.active_object
            if stay_in_sculpt_mode:
                if act_ob is not None and act_ob.type == 'MESH':
                    bpy.ops.object.mode_set(mode='SCULPT', toggle=False)
                    

            if create_and_center:
                bpy.ops.object.center_obj_cbox()

            if create_and_mirror:
                bpy.ops.object.mirror_mod_cbox()                


            return {'FINISHED'}

       
        return {'RUNNING_MODAL'}


    def invoke(self, context, event):

        mode = bpy.context.mode
        if mode != 'OBJECT':
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)



        bpy.ops.object.select_all(action='DESELECT')

        context.window_manager.modal_handler_add(self)

        return {'RUNNING_MODAL'}




#--------------------------------------------------------------------------
#----------------------------Short Cylinder End----------------------------
#--------------------------------------------------------------------------



#--------------------------- Middle Cylinder -------------------------------
#---------------------------------------------------------------------------
#---------------------------------------------------------------------------

class OBJECT_OT_create_middle_cylinder_cbox(bpy.types.Operator):
    '''Create Middle Cylinder'''
    bl_idname = "object.create_middle_cylinder_cbox"
    bl_label = "Middle Cylinde"
    bl_options = {'REGISTER', 'UNDO'}


    init_mouse_x: IntProperty()
    init_value: FloatProperty()

    def modal(self, context, event):


        kc = bpy.context.window_manager.keyconfigs
        km = kc.active.keymaps['3D View'].keymap_items['view3d.cursor3d']


        # temp. change
        km.type = 'LEFTMOUSE'
        km.shift = 0  


        # Ref
        stay_in_sculpt_mode = context.scene.stay_in_sculpt_mode_cx 
        create_and_center = context.scene.create_and_center_x_axis_cx
        create_and_mirror = context.scene.create_and_mirror_cx
        

        def main_cylinder():

            mode = bpy.context.mode
            if mode != 'OBJECT':
                bpy.ops.object.mode_set(mode='OBJECT', toggle=False)


            cursor = bpy.context.scene.cursor

            c_x = cursor.location[0]
            c_y = cursor.location[1]
            c_z = cursor.location[2]        
            
            # Cube --------------------------------------------------------
            
            def add_cylinder():
                
                bpy.ops.mesh.primitive_cube_add(size=2, 
                                                enter_editmode=True, 
                                                align='WORLD', 
                                                location=(c_x, c_y, c_z), 
                                                scale=(0.5, 0.5, 1))
                
            #---------------------------------------------------------------
                                 
            add_cylinder()
            
            #---------------------------------------------------------------        
            def add_loop():
                
                bpy.ops.mesh.loopcut_slide(MESH_OT_loopcut={"number_cuts":1,                                                     
                                                            "object_index":0, 
                                                            "edge_index":9, 
                                                            "mesh_select_mode_init":(False, True, False)},
                                            TRANSFORM_OT_edge_slide={"value":-0.9})
                                                    
            
            
                bpy.ops.mesh.loopcut_slide(MESH_OT_loopcut={"number_cuts":1,                                                     
                                                            "object_index":0, 
                                                            "edge_index":9, 
                                                            "mesh_select_mode_init":(False, True, False)},
                                            TRANSFORM_OT_edge_slide={"value":0.9})
                                                    
                    
                
            
            
                bpy.ops.mesh.loopcut_slide(MESH_OT_loopcut={"number_cuts":2,                                                     
                                                            "object_index":0, 
                                                            "edge_index":22, 
                                                            "mesh_select_mode_init":(False, True, False)},
                                            TRANSFORM_OT_edge_slide={"value":0.0})
                                                    

                bpy.ops.mesh.select_all(action='DESELECT')               
            #---------------------------------------------------------------               
            
            
            add_loop()
            
            
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            
            ob = bpy.context.object
            ob.name = 'mid_cylinder' 
            ob.data.name = 'mid_cylinder'

            ob.scale.x /= 10
            ob.scale.y /= 10
            ob.scale.z /= 10             
            
            # add and apply sub mod
            ob.modifiers.new('Subdivision', type='SUBSURF')
            ob.modifiers["Subdivision"].levels = 2
            bpy.ops.object.modifier_apply(modifier="Subdivision")



        # Place 3D Cursor
        if event.type == 'LEFTMOUSE' and event.value == 'PRESS':
            return {'PASS_THROUGH'}  


        if event.type == 'LEFTMOUSE' and event.value == 'RELEASE':
            main_cylinder()
           
                
            return {'RUNNING_MODAL'}



                    
        sel_ob = bpy.context.selected_objects
        
        if sel_ob:
            obj = sel_ob[0]
            
            if event.type == 'MOUSEMOVE':
                delta = self.init_mouse_x - (event.mouse_x - event.mouse_prev_press_x)
                obj.scale.x = self.init_value + delta * -0.01
                obj.scale.y = self.init_value + delta * -0.01
                obj.scale.z = self.init_value + delta * -0.01


                # change mouse cursor
                bpy.context.window.cursor_modal_set("MOVE_X")

                # header info
                bpy.context.area.header_text_set('Right Click or press  ESC | SHIFT | CTRL | ALT | SPACEBAR | ENTER  -  to Confirm')


        if event.type in {'RIGHTMOUSE', 'SPACE', 'RET', 'LEFT_CTRL', 'LEFT_ALT', 'LEFT_SHIFT', 'ESC'} and event.value == 'PRESS':

            # restore keymaps
            km.type = 'RIGHTMOUSE'
            km.shift = 1           
            

            # restore mouse cursor
            bpy.context.window.cursor_modal_restore()

            # restore info header
            bpy.context.area.header_text_set(None)

            # apply scale
            bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)


            act_ob = bpy.context.active_object
            if stay_in_sculpt_mode:
                if act_ob is not None and act_ob.type == 'MESH':
                    bpy.ops.object.mode_set(mode='SCULPT', toggle=False)
                    

            if create_and_center:
                bpy.ops.object.center_obj_cbox()

            if create_and_mirror:
                bpy.ops.object.mirror_mod_cbox()                


            return {'FINISHED'}


       
        return {'RUNNING_MODAL'}



    def invoke(self, context, event):

        mode = bpy.context.mode
        if mode != 'OBJECT':
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)



        bpy.ops.object.select_all(action='DESELECT')

        context.window_manager.modal_handler_add(self)

        return {'RUNNING_MODAL'}




#--------------------------------------------------------------------------
#---------------------------- Middle Cylinder End--------------------------
#--------------------------------------------------------------------------










classes = [ 

            OBJECT_OT_create_cylinder_low_cbox,
            OBJECT_OT_create_cylinder_mid_cbox,
            OBJECT_OT_create_cylinder_high_cbox,
            OBJECT_OT_create_short_cylinder_cbox,
            OBJECT_OT_create_middle_cylinder_cbox,

            
          ]
 
 
 
def register():
    for cls in classes:
        bpy.utils.register_class(cls)
        

 
def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)
