import bpy



#-------------------------------------------------------------------------
#-------------------------------- INFO -----------------------------------
#-------------------------------------------------------------------------

class OBJECT_OT_claybox_info_cx(bpy.types.Operator):
    '''How to create (invoke) and place sculpt blocks'''
    bl_idname = "object.claybox_info_cx"
    bl_label = "Info"
    bl_options = {'REGISTER', 'UNDO'}

 
    def execute(self, context):
        
        def ShowMessageBox(title = "Create various Shapes for Sculpting / Blocking Out", icon = 'INFO', lines=""):
            
            info_lines=lines
            def draw(self, context):
                for n in info_lines:
                    self.layout.label(text=n)
            bpy.context.window_manager.popup_menu(draw, title = title, icon = icon)
                              
        info_lines=("1) Select (Invoke) Shape ( by selecting the relevant Shape button )",
                    "2) Left click (3d Viewport) -> release -> and move on X-axis to adjust the size.",
                    "3) Right click to confirm.",
                    "4) Or press ESC / SHIFT / CTRL / ALT / SPACEBAR / ENTER - to confirm." 
                
                    )
                
        ShowMessageBox(lines=info_lines)

        return {'FINISHED'}


 


#-------------------------------------------------------------------------
#-------------------------------- INFO End -------------------------------
#-------------------------------------------------------------------------




#--------------------------------------------------------------------------    
#------------Apply Scale / Switch between Sculpt / Object Mode-------------   
#--------------------------------------------------------------------------    
    
 
class OBJECT_OT_apply_scale_back2sculpt_cbox(bpy.types.Operator):
    '''Apply Scale for all mesh objects, Switch between Sculpt / Object Mode'''
    bl_idname = "object.apply_scale_back2sculpt_cbox"
    bl_label = "Sculpt Mode"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.type == 'MESH')


                
    def execute(self, context):
   

        def apply_scale_for_all_mesh_obj():

            act_ob = bpy.context.active_object

            obj =  bpy.data.objects

            for o in obj:
                if o.type == 'MESH':
                    o.select_set(1)
                    
                    
            sel_obj = bpy.context.selected_objects      

            for o in sel_obj:
                bpy.context.view_layer.objects.active = o  
                    
                bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)

                bpy.ops.object.select_all(action='DESELECT')


            bpy.context.view_layer.objects.active = act_ob
            act_ob.select_set(1)


        #--------------------------------------------------------

        mode = bpy.context.mode
        if mode == 'OBJECT':
            apply_scale_for_all_mesh_obj()

            bpy.ops.sculpt.sculptmode_toggle()


        if mode != 'OBJECT':
            bpy.ops.sculpt.sculptmode_toggle()




        return {'FINISHED'}    


#--------------------------------------------------------------------------    
#------------Apply Scale / Switch between Sculpt / Object Mode End--------- 
#--------------------------------------------------------------------------    





#--------------------------------------------------------------------------    
#-------After transforming in Sculpt mode -> Refresh current Tool ---------
#--------------------------------------------------------------------------    
    
 
class OBJECT_OT_tool_update_cbox(bpy.types.Operator):
    '''After transforming in Sculpt mode -> "Refresh" current Tool
- Set Origing to Geometry'''
    bl_idname = "object.tool_update_cbox"
    bl_label = ""
    # bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.mode == 'SCULPT')

    def execute(self, context):

        bpy.ops.sculpt.sculptmode_toggle()
        bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='MEDIAN')
        bpy.ops.sculpt.sculptmode_toggle()


        return {'FINISHED'}    



#--------------------------------------------------------------------------
#--------------------------------------------------------------------------
#--------------------------------------------------------------------------



#--------------------------------------------------------------------------
#------------------------------ADD RES------------------------------------- 
#-------------------------------------------------------------------------- 
 

class OBJECT_OT_add_subdiv_res_cbox(bpy.types.Operator):
    '''Add 1 subdivision level'''
    bl_idname = "object.add_subdiv_res_cbox"
    bl_label = "Add Res"
    bl_options = {'REGISTER', 'UNDO'}

    @classmethod
    def poll(cls, context):
        modes = ['OBJECT', 'EDIT', 'SCULPT'] 
        return (context.object is not None and
                context.object.type == 'MESH' and
                context.object.mode in modes and
                len(context.selected_objects) != 0)


    def execute(self, context):
        
        modes = ['OBJECT', 'EDIT', 'SCULPT']    

        def add_res():     
            sel_obj = bpy.context.selected_objects

            for o in sel_obj:
                bpy.context.view_layer.objects.active = o                        
                  
                      
                subdiv_mods = [mod for mod in o.modifiers if mod.type == 'SUBSURF']
                if subdiv_mods:
                    bpy.ops.object.modifier_move_to_index(modifier="Subdivision", index=0)
                    bpy.ops.object.modifier_apply(modifier="Subdivision")  
                    
                # add and apply sub mod
                o.modifiers.new('Subdivision', type='SUBSURF')
                o.modifiers["Subdivision"].levels = 1
                bpy.ops.object.modifier_move_to_index(modifier="Subdivision", index=0)
                bpy.ops.object.modifier_apply(modifier="Subdivision")                  
              

        cur_mode = bpy.context.mode

        if cur_mode == "EDIT_MESH":
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            add_res()
            bpy.ops.object.editmode_toggle()


        if cur_mode in modes:
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            add_res()

            bpy.ops.object.mode_set(mode=cur_mode, toggle=False)


        return {'FINISHED'}    

 
 
#--------------------------------------------------------------------------
#-------------------------ADD RES End--------------------------------------
#--------------------------------------------------------------------------
  


#--------------------------------------------------------------------------
#-------------------------Create Dynamic Mirror  --------------------------
#--------------------------------------------------------------------------    
    
 
class OBJECT_OT_mirror_mod_cbox(bpy.types.Operator):
    '''Add Mirror Modifier for Mesh object(s)'''
    bl_idname = "object.mirror_mod_cbox"
    bl_label = "Mirror"
    bl_options = {'REGISTER', 'UNDO'}


   

    def execute(self, context):
        
        mode = bpy.context.mode
        if mode != 'OBJECT':
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)

        # Ref
        stay_in_sculpt_mode = context.scene.stay_in_sculpt_mode_cx 


        def get_empty():
            for m_empty in bpy.data.objects:
                if m_empty.type == 'EMPTY' and m_empty.name.startswith('mirror_empty'):
                    return m_empty  



        # Mirror Empty --------------------------------------------------------
        
        def create_mirror_empty():


            # Active Collection
            act_col = bpy.context.view_layer.active_layer_collection
            
            empty_name = "mirror_empty"
            empty = bpy.data.objects.new(empty_name, None )
            empty.empty_display_type = 'PLAIN_AXES'
            empty.empty_display_size = 0.1


            # Link to Collection
            act_col.collection.objects.link(empty)

            empty.location[0] = 0.0
            empty.location[1] = 0.0
            empty.location[2] = 0.0

            empty.hide_select = True
                       
        # ----------------------------------------------
        
        def add_mirror_mod():
            
            ob = bpy.context.object
            mod_name = 'mirror_clay'
            modifier = 'MIRROR'

            
            mirror_mods = [mod for mod in ob.modifiers if mod.type == 'MIRROR']
            if mirror_mods:
                ob.modifiers.remove(mirror_mods[-1])            
            
            
            empty_name = "mirror_empty"

            ob.modifiers.new(mod_name, type='MIRROR')
            ob.modifiers[mod_name].mirror_object = bpy.data.objects[empty_name]

        # ----------------------------------------------               
        sel_obj = bpy.context.selected_objects
        
        
        mirror_empty = get_empty()
        if not mirror_empty:
            create_mirror_empty()    
                
                
        for o in sel_obj:
            if o.type == 'MESH':
                bpy.context.view_layer.objects.active = o                        
                add_mirror_mod()            
                        


        
        act_ob = bpy.context.active_object
        if stay_in_sculpt_mode:
            if act_ob is not None and act_ob.type == 'MESH':

                bpy.ops.object.mode_set(mode='SCULPT', toggle=False)
            


        bpy.context.evaluated_depsgraph_get().update()
            

        return {'FINISHED'}    



#--------------------------------------------------------------------------
#----------------------------Dynamic Mirror End----------------------------    
#--------------------------------------------------------------------------    




#-------------------------------------------------------------------------------------------
#--------------------------Fill selected objects with vertex color--------------------------
#-------------------------------------------------------------------------------------------
 

 
class OBJECT_OT_fill_obj_vertcolor_cbox(bpy.types.Operator):
    '''Fill selected objects with vertex color'''
    bl_idname = "object.fill_obj_vertcolor_cbox"
    bl_label = "FillObject"
    bl_options = {'REGISTER', 'UNDO'}

    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                # context.object.mode == 'SCULPT' and
                context.object.type == 'MESH')

    def execute(self, context):
        
        # Viewport shading to "Attribute"
        bpy.context.space_data.shading.color_type = 'VERTEX'

        d_ob = bpy.data.objects

        for o in d_ob:
            if o.type != 'MESH':
                o.select_set(0)
        

        
        def blender_31_fill_color():

            
            ob = bpy.context.object
            sel_ob = bpy.context.selected_objects
             
            for o in sel_ob:
                for c in o.data.vertex_colors:
                    o.data.vertex_colors.remove(c)
       
                o.update_tag()
                
            for area in bpy.context.screen.areas:
                area.tag_redraw()   
                
                
            bpy.ops.object.mode_set(mode='VERTEX_PAINT')
            bpy.ops.paint.vertex_color_set()         
            bpy.ops.object.mode_set(mode='SCULPT')
            

        def fill_color():
        
                # Refs
                sel_color = context.scene.fill_object_color_cx
                
                color_one = sel_color[0]
                color_two = sel_color[1]
                color_tree = sel_color[2]
                
                
                ob = bpy.context.object
                sel_ob = bpy.context.selected_objects
                 
                for o in sel_ob:
                    for c in o.data.vertex_colors:
                        o.data.vertex_colors.remove(c)
                    for attr in ob.data.color_attributes:
                        ob.data.color_attributes.remove(attr)
                        
                        
                    o.update_tag()

                for area in bpy.context.screen.areas:
                    area.tag_redraw()
                    
                
                bpy.ops.geometry.color_attribute_add(color=(color_one, color_two, color_tree, 0))



        bv = bpy.app.version_string


        sel_ob = bpy.context.selected_objects
        
        for o in sel_ob:
            bpy.context.view_layer.objects.active = o

            if bv[0] == '3' and bv[2] == '1':
                blender_31_fill_color()


            if bv[0] == '3' and bv[2] != '1':
                fill_color()

            if bv[0] >= '4':
                fill_color()


        
        return {'FINISHED'}    
    
    

#--------------------------------------------------------------------------
#---------------------------End Fill with verts color---------------------- 
#--------------------------------------------------------------------------







#--------------------------------------------------------------------------    
#-----------------Create Material with vertex color connected-------------- 
#--------------------------------------------------------------------------    
    
 
class OBJECT_OT_create_vert_color_mat_cbox(bpy.types.Operator):
    '''Create new Material with connected Color/Vertex Attribute'''
    bl_idname = "object.create_vert_color_mat_cbox"
    bl_label = "Vertex Color Material"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.type == 'MESH') 
 
                
    
    def execute(self, context):
        
        current_language = bpy.context.preferences.view.language

        bpy.context.preferences.view.language = 'en_US'


        bv = bpy.app.version_string

        def new_mat_vertcolor():

            ob = bpy.context.object
            mats = bpy.data.materials

            # Clean not used mats
            bpy.ops.outliner.orphans_purge(do_local_ids=True, do_linked_ids=True, do_recursive=True)


            # Create new mat
            new_mat = mats.new(ob.name + '_mat')
            ob.data.materials.append(new_mat)

            new_mat.use_nodes = True

            ob.active_material = new_mat

            main_mat_node = new_mat.node_tree.nodes.get('Principled BSDF')
            vert_color_node = new_mat.node_tree.nodes.new(type='ShaderNodeVertexColor')

            vert_color_node.location = main_mat_node.location
            vert_color_node.location[0] -= 300

            new_mat.node_tree.links.new(vert_color_node.outputs[0], main_mat_node.inputs[0])




            if bv[0] == '3' and bv[2] == '1':    
                mats[new_mat.name].node_tree.nodes["Vertex Color"].layer_name = "Col"


            if bv[0] == '3' and bv[2] != '1':
                mats[new_mat.name].node_tree.nodes["Color Attribute"].layer_name = "Color"



        sel_ob = bpy.context.selected_objects

        for o in sel_ob:
            bpy.context.view_layer.objects.active = o
            ob_mats = o.data.materials

            if not ob_mats:
                new_mat_vertcolor()


        bpy.context.preferences.view.language = current_language


        return {'FINISHED'}    


#--------------------------------------------------------------------------
#-------------------Create Material with vertex color End------------------
#--------------------------------------------------------------------------




#--------------------------------------------------------------------------
#-------------------Center Object on X Axis--------------------------------
#--------------------------------------------------------------------------


 
class OBJECT_OT_center_obj_cbox(bpy.types.Operator):
    '''Center Selected Object on X Axis 
- Lock X Location / Y, Z Rotation'''
    bl_idname = "object.center_obj_cbox"
    bl_label = "Center X"
    bl_options = {'REGISTER', 'UNDO'}

   

    def execute(self, context):
        
        modes = ['OBJECT', 'EDIT', 'SCULPT'] 


        def unlock_transform():
            sel_obj = bpy.context.selected_objects

            for o in sel_obj:
                bpy.context.view_layer.objects.active = o 

                o.lock_location[0] = False
                o.lock_rotation[1] = False
                o.lock_rotation[2] = False


        def center_sel():
            sel_obj = bpy.context.selected_objects

            for o in sel_obj:
                bpy.context.view_layer.objects.active = o 
                bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='MEDIAN')
                o.location[0] = 0 
                o.lock_location[0] = True
                o.lock_rotation[1] = True
                o.lock_rotation[2] = True

                o.use_mesh_mirror_x = True

        cur_mode = bpy.context.mode
        ob = bpy.context.object



        if cur_mode == "EDIT_MESH":
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            center_sel()


        if cur_mode == "OBJECT":
            if ob is not None:
                center_sel()

        if cur_mode not in modes:
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            center_sel()



        if cur_mode == "SCULPT":
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            center_sel()

            bpy.ops.object.mode_set(mode="SCULPT", toggle=False)





        return {'FINISHED'}    


#--------------------------------------------------------------------------
#----------------------Center Object on X Axis End------------------------- 
#--------------------------------------------------------------------------





#--------------------------------------------------------------------------
#----------------- Rotate Selected Object 90 on X Axis --------------------
#--------------------------------------------------------------------------


 
class OBJECT_OT_rotate_90_x_axis_cbox(bpy.types.Operator):
    '''Rotate Selected Object 90 on X Axis and apply rotation'''
    bl_idname = "object.rotate_90_x_axis_cbox"
    bl_label = "Rotate 90 X Axis"
    bl_options = {'REGISTER', 'UNDO'}



    def execute(self, context):
        
        modes = ['OBJECT', 'EDIT', 'SCULPT'] 

        def rotate_sel():
            sel_obj = bpy.context.selected_objects

            for o in sel_obj:
                bpy.context.view_layer.objects.active = o 
                bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='MEDIAN')


                o.rotation_euler[0] = 1.5708
                bpy.ops.object.transform_apply(location=False, rotation=True, scale=False)



        cur_mode = bpy.context.mode
        ob = bpy.context.object

        if cur_mode == "EDIT_MESH":
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            rotate_sel()


        if cur_mode == "OBJECT":
            if ob is not None:
                rotate_sel()

 
        if cur_mode not in modes:
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            rotate_sel()


        if cur_mode == "SCULPT":
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            rotate_sel()

            bpy.ops.object.mode_set(mode="SCULPT", toggle=False)




        return {'FINISHED'}    


#--------------------------------------------------------------------------
#--------------- Rotate Selected Object 90 on X Axis End ------------------ 
#--------------------------------------------------------------------------






#--------------------------------------------------------------------------
#----------------- Rotate Selected Object 90 on Y Axis --------------------
#--------------------------------------------------------------------------


 
class OBJECT_OT_rotate_90_y_axis_cbox(bpy.types.Operator):
    '''Rotate Selected Object 90 on Y Axis and apply rotation'''
    bl_idname = "object.rotate_90_y_axis_cbox"
    bl_label = "Rotate 90 Y Axis"
    bl_options = {'REGISTER', 'UNDO'}



    def execute(self, context):
        
        modes = ['OBJECT', 'EDIT', 'SCULPT'] 

        def rotate_sel():
            sel_obj = bpy.context.selected_objects

            for o in sel_obj:
                bpy.context.view_layer.objects.active = o 
                bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='MEDIAN')


                o.rotation_euler[1] = 1.5708
                bpy.ops.object.transform_apply(location=False, rotation=True, scale=False)



        cur_mode = bpy.context.mode
        ob = bpy.context.object

        if cur_mode == "EDIT_MESH":
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            rotate_sel()


        if cur_mode == "OBJECT":
            if ob is not None:
                rotate_sel()

 
        if cur_mode not in modes:
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            rotate_sel()


        if cur_mode == "SCULPT":
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            rotate_sel()

            bpy.ops.object.mode_set(mode="SCULPT", toggle=False)




        return {'FINISHED'}    


#--------------------------------------------------------------------------
#--------------- Rotate Selected Object 90 on Y Axis End ------------------ 
#--------------------------------------------------------------------------






#--------------------------------------------------------------------------
#----------------- Rotate Selected Object 90 on Z Axis --------------------
#--------------------------------------------------------------------------


 
class OBJECT_OT_rotate_90_z_axis_cbox(bpy.types.Operator):
    '''Rotate Selected Object 90 on Z Axis and apply rotation'''
    bl_idname = "object.rotate_90_z_axis_cbox"
    bl_label = "Rotate 90 Z Axis"
    bl_options = {'REGISTER', 'UNDO'}



    def execute(self, context):
        
        modes = ['OBJECT', 'EDIT', 'SCULPT'] 

        def rotate_sel():
            sel_obj = bpy.context.selected_objects

            for o in sel_obj:
                bpy.context.view_layer.objects.active = o 
                bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='MEDIAN')


                o.rotation_euler[2] = 1.5708
                bpy.ops.object.transform_apply(location=False, rotation=True, scale=False)



        cur_mode = bpy.context.mode
        ob = bpy.context.object

        if cur_mode == "EDIT_MESH":
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            rotate_sel()


        if cur_mode == "OBJECT":
            if ob is not None:
                rotate_sel()

 
        if cur_mode not in modes:
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            rotate_sel()


        if cur_mode == "SCULPT":
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            rotate_sel()

            bpy.ops.object.mode_set(mode="SCULPT", toggle=False)




        return {'FINISHED'}    


#--------------------------------------------------------------------------
#--------------- Rotate Selected Object 90 on Z Axis End ------------------ 
#--------------------------------------------------------------------------





#--------------------------------------------------------------------------
#------------------- Unlock Transformation --------------------------------
#--------------------------------------------------------------------------


 
class OBJECT_OT_unlock_transform_cbox(bpy.types.Operator):
    '''Unlock Transformation'''
    bl_idname = "object.unlock_transform_cbox"
    bl_label = ""
    bl_options = {'REGISTER', 'UNDO'}


                
    def execute(self, context):
        

        def unlock_transform():
            sel_obj = bpy.context.selected_objects

            for o in sel_obj:
                bpy.context.view_layer.objects.active = o 

                o.lock_location[0] = False
                o.lock_location[1] = False
                o.lock_location[2] = False
                o.lock_rotation[0] = False
                o.lock_rotation[1] = False
                o.lock_rotation[2] = False


        cur_mode = bpy.context.mode


        ob = bpy.context.object

        if cur_mode != "OBJECT":
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            unlock_transform()


        if cur_mode == "OBJECT":
            if ob is not None:
                unlock_transform()

 
        return {'FINISHED'}    


#--------------------------------------------------------------------------
#---------------------- Unlock Transformation End  ------------------------
#--------------------------------------------------------------------------








classes = [ 
            OBJECT_OT_claybox_info_cx,
            OBJECT_OT_apply_scale_back2sculpt_cbox,
            OBJECT_OT_tool_update_cbox,
            OBJECT_OT_add_subdiv_res_cbox,
            OBJECT_OT_mirror_mod_cbox,
            OBJECT_OT_fill_obj_vertcolor_cbox,
            OBJECT_OT_create_vert_color_mat_cbox,
            OBJECT_OT_center_obj_cbox,
            OBJECT_OT_rotate_90_x_axis_cbox,
            OBJECT_OT_rotate_90_y_axis_cbox,
            OBJECT_OT_rotate_90_z_axis_cbox,
            OBJECT_OT_unlock_transform_cbox,

          ]
 
 
 
def register():
    for cls in classes:
        bpy.utils.register_class(cls)
        

 
def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)
