import bpy
from bpy.props import (FloatProperty, IntProperty)


#---------------------------Cuboid Prism-------------------------------------
#----------------------------------------------------------------------------
#----------------------------------------------------------------------------


class OBJECT_OT_create_cuboid_prism_cbox(bpy.types.Operator):
    '''Create Cuboid Prism'''
    bl_idname = "object.create_cuboid_prism_cbox"
    bl_label = "Cub.Prism"
    bl_options = {'REGISTER', 'UNDO'}


    init_mouse_x: IntProperty()
    init_value: FloatProperty()

    def modal(self, context, event):

        kc = bpy.context.window_manager.keyconfigs
        km = kc.active.keymaps['3D View'].keymap_items['view3d.cursor3d']


        # temp. change
        km.type = 'LEFTMOUSE'
        km.shift = 0  


        # Ref
        stay_in_sculpt_mode = context.scene.stay_in_sculpt_mode_cx  
        create_and_center = context.scene.create_and_center_x_axis_cx
        create_and_mirror = context.scene.create_and_mirror_cx
        

        def main_prism():

            mode = bpy.context.mode
            if mode != 'OBJECT':
                bpy.ops.object.mode_set(mode='OBJECT', toggle=False)


            cursor = bpy.context.scene.cursor

            c_x = cursor.location[0]
            c_y = cursor.location[1]
            c_z = cursor.location[2]

            # Cube --------------------------------------------------------
            def add_cube():
                
                bpy.ops.mesh.primitive_cube_add(size=2, 
                                                enter_editmode=True, 
                                                align='WORLD', 
                                                location=(c_x, c_y, c_z), 
                                                scale=(0.2, 0.2, 0.5))
                
                
               
                bpy.ops.mesh.loopcut_slide(MESH_OT_loopcut={"number_cuts":2, 
                                                            "object_index":0, 
                                                            "edge_index":9, 
                                                            "mesh_select_mode_init":(True, False, False)})
                          
                bpy.ops.mesh.select_all(action='SELECT')                                            
                bpy.ops.mesh.subdivide(number_cuts=2)
                
                bpy.ops.mesh.select_all(action='DESELECT')
                bpy.ops.object.mode_set(mode='OBJECT')

            #---------------------------------------------------------------
                                 
            add_cube()
       
            ob = bpy.context.object
            ob.name = 'cub_prism' 
            ob.data.name = 'cub_prism' 

            ob.scale.x /= 10
            ob.scale.y /= 10
            ob.scale.z /= 10            
            
            ob.modifiers.new('Subdivision', type='SUBSURF')
            ob.modifiers["Subdivision"].levels = 1
            bpy.ops.object.modifier_apply(modifier="Subdivision")        
        




        #------------------------------------------------------------------------

        # Place 3D Cursor
        if event.type == 'LEFTMOUSE' and event.value == 'PRESS':
            return {'PASS_THROUGH'}  


        if event.type == 'LEFTMOUSE' and event.value == 'RELEASE':
            main_prism()
           
                
            return {'RUNNING_MODAL'}
        
 
        sel_ob = bpy.context.selected_objects
        
        if sel_ob:
            obj = sel_ob[0]
            
            if event.type == 'MOUSEMOVE':
                delta = self.init_mouse_x - (event.mouse_x - event.mouse_prev_press_x)
                obj.scale.x = self.init_value + delta * -0.05
                obj.scale.y = self.init_value + delta * -0.05
                obj.scale.z = self.init_value + delta * -0.05


                # change mouse cursor
                bpy.context.window.cursor_modal_set("MOVE_X")

                # header info
                bpy.context.area.header_text_set('Right Click or press  ESC | SHIFT | CTRL | ALT | SPACEBAR | ENTER  -  to Confirm')


        if event.type in {'RIGHTMOUSE', 'SPACE', 'RET', 'LEFT_CTRL', 'LEFT_ALT', 'LEFT_SHIFT', 'ESC'} and event.value == 'PRESS':

            # restore keymaps
            km.type = 'RIGHTMOUSE'
            km.shift = 1           
            

            # restore mouse cursor
            bpy.context.window.cursor_modal_restore()

            # restore info header
            bpy.context.area.header_text_set(None)

            # apply scale
            bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)


            act_ob = bpy.context.active_object
            if stay_in_sculpt_mode:
                if act_ob is not None and act_ob.type == 'MESH':
                    bpy.ops.object.mode_set(mode='SCULPT', toggle=False)
                    

            if create_and_center:
                bpy.ops.object.center_obj_cbox()

            if create_and_mirror:
                bpy.ops.object.mirror_mod_cbox() 

            return {'FINISHED'}

        
        return {'RUNNING_MODAL'}



    def invoke(self, context, event):

        mode = bpy.context.mode
        if mode != 'OBJECT':
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)


        bpy.ops.object.select_all(action='DESELECT')

        context.window_manager.modal_handler_add(self)

        return {'RUNNING_MODAL'}



#--------------------------------------------------------------------------
#----------------------------Cuboid Prism End------------------------------
#--------------------------------------------------------------------------




#---------------------------Triangular Prism-------------------------------
#--------------------------------------------------------------------------
#--------------------------------------------------------------------------


class OBJECT_OT_create_triangular_prism_cbox(bpy.types.Operator):
    '''Create Triangular Prism'''
    bl_idname = "object.create_triangular_prism_cbox"
    bl_label = "Tri.Prism"
    bl_options = {'REGISTER', 'UNDO'}


    init_mouse_x: IntProperty()
    init_value: FloatProperty()

    def modal(self, context, event):

        kc = bpy.context.window_manager.keyconfigs
        km = kc.active.keymaps['3D View'].keymap_items['view3d.cursor3d']


        # temp. change
        km.type = 'LEFTMOUSE'
        km.shift = 0  


        # Ref
        stay_in_sculpt_mode = context.scene.stay_in_sculpt_mode_cx  
        create_and_center = context.scene.create_and_center_x_axis_cx
        create_and_mirror = context.scene.create_and_mirror_cx        

        def main_triprism():

        
            mode = bpy.context.mode
            if mode != 'OBJECT':
                bpy.ops.object.mode_set(mode='OBJECT', toggle=False)


            cursor = bpy.context.scene.cursor

            c_x = cursor.location[0]
            c_y = cursor.location[1]
            c_z = cursor.location[2]


            # Cylinder --------------------------------------------------------
            def add_cylinder():
                
                bpy.ops.mesh.primitive_cylinder_add(vertices=3, 
                                                    radius=1, 
                                                    depth=2, 
                                                    enter_editmode=True, 
                                                    align='WORLD', 
                                                    location=(c_x, c_y, c_z), 
                                                    scale=(0.6, 0.6, 1))
                                                    
                
                bpy.ops.mesh.bevel(offset=0.14, offset_pct=0, segments=2, affect='EDGES')
             
                         
                 
                bpy.ops.mesh.loopcut_slide(MESH_OT_loopcut={"number_cuts":2, 
                                                            "smoothness":0,                               
                                                            "object_index":0, 
                                                            "edge_index":63, 
                                                            "mesh_select_mode_init":(False, True, False)})


                bpy.ops.mesh.select_all(action='DESELECT')
                bpy.ops.object.mode_set(mode='OBJECT')
            #---------------------------------------------------------------
                                 
            add_cylinder()
       
            ob = bpy.context.object
            ob.name = 'tri_prism' 
            ob.data.name = 'tri_prism' 


            ob.modifiers.new('Subdivision', type='SUBSURF')
            ob.modifiers["Subdivision"].levels = 2
            bpy.ops.object.modifier_apply(modifier="Subdivision")        

            ob.scale.x /= 10
            ob.scale.y /= 10
            ob.scale.z /= 10            



        #------------------------------------------------------------------------

        # Place 3D Cursor
        if event.type == 'LEFTMOUSE' and event.value == 'PRESS':
            return {'PASS_THROUGH'}  


        if event.type == 'LEFTMOUSE' and event.value == 'RELEASE':
            main_triprism()
           
                
            return {'RUNNING_MODAL'}
        
 
        sel_ob = bpy.context.selected_objects
        
        if sel_ob:
            obj = sel_ob[0]
            
            if event.type == 'MOUSEMOVE':
                delta = self.init_mouse_x - (event.mouse_x - event.mouse_prev_press_x)
                obj.scale.x = self.init_value + delta * -0.05
                obj.scale.y = self.init_value + delta * -0.05
                obj.scale.z = self.init_value + delta * -0.05


                # change mouse cursor
                bpy.context.window.cursor_modal_set("MOVE_X")

                # header info
                bpy.context.area.header_text_set('Right Click or press  ESC | SHIFT | CTRL | ALT | SPACEBAR | ENTER  -  to Confirm')


        if event.type in {'RIGHTMOUSE', 'SPACE', 'RET', 'LEFT_CTRL', 'LEFT_ALT', 'LEFT_SHIFT', 'ESC'} and event.value == 'PRESS':

            # restore keymaps
            km.type = 'RIGHTMOUSE'
            km.shift = 1           
            

            # restore mouse cursor
            bpy.context.window.cursor_modal_restore()

            # restore info header
            bpy.context.area.header_text_set(None)

            # apply scale
            bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)


            act_ob = bpy.context.active_object
            if stay_in_sculpt_mode:
                if act_ob is not None and act_ob.type == 'MESH':
                    bpy.ops.object.mode_set(mode='SCULPT', toggle=False)
                    

            if create_and_center:
                bpy.ops.object.center_obj_cbox()

            if create_and_mirror:
                bpy.ops.object.mirror_mod_cbox() 


            return {'FINISHED'}

        
        return {'RUNNING_MODAL'}



    def invoke(self, context, event):

        mode = bpy.context.mode
        if mode != 'OBJECT':
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)


        bpy.ops.object.select_all(action='DESELECT')

        context.window_manager.modal_handler_add(self)

        return {'RUNNING_MODAL'}



#--------------------------------------------------------------------------
#----------------------------Triangular Prism End--------------------------
#--------------------------------------------------------------------------



#--------------------------------------------------------------------------
#---------------------------Rectangular Quader----------------------------- 
#--------------------------------------------------------------------------


class OBJECT_OT_create_rect_quader_cbox(bpy.types.Operator):
    '''Create Rectangular Quader'''
    bl_idname = "object.create_rect_quader_cbox"
    bl_label = "RQuader"
    bl_options = {'REGISTER', 'UNDO'}


    init_mouse_x: IntProperty()
    init_value: FloatProperty()

    def modal(self, context, event):

        kc = bpy.context.window_manager.keyconfigs
        km = kc.active.keymaps['3D View'].keymap_items['view3d.cursor3d']


        # temp. change
        km.type = 'LEFTMOUSE'
        km.shift = 0  


        # Ref
        stay_in_sculpt_mode = context.scene.stay_in_sculpt_mode_cx          
        create_and_center = context.scene.create_and_center_x_axis_cx
        create_and_mirror = context.scene.create_and_mirror_cx


        def main_RQuader():
        
            mode = bpy.context.mode
            if mode != 'OBJECT':
                bpy.ops.object.mode_set(mode='OBJECT', toggle=False)


            cursor = bpy.context.scene.cursor


            c_x = cursor.location[0]
            c_y = cursor.location[1]
            c_z = cursor.location[2]

            # Cube --------------------------------------------------------
            def add_cube():
                
                bpy.ops.mesh.primitive_cube_add(size=2, 
                                                enter_editmode=True, 
                                                align='WORLD', 
                                                location=(c_x, c_y, c_z), 
                                                scale=(0.5, 0.3, 0.12))
                
                

                bpy.ops.mesh.select_all(action='SELECT')                                            
                bpy.ops.mesh.subdivide(number_cuts=3)
                
                bpy.ops.mesh.select_all(action='DESELECT')
                bpy.ops.object.mode_set(mode='OBJECT')

            #---------------------------------------------------------------
                                 
            add_cube()
       

            ob = bpy.context.object
            ob.name = 'r_quader' 
            ob.data.name = 'r_quader' 
            
            ob.modifiers.new('Subdivision', type='SUBSURF')
            ob.modifiers["Subdivision"].levels = 1
            bpy.ops.object.modifier_apply(modifier="Subdivision")        
        
            ob.scale.x /= 10
            ob.scale.y /= 10
            ob.scale.z /= 10




        #------------------------------------------------------------------------

        # Place 3D Cursor
        if event.type == 'LEFTMOUSE' and event.value == 'PRESS':
            return {'PASS_THROUGH'}  


        if event.type == 'LEFTMOUSE' and event.value == 'RELEASE':
            main_RQuader()
           
                
            return {'RUNNING_MODAL'}
        
 
        sel_ob = bpy.context.selected_objects
        
        if sel_ob:
            obj = sel_ob[0]
            
            if event.type == 'MOUSEMOVE':
                delta = self.init_mouse_x - (event.mouse_x - event.mouse_prev_press_x)
                obj.scale.x = self.init_value + delta * -0.05
                obj.scale.y = self.init_value + delta * -0.05
                obj.scale.z = self.init_value + delta * -0.05


                # change mouse cursor
                bpy.context.window.cursor_modal_set("MOVE_X")

                # header info
                bpy.context.area.header_text_set('Right Click or press  ESC | SHIFT | CTRL | ALT | SPACEBAR | ENTER  -  to Confirm')


        if event.type in {'RIGHTMOUSE', 'SPACE', 'RET', 'LEFT_CTRL', 'LEFT_ALT', 'LEFT_SHIFT', 'ESC'} and event.value == 'PRESS':

            # restore keymaps
            km.type = 'RIGHTMOUSE'
            km.shift = 1           
            

            # restore mouse cursor
            bpy.context.window.cursor_modal_restore()

            # restore info header
            bpy.context.area.header_text_set(None)

            # apply scale
            bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)


            act_ob = bpy.context.active_object
            if stay_in_sculpt_mode:
                if act_ob is not None and act_ob.type == 'MESH':
                    bpy.ops.object.mode_set(mode='SCULPT', toggle=False)
                    

            if create_and_center:
                bpy.ops.object.center_obj_cbox()

            if create_and_mirror:
                bpy.ops.object.mirror_mod_cbox() 


            return {'FINISHED'}

        
        return {'RUNNING_MODAL'}



    def invoke(self, context, event):

        mode = bpy.context.mode
        if mode != 'OBJECT':
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)


        bpy.ops.object.select_all(action='DESELECT')

        context.window_manager.modal_handler_add(self)

        return {'RUNNING_MODAL'}


#--------------------------------------------------------------------------
#----------------------------Rectangular Quader End------------------------
#--------------------------------------------------------------------------



#--------------------------------------------------------------------------
#---------------------------------Quader-----------------------------------
#--------------------------------------------------------------------------


class OBJECT_OT_create_quader_cbox(bpy.types.Operator):
    '''Create Quader'''
    bl_idname = "object.create_quader_cbox"
    bl_label = "Quader"
    bl_options = {'REGISTER', 'UNDO'}


    init_mouse_x: IntProperty()
    init_value: FloatProperty()

    def modal(self, context, event):

        kc = bpy.context.window_manager.keyconfigs
        km = kc.active.keymaps['3D View'].keymap_items['view3d.cursor3d']


        # temp. change
        km.type = 'LEFTMOUSE'
        km.shift = 0  


        # Ref
        stay_in_sculpt_mode = context.scene.stay_in_sculpt_mode_cx          
        create_and_center = context.scene.create_and_center_x_axis_cx
        create_and_mirror = context.scene.create_and_mirror_cx


        def main_quader():
        
            mode = bpy.context.mode
            if mode != 'OBJECT':
                bpy.ops.object.mode_set(mode='OBJECT', toggle=False)


            cursor = bpy.context.scene.cursor


            c_x = cursor.location[0]
            c_y = cursor.location[1]
            c_z = cursor.location[2]

            # Cube --------------------------------------------------------
            def add_cube():
                
                bpy.ops.mesh.primitive_cube_add(size=2, 
                                                enter_editmode=True, 
                                                align='WORLD', 
                                                location=(c_x, c_y, c_z), 
                                                scale=(0.4, 0.4, 0.12))
                
                

                bpy.ops.mesh.select_all(action='SELECT')                                            
                bpy.ops.mesh.subdivide(number_cuts=3)
                
                bpy.ops.mesh.select_all(action='DESELECT')
                bpy.ops.object.mode_set(mode='OBJECT')

            #---------------------------------------------------------------
                                 
            add_cube()
       

            ob = bpy.context.object
            ob.name = 'quader' 
            ob.data.name = 'quader' 
            
            ob.modifiers.new('Subdivision', type='SUBSURF')
            ob.modifiers["Subdivision"].levels = 1
            bpy.ops.object.modifier_apply(modifier="Subdivision")        
        
            ob.scale.x /= 10
            ob.scale.y /= 10
            ob.scale.z /= 10



        #------------------------------------------------------------------------

        # Place 3D Cursor
        if event.type == 'LEFTMOUSE' and event.value == 'PRESS':
            return {'PASS_THROUGH'}  


        if event.type == 'LEFTMOUSE' and event.value == 'RELEASE':
            main_quader()
           
                
            return {'RUNNING_MODAL'}
        
 
        sel_ob = bpy.context.selected_objects
        
        if sel_ob:
            obj = sel_ob[0]
            
            if event.type == 'MOUSEMOVE':
                delta = self.init_mouse_x - (event.mouse_x - event.mouse_prev_press_x)
                obj.scale.x = self.init_value + delta * -0.05
                obj.scale.y = self.init_value + delta * -0.05
                obj.scale.z = self.init_value + delta * -0.05


                # change mouse cursor
                bpy.context.window.cursor_modal_set("MOVE_X")

                # header info
                bpy.context.area.header_text_set('Right Click or press  ESC | SHIFT | CTRL | ALT | SPACEBAR | ENTER  -  to Confirm')


        if event.type in {'RIGHTMOUSE', 'SPACE', 'RET', 'LEFT_CTRL', 'LEFT_ALT', 'LEFT_SHIFT', 'ESC'} and event.value == 'PRESS':

            # restore keymaps
            km.type = 'RIGHTMOUSE'
            km.shift = 1           
            

            # restore mouse cursor
            bpy.context.window.cursor_modal_restore()

            # restore info header
            bpy.context.area.header_text_set(None)

            # apply scale
            bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)


            act_ob = bpy.context.active_object
            if stay_in_sculpt_mode:
                if act_ob is not None and act_ob.type == 'MESH':
                    bpy.ops.object.mode_set(mode='SCULPT', toggle=False)
                    

            if create_and_center:
                bpy.ops.object.center_obj_cbox()

            if create_and_mirror:
                bpy.ops.object.mirror_mod_cbox() 


            return {'FINISHED'}

        
        return {'RUNNING_MODAL'}



    def invoke(self, context, event):

        mode = bpy.context.mode
        if mode != 'OBJECT':
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)


        bpy.ops.object.select_all(action='DESELECT')

        context.window_manager.modal_handler_add(self)

        return {'RUNNING_MODAL'}



#--------------------------------------------------------------------------
#----------------------------Quader End------------------------------
#--------------------------------------------------------------------------







classes = [ 

            OBJECT_OT_create_cuboid_prism_cbox,
            OBJECT_OT_create_triangular_prism_cbox,
            OBJECT_OT_create_rect_quader_cbox,
            OBJECT_OT_create_quader_cbox,

          ]
 
 
 
def register():
    for cls in classes:
        bpy.utils.register_class(cls)
        

 
def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)
