import bpy
from bpy.props import (FloatProperty, IntProperty)



#--------------------------------------------------------------------------  
#-------------------Round Cube---------------------------------------------    
#--------------------------------------------------------------------------  
    
# Low Res
 
class OBJECT_OT_create_rcube_low_cbox(bpy.types.Operator):
    '''Create round cube with 1 subdivision levels'''
    bl_idname = "object.create_rcube_low_cbox"
    bl_label = "Round Cube x1"
    bl_options = {'REGISTER', 'UNDO'}

    init_mouse_x: IntProperty()
    init_value: FloatProperty()

    def modal(self, context, event):


        kc = bpy.context.window_manager.keyconfigs
        km = kc.active.keymaps['3D View'].keymap_items['view3d.cursor3d']


        # temp. change
        km.type = 'LEFTMOUSE'
        km.shift = 0  


        # Ref
        stay_in_sculpt_mode = context.scene.stay_in_sculpt_mode_cx  
        create_and_center = context.scene.create_and_center_x_axis_cx
        create_and_mirror = context.scene.create_and_mirror_cx        


        def main_rcube():

        
            mode = bpy.context.mode
            if mode != 'OBJECT':
                bpy.ops.object.mode_set(mode='OBJECT', toggle=False)



            cursor = bpy.context.scene.cursor

            c_x = cursor.location[0]
            c_y = cursor.location[1]
            c_z = cursor.location[2]        
            
            # Cube --------------------------------------------------------
            def add_cube():
                
                bpy.ops.mesh.primitive_cube_add(size=2, 
                                                enter_editmode=True, 
                                                align='WORLD', 
                                                location=(c_x, c_y, c_z), 
                                                scale=(0.2, 0.2, 0.2))
                
                bpy.ops.mesh.subdivide(number_cuts=1)

            #---------------------------------------------------------------
                                 
            add_cube()
            
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            
            ob = bpy.context.object
            ob.name = 'rcube_block_out' 
            ob.data.name = 'rcube_block_out' 

            ob.scale.x /= 10
            ob.scale.y /= 10
            ob.scale.z /= 10

            # add and apply sub mod
            ob.modifiers.new('Subdivision', type='SUBSURF')
            ob.modifiers["Subdivision"].levels = 1
            bpy.ops.object.modifier_apply(modifier="Subdivision")



        #------------------------------------------------------------------------

        # Place 3D Cursor
        if event.type == 'LEFTMOUSE' and event.value == 'PRESS':
            return {'PASS_THROUGH'}  


        if event.type == 'LEFTMOUSE' and event.value == 'RELEASE':
            main_rcube()
           
                
            return {'RUNNING_MODAL'}
        
 
        sel_ob = bpy.context.selected_objects
        
        if sel_ob:
            obj = sel_ob[0]
            
            if event.type == 'MOUSEMOVE':
                delta = self.init_mouse_x - (event.mouse_x - event.mouse_prev_press_x)
                obj.scale.x = self.init_value + delta * -0.05
                obj.scale.y = self.init_value + delta * -0.05
                obj.scale.z = self.init_value + delta * -0.05


                # change mouse cursor
                bpy.context.window.cursor_modal_set("MOVE_X")

                # header info
                bpy.context.area.header_text_set('Right Click or press  ESC | SHIFT | CTRL | ALT | SPACEBAR | ENTER  -  to Confirm')


        if event.type in {'RIGHTMOUSE', 'SPACE', 'RET', 'LEFT_CTRL', 'LEFT_ALT', 'LEFT_SHIFT', 'ESC'} and event.value == 'PRESS':

            # restore keymaps
            km.type = 'RIGHTMOUSE'
            km.shift = 1           
            

            # restore mouse cursor
            bpy.context.window.cursor_modal_restore()

            # restore info header
            bpy.context.area.header_text_set(None)

            # apply scale
            bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)



            act_ob = bpy.context.active_object
            if stay_in_sculpt_mode:
                if act_ob is not None and act_ob.type == 'MESH':
                    bpy.ops.object.mode_set(mode='SCULPT', toggle=False)
                    

            if create_and_center:
                bpy.ops.object.center_obj_cbox()

            if create_and_mirror:
                bpy.ops.object.mirror_mod_cbox() 
                

            return {'FINISHED'}

        
        return {'RUNNING_MODAL'}


    def invoke(self, context, event):

        mode = bpy.context.mode
        if mode != 'OBJECT':
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)


        bpy.ops.object.select_all(action='DESELECT')

        context.window_manager.modal_handler_add(self)

        return {'RUNNING_MODAL'}



#--------------------------------------------------------------------------
#--------------------------------------------------------------------------
#--------------------------------------------------------------------------
    
    
# Mid Res
 
class OBJECT_OT_create_rcube_mid_cbox(bpy.types.Operator):
    '''Create round cube with 2 subdivision levels'''
    bl_idname = "object.create_rcube_mid_cbox"
    bl_label = "Round Cube x2"
    bl_options = {'REGISTER', 'UNDO'}


    init_mouse_x: IntProperty()
    init_value: FloatProperty()

    def modal(self, context, event):


        kc = bpy.context.window_manager.keyconfigs
        km = kc.active.keymaps['3D View'].keymap_items['view3d.cursor3d']


        # temp. change
        km.type = 'LEFTMOUSE'
        km.shift = 0  


        # Ref
        stay_in_sculpt_mode = context.scene.stay_in_sculpt_mode_cx  
        create_and_center = context.scene.create_and_center_x_axis_cx
        create_and_mirror = context.scene.create_and_mirror_cx

        def main_rcube():

            mode = bpy.context.mode
            if mode != 'OBJECT':
                bpy.ops.object.mode_set(mode='OBJECT', toggle=False)

     

            cursor = bpy.context.scene.cursor

            c_x = cursor.location[0]
            c_y = cursor.location[1]
            c_z = cursor.location[2]        
            
            # Cube --------------------------------------------------------
            def add_cube():
                
                bpy.ops.mesh.primitive_cube_add(size=2, 
                                                enter_editmode=True, 
                                                align='WORLD', 
                                                location=(c_x, c_y, c_z), 
                                                scale=(0.2, 0.2, 0.2))
                
                bpy.ops.mesh.subdivide(number_cuts=1)

            #---------------------------------------------------------------
                                 
            add_cube()
            
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            
            ob = bpy.context.object
            ob.name = 'rcube_block_out' 
            ob.data.name = 'rcube_block_out' 

            ob.scale.x /= 10
            ob.scale.y /= 10
            ob.scale.z /= 10            
            
            # add and apply sub mod
            ob.modifiers.new('Subdivision', type='SUBSURF')
            ob.modifiers["Subdivision"].levels = 2
            bpy.ops.object.modifier_apply(modifier="Subdivision")



        #------------------------------------------------------------------------

        # Place 3D Cursor
        if event.type == 'LEFTMOUSE' and event.value == 'PRESS':
            return {'PASS_THROUGH'}  


        if event.type == 'LEFTMOUSE' and event.value == 'RELEASE':
            main_rcube()
           
                
            return {'RUNNING_MODAL'}
        
 
        sel_ob = bpy.context.selected_objects
        
        if sel_ob:
            obj = sel_ob[0]
            
            if event.type == 'MOUSEMOVE':
                delta = self.init_mouse_x - (event.mouse_x - event.mouse_prev_press_x)
                obj.scale.x = self.init_value + delta * -0.05
                obj.scale.y = self.init_value + delta * -0.05
                obj.scale.z = self.init_value + delta * -0.05


                # change mouse cursor
                bpy.context.window.cursor_modal_set("MOVE_X")

                # header info
                bpy.context.area.header_text_set('Right Click or press  ESC | SHIFT | CTRL | ALT | SPACEBAR | ENTER  -  to Confirm')


        if event.type in {'RIGHTMOUSE', 'SPACE', 'RET', 'LEFT_CTRL', 'LEFT_ALT', 'LEFT_SHIFT', 'ESC'} and event.value == 'PRESS':

            # restore keymaps
            km.type = 'RIGHTMOUSE'
            km.shift = 1           
            

            # restore mouse cursor
            bpy.context.window.cursor_modal_restore()

            # restore info header
            bpy.context.area.header_text_set(None)

            # apply scale
            bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)


            act_ob = bpy.context.active_object
            if stay_in_sculpt_mode:
                if act_ob is not None and act_ob.type == 'MESH':
                    bpy.ops.object.mode_set(mode='SCULPT', toggle=False)
                    

            if create_and_center:
                bpy.ops.object.center_obj_cbox()

            if create_and_mirror:
                bpy.ops.object.mirror_mod_cbox() 


            return {'FINISHED'}

        
        return {'RUNNING_MODAL'}


    def invoke(self, context, event):

        mode = bpy.context.mode
        if mode != 'OBJECT':
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)


        bpy.ops.object.select_all(action='DESELECT')

        context.window_manager.modal_handler_add(self)

        return {'RUNNING_MODAL'}



#--------------------------------------------------------------------------    
#--------------------------------------------------------------------------    
#--------------------------------------------------------------------------    
    
# High Res
 
class OBJECT_OT_create_rcube_high_cbox(bpy.types.Operator):
    '''Create round cube with 3 subdivision levels'''
    bl_idname = "object.create_rcube_high_cbox"
    bl_label = "Round Cube x3"
    bl_options = {'REGISTER', 'UNDO'}


    init_mouse_x: IntProperty()
    init_value: FloatProperty()

    def modal(self, context, event):


        kc = bpy.context.window_manager.keyconfigs
        km = kc.active.keymaps['3D View'].keymap_items['view3d.cursor3d']


        # temp. change
        km.type = 'LEFTMOUSE'
        km.shift = 0  


        # Ref
        stay_in_sculpt_mode = context.scene.stay_in_sculpt_mode_cx  
        create_and_center = context.scene.create_and_center_x_axis_cx
        create_and_mirror = context.scene.create_and_mirror_cx

        def main_rcube():


            mode = bpy.context.mode
            if mode != 'OBJECT':
                bpy.ops.object.mode_set(mode='OBJECT', toggle=False)

     
     
            cursor = bpy.context.scene.cursor

            c_x = cursor.location[0]
            c_y = cursor.location[1]
            c_z = cursor.location[2]        
            
            # Cube --------------------------------------------------------
            
            def add_cube():
                
                bpy.ops.mesh.primitive_cube_add(size=2, 
                                                enter_editmode=True, 
                                                align='WORLD', 
                                                location=(c_x, c_y, c_z), 
                                                scale=(0.2, 0.2, 0.2))
                
                bpy.ops.mesh.subdivide(number_cuts=1)

            #---------------------------------------------------------------
                                 
            add_cube()
            
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            
            ob = bpy.context.object
            ob.name = 'rcube_block_out' 
            ob.data.name = 'rcube_block_out' 

            ob.scale.x /= 10
            ob.scale.y /= 10
            ob.scale.z /= 10            
            
            # add and apply sub mod
            ob.modifiers.new('Subdivision', type='SUBSURF')
            ob.modifiers["Subdivision"].levels = 3
            bpy.ops.object.modifier_apply(modifier="Subdivision")



        #------------------------------------------------------------------------

        # Place 3D Cursor
        if event.type == 'LEFTMOUSE' and event.value == 'PRESS':
            return {'PASS_THROUGH'}  


        if event.type == 'LEFTMOUSE' and event.value == 'RELEASE':
            main_rcube()
           
                
            return {'RUNNING_MODAL'}
        
 
        sel_ob = bpy.context.selected_objects
        
        if sel_ob:
            obj = sel_ob[0]
            
            if event.type == 'MOUSEMOVE':
                delta = self.init_mouse_x - (event.mouse_x - event.mouse_prev_press_x)
                obj.scale.x = self.init_value + delta * -0.05
                obj.scale.y = self.init_value + delta * -0.05
                obj.scale.z = self.init_value + delta * -0.05


                # change mouse cursor
                bpy.context.window.cursor_modal_set("MOVE_X")

                # header info
                bpy.context.area.header_text_set('Right Click or press  ESC | SHIFT | CTRL | ALT | SPACEBAR | ENTER  -  to Confirm')


        if event.type in {'RIGHTMOUSE', 'SPACE', 'RET', 'LEFT_CTRL', 'LEFT_ALT', 'LEFT_SHIFT', 'ESC'} and event.value == 'PRESS':

            # restore keymaps
            km.type = 'RIGHTMOUSE'
            km.shift = 1           
            

            # restore mouse cursor
            bpy.context.window.cursor_modal_restore()

            # restore info header
            bpy.context.area.header_text_set(None)

            # apply scale
            bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)


            act_ob = bpy.context.active_object
            if stay_in_sculpt_mode:
                if act_ob is not None and act_ob.type == 'MESH':
                    bpy.ops.object.mode_set(mode='SCULPT', toggle=False)
                    

            if create_and_center:
                bpy.ops.object.center_obj_cbox()

            if create_and_mirror:
                bpy.ops.object.mirror_mod_cbox() 


            return {'FINISHED'}

        
        return {'RUNNING_MODAL'}



    def invoke(self, context, event):

        mode = bpy.context.mode
        if mode != 'OBJECT':
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)


        bpy.ops.object.select_all(action='DESELECT')

        context.window_manager.modal_handler_add(self)

        return {'RUNNING_MODAL'}


#--------------------------------------------------------------------------

#---------------------Round Cube End---------------------------------------

#--------------------------------------------------------------------------













classes = [ 

            OBJECT_OT_create_rcube_low_cbox,
            OBJECT_OT_create_rcube_mid_cbox,
            OBJECT_OT_create_rcube_high_cbox,

            
          ]
 
 
 
def register():
    for cls in classes:
        bpy.utils.register_class(cls)
        

 
def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)
