import bpy
from bpy.props import (FloatVectorProperty, FloatProperty)



# Function
#=====================================

# Emission Color
#====================================

def change_emission_color_fun_anbx(self, context):
    
    sel_ob = bpy.context.selected_objects
    act_ob = bpy.context.active_object
    cur_frame = bpy.context.scene.frame_current

    # location -> material_ax.py
    add_keyframe = context.scene.mat_light_add_keyframe_anbx

    for o in sel_ob:
        mat_object_types = ['CURVE', 'MESH', 'FONT', 'SURFACE', 'META']
        bpy.context.view_layer.objects.active = o

        if o.type in mat_object_types:

            for m in o.data.materials:
                mat_name = m.name

                num = 17
                # Check Blender Version
                blender_version = bpy.app.version[0]

                if blender_version == 2:
                    num = 17

                if blender_version == 3:
                    num = 19

                if blender_version == 4:
                    num = 26


                # Emission
                for n in bpy.data.materials[mat_name].node_tree.nodes:
                    if n.name == "Principled BSDF":
                        n.inputs[num].default_value  = self.emission_color_anbx

                        # if bool 'add keyframe'
                        if add_keyframe:                        
                            n.inputs[num].keyframe_insert(data_path="default_value", frame=cur_frame)



                for n in bpy.data.materials[mat_name].node_tree.nodes:
                    if n.name == "Emission":
                        n.inputs[0].default_value  = self.emission_color_anbx

                        # if bool 'add keyframe'
                        if add_keyframe:                        
                            n.inputs[0].keyframe_insert(data_path="default_value", frame=cur_frame)

# End Emission Color
#===========================================================================



# Emission Strength
#===========================================================================
def emission_strength_update_fun_anbx(self, context):

    sel_ob = bpy.context.selected_objects
    act_ob = bpy.context.active_object
    cur_frame = bpy.context.scene.frame_current

    # location -> material_mdh.py
    add_keyframe = context.scene.mat_light_add_keyframe_anbx   

    #======================================
    # Surfaces


    # Emission Strength
    for o in sel_ob:
        mat_object_types = ['CURVE', 'MESH', 'FONT', 'SURFACE', 'META']
        bpy.context.view_layer.objects.active = o

        if o.type in mat_object_types:

            for m in o.data.materials:
                mat_name = m.name


                num = 18
                # Check Blender Version
                blender_version = bpy.app.version[0]

                if blender_version == 2:
                    num = 18

                if blender_version == 3:
                    num = 20

                if blender_version == 4:
                    num = 27



                for n in bpy.data.materials[mat_name].node_tree.nodes:
                    if n.name == "Principled BSDF":
                        n.inputs[num].default_value = self.emission_strength_anbx

                        # if bool 'add keyframe'
                        if add_keyframe:   
                            n.inputs[num].keyframe_insert(data_path="default_value", frame=cur_frame)


                    if n.name == "Emission":
                        n.inputs[1].default_value = self.emission_strength_anbx

                        # if bool 'add keyframe'
                        if add_keyframe:   
                            n.inputs[1].keyframe_insert(data_path="default_value", frame=cur_frame)



# End Emission Strength
#===========================================================================





def register():


    bpy.types.Scene.emission_color_anbx = bpy.props.FloatVectorProperty(
            name="Emission",
            subtype="COLOR",
            default=(1.0, 1.0, 1.0, 1.0),
            size=4,
            min=0.0,max=1.0,
            description="Selected Objects",
            update= change_emission_color_fun_anbx
            )


    bpy.types.Scene.emission_strength_anbx = bpy.props.FloatProperty(
            name="Emission Strength",
            default=1.0,
            precision=3,
            step=10,
            min=0,
            description="Selected Objects",
            update = emission_strength_update_fun_anbx
            )



def unregister():

    del bpy.types.Scene.emission_color_anbx
    del bpy.types.Scene.emission_strength_anbx



if __name__ == "__main__":
    register()





