import bpy
import random



#=====================================================
#                Diffuse Colors Random
#=====================================================



class OBJECT_OT_diffuse_color_random_anbx(bpy.types.Operator):
    '''Diffuse Color Random - Selected Objects
- Create Material(s) - if not exists '''

    bl_idname = "object.diffuse_color_random_anbx"
    bl_label = "Color Random"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):

        object_types = ['CURVE', 'MESH', 'FONT', 'SURFACE']

        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) != 0 and
                context.object.type in object_types)
        

    def execute(self, context):

        mat_object_types = ['CURVE', 'MESH', 'FONT', 'SURFACE']

        sel_ob = bpy.context.selected_objects
        act_ob = bpy.context.active_object
        cur_frame = bpy.context.scene.frame_current

        # location -> material_ax.py
        add_keyframe = context.scene.mat_light_add_keyframe_anbx


        # Add Material if not Exist

        def add_new_mat_to_active():
            mat = act_ob.data.materials
            if not mat:
                new_mat = bpy.data.materials.new(act_ob.name + '_mat')
                act_ob.data.materials.append(new_mat)
                bpy.context.object.active_material.use_nodes = True


        def add_new_mat_to_multi_selected():
            for o in sel_ob:
                bpy.context.view_layer.objects.active = o

                if o.type in mat_object_types:

                    mat = o.data.materials
                    if not mat:
                        new_mat = bpy.data.materials.new(o.name + '_mat')
                        o.data.materials.append(new_mat)
                        bpy.context.object.active_material.use_nodes = True
                        bpy.ops.object.make_single_user(object=False, obdata=False, material=True, animation=False)


        #---------------------------------------

        if len(sel_ob) == 1:
            add_new_mat_to_active()


        if len(sel_ob) > 1:
            add_new_mat_to_multi_selected()
            bpy.ops.object.make_single_user(object=False, obdata=False, material=True, animation=False)

        #---------------------------------------

        # Set First Material Slot as Active
        for o in bpy.context.selected_objects:
            o.active_material_index = 0

            bpy.ops.object.material_slot_remove_unused()

        #======================================
        # Surfaces

        node_list = ["Principled BSDF",
                     "Diffuse BSDF", "Emission",
                     "Glass BSDF", "Anisotropic BSDF",
                     "Glossy BSDF", "Principled Hair BSDF",
                     "Principled Volume", "Refraction BSDF",
                     "Specular", "Subsurface Scattering",
                     "Toon BSDF", "Translucent BSDF",
                     "Transparent BSDF", "Velvet BSDF",
                     ]


        #======================================
        # Get Colors

        for o in sel_ob:
            o_name = o.name

            if o.type in mat_object_types:

                for m in o.data.materials:
                    for n in m.node_tree.nodes:
                        if n.name in node_list:
                            node_name = n.name
                            inputs = n.inputs
                            color = inputs[0].default_value

                            r = color[0]
                            g = color[1]
                            b = color[2]
                            a = color[3]

        #============================================


        def rand_diff_col():

            act_mat = bpy.context.object.active_material
            if act_mat:

                mat_name = act_mat.name
                diff_col = act_mat.diffuse_color

                diff_col[0] = random.random()
                diff_col[1] = random.random()
                diff_col[2] = random.random()
                diff_col[3] = 1.0

                # if bool 'add keyframe'
                if add_keyframe:

                    act_mat.diffuse_color.data.keyframe_insert(data_path="diffuse_color", frame=cur_frame)


        def rand_render_diff_col():

            o = bpy.context.object

            if o.type in mat_object_types:

                for m in o.data.materials:
                    mat_name = m.name
                    r = random.random()
                    g = random.random()
                    b = random.random()
                    a = 1.0

                    for n in bpy.data.materials[mat_name].node_tree.nodes:
                        if n.name in node_list:
                            n.inputs[0].default_value  = (r,g,b,a)

                            # if bool 'add keyframe'
                            if add_keyframe:

                               # Insert Keyframe - Diffuse
                                n.inputs[0].keyframe_insert(data_path="default_value", frame=cur_frame)


        for o in sel_ob:
            bpy.context.view_layer.objects.active = o
            rand_diff_col()
            rand_render_diff_col()



        return {'FINISHED'}




#=====================================================
#            End Diffuse Colors Random
#=====================================================




#=====================================================
#                Emission Colors Random
#=====================================================



class OBJECT_OT_emission_color_random_anbx(bpy.types.Operator):
    '''Emission Color Random - Selected Objects
- Create Material(s) - if not exists'''

    bl_idname = "object.emission_color_random_anbx"
    bl_label = "Color Random"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):

        object_types = ['CURVE', 'MESH', 'FONT', 'SURFACE']

        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) != 0 and
                context.object.type in object_types)
        

    def execute(self, context):

        mat_object_types = ['CURVE', 'MESH', 'FONT', 'SURFACE']

        sel_ob = bpy.context.selected_objects
        act_ob = bpy.context.active_object
        cur_frame = bpy.context.scene.frame_current

        # location -> material_mdh.py
        add_keyframe = context.scene.mat_light_add_keyframe_anbx


        # Add Material if not Exist

        def add_new_mat_to_active():
            mat = act_ob.data.materials
            if not mat:
                new_mat = bpy.data.materials.new(act_ob.name + '_mat')
                act_ob.data.materials.append(new_mat)
                bpy.context.object.active_material.use_nodes = True


        def add_new_mat_to_multi_selected():
            for o in sel_ob:
                bpy.context.view_layer.objects.active = o

                if o.type in mat_object_types:

                    mat = o.data.materials
                    if not mat:
                        new_mat = bpy.data.materials.new(o.name + '_mat')
                        o.data.materials.append(new_mat)
                        bpy.context.object.active_material.use_nodes = True
                        bpy.ops.object.make_single_user(object=False, obdata=False, material=True, animation=False)
                        

        #---------------------------------------

        if len(sel_ob) == 1:
            add_new_mat_to_active()


        if len(sel_ob) > 1:
            add_new_mat_to_multi_selected()
            bpy.ops.object.make_single_user(object=False, obdata=False, material=True, animation=False)

        #---------------------------------------

        # Set First Material Slot as Active
        for o in bpy.context.selected_objects:
            o.active_material_index = 0

            bpy.ops.object.material_slot_remove_unused()

        #======================================
        # Surfaces

        node_list = ["Principled BSDF", "Emission"]

 
        for o in sel_ob:
            o_name = o.name

            if o.type in mat_object_types:

                for m in o.data.materials:
                    for n in m.node_tree.nodes:
                        if n.name in node_list:
                            node_name = n.name
                            inputs = n.inputs
                            color = inputs[0].default_value

                            r = color[0]
                            g = color[1]
                            b = color[2]
                            a = color[3]
                         
        #============================================
                  
        def random_emission_col():
            
            o = bpy.context.object 

            if o.type in mat_object_types:
                           
                for m in o.data.materials:
                    mat_name = m.name              
                    r = random.random()
                    g = random.random()
                    b = random.random()
                    a = 1.0
                    
             
                    num = 17
                    # Check Blender Version
                    blender_version = bpy.app.version[0]

                    if blender_version == 2:
                        num = 17
            
                    if blender_version == 3: 
                        num = 19

                    if blender_version == 4:
                        num = 26
                        

                    # Emission
                    for n in bpy.data.materials[mat_name].node_tree.nodes:
                        if n.name == "Principled BSDF":
                            n.inputs[num].default_value  = (r,g,b,a)

                            # if bool 'add keyframe'
                            if add_keyframe:

                                n.inputs[num].keyframe_insert(data_path="default_value", frame=cur_frame)
                             
              
              
                    for n in bpy.data.materials[mat_name].node_tree.nodes:
                        if n.name == "Emission":
                            n.inputs[0].default_value  = (r,g,b,a)

                            # if bool 'add keyframe'
                            if add_keyframe:

                                n.inputs[0].keyframe_insert(data_path="default_value", frame=cur_frame)
                             
              


      
        for o in sel_ob:
            bpy.context.view_layer.objects.active = o
            random_emission_col()
            
          
            
        return {'FINISHED'}


#=====================================================
#            End Emission Colors Random
#=====================================================





classes = [OBJECT_OT_diffuse_color_random_anbx,
           OBJECT_OT_emission_color_random_anbx,
           ]




def register():
    for cls in classes:
        bpy.utils.register_class(cls)




def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)



if __name__ == "__main__":
    register()



