import bpy
import random 
from bpy.props import (BoolProperty, FloatProperty)




#=====================================================
#             Shift Color
#=====================================================


class OBJECT_OT_offset_colors_anbx(bpy.types.Operator):
    '''Shift Diffuse Color - Selected Objects.
- Create Material(s) - if not exists'''



    bl_idname = "object.offset_colors_anbx"
    bl_label = "Shift Color"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):

        object_types = ['CURVE', 'MESH', 'FONT', 'SURFACE']

        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) > 1 and
                context.object.type in object_types)
        

    def execute(self, context):

        mat_object_types = ['CURVE', 'MESH', 'FONT', 'SURFACE']

        sel_ob = bpy.context.selected_objects
        act_ob = bpy.context.active_object
        cur_frame = bpy.context.scene.frame_current
        col_offset = 0

     

        # Bool Ref
        rev_color_bool = context.scene.revers_color_bool   

        # location -> material_mdh.py
        add_keyframe = context.scene.mat_light_add_keyframe_anbx   

         
        # Shift Color Amount  Ref
        shift_amount = context.scene.shift_color_factor



        # Add Material if not Exist

        def add_new_mat_to_active():
            mat = act_ob.data.materials
            if not mat:
                new_mat = bpy.data.materials.new('offset_mat')
                act_ob.data.materials.append(new_mat)
                bpy.context.object.active_material.use_nodes = True


        def add_new_mat_to_multi_selected():
            for o in sel_ob:
                bpy.context.view_layer.objects.active = o

                if o.type in mat_object_types:

                    mat = o.data.materials
                    if not mat:
                        new_mat = bpy.data.materials.new('offset_mat')
                        o.data.materials.append(new_mat)
                        bpy.context.object.active_material.use_nodes = True
                        bpy.ops.object.make_single_user(object=False, obdata=False, material=True, animation=False)
                        

        #---------------------------------------

        if len(sel_ob) == 1:
            add_new_mat_to_active()


        if len(sel_ob) > 1:
            add_new_mat_to_multi_selected()


        #---------------------------------------

        # Set First Material Slot as Active
        for o in bpy.context.selected_objects:
            o.active_material_index = 0

            bpy.ops.object.material_slot_remove_unused()

         
        #======================================
        # Surfaces

        node_list = ["Principled BSDF",
                     "Diffuse BSDF", "Emission",
                     "Glass BSDF", "Anisotropic BSDF",
                     "Glossy BSDF", "Principled Hair BSDF",
                     "Principled Volume", "Refraction BSDF",
                     "Specular", "Subsurface Scattering",
                     "Toon BSDF", "Translucent BSDF",
                     "Transparent BSDF", "Velvet BSDF",
                     ] 

        #======================================

        # Get Colors

                    
        for o in sel_ob:
            if o == act_ob:
                o_name = o.name
                for m in o.data.materials:
                    for n in m.node_tree.nodes:
                        if n.name in node_list:
                            node_name = n.name
                            inputs = n.inputs
                            color = inputs[0].default_value

                            r = color[0]
                            g = color[1]
                            b = color[2]
                            a = color[3]
                         
        #============================================


        # Set Colors

        def set_color(objects):
            col_offset = 0
            for o in objects:
                bpy.context.view_layer.objects.active = o

                if o.type in mat_object_types:


                    for m in o.data.materials:
                        mat_name = m.name
                        
                        m.diffuse_color = (r + col_offset,
                                           g + col_offset,
                                           b + col_offset, a)
                                            
                        
                        # if bool 'add keyframe'
                        if add_keyframe:              

                            # Insert Keyframe - Diffuse                   
                            m.diffuse_color.data.keyframe_insert(data_path="diffuse_color", frame=cur_frame)
                            

                        for n in bpy.data.materials[mat_name].node_tree.nodes:
                            if n.name in node_list:
                                n.inputs[0].default_value  = (r + col_offset,
                                                              g + col_offset,
                                                              b + col_offset, a)

                                       
                                # if bool 'add keyframe'
                                if add_keyframe: 

                                    # Insert Keyframe - Diffuse 
                                    n.inputs[0].keyframe_insert(data_path="default_value", frame=cur_frame)
                                
                        col_offset += shift_amount / (len(sel_ob))
                 


        sel_ob = bpy.context.selected_objects
        if rev_color_bool:
            set_color(reversed(sel_ob))

        else:
            set_color(sel_ob)

          
             
        return {'FINISHED'}



#=====================================================
#             End Shift Color
#=====================================================







#=====================================================
#             Shift Color Random
#=====================================================



class OBJECT_OT_offset_colors_random_anbx(bpy.types.Operator):
    '''Shift Diffuse Color Random - Selected Objects
- Create Material(s) - if not exists'''

    bl_idname = "object.offset_colors_random_anbx"
    bl_label = "Shift Random"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):

        object_types = ['CURVE', 'MESH', 'FONT', 'SURFACE']

        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) > 1 and
                context.object.type in object_types)
        

    def execute(self, context):

        mat_object_types = ['CURVE', 'MESH', 'FONT', 'SURFACE']

        sel_ob = bpy.context.selected_objects
        act_ob = bpy.context.active_object
        cur_frame = bpy.context.scene.frame_current
        col_offset = 0


        # Bool Ref
        rev_color_bool = context.scene.revers_color_bool 

        # location -> material_mdh.py
        add_keyframe = context.scene.mat_light_add_keyframe_anbx  


        # Shift Color Amount  Ref
        shift_amount = context.scene.shift_color_factor

                  

        # Add Material if not Exist

        def add_new_mat_to_active():
            mat = act_ob.data.materials
            if not mat:
                new_mat = bpy.data.materials.new('offset_mat')
                act_ob.data.materials.append(new_mat)
                bpy.context.object.active_material.use_nodes = True


        def add_new_mat_to_multi_selected():
            for o in sel_ob:
                bpy.context.view_layer.objects.active = o

                if o.type in mat_object_types:

                    mat = o.data.materials
                    if not mat:
                        new_mat = bpy.data.materials.new('offset_mat')
                        o.data.materials.append(new_mat)
                        bpy.context.object.active_material.use_nodes = True
                        bpy.ops.object.make_single_user(object=False, obdata=False, material=True, animation=False)
                        

        #---------------------------------------

        if len(sel_ob) == 1:
            add_new_mat_to_active()


        if len(sel_ob) > 1:
            add_new_mat_to_multi_selected()
            bpy.ops.object.make_single_user(object=False, obdata=False, material=True, animation=False)


        #---------------------------------------

        # Set First Material Slot as Active
        for o in bpy.context.selected_objects:
            o.active_material_index = 0

            bpy.ops.object.material_slot_remove_unused()

        #======================================
        # Surfaces

        node_list = ["Principled BSDF",
                     "Diffuse BSDF", "Emission",
                     "Glass BSDF", "Anisotropic BSDF",
                     "Glossy BSDF", "Principled Hair BSDF",
                     "Principled Volume", "Refraction BSDF",
                     "Specular", "Subsurface Scattering",
                     "Toon BSDF", "Translucent BSDF",
                     "Transparent BSDF", "Velvet BSDF",
                     ] 


        #======================================
        # Get Colors

                    
        for o in sel_ob:
            o_name = o.name

            if o.type in mat_object_types:

                for m in o.data.materials:
                    for n in m.node_tree.nodes:
                        if n.name in node_list:
                            node_name = n.name
                            inputs = n.inputs
                            color = inputs[0].default_value


                            color[0] = random.random()
                            color[1] = random.random()
                            color[2] = random.random()
                            color[3] = 1.0    
                                

                            r = color[0]
                            g = color[1]
                            b = color[2]
                            a = color[3]
                         
        #============================================

        # Set Colors

        def set_color(objects):
            col_offset = 0
            for o in objects:
                bpy.context.view_layer.objects.active = o

                if o.type in mat_object_types:
                    
                    for m in o.data.materials:
                        mat_name = m.name
                        m.diffuse_color = (r + col_offset,
                                           g + col_offset,
                                           b + col_offset, a)


                        # if bool 'add keyframe'
                        if add_keyframe:   

                            # Insert Keyframe - Diffuse                   
                            m.diffuse_color.data.keyframe_insert(data_path="diffuse_color", frame=cur_frame)
                        

                        for n in bpy.data.materials[mat_name].node_tree.nodes:
                            if n.name in node_list:
                                n.inputs[0].default_value  = (r + col_offset,
                                                              g + col_offset,
                                                              b + col_offset, a)
                                       

                                # if bool 'add keyframe'
                                if add_keyframe: 
                                             
                                    # Insert Keyframe - Diffuse 
                                    n.inputs[0].keyframe_insert(data_path="default_value", frame=cur_frame)
                                

                        col_offset += shift_amount / (len(sel_ob))
                 


        sel_ob = bpy.context.selected_objects

        if rev_color_bool:
            set_color(reversed(sel_ob))

        else:
            set_color(sel_ob)

          

         
        return {'FINISHED'}




#=====================================================
#            End Shift Color Random
#=====================================================







classes = [OBJECT_OT_offset_colors_anbx,
           OBJECT_OT_offset_colors_random_anbx,
           ]




def register():
    for cls in classes:
        bpy.utils.register_class(cls)




    bpy.types.Scene.revers_color_bool = bpy.props.BoolProperty(
        name="",
        description="REVERSED COLOR SHIFT",
        default = False
    )


 

    bpy.types.Scene.shift_color_factor = bpy.props.FloatProperty(
        name="Shift Amount",
        description="Shift Amount: bigger Number = stronger Color Shift ",
        default = 1.0,
        min=0.2, max=2.0,
    )





def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)

    del bpy.types.Scene.revers_color_bool
    del bpy.types.Scene.shift_color_factor


if __name__ == "__main__":
    register()

