import bpy
import bmesh
from mathutils import Matrix, Vector
import random
import string




#========================================================================
# Copy Active To Selected - To Center
#========================================================================



class OBJECT_OT_copy_active_to_selected_center_anbx(bpy.types.Operator):
    '''Copy Active to the center of the Selected'''

    bl_idname = "object.copy_active_to_selected_center_anbx"
    bl_label = "To Center"
    bl_options = {'REGISTER', 'UNDO'}

    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) > 1)


    def execute(self, context):

        ob = bpy.context.object
        sel_ob = bpy.context.selected_objects

        act_ob = bpy.context.active_object

        cursor = bpy.context.scene.cursor
        cursor_location = bpy.context.scene.cursor.location
        cursor_previous_location = cursor.location.copy()


        # Apply Scale
        bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)


        act_ob_name = act_ob.name


        def random_name(length):

            source = string.ascii_lowercase + string.digits
            result_str = ''.join((random.choice(source) for i in range(length)))

            act_name = bpy.context.view_layer.objects.active.name
            act_type = bpy.context.view_layer.objects.active.type

            name_spl = act_name[:3]
            rand_name = ( '_' + result_str + '_')

            act_ob.name = name_spl + rand_name
            if act_ob.data:
                act_ob.data.name = act_ob.name


        random_name(6)


        #----------------------------------------------

        act_col = bpy.context.view_layer.active_layer_collection


        def new_ob_main():

            def new_ob(): 
 
                orig_ob = bpy.data.objects[ob.name]
                dupl_ob = bpy.data.objects.new(ob.name, ob.data)


                act_col.collection.objects.link(dupl_ob)
                dupl_ob.location = cursor.location

                bpy.ops.object.make_single_user(object=False, obdata=True, material=True, animation=True)
                
                if dupl_ob.data:      
                    dupl_ob.name = dupl_ob.name
                    dupl_ob.data.name = dupl_ob.name
                
                            
                dupl_ob.select_set(1)
                o.select_set(0)
                                                       
            #-----------------------------------------------------------

            act_ob = bpy.context.active_object
            for o in bpy.context.selected_objects:
                
                if o != bpy.context.active_object and o.type == 'MESH' and o.name != act_ob.name:
                    bpy.context.view_layer.objects.active = o 
                                             
                    cursor.location = o.location
                    new_ob()
                                 
                else:
                    if o != bpy.context.active_object and o.type != 'MESH' and o.name != act_ob.name:
                        bpy.context.view_layer.objects.active = o 
                        cursor.location = o.location               
                        new_ob()
            
        new_ob_main()

        # Name Back
        act_ob.name = act_ob_name
        act_ob.select_set(0)


        # Cursor back
        cursor.location = cursor_previous_location

 
        # Adds
        bpy.context.scene.transform_orientation_slots[0].type = 'LOCAL'
        bpy.context.scene.tool_settings.transform_pivot_point = 'INDIVIDUAL_ORIGINS'
        

        # Scene Update
        bpy.context.view_layer.update()
 

        return {'FINISHED'}


# End to Center
#==================================================================================




#========================================================================
# Copy Active To Selected - as Child - To Center
#========================================================================


class OBJECT_OT_copy_active_to_selected_center_child_anbx(bpy.types.Operator):
    '''Copy Active to the center of the Selected
- as Child''' 

    bl_idname = "object.copy_active_to_selected_center_child_anbx"
    bl_label = "As Child"
    bl_options = {'REGISTER', 'UNDO'}

    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) > 1)


    def execute(self, context):

        ob = bpy.context.object
        sel_ob = bpy.context.selected_objects

        act_ob = bpy.context.active_object

        cursor = bpy.context.scene.cursor
        cursor_location = bpy.context.scene.cursor.location
        cursor_previous_location = cursor.location.copy()


        # Apply Scale
        bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)


        act_ob_name = act_ob.name


        def random_name(length):

            source = string.ascii_lowercase + string.digits
            result_str = ''.join((random.choice(source) for i in range(length)))

            act_name = bpy.context.view_layer.objects.active.name
            act_type = bpy.context.view_layer.objects.active.type

            name_spl = act_name[:3]
            rand_name = ( '_' + result_str + '_')

            act_ob.name = name_spl + rand_name
            if act_ob.data:
                act_ob.data.name = act_ob.name


        random_name(6)

        #----------------------------------------------

        act_col = bpy.context.view_layer.active_layer_collection


        def new_ob_main():

            def new_ob(): 
 
                orig_ob = bpy.data.objects[ob.name]
                dupl_ob = bpy.data.objects.new(ob.name, ob.data)


                act_col.collection.objects.link(dupl_ob)
                dupl_ob.location = cursor.location

                bpy.ops.object.make_single_user(object=False, obdata=True, material=True, animation=True)
                
                if dupl_ob.data:      
                    dupl_ob.name = dupl_ob.name
                    dupl_ob.data.name = dupl_ob.name
                
                           
                dupl_ob.parent = o
                dupl_ob.matrix_parent_inverse = o.matrix_world.inverted()
                dupl_ob.select_set(1)
                o.select_set(0)
                                                       
            #-----------------------------------------------------------

            act_ob = bpy.context.active_object
            for o in bpy.context.selected_objects:
                
                if o != bpy.context.active_object and o.type == 'MESH' and o.name != act_ob.name:
                    bpy.context.view_layer.objects.active = o 
                                 
                    cursor.location = o.location
                    new_ob()
                                 
                else:
                    if o != bpy.context.active_object and o.type != 'MESH' and o.name != act_ob.name:
                        bpy.context.view_layer.objects.active = o 
                        cursor.location = o.location               
                        new_ob()
   
                        
        new_ob_main()

        # Name Back
        act_ob.name = act_ob_name
        act_ob.select_set(0)


        # Cursor back
        cursor.location = cursor_previous_location

 
        # Adds
        bpy.context.scene.transform_orientation_slots[0].type = 'LOCAL'
        bpy.context.scene.tool_settings.transform_pivot_point = 'INDIVIDUAL_ORIGINS'
        

        # Scene Update
        bpy.context.view_layer.update()
 

        return {'FINISHED'}


# End Copy To Center as Child
#==================================================================================




#========================================================================
# Copy Active To Selected - as Parent - To Center
#========================================================================


class OBJECT_OT_copy_active_to_selected_center_parent_anbx(bpy.types.Operator):
    '''Copy Active to the center of the Selected
- as Parent''' 

    bl_idname = "object.copy_active_to_selected_center_parent_anbx"
    bl_label = "As Parent"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) > 1)


    def execute(self, context):


        ob = bpy.context.object
        sel_ob = bpy.context.selected_objects

        act_ob = bpy.context.active_object

        cursor = bpy.context.scene.cursor
        cursor_location = bpy.context.scene.cursor.location
        cursor_previous_location = cursor.location.copy()


        # Apply Scale
        bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)


        act_ob_name = act_ob.name


        def random_name(length):

            source = string.ascii_lowercase + string.digits
            result_str = ''.join((random.choice(source) for i in range(length)))

            act_name = bpy.context.view_layer.objects.active.name
            act_type = bpy.context.view_layer.objects.active.type

            name_spl = act_name[:3]
            rand_name = ( '_' + result_str + '_')

            act_ob.name = name_spl + rand_name
            if act_ob.data:
                act_ob.data.name = act_ob.name


        random_name(6)

        #----------------------------------------------


        act_col = bpy.context.view_layer.active_layer_collection


        def new_ob_main():

            def new_ob(): 
 
                orig_ob = bpy.data.objects[ob.name]
                dupl_ob = bpy.data.objects.new(ob.name, ob.data)


                act_col.collection.objects.link(dupl_ob)
                dupl_ob.location = cursor.location

                bpy.ops.object.make_single_user(object=False, obdata=True, material=True, animation=True)
                
                if dupl_ob.data:      
                    dupl_ob.name = dupl_ob.name
                    dupl_ob.data.name = dupl_ob.name
                
             
                o.parent = dupl_ob
                o.matrix_parent_inverse = dupl_ob.matrix_world.inverted()
                dupl_ob.select_set(1)
                o.select_set(0)
                                                       
            #-----------------------------------------------------------

            act_ob = bpy.context.active_object
            for o in bpy.context.selected_objects:
                
                if o != bpy.context.active_object and o.type == 'MESH' and o.name != act_ob.name:
                    bpy.context.view_layer.objects.active = o 

                    cursor.location = o.location
                    new_ob()
                                 
                else:
                    if o != bpy.context.active_object and o.type != 'MESH' and o.name != act_ob.name:
                        bpy.context.view_layer.objects.active = o 
                        cursor.location = o.location               
                        new_ob()
   
                        
        new_ob_main()

        # Name Back
        act_ob.name = act_ob_name
        act_ob.select_set(0)


        # Cursor back
        cursor.location = cursor_previous_location

 
        # Adds
        bpy.context.scene.transform_orientation_slots[0].type = 'LOCAL'
        bpy.context.scene.tool_settings.transform_pivot_point = 'INDIVIDUAL_ORIGINS'
        

        # Scene Update
        bpy.context.view_layer.update()
 

        return {'FINISHED'}


# End Copy To Center as Parent
#==================================================================================






#========================================================================
# Copy Active To Selected - To Center - with Copy Location Constraint 
#========================================================================


class OBJECT_OT_copy_active_to_sel_with_copyloc_constr_anbx(bpy.types.Operator):
    '''Copy Active to the center of the Selected Objects
- with Copy Location Constraint ''' 

    bl_idname = "object.copy_active_to_sel_with_copyloc_constr_anbx"
    bl_label = "Add Copy Location Constraint"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) > 1)


    def execute(self, context):

        ob = bpy.context.object
        sel_ob = bpy.context.selected_objects
        act_ob = bpy.context.active_object

        cursor = bpy.context.scene.cursor
        cursor_location = bpy.context.scene.cursor.location
        cursor_previous_location = cursor.location.copy()


        # Apply Scale
        bpy.ops.object.transform_apply(location=False, rotation=True, scale=True)

        act_ob_name = act_ob.name

        def random_name(length):

            source = string.ascii_lowercase + string.digits
            result_str = ''.join((random.choice(source) for i in range(length)))

            act_name = bpy.context.view_layer.objects.active.name
         
            name_spl = act_name[:4]
         
            rand_name = ( '_' + result_str + '_')

            act_ob.name = name_spl + rand_name
            if act_ob.data:
                act_ob.data.name = act_ob.name


        random_name(6)

        #----------------------------------------------

        act_col = bpy.context.view_layer.active_layer_collection


        def new_ob_main():

            def new_ob(): 
         
                orig_ob = bpy.data.objects[ob.name]
                dupl_ob = bpy.data.objects.new(ob.name, ob.data)


                act_col.collection.objects.link(dupl_ob)
                dupl_ob.location = cursor.location

                bpy.ops.object.make_single_user(object=False, obdata=True, material=True, animation=True)
                
                if dupl_ob.data:      
                    dupl_ob.name = dupl_ob.name
                    dupl_ob.data.name = dupl_ob.name
                
                
                o.location = cursor_location 
                if o.location == cursor_location:
                    Empty_Name = o.name


                bc = dupl_ob.constraints.new(type='COPY_LOCATION')
                
                bc.target = bpy.data.objects[Empty_Name]
                
                dupl_ob.select_set(1)
                o.select_set(0)

            #-----------------------------------------------------------

            act_ob = bpy.context.active_object
            for o in bpy.context.selected_objects:
                
         
                if o != bpy.context.active_object and o.name != act_ob.name:
                    bpy.context.view_layer.objects.active = o 
                                 
                    cursor.location = o.location
                    new_ob()
                                 
              
        new_ob_main()

        # Name Back
        act_ob.name = act_ob_name
        act_ob.select_set(0)


        # Cursor back
        cursor.location = cursor_previous_location

         
        # Adds
        bpy.context.scene.transform_orientation_slots[0].type = 'LOCAL'
        bpy.context.scene.tool_settings.transform_pivot_point = 'INDIVIDUAL_ORIGINS'


        # Scene Update
        bpy.context.view_layer.update()
         

        return {'FINISHED'}


# End Copy To Center with Copy Location Constraint 
#==================================================================================






#========================================================================
# Copy Active To Selected - Put on Top
#========================================================================

class OBJECT_OT_copy_active_to_selected_top_anbx(bpy.types.Operator):
    '''Copy Active To Selected
- Place on Top
- Origin to Bottom'''

    bl_idname = "object.copy_active_to_selected_top_anbx"
    bl_label = "On Top"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) > 1)



    def execute(self, context):


        ob = bpy.context.object
        sel_ob = bpy.context.selected_objects

        act_ob = bpy.context.active_object

        cursor = bpy.context.scene.cursor
        cursor_location = bpy.context.scene.cursor.location
        cursor_previous_location = cursor.location.copy()


        # Apply Scale
        bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)


        act_ob_name = act_ob.name


        def random_name(length):

            source = string.ascii_lowercase + string.digits
            result_str = ''.join((random.choice(source) for i in range(length)))

            act_name = bpy.context.view_layer.objects.active.name
            act_type = bpy.context.view_layer.objects.active.type

            name_spl = act_name[:3]
            rand_name = ( '_' + result_str + '_')

            act_ob.name = name_spl + rand_name
            if act_ob.data:
                act_ob.data.name = act_ob.name


        random_name(6)


        #----------------------------------------------

        def origin_to_top(ob, matrix=Matrix()):
            me = ob.data
            mw = ob.matrix_world
            local_verts = [matrix @ Vector(v[:]) for v in ob.bound_box]
            o = sum(local_verts, Vector()) / 8
            o.z = max(v.z for v in local_verts)
            o = matrix.inverted() @ o
            me.transform(Matrix.Translation(-o))

            mw.translation = mw @ o


        def origin_to_bottom(ob, matrix=Matrix()):
            if bpy.context.active_object.type == 'MESH':
            
                me = ob.data
                mw = ob.matrix_world
                local_verts = [matrix @ Vector(v[:]) for v in ob.bound_box]
                o = sum(local_verts, Vector()) / 8
                o.z = min(v.z for v in local_verts)
                o = matrix.inverted() @ o
                me.transform(Matrix.Translation(-o))

                mw.translation = mw @ o

        act_ob = bpy.context.active_object
        if bpy.context.active_object.type == 'MESH':
            
            origin_to_bottom(act_ob)


        act_col = bpy.context.view_layer.active_layer_collection


        def new_ob_main():

            def new_ob(): 
 
                orig_ob = bpy.data.objects[ob.name]
                dupl_ob = bpy.data.objects.new(ob.name, ob.data)


                act_col.collection.objects.link(dupl_ob)
                dupl_ob.location = cursor.location

                bpy.ops.object.make_single_user(object=False, obdata=True, material=True, animation=True)
                
                if dupl_ob.data:      
                    dupl_ob.name = dupl_ob.name
                    dupl_ob.data.name = dupl_ob.name
                
                origin_to_bottom(o) 
                            

                dupl_ob.select_set(1)
                o.select_set(0)
                                                       
            #-----------------------------------------------------------

            act_ob = bpy.context.active_object
            for o in bpy.context.selected_objects:
                
                if o != bpy.context.active_object and o.type == 'MESH' and o.name != act_ob.name:
                    bpy.context.view_layer.objects.active = o 
                    
                    origin_to_top(o)
                                 
                    cursor.location = o.location
                    new_ob()
                                 
                else:
                    if o != bpy.context.active_object and o.type != 'MESH' and o.name != act_ob.name:
                        bpy.context.view_layer.objects.active = o 
                        cursor.location = o.location               
                        new_ob()
   
                        
        new_ob_main()

        # Name Back
        act_ob.name = act_ob_name
        act_ob.select_set(0)


        # Cursor back
        cursor.location = cursor_previous_location

 
        # Adds
        bpy.context.scene.transform_orientation_slots[0].type = 'LOCAL'
        bpy.context.scene.tool_settings.transform_pivot_point = 'INDIVIDUAL_ORIGINS'
        

 
        # Scene Update
        bpy.context.view_layer.update()
 


        return {'FINISHED'}


# End Put on Top
#==================================================================================






#========================================================================
# Copy Active To Selected - as Child - Put on Top
#========================================================================

class OBJECT_OT_copy_active_to_selected_top_child_anbx(bpy.types.Operator):
    '''Copy Active To Selected - as Child
- Place on Top
- Origin to Bottom'''

    bl_idname = "object.copy_active_to_selected_top_child_anbx"
    bl_label = "As Child"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) > 1)



    def execute(self, context):


        ob = bpy.context.object
        sel_ob = bpy.context.selected_objects

        act_ob = bpy.context.active_object

        cursor = bpy.context.scene.cursor
        cursor_location = bpy.context.scene.cursor.location
        cursor_previous_location = cursor.location.copy()


        # Apply Scale
        bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)


        act_ob_name = act_ob.name


        def random_name(length):

            source = string.ascii_lowercase + string.digits
            result_str = ''.join((random.choice(source) for i in range(length)))

            act_name = bpy.context.view_layer.objects.active.name
            act_type = bpy.context.view_layer.objects.active.type

            name_spl = act_name[:3]
            rand_name = ( '_' + result_str + '_')

            act_ob.name = name_spl + rand_name
            if act_ob.data:
                act_ob.data.name = act_ob.name


        random_name(6)


        #----------------------------------------------

        def origin_to_top(ob, matrix=Matrix()):
            me = ob.data
            mw = ob.matrix_world
            local_verts = [matrix @ Vector(v[:]) for v in ob.bound_box]
            o = sum(local_verts, Vector()) / 8
            o.z = max(v.z for v in local_verts)
            o = matrix.inverted() @ o
            me.transform(Matrix.Translation(-o))

            mw.translation = mw @ o


        def origin_to_bottom(ob, matrix=Matrix()):
            if bpy.context.active_object.type == 'MESH':
            
                me = ob.data
                mw = ob.matrix_world
                local_verts = [matrix @ Vector(v[:]) for v in ob.bound_box]
                o = sum(local_verts, Vector()) / 8
                o.z = min(v.z for v in local_verts)
                o = matrix.inverted() @ o
                me.transform(Matrix.Translation(-o))

                mw.translation = mw @ o

        act_ob = bpy.context.active_object
        if bpy.context.active_object.type == 'MESH':
            
            origin_to_bottom(act_ob)


        act_col = bpy.context.view_layer.active_layer_collection


        def new_ob_main():

            def new_ob(): 
 
                orig_ob = bpy.data.objects[ob.name]
                dupl_ob = bpy.data.objects.new(ob.name, ob.data)


                act_col.collection.objects.link(dupl_ob)
                dupl_ob.location = cursor.location

                bpy.ops.object.make_single_user(object=False, obdata=True, material=True, animation=True)
                
                if dupl_ob.data:      
                    dupl_ob.name = dupl_ob.name
                    dupl_ob.data.name = dupl_ob.name
                
                origin_to_bottom(o) 
                            
                dupl_ob.parent = o
                dupl_ob.matrix_parent_inverse = o.matrix_world.inverted()
                dupl_ob.select_set(1)
                o.select_set(0)
                                                       
            #-----------------------------------------------------------

            act_ob = bpy.context.active_object
            for o in bpy.context.selected_objects:
                
                if o != bpy.context.active_object and o.type == 'MESH' and o.name != act_ob.name:
                    bpy.context.view_layer.objects.active = o 
                    
                    origin_to_top(o)
                                 
                    cursor.location = o.location
                    new_ob()
                                 
                else:
                    if o != bpy.context.active_object and o.type != 'MESH' and o.name != act_ob.name:
                        bpy.context.view_layer.objects.active = o 
                        cursor.location = o.location               
                        new_ob()
   
                        
        new_ob_main()

        # Name Back
        act_ob.name = act_ob_name
        act_ob.select_set(0)


        # Cursor back
        cursor.location = cursor_previous_location

 
        # Adds
        bpy.context.scene.transform_orientation_slots[0].type = 'LOCAL'
        bpy.context.scene.tool_settings.transform_pivot_point = 'INDIVIDUAL_ORIGINS'
        

 
        # Scene Update
        bpy.context.view_layer.update()
 


        return {'FINISHED'}


# End Put on Top as Child
#==================================================================================






#========================================================================
# Copy Active To Selected - as Parent - Put on Top
#========================================================================

class OBJECT_OT_copy_active_to_selected_top_parent_anbx(bpy.types.Operator):
    '''Copy Active To Selected - as Parent (Only Mesh)
- Place on Top
- Origin to Bottom'''

    bl_idname = "object.copy_active_to_selected_top_parent_anbx"
    bl_label = "As Parent"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) > 1)



    def execute(self, context):


        ob = bpy.context.object
        sel_ob = bpy.context.selected_objects

        act_ob = bpy.context.active_object

        cursor = bpy.context.scene.cursor
        cursor_location = bpy.context.scene.cursor.location
        cursor_previous_location = cursor.location.copy()


        # Apply Scale
        bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)


        act_ob_name = act_ob.name


        def random_name(length):

            source = string.ascii_lowercase + string.digits
            result_str = ''.join((random.choice(source) for i in range(length)))

            act_name = bpy.context.view_layer.objects.active.name
            act_type = bpy.context.view_layer.objects.active.type

            name_spl = act_name[:3]
            rand_name = ( '_' + result_str + '_')

            act_ob.name = name_spl + rand_name
            if act_ob.data:
                act_ob.data.name = act_ob.name


        random_name(6)


        #----------------------------------------------

        def origin_to_top(ob, matrix=Matrix()):
            if bpy.context.active_object.type == 'MESH':

                me = ob.data
                mw = ob.matrix_world
                local_verts = [matrix @ Vector(v[:]) for v in ob.bound_box]
                o = sum(local_verts, Vector()) / 8
                o.z = max(v.z for v in local_verts)
                o = matrix.inverted() @ o
                me.transform(Matrix.Translation(-o))

                mw.translation = mw @ o


        def origin_to_bottom(ob, matrix=Matrix()):
            if bpy.context.active_object.type == 'MESH':
            
                me = ob.data
                mw = ob.matrix_world
                local_verts = [matrix @ Vector(v[:]) for v in ob.bound_box]
                o = sum(local_verts, Vector()) / 8
                o.z = min(v.z for v in local_verts)
                o = matrix.inverted() @ o
                me.transform(Matrix.Translation(-o))

                mw.translation = mw @ o

        act_ob = bpy.context.active_object
        if bpy.context.active_object.type == 'MESH':
            
            origin_to_bottom(act_ob)


        act_col = bpy.context.view_layer.active_layer_collection


        def new_ob_main():

            def new_ob(): 
 
                orig_ob = bpy.data.objects[ob.name]
                dupl_ob = bpy.data.objects.new(ob.name, ob.data)


                act_col.collection.objects.link(dupl_ob)
                dupl_ob.location = cursor.location

                bpy.ops.object.make_single_user(object=False, obdata=True, material=True, animation=True)
                
                if dupl_ob.data:      
                    dupl_ob.name = dupl_ob.name
                    dupl_ob.data.name = dupl_ob.name
                
 
                origin_to_top(dupl_ob) 
                            
                o.parent = dupl_ob
                o.matrix_parent_inverse = dupl_ob.matrix_world.inverted()
                dupl_ob.select_set(1)
                o.select_set(0)
                                                       
            #-----------------------------------------------------------

            act_ob = bpy.context.active_object
            for o in bpy.context.selected_objects:
                
                if o != bpy.context.active_object and o.type == 'MESH' and o.name != act_ob.name:
                    bpy.context.view_layer.objects.active = o 
                    
                    origin_to_top(o)
                                 
                    cursor.location = o.location
                    new_ob()
                                 
                else:
                    if o != bpy.context.active_object and o.type != 'MESH' and o.name != act_ob.name:
                        bpy.context.view_layer.objects.active = o 
                        cursor.location = o.location               
                        new_ob()
   
                        
        new_ob_main()

        # Name Back
        act_ob.name = act_ob_name
        act_ob.select_set(0)


        # Cursor back
        cursor.location = cursor_previous_location

 
        # Adds
        bpy.context.scene.transform_orientation_slots[0].type = 'LOCAL'
        bpy.context.scene.tool_settings.transform_pivot_point = 'INDIVIDUAL_ORIGINS'
        

 
        # Scene Update
        bpy.context.view_layer.update()
 

        return {'FINISHED'}


# End Put on Top as Parent
#==================================================================================



#===========================End Top================================================







#=========================== Bottom ================================================





#========================================================================
# Copy Active To Selected - to Bottom 
#========================================================================

class OBJECT_OT_copy_active_to_selected_bottom_anbx(bpy.types.Operator):
    '''Copy Active To Selected
- Place at the Bottom
- Origin to Top'''

    bl_idname = "object.copy_active_to_selected_bottom_anbx"
    bl_label = "To Bottom"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) > 1)



    def execute(self, context):


        ob = bpy.context.object
        sel_ob = bpy.context.selected_objects

        act_ob = bpy.context.active_object

        cursor = bpy.context.scene.cursor
        cursor_location = bpy.context.scene.cursor.location
        cursor_previous_location = cursor.location.copy()


        # Apply Scale
        bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)


        act_ob_name = act_ob.name


        def random_name(length):

            source = string.ascii_lowercase + string.digits
            result_str = ''.join((random.choice(source) for i in range(length)))

            act_name = bpy.context.view_layer.objects.active.name
            act_type = bpy.context.view_layer.objects.active.type

            name_spl = act_name[:3]
            rand_name = ( '_' + result_str + '_')

            act_ob.name = name_spl + rand_name
            if act_ob.data:
                act_ob.data.name = act_ob.name


        random_name(6)


        #----------------------------------------------

        def origin_to_top(ob, matrix=Matrix()):
            me = ob.data
            mw = ob.matrix_world
            local_verts = [matrix @ Vector(v[:]) for v in ob.bound_box]
            o = sum(local_verts, Vector()) / 8
            o.z = max(v.z for v in local_verts)
            o = matrix.inverted() @ o
            me.transform(Matrix.Translation(-o))

            mw.translation = mw @ o


        def origin_to_bottom(ob, matrix=Matrix()):
            if bpy.context.active_object.type == 'MESH':
            
                me = ob.data
                mw = ob.matrix_world
                local_verts = [matrix @ Vector(v[:]) for v in ob.bound_box]
                o = sum(local_verts, Vector()) / 8
                o.z = min(v.z for v in local_verts)
                o = matrix.inverted() @ o
                me.transform(Matrix.Translation(-o))

                mw.translation = mw @ o

        act_ob = bpy.context.active_object
        if bpy.context.active_object.type == 'MESH':
            
            origin_to_top(act_ob)


        act_col = bpy.context.view_layer.active_layer_collection



        def new_ob_main():

            def new_ob(): 
 
                orig_ob = bpy.data.objects[ob.name]
                dupl_ob = bpy.data.objects.new(ob.name, ob.data)


                act_col.collection.objects.link(dupl_ob)
                dupl_ob.location = cursor.location

                bpy.ops.object.make_single_user(object=False, obdata=True, material=True, animation=True)
                
                if dupl_ob.data:      
                    dupl_ob.name = dupl_ob.name
                    dupl_ob.data.name = dupl_ob.name
                
             
                dupl_ob.select_set(1)
                o.select_set(0)
                                                       
            #-----------------------------------------------------------

            act_ob = bpy.context.active_object
            for o in bpy.context.selected_objects:
                
                if o != bpy.context.active_object and o.type == 'MESH' and o.name != act_ob.name:
                    bpy.context.view_layer.objects.active = o 
                    
                    origin_to_bottom(o)
                                 
                    cursor.location = o.location
                    new_ob()
                                 
                else:
                    if o != bpy.context.active_object and o.type != 'MESH' and o.name != act_ob.name:
                        bpy.context.view_layer.objects.active = o 
                        cursor.location = o.location               
                        new_ob()
   
                        
        new_ob_main()

        # Name Back
        act_ob.name = act_ob_name
        act_ob.select_set(0)


        # Cursor back
        cursor.location = cursor_previous_location

 
        # Adds
        bpy.context.scene.transform_orientation_slots[0].type = 'LOCAL'
        bpy.context.scene.tool_settings.transform_pivot_point = 'INDIVIDUAL_ORIGINS'
        

 
        # Scene Update
        bpy.context.view_layer.update()
 


        return {'FINISHED'}


# End Put to Bottom 
#==================================================================================







#========================================================================
# Copy Active To Selected - to Bottom as Child
#========================================================================

class OBJECT_OT_copy_active_to_selected_bottom_child_anbx(bpy.types.Operator):
    '''Copy Active To Selected as Child
- Place at the Bottom
- Origin to Top'''

    bl_idname = "object.copy_active_to_selected_bottom_child_anbx"
    bl_label = "As Child"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) > 1)



    def execute(self, context):


        ob = bpy.context.object
        sel_ob = bpy.context.selected_objects

        act_ob = bpy.context.active_object

        cursor = bpy.context.scene.cursor
        cursor_location = bpy.context.scene.cursor.location
        cursor_previous_location = cursor.location.copy()


        # Apply Scale
        bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)


        act_ob_name = act_ob.name


        def random_name(length):

            source = string.ascii_lowercase + string.digits
            result_str = ''.join((random.choice(source) for i in range(length)))

            act_name = bpy.context.view_layer.objects.active.name
            act_type = bpy.context.view_layer.objects.active.type

            name_spl = act_name[:3]
            rand_name = ( '_' + result_str + '_')

            act_ob.name = name_spl + rand_name
            if act_ob.data:
                act_ob.data.name = act_ob.name


        random_name(6)


        #----------------------------------------------

        def origin_to_top(ob, matrix=Matrix()):
            if bpy.context.active_object.type == 'MESH':

                me = ob.data
                mw = ob.matrix_world
                local_verts = [matrix @ Vector(v[:]) for v in ob.bound_box]
                o = sum(local_verts, Vector()) / 8
                o.z = max(v.z for v in local_verts)
                o = matrix.inverted() @ o
                me.transform(Matrix.Translation(-o))

                mw.translation = mw @ o


        def origin_to_bottom(ob, matrix=Matrix()):
            if bpy.context.active_object.type == 'MESH':
            
                me = ob.data
                mw = ob.matrix_world
                local_verts = [matrix @ Vector(v[:]) for v in ob.bound_box]
                o = sum(local_verts, Vector()) / 8
                o.z = min(v.z for v in local_verts)
                o = matrix.inverted() @ o
                me.transform(Matrix.Translation(-o))

                mw.translation = mw @ o

        act_ob = bpy.context.active_object
        if bpy.context.active_object.type == 'MESH':
            
            origin_to_top(act_ob)


        act_col = bpy.context.view_layer.active_layer_collection

 

        def new_ob_main():

            def new_ob(): 
 
                orig_ob = bpy.data.objects[ob.name]
                dupl_ob = bpy.data.objects.new(ob.name, ob.data)


                act_col.collection.objects.link(dupl_ob)
                dupl_ob.location = cursor.location

                bpy.ops.object.make_single_user(object=False, obdata=True, material=True, animation=True)
                
                if dupl_ob.data:      
                    dupl_ob.name = dupl_ob.name
                    dupl_ob.data.name = dupl_ob.name
                

                origin_to_top(o)


                dupl_ob.parent = o
                dupl_ob.matrix_parent_inverse = o.matrix_world.inverted()
                dupl_ob.select_set(1)
                o.select_set(0)
                                                       
            #-----------------------------------------------------------

            act_ob = bpy.context.active_object
            for o in bpy.context.selected_objects:
                
                if o != bpy.context.active_object and o.type == 'MESH' and o.name != act_ob.name:
                    bpy.context.view_layer.objects.active = o 
                    
                    origin_to_bottom(o)
                                 
                    cursor.location = o.location
                    new_ob()

                                 
                else:
                    if o != bpy.context.active_object and o.type != 'MESH' and o.name != act_ob.name:
                        bpy.context.view_layer.objects.active = o 
                        cursor.location = o.location               
                        new_ob()
   
                        
        new_ob_main()

        # Name Back
        act_ob.name = act_ob_name
        act_ob.select_set(0)


        # Cursor back
        cursor.location = cursor_previous_location

 
        # Adds
        bpy.context.scene.transform_orientation_slots[0].type = 'LOCAL'
        bpy.context.scene.tool_settings.transform_pivot_point = 'INDIVIDUAL_ORIGINS'
        

 
        # Scene Update
        bpy.context.view_layer.update()
 


        return {'FINISHED'}


# End Put to Bottom as Child
#==================================================================================









#========================================================================
# Copy Active To Selected - to Bottom as Parent
#========================================================================

class OBJECT_OT_copy_active_to_selected_bottom_parent_anbx(bpy.types.Operator):
    '''Copy Active To Selected as Parent (Only Mesh)
- Place at the Bottom
- Origin to Top'''

    bl_idname = "object.copy_active_to_selected_bottom_parent_anbx"
    bl_label = "As Parent"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) > 1)



    def execute(self, context):


        ob = bpy.context.object
        sel_ob = bpy.context.selected_objects

        act_ob = bpy.context.active_object

        cursor = bpy.context.scene.cursor
        cursor_location = bpy.context.scene.cursor.location
        cursor_previous_location = cursor.location.copy()


        # Apply Scale
        bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)


        act_ob_name = act_ob.name


        def random_name(length):

            source = string.ascii_lowercase + string.digits
            result_str = ''.join((random.choice(source) for i in range(length)))

            act_name = bpy.context.view_layer.objects.active.name
            act_type = bpy.context.view_layer.objects.active.type

            name_spl = act_name[:3]
            rand_name = ( '_' + result_str + '_')

            act_ob.name = name_spl + rand_name
            if act_ob.data:
                act_ob.data.name = act_ob.name


        random_name(6)


        #----------------------------------------------

        def origin_to_top(ob, matrix=Matrix()):
            if bpy.context.active_object.type == 'MESH':
                
                me = ob.data
                mw = ob.matrix_world
                local_verts = [matrix @ Vector(v[:]) for v in ob.bound_box]
                o = sum(local_verts, Vector()) / 8
                o.z = max(v.z for v in local_verts)
                o = matrix.inverted() @ o
                me.transform(Matrix.Translation(-o))

                mw.translation = mw @ o


        def origin_to_bottom(ob, matrix=Matrix()):
            if bpy.context.active_object.type == 'MESH':
            
                me = ob.data
                mw = ob.matrix_world
                local_verts = [matrix @ Vector(v[:]) for v in ob.bound_box]
                o = sum(local_verts, Vector()) / 8
                o.z = min(v.z for v in local_verts)
                o = matrix.inverted() @ o
                me.transform(Matrix.Translation(-o))

                mw.translation = mw @ o

        act_ob = bpy.context.active_object
        if bpy.context.active_object.type == 'MESH':
            
            origin_to_top(act_ob)


        act_col = bpy.context.view_layer.active_layer_collection

 

        def new_ob_main():

            def new_ob(): 
 
                orig_ob = bpy.data.objects[ob.name]
                dupl_ob = bpy.data.objects.new(ob.name, ob.data)

                act_col.collection.objects.link(dupl_ob)
                dupl_ob.location = cursor.location

                bpy.ops.object.make_single_user(object=False, obdata=True, material=True, animation=True)
                
                if dupl_ob.data:      
                    dupl_ob.name = dupl_ob.name
                    dupl_ob.data.name = dupl_ob.name
                
                origin_to_bottom(dupl_ob)

                o.parent = dupl_ob
                o.matrix_parent_inverse = dupl_ob.matrix_world.inverted()
                dupl_ob.select_set(1)
                o.select_set(0)
                                                       
            #-----------------------------------------------------------

            act_ob = bpy.context.active_object
            for o in bpy.context.selected_objects:
                
                if o != bpy.context.active_object and o.type == 'MESH' and o.name != act_ob.name:
                    bpy.context.view_layer.objects.active = o 
                    
                    origin_to_bottom(o)
                                 
                    cursor.location = o.location
                    new_ob()
                                 
                else:
                    if o != bpy.context.active_object and o.type != 'MESH' and o.name != act_ob.name:
                        bpy.context.view_layer.objects.active = o 
                        cursor.location = o.location               
                        new_ob()
   
                        
        new_ob_main()

        # Name Back
        act_ob.name = act_ob_name
        act_ob.select_set(0)


        # Cursor back
        cursor.location = cursor_previous_location

 
        # Adds
        bpy.context.scene.transform_orientation_slots[0].type = 'LOCAL'
        bpy.context.scene.tool_settings.transform_pivot_point = 'INDIVIDUAL_ORIGINS'
        

 
        # Scene Update
        bpy.context.view_layer.update()
 


        return {'FINISHED'}


# End Put to Bottom as Parent
#==================================================================================













# to  Vertex / Vertices
#==================================================================================

# Copy Active To Selected - Vertex / Vertices
class OBJECT_OT_copy_active_to_vertex_anbx(bpy.types.Operator):
    '''Copy Active To Selected - Vertex / Vertices

-Alt Click = Insert Keys (Loc / Rot / Scale)'''
    bl_idname = "object.copy_active_to_vertex_anbx"
    bl_label = "Copy Active to Selected V"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) == 2)



    def invoke(self, context, event):

        exept_ob = ['CAMERA', 'EMPTY', 'LIGHT', 'ARMATURE', 
                    'SURFACE', 'CURVE', 'META', 'FONT', 'GPENCIL', 'LATTICE', 'SPEAKER']

  
        ob = bpy.context.object
        sel_ob = bpy.context.selected_objects
        cursor = bpy.context.scene.cursor
        cursor_location = bpy.context.scene.cursor.location
        act_ob = bpy.context.active_object


        act_ob_name = act_ob.name

        # Apply Scale
        bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)


        def random_name(length):

            source = string.ascii_lowercase + string.digits
            result_str = ''.join((random.choice(source) for i in range(length)))

            act_name = bpy.context.view_layer.objects.active.name
            act_type = bpy.context.view_layer.objects.active.type

            name_spl = act_name[:3]
            rand_name = ( '_' + result_str + '_')

            act_ob.name = name_spl + rand_name
            act_ob.data.name = act_ob.name

        if act_ob.type not in exept_ob:
            random_name(6)



        def insert_keys():
            for o in bpy.context.selected_objects:
                o.keyframe_insert(data_path='location')
                o.keyframe_insert(data_path='scale')
                o.keyframe_insert(data_path='rotation_euler')



        # Apply Scale
        bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)



        def origin_to_bottom(ob, matrix=Matrix()):
            me = ob.data
            mw = ob.matrix_world
            local_verts = [matrix @ Vector(v[:]) for v in ob.bound_box]
            o = sum(local_verts, Vector()) / 8
            o.z = min(v.z for v in local_verts)
            o = matrix.inverted() @ o
            me.transform(Matrix.Translation(-o))

            mw.translation = mw @ o


        act_col = bpy.context.view_layer.active_layer_collection

        #----------------------------------------

        def active_to_sel_verts():

            norm_length = 2

            context = bpy.context

            for o in bpy.context.selected_objects:

                if o == bpy.context.active_object and o.type == 'MESH':
                    origin_to_bottom(o)

                if o!= bpy.context.active_object and o.type not in exept_ob:
                    obj_2 = o

                    mw = obj_2.matrix_world
                    N = mw.inverted_safe().transposed().to_3x3()


                    for v in obj_2.data.vertices:
                        if v.select:
                            n = N @ v.normal
                            act_ob = bpy.context.active_object
                            o_name = bpy.context.active_object.name
                            orig_ob = bpy.data.objects[o_name]
                            dupl_ob = bpy.data.objects.new(o_name, act_ob.data)
                            dupl_ob.location = mw @ v.co
                            dupl_ob.rotation_euler =  n.to_track_quat().to_euler()
                            act_col.collection.objects.link(dupl_ob)

                            act_ob.select_set(0)
                            dupl_ob.select_set(1)

            for o in bpy.context.selected_objects:
                o_name = bpy.context.active_object.name
                if o.name.startswith(o_name):
                    o.select_set(1)
                    bpy.ops.object.make_single_user(object=False, obdata=True, material=True, animation=True)

                    if o.data:
                        o.name = o.name
                        o.data.name = o.name

                else:
                    o.select_set(0)

        #----------------------------------------


        active_to_sel_verts()


        act_ob.name = act_ob_name
        act_ob.select_set(0)



        if event.alt:
            insert_keys()


        # Adds
        bpy.context.scene.transform_orientation_slots[0].type = 'LOCAL'
        bpy.context.scene.tool_settings.transform_pivot_point = 'INDIVIDUAL_ORIGINS'

        # Scene Update
        bpy.context.view_layer.update()
 


        return {'FINISHED'}


# End Vertices
#=================================================




# To Faces
#=================================================



# Copy Active To Selected - Face(s)
class OBJECT_OT_copy_active_to_faces_anbx(bpy.types.Operator):
    '''Copy Active To Selected - Face(s)

-Alt Click = Insert Keys (Loc / Rot / Scale)'''
    bl_idname = "object.copy_active_to_faces_anbx"
    bl_label = "Copy Active to Selected F"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) == 2)



    def invoke(self, context, event):

        exept_ob = ['CAMERA', 'EMPTY', 'LIGHT', 'ARMATURE',
                    'SURFACE', 'CURVE', 'META', 'FONT', 'GPENCIL', 'LATTICE', 'SPEAKER']

  

        ob = bpy.context.object
        sel_ob = bpy.context.selected_objects
        cursor = bpy.context.scene.cursor
        cursor_location = bpy.context.scene.cursor.location
        act_ob = bpy.context.active_object


        act_ob_name = act_ob.name

        # Apply Scale
        bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)


        def random_name(length):

            source = string.ascii_lowercase + string.digits
            result_str = ''.join((random.choice(source) for i in range(length)))

            act_name = bpy.context.view_layer.objects.active.name
            act_type = bpy.context.view_layer.objects.active.type

            name_spl = act_name[:3]
            rand_name = ( '_' + result_str + '_')

            act_ob.name = name_spl + rand_name
            act_ob.data.name = act_ob.name


        if act_ob.type not in exept_ob:
            random_name(6)



        def insert_keys():
            for o in bpy.context.selected_objects:
                o.keyframe_insert(data_path='location')
                o.keyframe_insert(data_path='scale')
                o.keyframe_insert(data_path='rotation_euler')



        # Apply Scale
        bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)



        def origin_to_bottom(ob, matrix=Matrix()):
            me = ob.data
            mw = ob.matrix_world
            local_verts = [matrix @ Vector(v[:]) for v in ob.bound_box]
            o = sum(local_verts, Vector()) / 8
            o.z = min(v.z for v in local_verts)
            o = matrix.inverted() @ o
            me.transform(Matrix.Translation(-o))

            mw.translation = mw @ o



        act_col = bpy.context.view_layer.active_layer_collection

        #----------------------------------------


        def active_to_sel_faces():

            norm_length = 2

            context = bpy.context

            for o in bpy.context.selected_objects:

                if o == bpy.context.active_object and o.type == 'MESH':
                    origin_to_bottom(o)

                if o!= bpy.context.active_object and o.type not in exept_ob:
                    obj_2 = o

                    mw = obj_2.matrix_world
                    N = mw.inverted_safe().transposed().to_3x3()

                    for f in obj_2.data.polygons:
                        if f.select:
                            n = N @ f.normal
                            act_ob = bpy.context.active_object
                            o_name = bpy.context.active_object.name
                            orig_ob = bpy.data.objects[o_name]
                            dupl_ob = bpy.data.objects.new(o_name, act_ob.data)
                            dupl_ob.location = mw @ f.center
                            dupl_ob.rotation_euler =  n.to_track_quat().to_euler()
                            act_col.collection.objects.link(dupl_ob)

                            act_ob.select_set(0)
                            dupl_ob.select_set(1)


            for o in bpy.context.selected_objects:
                o_name = bpy.context.active_object.name
                if o.name.startswith(o_name):
                    o.select_set(1)
                    bpy.ops.object.make_single_user(object=False, obdata=True, material=True, animation=True)

                    if o.data:
                        o.name = o.name
                        o.data.name = o.name

                else:
                    o.select_set(0)


        #----------------------------------------


        active_to_sel_faces()


        act_ob.name = act_ob_name
        act_ob.select_set(0)


        if event.alt:
            insert_keys()


        # Adds
        
        bpy.context.scene.transform_orientation_slots[0].type = 'LOCAL'
        bpy.context.scene.tool_settings.transform_pivot_point = 'INDIVIDUAL_ORIGINS'


        # Scene Update
        bpy.context.view_layer.update()
 

        return {'FINISHED'}





# End Faces
#=================================================

 


classes = [ OBJECT_OT_copy_active_to_selected_center_anbx,
            OBJECT_OT_copy_active_to_selected_center_child_anbx,
            OBJECT_OT_copy_active_to_selected_center_parent_anbx,
            OBJECT_OT_copy_active_to_sel_with_copyloc_constr_anbx,
            
            OBJECT_OT_copy_active_to_selected_top_anbx,
            OBJECT_OT_copy_active_to_selected_top_child_anbx,
            OBJECT_OT_copy_active_to_selected_top_parent_anbx,

            OBJECT_OT_copy_active_to_selected_bottom_anbx,
            OBJECT_OT_copy_active_to_selected_bottom_child_anbx,
            OBJECT_OT_copy_active_to_selected_bottom_parent_anbx,

            OBJECT_OT_copy_active_to_vertex_anbx,
            OBJECT_OT_copy_active_to_faces_anbx,


          ]



def register():
    for cls in classes:
        bpy.utils.register_class(cls)



def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)




if __name__ == "__main__":
    register()



