import bpy




# Add Shared Lattice to Selected Object(s)
class OBJECT_OT_add_shared_lattice_to_sel_obj_anbx(bpy.types.Operator):
    '''Add Shared Lattice to Selected Object(s)

-Auto Convert Font to Mesh'''
    bl_idname = "object.add_shared_lattice_to_sel_obj_anbx"
    bl_label = "Shared Lattice"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        ob_types = ('FONT', 'MESH', 'CURVE', 'SURFACE')
        return (context.object is not None and
                len(context.selected_objects) !=0 and
                context.object.type in ob_types and
                context.object.mode == 'OBJECT')


    def execute(self, context):


        ob = bpy.context.object
        sel_ob = bpy.context.selected_objects
        cursor = bpy.context.scene.cursor
        cursor_location = bpy.context.scene.cursor.location


        #-------------------------------------

        # Text to Mesh

        act_ob = bpy.context.active_object

        def text_to_mesh():
            bpy.ops.object.convert(target='MESH')
            bpy.ops.object.mode_set(mode='EDIT')
            bpy.ops.mesh.select_all(action='SELECT')
            bpy.ops.mesh.remove_doubles()

            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            bpy.context.scene.tool_settings.transform_pivot_point = 'INDIVIDUAL_ORIGINS'

            bpy.ops.object.origin_set(type='ORIGIN_CENTER_OF_VOLUME', center='MEDIAN')



        # Add Edge_Split Modifier
        for o in bpy.context.selected_objects:
            bpy.context.view_layer.objects.active = o
            if o.type == 'FONT':
                text_to_mesh()
                mod = o.modifiers.get("Font_Split")
                if mod is None:
                    o.modifiers.new('Font_Split', 'EDGE_SPLIT')



        names = []

        for o in sel_ob:
            rem_names = names.append(o.name)


        if len(sel_ob) >= 1:
            bpy.ops.object.duplicate(linked=False)
            bpy.ops.object.join()

            # Copy dim, rot
            main_dim = bpy.context.object.dimensions.copy()
            main_rot = bpy.context.object.rotation_euler.copy()

            bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='BOUNDS')


            cursor_location = bpy.context.object.location

            bpy.ops.object.delete(use_global=False)


        bpy.context.scene.tool_settings.transform_pivot_point = 'BOUNDING_BOX_CENTER'


        # Active Collection
        act_col = bpy.context.view_layer.active_layer_collection

        # Add Lattice Object-------------
        lattice = bpy.data.lattices.new("Lattice_motion")
        lattice_ob = bpy.data.objects.new("Lattice_motion", lattice)


        od_x = main_dim.x 
        od_y = main_dim.y
        od_z = main_dim.z
        
        lattice_ob.dimensions = (od_x+0.1, od_y+0.1, od_z+0.1)

        lattice_ob.rotation_euler = main_rot

        lattice_ob.location = cursor_location


        lattice_ob.data.points_u = 2
        lattice_ob.data.points_v = 2
        lattice_ob.data.points_w = 2
        # Lattice Object-------------

        act_col.collection.objects.link(lattice_ob)

        for o in bpy.data.objects:
            if o.name in names:
                o.select_set(1)

        ob_types = ('MESH', 'CURVE', 'SURFACE')

        for o in sel_ob:
            if o.type in ob_types:
                bpy.context.view_layer.objects.active = o

                mod = o.modifiers.get("Lattice_motion")

                if mod is None:
                    o.modifiers.new("Lattice_motion", 'LATTICE')
                    o.modifiers["Lattice_motion"].object = lattice_ob

                else:
                    o.modifiers["Lattice_motion"].object = lattice_ob

        bpy.context.scene.tool_settings.transform_pivot_point = 'INDIVIDUAL_ORIGINS'



        sel_o = bpy.context.selected_objects
        # Select Lattice - go to Edit Mode
        for o in sel_o:
            o.select_set(0)

        ob_mod = bpy.context.object.modifiers

        for m in ob_mod:
            print(m.object.name)
            m.object.select_set(1)

            bpy.context.view_layer.objects.active = m.object

            bpy.context.evaluated_depsgraph_get().update()

            bpy.ops.object.editmode_toggle()




        return {'FINISHED'}






# Add Individual Lattice to Selected Object(s)
class OBJECT_OT_add_individual_lattice_to_sel_obj_anbx(bpy.types.Operator):
    '''Add Individual Lattice to Selected Object(s)

-Auto Convert Font to Mesh'''
    bl_idname = "object.add_individual_lattice_to_sel_obj_anbx"
    bl_label = "Individual Lattice"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        ob_types = ('FONT', 'MESH', 'CURVE', 'SURFACE')
        return (context.object is not None and
                len(context.selected_objects) !=0 and
                context.object.type in ob_types and
                context.object.mode == 'OBJECT')


    def execute(self, context):

        ob = bpy.context.object
        sel_ob = bpy.context.selected_objects
        cursor = bpy.context.scene.cursor
        cursor_location = bpy.context.scene.cursor.location

        #-------------------------------------

        # Text to Mesh

        act_ob = bpy.context.active_object

        def text_to_mesh():
            bpy.ops.object.convert(target='MESH')
            bpy.ops.object.mode_set(mode='EDIT')
            bpy.ops.mesh.select_all(action='SELECT')
            bpy.ops.mesh.remove_doubles()

            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            bpy.context.scene.tool_settings.transform_pivot_point = 'INDIVIDUAL_ORIGINS'

            bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='BOUNDS')



        # Add Edge_Split Modifier
        for o in bpy.context.selected_objects:
            bpy.context.view_layer.objects.active = o
            if o.type == 'FONT':
                text_to_mesh()
                mod = o.modifiers.get("Font_Split")
                if mod is None:
                    o.modifiers.new('Font_Split', 'EDGE_SPLIT')


        # Active Collection
        act_col = bpy.context.view_layer.active_layer_collection

        ob_types = ('MESH', 'CURVE', 'SURFACE')

        for o in sel_ob:
            if o.type in ob_types:
                bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='BOUNDS')

        for o in sel_ob:
            bpy.context.view_layer.objects.active = o

            lattice = bpy.data.lattices.new("Lattice_motion")
            lattice_ob = bpy.data.objects.new("Lattice_motion", lattice)

            
            od_x = o.dimensions.x 
            od_y = o.dimensions.y
            od_z = o.dimensions.z
            
            lattice_ob.dimensions = (od_x+0.1, od_y+0.1, od_z+0.1)

            lattice_ob.rotation_euler = o.rotation_euler
            lattice_ob.location = o.location
            lattice_ob.data.points_u = 2
            lattice_ob.data.points_v = 2
            lattice_ob.data.points_w = 2
            act_col.collection.objects.link(lattice_ob)



            mod = o.modifiers.get("Lattice_motion")


            if mod is None:
                o.modifiers.new("Lattice_motion", 'LATTICE')
                o.modifiers["Lattice_motion"].object = lattice_ob


            else:
                o.modifiers["Lattice_motion"].object = lattice_ob


        bpy.context.scene.tool_settings.transform_pivot_point = 'INDIVIDUAL_ORIGINS'



        sel_o = bpy.context.selected_objects
        # Select Lattice - go to Edit Mode
        for o in sel_o:
            o.select_set(0)

        ob_mod = bpy.context.object.modifiers

        for m in ob_mod:
            print(m.object.name)
            m.object.select_set(1)

            bpy.context.view_layer.objects.active = m.object

            bpy.context.evaluated_depsgraph_get().update()
            
            bpy.ops.object.editmode_toggle()        


        return {'FINISHED'}








classes = [ OBJECT_OT_add_shared_lattice_to_sel_obj_anbx,
            OBJECT_OT_add_individual_lattice_to_sel_obj_anbx,

          ]



def register():
    for cls in classes:
        bpy.utils.register_class(cls)



def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)




if __name__ == "__main__":
    register()



