import bpy
from bpy.props import (BoolProperty, FloatProperty, IntProperty)


#---------------------- Add Shape Keys -----------------------------

class OBJECT_OT_add_shape_key_to_sel_anbx(bpy.types.Operator):
    '''Add Shape Key to Selected'''
    bl_idname = "object.add_shape_key_to_sel_anbx"
    bl_label = "Add Shape Key"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                len(context.selected_objects) != 0)

    def execute(self, context):

        sel_ob = bpy.context.selected_objects

        def add_sh_keys_to_sel():
            for o in sel_ob:
                bpy.context.view_layer.objects.active = o
                bpy.ops.object.shape_key_add(from_mix=False)


        add_sh_keys_to_sel()


        return {'FINISHED'}



# ----------------------- Remove Shape Keys - Active Index --------

class OBJECT_OT_remove_shape_key_active_index_anbx(bpy.types.Operator):
    '''Remove Active Shape Key Index
-Selected Object(s)'''
    bl_idname = "object.remove_shape_key_active_index_anbx"
    bl_label = ""
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                len(context.selected_objects) > 1)

    def execute(self, context):

        sel_ob = bpy.context.selected_objects
        act_ob = bpy.context.active_object
        act_ob_index = act_ob.active_shape_key_index


        def remove_sh_keys_active():

            for o in sel_ob:
                bpy.context.view_layer.objects.active = o
                o.active_shape_key_index = act_ob_index
                bpy.ops.object.shape_key_remove(all=False)


        remove_sh_keys_active()


        return {'FINISHED'}



# ----------------------- Remove Shape Keys - All --------

class OBJECT_OT_remove_shape_key_all_anbx(bpy.types.Operator):
    '''Remove All Shape Keys
-Selected Object(s)'''
    bl_idname = "object.remove_shape_key_all_anbx"
    bl_label = ""
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                len(context.selected_objects) > 1)

    def execute(self, context):

        sel_ob = bpy.context.selected_objects
        act_ob = bpy.context.active_object

        def remove_sh_keys_all():

            for o in sel_ob:
                bpy.context.view_layer.objects.active = o
                bpy.ops.object.shape_key_remove(all=True)

        remove_sh_keys_all()


        return {'FINISHED'}




 


#-- Shape Key Value All-----------------------------------------

def set_key_shape_key_blocks_value_all_anbx(self, context):
    sel_ob = bpy.context.selected_objects


    for o in sel_ob:
        bpy.context.view_layer.objects.active = o

        if o.data.shape_keys:
            c_blocks = o.data.shape_keys.key_blocks


            for c in c_blocks:
                if c != c_blocks[0]:

                    c.value = self.shape_key_value_all_anbx
                    c.keyframe_insert(data_path='value')


#-- Shape Key Value Active-----------------------------------------

def set_key_shape_key_blocks_value_active_anbx(self, context):
    sel_ob = bpy.context.selected_objects
    act_ob = bpy.context.active_object
    act_ob_index = act_ob.active_shape_key_index

    for o in sel_ob:
        bpy.context.view_layer.objects.active = o
        o.active_shape_key_index = act_ob_index

        if o.data.shape_keys:
            c_blocks = o.data.shape_keys.key_blocks


            for c in c_blocks:
                if c == c_blocks[act_ob_index]:

                    c.value = self.shape_key_value_active_anbx
                    c.keyframe_insert(data_path='value')

#------------------------------------------------------




classes = [ OBJECT_OT_add_shape_key_to_sel_anbx,
            OBJECT_OT_remove_shape_key_active_index_anbx,
            OBJECT_OT_remove_shape_key_all_anbx,

          ]



def register():
    for cls in classes:
        bpy.utils.register_class(cls)


    # Value All
    bpy.types.Scene.shape_key_value_all_anbx = bpy.props.FloatProperty(
        name="Value All",
        default=0.0,
        description="Shape Key All Value - Selected Objects\n-Auto Keys",
        min=0.0, max = 1.0,
        precision=3,
        update = set_key_shape_key_blocks_value_all_anbx
    )


    # Value Active
    bpy.types.Scene.shape_key_value_active_anbx = bpy.props.FloatProperty(
        name="Value Active",
        default=0.0,
        description="Shape Key Active Value - Selected Objects\n-Auto Keys",
        min=0.0, max = 1.0,
        precision=3,
        update = set_key_shape_key_blocks_value_active_anbx
    )




def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)

    del bpy.types.Scene.shape_key_value_all_anbx
    del bpy.types.Scene.shape_key_value_active_anbx


if __name__ == "__main__":
    register()
