import bpy
from bpy.props import (BoolProperty, FloatProperty, IntProperty)



#-----------------------------------------------------------------
#--------------------- Create Smooth Emtpy Hook ------------------
#-----------------------------------------------------------------

class OBJECT_OT_create_smooth_empty_hook_anbx(bpy.types.Operator):
    '''- Can be parented to a Bone only if Mesh is parented to a Bone !!!
- In Edit Mode select Vertex / Vertices
- More Selected Vertices = Stronger and Smoother Influence'''

    bl_idname = "object.create_smooth_empty_hook_anbx"
    bl_label = "Create Smooth Hook - EMPTY"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        ob = context.object
        return (ob is not None and ob.type == 'MESH' and
        ob.mode == 'EDIT' and ob.data.total_vert_sel != 0)


    def execute(self, context):
        
        ob = bpy.context.object
        ob_name = ob.name
        
        # Ref        
        parent_to_parent = context.scene.smooth_empty_parent_to_parent_obj_anbx
             
        # Repeat Smooth Vertex Group  
        vert_gr_smooth_repeat = context.scene.vertex_group_smooth_repeat_anbx   
        # Levels Gain    
        vert_gr_level_gain = context.scene.vertex_group_levels_gain_anbx      

        def select_active_object():

            act_ob = bpy.context.active_object
            bpy.ops.object.select_all(action='DESELECT')
            act_ob.select_set(1) 

        
        # Cursor -----------------------------------------
        cursor = bpy.context.scene.cursor
        cursor_location = bpy.context.scene.cursor.location
        cursor_previous_location = cursor.location.copy()   
        #-------------------------------------------------  

        #  Child Of Constraint - Get Target (Object Name)
        def child_of_constraint_object():
            ob = bpy.context.object
            
            for const in ob.constraints:
                if const.type == 'CHILD_OF':
                    c_t = const.target.name
                    return c_t
              

        # Child Of Constraint - Get Armature Name , Bone Name    
        def child_of_constraint_bone():
            ob = bpy.context.object
            
            for const in ob.constraints:
                if const.type == 'CHILD_OF' and const.target.type == 'ARMATURE':
                    arm_name = const.target.name
                    bone_name = const.subtarget
                    return arm_name, bone_name  


        # Parent Object 
        def parent_object():
            
            ob = bpy.context.object
            if ob.parent is not None and ob.parent.type != 'ARMATURE':
                return ob.parent 
  

        # Parent Armature  
        def parent_armature():
            
            ob = bpy.context.object
            if ob.parent is not None and ob.parent.type == 'ARMATURE' and ob.parent_type == 'BONE':
                return ob.parent 
  

        def selected_bone():
            
            arm = parent_armature()   
            arm_name = arm.id_data.data.name
             
            act_bone = bpy.data.armatures[arm_name].bones.active
            if act_bone:
                
                return act_bone.name, act_bone.length


        # New Vertex Group
        def new_vertex_group():
            vert_gp = ob.vertex_groups.new(name = 'smooth_hook_grp_' + ob_name)
            bpy.ops.object.vertex_group_assign()    
            
        # Smooth Weight Paint
        def smooth_weight_paint():
            bpy.ops.object.mode_set(mode='WEIGHT_PAINT', toggle=False)
            
            bpy.context.object.vertex_groups.active = ob.vertex_groups[-1]
            vertex_group_name = ob.vertex_groups[-1].name             
                
            # Smooth
            bpy.ops.object.vertex_group_smooth()
            bpy.ops.object.vertex_group_smooth(repeat=vert_gr_smooth_repeat) 
            # Levels
            bpy.ops.object.vertex_group_levels()
            bpy.ops.object.vertex_group_levels(gain=vert_gr_level_gain)
            # Smooth
            bpy.ops.object.vertex_group_smooth()
            bpy.ops.object.vertex_group_smooth(repeat=vert_gr_smooth_repeat)
        
        
        # Add Empty
        def add_empty():
            bpy.ops.object.mode_set(mode='EDIT', toggle=False)
            
            ob = bpy.context.object
       
            # Active Vertex Group
            active_vert_gr = ob.vertex_groups.active
            
            cursor_location = bpy.context.scene.cursor.location
            empty = bpy.data.objects.new( "Smooth_Empty", None)
            empty.empty_display_type = 'SPHERE'
            empty.empty_display_size = 0.2

            # link to "collection"
            bpy.context.scene.collection.objects.link( empty )
        
            empty.location = cursor_location 
            if empty.location == cursor_location:
                empty_name = empty.name
            
            
            
            # Add 'Hook' Modifier

            modifier = ob.modifiers.new(name="Smooth_Hook", type='HOOK')
            modifier.vertex_group = active_vert_gr.name
            
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)

            modifier.object = bpy.data.objects[empty_name]


            
            # Get Armature
            arm = parent_armature()
            # Get Parent Object
            parent_obj = parent_object()
            const_target = child_of_constraint_object()
            const_target_arm_bone = child_of_constraint_bone()
             

                
            if arm and parent_to_parent and not parent_obj and not const_target and not const_target_arm_bone:
                    
                sel_bone = selected_bone()
                if sel_bone:
                    sel_bone_name = sel_bone[0]
                    
                    bone_length = sel_bone[1]
                    
                    arm_name = arm.id_data.data.name
     
                    arm_bones = bpy.data.armatures[arm_name].bones

                    empty.select_set(1)
                    empty.empty_display_size = bone_length / 4
                    
                    arm.select_set(1)
                    bpy.context.view_layer.objects.active = arm 

                    bpy.ops.object.parent_set(type='BONE') 

                    bpy.context.view_layer.objects.active = empty
                     
                    # Select Empty
                    select_active_object()

                    bpy.ops.object.transforms_to_deltas(mode='ALL')
                

            if arm and not parent_to_parent and not parent_obj and not const_target and not const_target_arm_bone:
                    
                empty.select_set(1)
                bpy.ops.object.transforms_to_deltas(mode='ALL')

                bpy.context.view_layer.objects.active = empty
                
                # Select Empty
                select_active_object() 
                
     
            
            if parent_obj and parent_to_parent and not arm and not const_target and not const_target_arm_bone:
                empty.select_set(1)
                bpy.ops.object.transforms_to_deltas(mode='ALL')

                parent_obj.select_set(1)
                bpy.context.view_layer.objects.active = parent_obj
                bpy.ops.object.parent_set(type='OBJECT', keep_transform=True)
                
                bpy.context.view_layer.objects.active = empty
                
                # Select Empty
                select_active_object()              


            if parent_obj and not parent_to_parent and not arm and not const_target and not const_target_arm_bone:
                empty.select_set(1)
                bpy.ops.object.transforms_to_deltas(mode='ALL')

                bpy.context.view_layer.objects.active = empty
                
                # Select Empty
                select_active_object()                


            if not parent_obj and not parent_to_parent and not arm and not const_target and not const_target_arm_bone:
                empty.select_set(1)
                bpy.ops.object.transforms_to_deltas(mode='ALL')

                bpy.context.view_layer.objects.active = empty
                
                # Select Empty
                select_active_object()    


            if not parent_obj and parent_to_parent and not arm and not const_target and not const_target_arm_bone:
                empty.select_set(1)
                bpy.ops.object.transforms_to_deltas(mode='ALL')

                bpy.context.view_layer.objects.active = empty
                
                # Select Empty
                select_active_object()  


            if const_target and parent_to_parent and not arm and not const_target_arm_bone:
                print(const_target)
                
                empty.select_set(1)
                bpy.ops.object.transforms_to_deltas(mode='ALL')
                
                bc = empty.constraints.new(type='CHILD_OF')
        
                bc.target = bpy.data.objects[const_target]                
                   
                bpy.context.view_layer.objects.active = empty
                
                # Select Empty
                select_active_object() 



            if const_target and not parent_to_parent and not arm and not const_target_arm_bone:

                empty.select_set(1)
                bpy.ops.object.transforms_to_deltas(mode='ALL')
                
                bpy.context.view_layer.objects.active = empty
                
                # Select Empty
                select_active_object() 



            if const_target_arm_bone and parent_to_parent and const_target and not arm:                 
                empty.select_set(1)
                bpy.ops.object.transforms_to_deltas(mode='ALL')
                
                bc = empty.constraints.new(type='CHILD_OF')                 
                 
                arm_name = const_target_arm_bone[0]
                bone_name = const_target_arm_bone[1]
                
                bc.target = bpy.data.objects[arm_name]    
                bc.subtarget = bone_name
    
                bpy.context.view_layer.objects.active = empty
                
                # Select Empty
                select_active_object()  


              
            if const_target_arm_bone and const_target and not parent_to_parent and not parent_obj and not arm:                 
                empty.select_set(1)
                bpy.ops.object.transforms_to_deltas(mode='ALL')

                bpy.context.view_layer.objects.active = empty
                
                # Select Empty
                select_active_object() 

 

        bpy.ops.view3d.snap_cursor_to_selected()
        new_vertex_group()
        smooth_weight_paint()
        add_empty()
        
        bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
        
 
        # Cursor back ------------------------------------
        cursor.location = cursor_previous_location  

        
        return {'FINISHED'}


#-----------------------------------------------------------------
#--------------------- End Create Smooth Emtpy Hook --------------
#-----------------------------------------------------------------





#-----------------------------------------------------------------
#--------------------- Create Smooth Bone ------------------------
#-----------------------------------------------------------------

class OBJECT_OT_create_smooth_bone_anbx(bpy.types.Operator):
    '''- Only for Meshes parented to the Armature with Automatic Weights !!!
- Created Bone can be parented to Active Bone
- More Selected Vertices = Stronger and Smoother Influence'''
    bl_idname = "object.create_smooth_bone_anbx"
    bl_label = "Create Smooth BONE"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        ob = context.object
        return (ob is not None and ob.type == 'MESH' and
        ob.mode == 'EDIT' and ob.data.total_vert_sel != 0
        and ob.parent is not None and ob.parent.type == 'ARMATURE'
        and ob.parent_type == 'OBJECT')


    def execute(self, context):
        
        ob = bpy.context.object
        ob_name = ob.name        

        # Ref 
        parent_to_act_bone = context.scene.smooth_empty_bone_parent_to_act_bone_anbx        
         
        # Repeat Smooth Vertex Group  
        vert_gr_smooth_repeat = context.scene.vertex_group_smooth_repeat_anbx   
        # Levels Gain    
        vert_gr_level_gain = context.scene.vertex_group_levels_gain_anbx      

         
         
         
        # Cursor -----------------------------------------
        cursor = bpy.context.scene.cursor
        cursor_location = bpy.context.scene.cursor.location
        cursor_previous_location = cursor.location.copy()   
        #-------------------------------------------------

        bpy.ops.view3d.snap_cursor_to_selected()
  
 
        # Parent Armature  
        def parent_armature():
            
            ob = bpy.context.object
            if ob.parent is not None and ob.parent.type == 'ARMATURE':
                return ob.parent    
  


        # New Vertex Group--------------------------------
            
        vert_gp = ob.vertex_groups.new(name = 'Smooth_Bone_' + ob_name)
        bpy.ops.object.vertex_group_assign() 
        
            
        # Smooth Weight Paint -------------------------------
            
        bpy.ops.object.mode_set(mode='WEIGHT_PAINT', toggle=False)
        
        bpy.context.object.vertex_groups.active = ob.vertex_groups[-1]
        vertex_group_name = ob.vertex_groups[-1].name 
        
        # Smooth
        bpy.ops.object.vertex_group_smooth()
        bpy.ops.object.vertex_group_smooth(repeat=vert_gr_smooth_repeat) 
        # Levels
        bpy.ops.object.vertex_group_levels()
        bpy.ops.object.vertex_group_levels(gain=vert_gr_level_gain)
        # Smooth
        bpy.ops.object.vertex_group_smooth()
        bpy.ops.object.vertex_group_smooth(repeat=vert_gr_smooth_repeat)
        
        # ---------------------------------------------------
        
        bpy.ops.object.mode_set(mode='EDIT', toggle=False)
        
        
        # Parent Armature -------------------------
        arm = parent_armature()
        bpy.context.view_layer.objects.active = arm
     
        # Cursor -----------------------------------------

        clx = cursor_location.x
        cly = cursor_location.y
        clz = cursor_location.z

        bpy.ops.object.mode_set(mode='EDIT', toggle=False)
        
        bpy.context.evaluated_depsgraph_get().update()

        act_bone = bpy.context.active_bone        
        
        # Add new Bone
        bpy.ops.armature.bone_primitive_add()

        last_bone = bpy.context.object.data.edit_bones[-1]
        
        last_bone.name = vertex_group_name
                 
  
        last_bone.head = (clx,cly,clz)
        last_bone.tail = (clx, cly, clz + 0.5) 
    
            
        if parent_to_act_bone and act_bone:
            bone_name = act_bone.name
            bone_length = act_bone.length  
     
            
            last_bone.parent = arm.data.edit_bones[bone_name]
            last_bone.length = bone_length / 2
                    
        
        bpy.ops.object.mode_set(mode='OBJECT', toggle=False)

        obj = bpy.data.objects.get(ob_name)
        bpy.context.view_layer.objects.active = obj
        
        
        # Pose Mode
        bpy.ops.object.mode_set(mode='OBJECT', toggle=False)

        bpy.ops.object.select_all(action='DESELECT')
        bpy.context.view_layer.objects.active = arm
        bpy.ops.object.posemode_toggle()                 
 
        
        # Cursor back ------------------------------------
        cursor.location = cursor_previous_location  

        return {'FINISHED'}


#-----------------------------------------------------------------
#--------------------- End Create Smooth Bone --------------------
#-----------------------------------------------------------------






def parent_to_act_bone_anbx(self, context):
    sc = bpy.context.scene

    sc.property_unset('smooth_empty_bone_parent_to_act_bone_anbx')
 
        
def parent_to_parent_anbx(self, context):
    sc = bpy.context.scene

    sc.property_unset('smooth_empty_parent_to_parent_obj_anbx')
 
        
                
        
    
        

classes = [ OBJECT_OT_create_smooth_empty_hook_anbx,
            OBJECT_OT_create_smooth_bone_anbx,

          ]
 
 
 
def register():
    for cls in classes:
        bpy.utils.register_class(cls)
        

    #------------------- Parent to Active Bone -------------------
    

    bpy.types.Scene.smooth_empty_bone_parent_to_act_bone_anbx = bpy.props.BoolProperty(
        name=" Parent to Active Bone",
        description="Parent to Active Bone",
        default = False,
        update = parent_to_parent_anbx,
    )

    #-------------------------------------------------------------
 

    #------------------- Parent to 'Parent' Object if exists------
    

    bpy.types.Scene.smooth_empty_parent_to_parent_obj_anbx = bpy.props.BoolProperty(
        name=" Attach to Parent",
        description="Parent created Empty to the parent object in hierarchy",
        default = False,
        update = parent_to_act_bone_anbx,
    )

    #-------------------------------------------------------------

    #--------------- Vertex Group Smooth - times to repeat -------
    
    bpy.types.Scene.vertex_group_smooth_repeat_anbx = bpy.props.IntProperty(
        name="",
        default=15,
        description="Smooth Vertex Weights - Iterations",
        min=0,
        
    )
 
    #-------------------------------------------------------------


    #--------------- Vertex Group Smooth - levels (gain) ---------
    
    bpy.types.Scene.vertex_group_levels_gain_anbx = bpy.props.IntProperty(
        name="",
        default=3,
        description="Vertex Group Levels Gain ( Influence Area )",
        min=0,
        
    )
 
    #-------------------------------------------------------------
 



 
def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)
        

    del bpy.types.Scene.smooth_empty_bone_parent_to_act_bone_anbx
    del bpy.types.Scene.smooth_empty_parent_to_parent_obj_anbx
    del bpy.types.Scene.vertex_group_smooth_repeat_anbx
    del bpy.types.Scene.vertex_group_levels_gain_anbx
 
 
if __name__ == "__main__":
    register() 