import bpy
import math
from bpy.props import (BoolProperty, FloatProperty, IntProperty)



#============================ Rotate Around Self ======================



class OBJECT_OT_driver_rotate_around_self_anbx(bpy.types.Operator):
    '''Rotate Selected Objects around the Origin'''

    bl_idname = "object.driver_rotate_around_self_anbx"
    bl_label = "Rotate Around Origin"
    bl_options = {'REGISTER', 'UNDO'}

    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) != 0)


    def execute(self, context):
  
        # Ref 
        sin_bool = context.scene.driver_sine_wave_anbx  
        offset_motion_bool = context.scene.driver_offset_motion_anbx        
        offset_motion_amp = context.scene.driver_offset_motion_amplitude_anbx      
       
        speed = context.scene.driver_speed_anbx
          
        x_axis = context.scene.driver_axis_x_anbx
        y_axis = context.scene.driver_axis_y_anbx
        z_axis = context.scene.driver_axis_z_anbx
        
        offset_speed = context.scene.driver_offset_speed_anbx
        
        sel_ob = bpy.context.selected_objects
        
        
        def rot_driver_offseted(transf, index):
            
            offset = 0
            for o in sel_ob:
                n_dr = o.driver_add(transf, index)
                n_dr.driver.type = 'SCRIPTED'
            
                amp = offset_motion_amp
                freq = speed / speed
                       
                if sin_bool:                         
                    n_dr.driver.expression = "{}  * sin({} + (frame * {}) /{})".format(amp, offset, speed, freq) 
                
                       
                else:                         
                    n_dr.driver.expression = "{}  * ({} + (frame * {}) /{})".format(amp, offset, speed, freq) 
                                
                
                offset += (1 + offset_speed)
     
 
 
        def rot_driver(transf, index):
            n_dr = o.driver_add(transf, index)
            n_dr.driver.type = 'SCRIPTED'
            
            amp = offset_motion_amp
            freq = speed / speed

 
            if sin_bool:
                n_dr.driver.expression = "{} * sin(frame * {}) /{}".format(amp, speed, freq)
            
            else:
                n_dr.driver.expression = "{} * (frame * {}) /{}".format(amp, speed, freq)
            
 
 
 

        def add_driver_offseted():
                   
            if x_axis:
                rot_driver_offseted('rotation_euler', 0)       
            if y_axis:
                rot_driver_offseted('rotation_euler', 1)       
            if z_axis:
                rot_driver_offseted('rotation_euler', 2)       
            
            else:
                pass                  
  
                
 

        def add_driver_main():
                   
            if x_axis:
                rot_driver('rotation_euler', 0)       
            if y_axis:
                rot_driver('rotation_euler', 1)       
            if z_axis:
                rot_driver('rotation_euler', 2)       
            
            else:
                pass                  

 
        for o in bpy.context.selected_objects:
            if offset_motion_bool:
                add_driver_offseted()
             
            if not offset_motion_bool:   
                speed += offset_speed
                add_driver_main()
      
 
 
    


        return {'FINISHED'}

#============================ End Rotate Around Self ======================





#============================ Rotate Around Active ======================

 
class OBJECT_OT_driver_rotate_around_active_anbx(bpy.types.Operator):
    '''Rotate Selected around Active
- Parent the Selected Objects to the Active Object
- Origin to Active Object'''

    bl_idname = "object.driver_rotate_around_active_anbx"
    bl_label = "Rotate Around Active"
    bl_options = {'REGISTER', 'UNDO'}

    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) != 0)


    def execute(self, context):
  
        # Ref 
        sin_bool = context.scene.driver_sine_wave_anbx 
        
        offset_motion_bool = context.scene.driver_offset_motion_anbx        
        offset_motion_amp = context.scene.driver_offset_motion_amplitude_anbx      
           
        speed = context.scene.driver_speed_anbx
              
        x_axis = context.scene.driver_axis_x_anbx
        y_axis = context.scene.driver_axis_y_anbx
        z_axis = context.scene.driver_axis_z_anbx
        
        offset_speed = context.scene.driver_offset_speed_anbx
        
        sel_ob = bpy.context.selected_objects
        cursor = bpy.context.scene.cursor
        act_ob = bpy.context.active_object
 
        
        cursor_previous_location = cursor.location.copy()
        
        for o in sel_ob:
            
            if o == act_ob:
                bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='MEDIAN')
                cursor.location = o.location

                
        for o in bpy.context.selected_objects:        
            if o != bpy.context.active_object:
                bpy.ops.object.origin_set(type='ORIGIN_CURSOR', center='MEDIAN')
                
 

        # Cursor back
        cursor.location = cursor_previous_location        
        
        
        
        def rot_driver_offseted(transf, index):
            
            offset = 0
            for o in sel_ob:
                n_dr = o.driver_add(transf, index)
                n_dr.driver.type = 'SCRIPTED'
            
                amp = offset_motion_amp
                freq = speed / speed
                       
                if sin_bool:                         
                    n_dr.driver.expression = "{}  * sin({} + (frame * {}) /{})".format(amp, offset, speed, freq) 
                
                       
                else:                         
                    n_dr.driver.expression = "{}  * ({} + (frame * {}) /{})".format(amp, offset, speed, freq) 
                                 
                
                offset += (1 + offset_speed)
     
 

          
 
        def rot_driver(transf, index):
            n_dr = o.driver_add(transf, index)
            n_dr.driver.type = 'SCRIPTED'

            amp = offset_motion_amp
            freq = speed / speed

 
            if sin_bool:
                n_dr.driver.expression = "{} * sin(frame * {}) /{}".format(amp, speed, freq)
            
            else:
                n_dr.driver.expression = "{} * (frame * {}) /{}".format(amp, speed, freq)
            
 
 
 
        def add_driver_offseted():
                   
            if x_axis:
                rot_driver_offseted('rotation_euler', 0)       
            if y_axis:
                rot_driver_offseted('rotation_euler', 1)       
            if z_axis:
                rot_driver_offseted('rotation_euler', 2)       
            
            else:
                pass                  
  
                 
            
        def add_driver_main():
                      
            if x_axis:
                rot_driver('rotation_euler', 0)       
            if y_axis:
                rot_driver('rotation_euler', 1)       
            if z_axis:
                rot_driver('rotation_euler', 2)       
            
            else:
                pass                  

 

        for o in bpy.context.selected_objects:
            if o != act_ob:
                if offset_motion_bool:
                    add_driver_offseted()
                    o.parent = act_ob
                    o.matrix_parent_inverse = act_ob.matrix_world.inverted()
                    
                if not offset_motion_bool: 
                    
                    speed += offset_speed
                    add_driver_main()
                    o.parent = act_ob 
                    o.matrix_parent_inverse = act_ob.matrix_world.inverted()

     
  
        return {'FINISHED'}

#============================ End Rotate Around Active ======================




#============================ Rotate Around Cursor ======================

 
class OBJECT_OT_driver_rotate_around_cursor_anbx(bpy.types.Operator):
    '''Rotate Selected Objects around 3D Cursor
- Origin to 3D Cursor'''

    bl_idname = "object.driver_rotate_around_cursor_anbx"
    bl_label = "Rotate Around Cursor"
    bl_options = {'REGISTER', 'UNDO'}

    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) != 0)


    def execute(self, context):
  
        # Ref 
        sin_bool = context.scene.driver_sine_wave_anbx 
        
        offset_motion_bool = context.scene.driver_offset_motion_anbx        
        offset_motion_amp = context.scene.driver_offset_motion_amplitude_anbx      
                  
        speed = context.scene.driver_speed_anbx
                
        x_axis = context.scene.driver_axis_x_anbx
        y_axis = context.scene.driver_axis_y_anbx
        z_axis = context.scene.driver_axis_z_anbx
        
        offset_speed = context.scene.driver_offset_speed_anbx
        
        sel_ob = bpy.context.selected_objects
        cursor = bpy.context.scene.cursor
 
          
        for o in bpy.context.selected_objects:        
            bpy.ops.object.origin_set(type='ORIGIN_CURSOR', center='MEDIAN')
                
 
        def rot_driver_offseted(transf, index):
            
            offset = 0
            for o in sel_ob:
                n_dr = o.driver_add(transf, index)
                n_dr.driver.type = 'SCRIPTED'
            
                amp = offset_motion_amp
                freq = speed / speed
                       
                if sin_bool:                         
                    n_dr.driver.expression = "{}  * sin({} + (frame * {}) /{})".format(amp, offset, speed, freq) 
                
                       
                else:                         
                    n_dr.driver.expression = "{}  * ({} + (frame * {}) /{})".format(amp, offset, speed, freq) 
                                 
                
                offset += (1 + offset_speed)
     
 

 
        def rot_driver(transf, index):
            n_dr = o.driver_add(transf, index)
            n_dr.driver.type = 'SCRIPTED'
 
            amp = offset_motion_amp
            freq = speed / speed

 
            if sin_bool:
                n_dr.driver.expression = "{} * sin(frame * {}) /{}".format(amp, speed, freq)
            
            else:
                n_dr.driver.expression = "{} * (frame * {}) /{}".format(amp, speed, freq)
            
          
 
 
        def add_driver_offseted():
                   
            if x_axis:
                rot_driver_offseted('rotation_euler', 0)       
            if y_axis:
                rot_driver_offseted('rotation_euler', 1)       
            if z_axis:
                rot_driver_offseted('rotation_euler', 2)       
            
            else:
                pass                  
                  
 

        def add_driver_main(): 
                      
            if x_axis:
                rot_driver('rotation_euler', 0)       
            if y_axis:
                rot_driver('rotation_euler', 1)       
            if z_axis:
                rot_driver('rotation_euler', 2)       
            
            else:
                pass                  

 

        for o in bpy.context.selected_objects:
            if offset_motion_bool:
                add_driver_offseted()
               
            if not offset_motion_bool: 
                speed += offset_speed
                add_driver_main()
 


        return {'FINISHED'}

#============================ End Rotate Around Cursor ======================







#============================ Remove Rotation Driver ======================

 
class OBJECT_OT_remove_rotation_driver_anbx(bpy.types.Operator):
    '''Remove Rotation Driver'''

    bl_idname = "object.remove_rotation_driver_anbx"
    bl_label = ""
    bl_options = {'REGISTER', 'UNDO'}

    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) != 0)


    def execute(self, context):
        
        bpy.context.scene.property_unset('driver_offset_motion_amplitude_anbx')               
        bpy.context.scene.property_unset('driver_offset_speed_anbx')        
        bpy.context.scene.property_unset('driver_speed_anbx')        
        
        
        act_ob_name = bpy.context.active_object.name
  
        def remove_driver(transf, index):
            r_dr = o.driver_remove(transf, index)
 

        def remove_driver_main():
            remove_driver('rotation_euler', 0)
            remove_driver('rotation_euler', 1)
            remove_driver('rotation_euler', 2)


        
        for o in bpy.context.selected_objects:
            bpy.context.view_layer.objects.active = o
            remove_driver_main()
            
        for o in bpy.context.selected_objects:
            if o.name == act_ob_name:
                bpy.context.view_layer.objects.active = o 
                
            
                  
        return {'FINISHED'}

#============================ End Remove Rotation Driver ====================== 


 
        

classes = [ OBJECT_OT_driver_rotate_around_self_anbx,
            OBJECT_OT_driver_rotate_around_active_anbx,
            OBJECT_OT_driver_rotate_around_cursor_anbx,
            OBJECT_OT_remove_rotation_driver_anbx,

            
          ]
 
 
 
def register():
    for cls in classes:
        bpy.utils.register_class(cls)
  



    bpy.types.Scene.driver_sine_wave_anbx = bpy.props.BoolProperty(
        name="",
        description="Sine Wave Motion (Checkbox)",
        default = False
    )





#========================= Offset Motion ==========================

    
    bpy.types.Scene.driver_offset_motion_anbx = bpy.props.BoolProperty(
        name="",
        description="Offset Motion (Checkbox)",
        default = False
    )



    # Amplitude 
    bpy.types.Scene.driver_offset_motion_amplitude_anbx = bpy.props.FloatProperty(
        name="Amplitude",
        default=0.2,
        description="Amplitude of Offset Motion",
        step=1,
 

    )

 

#=================== Axis ======================

    bpy.types.Scene.driver_axis_x_anbx = bpy.props.BoolProperty(
        name="",
        description="X Axis (Checkbox)",
        default = False
    )

    bpy.types.Scene.driver_axis_y_anbx = bpy.props.BoolProperty(
        name="",
        description="Y Axis (Checkbox)",
        default = False
    )


    bpy.types.Scene.driver_axis_z_anbx = bpy.props.BoolProperty(
        name="",
        description="Z Axis (Checkbox)",
        default = False
    )

#=================== Axis ======================


  
    bpy.types.Scene.driver_speed_anbx = bpy.props.FloatProperty(
        name="Speed", 
        default=0.1, 
        description="Higher Value = greater Speed",
        step=1,
        precision=2,
 
    )
  

    bpy.types.Scene.driver_offset_speed_anbx = bpy.props.FloatProperty(
        name="Offset Speed", 
        default=0.0, 
        description="Offset Speed - Selected Objects",
        step=1,
        precision=2,
 
    )
  

 
def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)

    del bpy.types.Scene.driver_sine_wave_anbx
    del bpy.types.Scene.driver_offset_motion_anbx
    del bpy.types.Scene.driver_offset_motion_amplitude_anbx

     
    del bpy.types.Scene.driver_axis_x_anbx
    del bpy.types.Scene.driver_axis_y_anbx
    del bpy.types.Scene.driver_axis_z_anbx
    
    del bpy.types.Scene.driver_speed_anbx      
    del bpy.types.Scene.driver_offset_speed_anbx
    
 
if __name__ == "__main__":
    register() 
    
    
     
