 
import bpy
import math
from bpy.props import (BoolProperty, FloatProperty, IntProperty)




#====================== Add Cycle Modifier ====================


 
#==============================================
#  ADD CYCLES MODIFIER - Extra Option
#==============================================


# Add CYCLES Modifier ( "Pre_Post_Infinity" ) to Selected Object
class OBJECT_OT_add_remove_cycle_modifier_anbx(bpy.types.Operator):
    '''Add CYCLES Modifier - To Selected Object(s)

- applied to the corresponding Keyframe(s)
- according to the selected Option:

- Location, Rotation, Scale.
- Light Parameters.
- Shape Keys.
- Materials.
- Curve Bevel Depth.

Alt + Click to Remove'''

    bl_idname = "object.add_remove_cycle_modifier_anbx"
    bl_label = "Cycle Modifier"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):    
        return (context.object is not None and
                 context.object.mode == 'OBJECT'
                 and len(context.selected_objects) != 0 )
 
 

    def invoke(self, context, event):



        # Ref
 
        cycle_loc = context.scene.cycle_modifier_fcurve_location_anbx
        cycle_rot = context.scene.cycle_modifier_fcurve_rotation_anbx
        cycle_scale = context.scene.cycle_modifier_fcurve_scale_anbx
        
 
        cycle_light = context.scene.cycle_modifier_fcurve_lights_anbx
        cycle_mat = context.scene.cycle_modifier_fcurve_materials_anbx
        cycle_sk = context.scene.cycle_modifier_fcurve_shape_keys_anbx
        cycle_curve_bevel = context.scene.cycle_modifier_fcurve_curve_bevel_anbx        
      
             

        anim_data = bpy.context.object.animation_data
        ob = bpy.context.object

        sel_ob = bpy.context.selected_objects
        act_ob = bpy.context.active_object
 


        # Location -----------------------------------------------------------
        
        def loc_cycle_mod():

            for o in sel_ob:
                bpy.context.view_layer.objects.active = o

                if o.animation_data:
                    if o.animation_data.action:
                        if o.animation_data.action.fcurves:


                            ob_fcurve = o.animation_data.action.fcurves

                            for fc in ob_fcurve:
                                d_path = ('location')

                                if fc.data_path.endswith((d_path)):

                                    if event:

                                        for m in fc.modifiers:
                                           if (m.type == 'CYCLES'):
                                               fc.modifiers.remove(m)
                                        fc.modifiers.new(type='CYCLES')

                                    if event.alt:   

                                        for m in fc.modifiers:
                                           if (m.type == 'CYCLES'):
                                               fc.modifiers.remove(m)


                                fc.update()


        for o in sel_ob:        
            if cycle_loc: 
                loc_cycle_mod()
        





        # Rotation -----------------------------------------------------------
        
        def rot_cycle_mod():

            for o in sel_ob:
                bpy.context.view_layer.objects.active = o

                if o.animation_data:
                    if o.animation_data.action:
                        if o.animation_data.action.fcurves:


                            ob_fcurve = o.animation_data.action.fcurves

                            for fc in ob_fcurve:
                                d_path = ('rotation_quaternion', 'rotation_euler')

                                if fc.data_path.endswith((d_path)):

                                    if event:

                                        for m in fc.modifiers:
                                           if (m.type == 'CYCLES'):
                                               fc.modifiers.remove(m)
                                        fc.modifiers.new(type='CYCLES')

                                    if event.alt:   

                                        for m in fc.modifiers:
                                           if (m.type == 'CYCLES'):
                                               fc.modifiers.remove(m)


                                fc.update()


        for o in sel_ob:        
            if cycle_rot: 
                rot_cycle_mod()
        







        # Scale -----------------------------------------------------------
        
        def scale_cycle_mod():

            for o in sel_ob:
                bpy.context.view_layer.objects.active = o

                if o.animation_data:
                    if o.animation_data.action:
                        if o.animation_data.action.fcurves:


                            ob_fcurve = o.animation_data.action.fcurves

                            for fc in ob_fcurve:
                                d_path = ('scale')

                                if fc.data_path.endswith((d_path)):

                                    if event:

                                        for m in fc.modifiers:
                                           if (m.type == 'CYCLES'):
                                               fc.modifiers.remove(m)
                                        fc.modifiers.new(type='CYCLES')

                                    if event.alt:   

                                        for m in fc.modifiers:
                                           if (m.type == 'CYCLES'):
                                               fc.modifiers.remove(m)


                                fc.update()


        for o in sel_ob:        
            if cycle_scale: 
                scale_cycle_mod()
        





        # Location Rotation Scale -----------------------------------------------

        def loc_rot_scale_cycle_mod():

            for o in sel_ob:
                bpy.context.view_layer.objects.active = o

                if o.animation_data:
                    if o.animation_data.action:
                        if o.animation_data.action.fcurves:


                            ob_fcurve = o.animation_data.action.fcurves

                            for fc in ob_fcurve:
                                d_path = ('location', 'rotation_quaternion', 'rotation_euler', 'scale')

                                if fc.data_path.endswith((d_path)):

                                    if event:

                                        for m in fc.modifiers:
                                           if (m.type == 'CYCLES'):
                                               fc.modifiers.remove(m)
                                        fc.modifiers.new(type='CYCLES')

                                    if event.alt:   

                                        for m in fc.modifiers:
                                           if (m.type == 'CYCLES'):
                                               fc.modifiers.remove(m)


                                fc.update()


        for o in sel_ob:        
            if not cycle_loc and not cycle_rot and not cycle_scale and not cycle_light and not cycle_mat and not cycle_sk and not cycle_curve_bevel:  
                loc_rot_scale_cycle_mod()          
 
 
 

        #=======================LIGHTS============

        lights = bpy.data.lights
            
        def lights_cycle_mod():

            for o in sel_ob:
                bpy.context.view_layer.objects.active = o
                o_name = o.name

                for l in lights:
                    if l.name == o_name:

                        if l.animation_data:
                            if l.animation_data.action:
                                if l.animation_data.action.fcurves:

                                    l_fcurve = l.animation_data.action.fcurves
                                    for fc in l_fcurve:
                                        d_path = ('energy', 'color')

                                        if fc.data_path.endswith((d_path)):

                                            if event:
                                 
                                                for m in fc.modifiers:
                                                   if (m.type == 'CYCLES'):
                                                       fc.modifiers.remove(m)
                                                fc.modifiers.new(type='CYCLES')

                                            if event.alt:

                                                for m in fc.modifiers:
                                                   if (m.type == 'CYCLES'):
                                                       fc.modifiers.remove(m)

                                        
                                        fc.update()               



        for o in sel_ob:
            if cycle_light:
                if o.type == 'LIGHT':       
                    lights_cycle_mod()



 

        #=======================Shape Keys============

        def shape_key_cycle_mod():
                
            for o in sel_ob:
                if o.type == 'MESH':

                    shape_keys = o.data.shape_keys

                    if shape_keys:

                        sk_blocks = o.data.shape_keys.key_blocks
                        for s in sk_blocks:
                            sk_block_name = s.name
                        
                        if shape_keys.animation_data:
                            if shape_keys.animation_data.action:
                                if shape_keys.animation_data.action.fcurves:
                                    
                                    shk_fcurve = shape_keys.animation_data.action.fcurves
                                    for fc in shk_fcurve:       
                                        if fc.data_path:
                                            
                                            if event:
                          
                                                for m in fc.modifiers:
                                                   if (m.type == 'CYCLES'):
                                                       fc.modifiers.remove(m)
                                                fc.modifiers.new(type='CYCLES')
                                
                                            if event.alt:
                                                
                                                for m in fc.modifiers:
                                                   if (m.type == 'CYCLES'):
                                                       fc.modifiers.remove(m)                  
                                            
                                        fc.update()



        for o in sel_ob:
            if cycle_sk:
                if o.type == 'MESH':
                    shape_key_cycle_mod()


 


        #=======================Materials============

        mat_object_types = ['CURVE', 'MESH', 'FONT', 'SURFACE']

         
        act_ob = bpy.context.active_object

        def animated_mat_diff_color_cycle_mod():
            sel_ob = bpy.context.selected_objects
                 
            for o in sel_ob:
                if o.type in mat_object_types:

                    for m in o.data.materials:
                        mat_name = m.name
                        if m.animation_data:
                            if m.animation_data.action:
                                if m.animation_data.action.fcurves:
                                    f_curve =  bpy.data.materials[mat_name].diffuse_color.data.animation_data.action.fcurves
                                
                                    for fc in f_curve:      
                                        if fc.data_path:
                    
                                            if event:
                          
                                                for m in fc.modifiers:
                                                   if (m.type == 'CYCLES'):
                                                       fc.modifiers.remove(m)
                                                fc.modifiers.new(type='CYCLES')
                                
                                            if event.alt:
                                                
                                                for m in fc.modifiers:
                                                   if (m.type == 'CYCLES'):
                                                       fc.modifiers.remove(m)                  
                                            
                                        
                                        fc.update()   

             


        def animated_material_cycle_mod():
            sel_ob = bpy.context.selected_objects
           
            for o in sel_ob:
                if o.type in mat_object_types:

                    for n in o.data.materials:
                        mat_name = n.name
                        if n.node_tree:
                            if n.node_tree.animation_data:
                                if n.node_tree.animation_data.action:
                                    if n.node_tree.animation_data.action.fcurves:
                                        fn_curve = n.node_tree.animation_data.action.fcurves
             
                                        for fc in fn_curve:       
                                            if fc.data_path:
                     
                                                if event:
                              
                                                    for m in fc.modifiers:
                                                       if (m.type == 'CYCLES'):
                                                           fc.modifiers.remove(m)
                                                    fc.modifiers.new(type='CYCLES')
                                    
                                                if event.alt:
                                                    
                                                    for m in fc.modifiers:
                                                       if (m.type == 'CYCLES'):
                                                           fc.modifiers.remove(m)                  
                                                                                    
                                       
                                            fc.update()                       
                                                
                                                                                  

        for o in sel_ob:
            if cycle_mat:
                if o.type in mat_object_types:
                    animated_material_cycle_mod()
                    animated_mat_diff_color_cycle_mod()


 


        #======================= Curves ============

        curves = bpy.data.curves
            
        def curve_bevel_depth_cycle_mod():

            for o in sel_ob:
                bpy.context.view_layer.objects.active = o
                o_name = o.name

                for c in curves:
                    if c.name == o_name:

                        if c.animation_data:
                            if c.animation_data.action:
                                if c.animation_data.action.fcurves:

                                    c_fcurve = c.animation_data.action.fcurves
                                    for fc in c_fcurve:
                                        d_path = ('bevel_depth')

                                        if fc.data_path.endswith((d_path)):

                                            if event:
                                 
                                                for m in fc.modifiers:
                                                   if (m.type == 'CYCLES'):
                                                       fc.modifiers.remove(m)
                                                fc.modifiers.new(type='CYCLES')

                                            if event.alt:

                                                for m in fc.modifiers:
                                                   if (m.type == 'CYCLES'):
                                                       fc.modifiers.remove(m)

                                        
                                        fc.update()               


        for o in sel_ob:
            if cycle_curve_bevel:
                if o.type == 'CURVE':
                    curve_bevel_depth_cycle_mod()

 




        return {'FINISHED'}


#==============================================
# END - ADD CYCLES MODIFIER - Extra Option
#==============================================




classes = [OBJECT_OT_add_remove_cycle_modifier_anbx]




def register():
    for cls in classes:
        bpy.utils.register_class(cls)





#========================= Loc Rot Scale ==========================


    bpy.types.Scene.cycle_modifier_fcurve_location_anbx = bpy.props.BoolProperty(
        name="",
        description="Cycle Modifier - Location (Checkbox)",
        default = False
    )

    bpy.types.Scene.cycle_modifier_fcurve_rotation_anbx = bpy.props.BoolProperty(
        name="",
        description="Cycle Modifier - Rotation (Checkbox)",
        default = False
    )

    bpy.types.Scene.cycle_modifier_fcurve_scale_anbx = bpy.props.BoolProperty(
        name="",
        description="Cycle Modifier - Scale (Checkbox)",
        default = False
    )


#========================= End "Loc Rot Scale" ==========================


#========================= Lights / Shape Keys / Materials / Curves ==========================


    bpy.types.Scene.cycle_modifier_fcurve_lights_anbx = bpy.props.BoolProperty(
        name="",
        description="Cycle Modifier - Lights (Checkbox)",
        default = False
    )


    bpy.types.Scene.cycle_modifier_fcurve_materials_anbx = bpy.props.BoolProperty(
        name="",
        description="Cycle Modifier - Materials (Checkbox)",
        default = False
    )



    bpy.types.Scene.cycle_modifier_fcurve_shape_keys_anbx = bpy.props.BoolProperty(
        name="",
        description="Cycle Modifier - Shape Keys (Checkbox)",
        default = False
    )


    bpy.types.Scene.cycle_modifier_fcurve_curve_bevel_anbx = bpy.props.BoolProperty(
        name="",
        description="Cycle Modifier - Curve Bevel (Checkbox)",
        default = False
    )


 


#========================= Lights / Shape Keys / Materials / Curves / Bones ==========================






def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)



    del bpy.types.Scene.cycle_modifier_fcurve_location_anbx
    del bpy.types.Scene.cycle_modifier_fcurve_rotation_anbx
    del bpy.types.Scene.cycle_modifier_fcurve_scale_anbx
    
    del bpy.types.Scene.cycle_modifier_fcurve_lights_anbx
    del bpy.types.Scene.cycle_modifier_fcurve_materials_anbx
    del bpy.types.Scene.cycle_modifier_fcurve_shape_keys_anbx
    del bpy.types.Scene.cycle_modifier_fcurve_curve_bevel_anbx
 



if __name__ == "__main__":
    register()





 
