import bpy




#==============================================
#  ADD CYCLES MODIFIER - "PRE POST INFINITY"
#==============================================


# Add CYCLES Modifier ( "Pre_Post_Infinity" ) to Selected Object
class OBJECT_OT_add_remove_infinity_cycle_anbx(bpy.types.Operator):
    '''Add CYCLES Modifier - To Selected Object(s)

All at Once:

- Location, Rotation, Scale.
- Light Parameters.
- Shape Keys.
- Materials.
- Curve Bevel Depth.

Alt + Click to Remove'''

    bl_idname = "object.add_remove_infinity_cycle_anbx"
    bl_label = "Pre_Post Infinity Cycle"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):

        return (context.object is not None and
                 context.object.mode == 'OBJECT'
                 and len(context.selected_objects) != 0)

 
    def invoke(self, context, event):


        anim_data = bpy.context.object.animation_data
        ob = bpy.context.object

        sel_ob = bpy.context.selected_objects
        act_ob = bpy.context.active_object
 


        def loc_rot_scale_pre_post_infinity_sel():

            for o in sel_ob:
                bpy.context.view_layer.objects.active = o

                if o.animation_data:
                    if o.animation_data.action:
                        if o.animation_data.action.fcurves:


                            ob_fcurve = o.animation_data.action.fcurves

                            for fc in ob_fcurve:
                                d_path = ('location', 'rotation_quaternion', 'rotation_euler', 'scale')

                                if fc.data_path.endswith((d_path)):

                                    if event:

                                        for m in fc.modifiers:
                                           if (m.type == 'CYCLES'):
                                               fc.modifiers.remove(m)
                                        fc.modifiers.new(type='CYCLES')

                                    if event.alt:   

                                        for m in fc.modifiers:
                                           if (m.type == 'CYCLES'):
                                               fc.modifiers.remove(m)


                                fc.update()


        loc_rot_scale_pre_post_infinity_sel()




        #=======================LIGHTS============

        lights = bpy.data.lights
            
        def lights_infinity_cycle():

            for o in sel_ob:
                bpy.context.view_layer.objects.active = o
                o_name = o.name

                for l in lights:
                    if l.name == o_name:

                        if l.animation_data:
                            if l.animation_data.action:
                                if l.animation_data.action.fcurves:

                                    l_fcurve = l.animation_data.action.fcurves
                                    for fc in l_fcurve:
                                        d_path = ('energy', 'color')

                                        if fc.data_path.endswith((d_path)):

                                            if event:
                                 
                                                for m in fc.modifiers:
                                                   if (m.type == 'CYCLES'):
                                                       fc.modifiers.remove(m)
                                                fc.modifiers.new(type='CYCLES')

                                            if event.alt:

                                                for m in fc.modifiers:
                                                   if (m.type == 'CYCLES'):
                                                       fc.modifiers.remove(m)

                                        
                                        fc.update()               



        for o in sel_ob:
            if o.type == 'LIGHT':
                lights_infinity_cycle()



        #=======================LIGHTS============           


        #=======================Shape Keys============

        def shape_key_infinity_cycle():
                
            for o in sel_ob:
                if o.type == 'MESH':

                    shape_keys = o.data.shape_keys

                    if shape_keys:

                        sk_blocks = o.data.shape_keys.key_blocks
                        for s in sk_blocks:
                            sk_block_name = s.name
                        
                        if shape_keys.animation_data:
                            if shape_keys.animation_data.action:
                                if shape_keys.animation_data.action.fcurves:
                                    
                                    shk_fcurve = shape_keys.animation_data.action.fcurves
                                    for fc in shk_fcurve:       
                                        if fc.data_path:
                                            
                                            if event:
                          
                                                for m in fc.modifiers:
                                                   if (m.type == 'CYCLES'):
                                                       fc.modifiers.remove(m)
                                                fc.modifiers.new(type='CYCLES')
                                
                                            if event.alt:
                                                
                                                for m in fc.modifiers:
                                                   if (m.type == 'CYCLES'):
                                                       fc.modifiers.remove(m)                  
                                            
                                        fc.update()



        for o in sel_ob:
            if o.type == 'MESH':
                shape_key_infinity_cycle()



        #=======================Shape Keys============



        #=======================Materials============

        mat_object_types = ['CURVE', 'MESH', 'FONT', 'SURFACE']

         
        act_ob = bpy.context.active_object

        def animated_mat_diff_color_infinity_cycle():
            sel_ob = bpy.context.selected_objects
                 
            for o in sel_ob:
                if o.type in mat_object_types:

                    for m in o.data.materials:
                        mat_name = m.name
                        if m.animation_data:
                            if m.animation_data.action:
                                if m.animation_data.action.fcurves:
                                    f_curve =  bpy.data.materials[mat_name].diffuse_color.data.animation_data.action.fcurves
                                
                                    for fc in f_curve:      
                                        if fc.data_path:
                    
                                            if event:
                          
                                                for m in fc.modifiers:
                                                   if (m.type == 'CYCLES'):
                                                       fc.modifiers.remove(m)
                                                fc.modifiers.new(type='CYCLES')
                                
                                            if event.alt:
                                                
                                                for m in fc.modifiers:
                                                   if (m.type == 'CYCLES'):
                                                       fc.modifiers.remove(m)                  
                                            
                                        
                                        fc.update()   

             


        def animated_material_infinity_cycle():
            sel_ob = bpy.context.selected_objects
           
            for o in sel_ob:
                if o.type in mat_object_types:

                    for n in o.data.materials:
                        mat_name = n.name
                        if n.node_tree:
                            if n.node_tree.animation_data:
                                if n.node_tree.animation_data.action:
                                    if n.node_tree.animation_data.action.fcurves:
                                        fn_curve = n.node_tree.animation_data.action.fcurves
             
                                        for fc in fn_curve:       
                                            if fc.data_path:
                     
                                                if event:
                              
                                                    for m in fc.modifiers:
                                                       if (m.type == 'CYCLES'):
                                                           fc.modifiers.remove(m)
                                                    fc.modifiers.new(type='CYCLES')
                                    
                                                if event.alt:
                                                    
                                                    for m in fc.modifiers:
                                                       if (m.type == 'CYCLES'):
                                                           fc.modifiers.remove(m)                  
                                                                                    
                                       
                                            fc.update()                       
                                                
                                                                                  

        for o in sel_ob:
            if o.type in mat_object_types:
                animated_material_infinity_cycle()
                animated_mat_diff_color_infinity_cycle()



        #=======================Materials============




        #======================= Curves ============

        curves = bpy.data.curves
            
        def curve_bevel_depth_cycle():

            for o in sel_ob:
                bpy.context.view_layer.objects.active = o
                o_name = o.name

                for c in curves:
                    if c.name == o_name:

                        if c.animation_data:
                            if c.animation_data.action:
                                if c.animation_data.action.fcurves:

                                    c_fcurve = c.animation_data.action.fcurves
                                    for fc in c_fcurve:
                                        d_path = ('bevel_depth')

                                        if fc.data_path.endswith((d_path)):

                                            if event:
                                 
                                                for m in fc.modifiers:
                                                   if (m.type == 'CYCLES'):
                                                       fc.modifiers.remove(m)
                                                fc.modifiers.new(type='CYCLES')

                                            if event.alt:

                                                for m in fc.modifiers:
                                                   if (m.type == 'CYCLES'):
                                                       fc.modifiers.remove(m)

                                        
                                        fc.update()               


        for o in sel_ob:
            if o.type == 'CURVE':
                curve_bevel_depth_cycle()

        #======================= Curves ============




        return {'FINISHED'}


#==============================================
# END - ADD CYCLES MODIFIER - PRE POST INFINITY
#==============================================




classes = [OBJECT_OT_add_remove_infinity_cycle_anbx]




def register():
    for cls in classes:
        bpy.utils.register_class(cls)




def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)



if __name__ == "__main__":
    register()





