import bpy
import random

from bpy.props import (
                       FloatVectorProperty,
                       BoolProperty,
                       IntProperty,
                       FloatProperty,
                       )




#=====================================================
#                Light Color Random
#=====================================================



class OBJECT_OT_light_color_random_anbx(bpy.types.Operator):
    '''Light Color Random'''

    bl_idname = "object.light_color_random_anbx"
    bl_label = "Light Color Random"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) != 0
                and context.object.type == 'LIGHT')


    def execute(self, context):

        sel_ob = bpy.context.selected_objects
        act_ob = bpy.context.active_object
        cur_frame = bpy.context.scene.frame_current 
         
        l_color = act_ob.data.color

        # location -> material_ax.py
        add_keyframe = context.scene.mat_light_add_keyframe_anbx

        for o in sel_ob:
            if o.type == 'LIGHT':
                bpy.context.view_layer.objects.active = o
                o.data.color = l_color
                
                o.data.color[0] = random.random()
                o.data.color[1] = random.random()
                o.data.color[2] = random.random()

                # if bool 'add keyframe'
                if add_keyframe:                
                
                    o.data.keyframe_insert(data_path="color", frame=cur_frame)
             


        return {'FINISHED'}
        


#=====================================================
#              End  Light Color Random
#=====================================================




# Light Intensity
def light_intensity_fun(self, context):
    sel_ob = bpy.context.selected_objects
    cur_frame = bpy.context.scene.frame_current

    # location -> material_mdh.py
    add_keyframe = context.scene.mat_light_add_keyframe_anbx    

    for o in sel_ob:
        if o.type == 'LIGHT':
            bpy.context.view_layer.objects.active = o
            o.data.energy = self.light_intensity_anbx

            # if bool 'add keyframe'
            if add_keyframe:
                o.data.keyframe_insert(data_path="energy", frame=cur_frame)


# Light Color
def light_color_update_fun(self, context):
    sel_ob = bpy.context.selected_objects
    cur_frame = bpy.context.scene.frame_current

    # location -> material_mdh.py
    add_keyframe = context.scene.mat_light_add_keyframe_anbx       

    for o in sel_ob:
        if o.type == 'LIGHT':
            bpy.context.view_layer.objects.active = o
            o.data.color = self.light_color_anbx

            # if bool 'add keyframe'
            if add_keyframe:            
                o.data.keyframe_insert(data_path="color", frame=cur_frame)




   


classes = [OBJECT_OT_light_color_random_anbx]




def register():
    for cls in classes:
        bpy.utils.register_class(cls)



    bpy.types.Scene.light_intensity_anbx = bpy.props.FloatProperty(
        name="Light Power",
        default=100.0,
        description="Light Intensity",
        min=0,
        update = light_intensity_fun
    )



    bpy.types.Scene.light_color_anbx  = bpy.props.FloatVectorProperty(
            name="Light Color",
            subtype='COLOR',
            default=(1.0, 1.0, 1.0),
            size=3,
            min=0.0, max=1.0,
            description="Selected Light(s)",
            update = light_color_update_fun
            )




def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)

    del bpy.types.Scene.light_intensity_anbx
    del bpy.types.Scene.light_color_anbx


if __name__ == "__main__":
    register()





 
