import bpy
 


#========================================================================
# Add Parent Empty to Selected ("Origin" Panel)
#========================================================================

class OBJECT_OT_add_parent_empty_2selected_anbx(bpy.types.Operator):
    '''Add Parent Empty to Selected'''

    bl_idname = "object.add_parent_empty_2selected_anbx"
    bl_label = "Parent Empty"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) != 0)

    def execute(self, context):
 
        for o in bpy.context.selected_objects:    

            empty = bpy.data.objects.new("Parent_Empty_" + o.name, None )
            o.empty_display_type = 'PLAIN_AXES'

            o.users_collection[-1].objects.link(empty)
            
            empty.location = o.location
            o.location = (0, 0, 0)
            
            display_size = o.dimensions.z
            if display_size < 1:
                empty.empty_display_size = 1
            
            elif display_size == 1:
                empty.empty_display_size = 1.2
                
            elif display_size > 1:
                empty.empty_display_size = 1 + (display_size - 1)
                
            o.parent = empty

            empty.select_set(1)
            o.select_set(0)

            bpy.context.view_layer.objects.active = empty



        return {'FINISHED'}


#------------------------------------------------------------------------



#========================================================================
# Add Child Empty to Selected ("Origin" Panel)
#========================================================================

class OBJECT_OT_add_child_empty_2selected_anbx(bpy.types.Operator):
    '''Add Child Empty to Selected'''

    bl_idname = "object.add_child_empty_2selected_anbx"
    bl_label = "Child Empty"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) != 0)

    def execute(self, context):
 


        for o in bpy.context.selected_objects:
            bpy.context.view_layer.objects.active = o


            empty = bpy.data.objects.new("Child_Empty_" + o.name, None )
            o.empty_display_type = 'PLAIN_AXES'

            o.users_collection[-1].objects.link(empty)

            empty.location = o.location
            empty.location = (0, 0, 0)

            # Size of Empty
            display_size = o.dimensions.z
            if display_size < 1:
                empty.empty_display_size = 1

            elif display_size == 1:
                empty.empty_display_size = 1.2

            elif display_size > 1:
                empty.empty_display_size = 1 + (display_size - 1)


            empty.parent = o

            empty.select_set(1)
            o.select_set(0)
            for o in bpy.context.selected_objects:
                bpy.context.view_layer.objects.active = o



        return {'FINISHED'}




#========================================================================
#  Shared Parent Empty  ("Origin" Panel)
#========================================================================

class OBJECT_OT_add_parent_empty_shared_anbx(bpy.types.Operator):
    '''Add Parent Empty to multiple objects'''

    bl_idname = "object.add_parent_empty_shared_anbx"
    bl_label = "Parent Empty Shared"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and 
                len(context.selected_objects) != 0)

    def execute(self, context):
 
        bpy.ops.view3d.snap_cursor_to_selected()
        cursor = bpy.context.scene.cursor
        cursor_location = bpy.context.scene.cursor.location 
 
        act_col = bpy.context.view_layer.active_layer_collection
         
        empty = bpy.data.objects.new("Parent_Empty_Shared" , None )
        empty.empty_display_type = 'PLAIN_AXES'   

        act_col.collection.objects.link(empty)
        
        empty.location = cursor_location    
        
        bpy.context.view_layer.objects.active = empty
        
        bpy.ops.object.parent_set(type='OBJECT', keep_transform=False)

        for o in bpy.context.selected_objects:    

            empty.select_set(1)
            o.select_set(0) 


        return {'FINISHED'}



#========================================================================
#  Shared Parent Empty at Cursor ("Origin" Panel)
#========================================================================

class OBJECT_OT_add_parent_empty_shared_at_cursor_anbx(bpy.types.Operator):
    '''Add Parent Empty to multiple objects - at Cursor'''

    bl_idname = "object.add_parent_empty_shared_at_cursor_anbx"
    bl_label = "Parent Empty Shared at Cursor"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and 
                len(context.selected_objects) != 0)

    def execute(self, context):
 
        cursor = bpy.context.scene.cursor
        cursor_location = bpy.context.scene.cursor.location 
 
        act_col = bpy.context.view_layer.active_layer_collection
         
        empty = bpy.data.objects.new("Parent_Empty_Shared" , None )
        empty.empty_display_type = 'PLAIN_AXES'   

        act_col.collection.objects.link(empty)
        
        empty.location = cursor_location    
        
        bpy.context.view_layer.objects.active = empty
        
        bpy.ops.object.parent_set(type='OBJECT', keep_transform=False)

 
        for o in bpy.context.selected_objects:    

            empty.select_set(1)
            o.select_set(0) 


        return {'FINISHED'}



#========================================================================
#========================================================================
#========================================================================

#========================================================================
# Drop to Floor  ("Origin" Panel)
#========================================================================

class OBJECT_OT_drop_to_floor_anbx(bpy.types.Operator):
    '''Drop Selected Object/s (Mesh) to Floor'''

    bl_idname = "object.drop_to_floor_anbx"
    bl_label = "Drop to Floor"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and 
                context.object.type == 'MESH' and
                len(context.selected_objects) != 0)

    def execute(self, context):
 
        
        sel_ob = bpy.context.selected_objects

        for obj in sel_ob:
            mx = obj.matrix_world
            minz = min((mx @ v.co)[2] for v in obj.data.vertices)
            mx.translation.z -= minz


        return {'FINISHED'}




#========================================================================
# Copy Selected at 3D Cursor  ( "Duplication" Panel)
#========================================================================

class OBJECT_OT_copy_selected_at_cursor_anbx(bpy.types.Operator):
    '''Copy Any Selected Object at 3D Cursor'''

    bl_idname = "object.copy_selected_at_cursor_anbx"
    bl_label = "Copy Sel to Cursor"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT')



    def execute(self, context):

        ob = bpy.context.object
        sel_ob = bpy.context.selected_objects

        cursor = bpy.context.scene.cursor
        cursor_location = bpy.context.scene.cursor.location

        act_col = bpy.context.view_layer.active_layer_collection


        def new_ob(): 
 
            orig_ob = bpy.data.objects[ob.name]
            dupl_ob = bpy.data.objects.new(ob.name, ob.data)
            

            act_col.collection.objects.link(dupl_ob)
            dupl_ob.location = cursor.location
            
            bpy.ops.object.make_single_user(object=False, obdata=True, material=True, animation=True)
            

        for o in sel_ob: 
          
            new_ob()
            
            o.data.name = o.name
            

        return {'FINISHED'}


# End  Copy Selected at 3D Cursor
#==================================================================================












classes = [ OBJECT_OT_add_parent_empty_2selected_anbx,
            OBJECT_OT_add_child_empty_2selected_anbx,
            OBJECT_OT_add_parent_empty_shared_anbx,
            OBJECT_OT_add_parent_empty_shared_at_cursor_anbx,
            OBJECT_OT_drop_to_floor_anbx,
            OBJECT_OT_copy_selected_at_cursor_anbx,

            
          ]
 


def register():
    for cls in classes:
        bpy.utils.register_class(cls)
        

 
def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)



if __name__ == "__main__":
    register()




