import bpy
import random
import string
from bpy.props import (IntProperty, BoolProperty)



# =====================================================
#                      Obj2Curve
# =====================================================

# Duplicate Selected Object along Selected Curve + Follow Path Animation
class OBJECT_OT_object_to_curve_anim_anbx(bpy.types.Operator):
    '''Duplicate Selected Object along Selected Curve -
- Follow Path Animation'''
    bl_idname = "object.object_to_curve_anim_anbx"
    bl_label = "Obj2Curve + Animate"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) > 1)


    def execute(self, context):
  
        ob = bpy.context.object
        sel_ob = bpy.context.selected_objects
        act_ob = bpy.context.active_object

        cursor = bpy.context.scene.cursor
        cursor_location = bpy.context.scene.cursor.location
        cursor_previous_location = cursor.location.copy()



        # Ref to Property -------------------
        dup_obj2curve = context.scene.dupl_object2curve_count
           
        path_anim_speed = context.scene.curve_frames_anim_speed
        #------------------------------------

        for c in bpy.data.objects:
            if c in sel_ob:
                if c.type == 'CURVE':
                    curve_name = c.name
                    
        path_curve = bpy.data.objects[curve_name]
        path_curve.data.path_duration = path_anim_speed
          
          
        sel_obj = bpy.context.selected_objects
        act_obj = bpy.context.view_layer.objects

        for o in bpy.context.selected_objects:
            if o == act_ob and o.type == 'CURVE':
                o.select_set(0)

                if act_obj.active == sel_obj[0]:
                    act_obj.active = sel_obj[-1]

                elif act_obj.active == sel_obj[-1]:
                    act_obj.active = sel_obj[0]

            elif o != act_ob and o.type == 'CURVE':
                o.select_set(0)            

        ob = bpy.context.view_layer.objects.active 
        ob.select_set(True)        
            
        # Placement
        bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='MEDIAN')
        bpy.ops.view3d.snap_cursor_to_center()
        bpy.ops.view3d.snap_selected_to_cursor(use_offset=False)
     
     

        def random_name(length):

            ob = bpy.context.object

            source = string.ascii_lowercase + string.digits
            result_str = ''.join((random.choice(source) for i in range(length)))


            name_spl = ob.name[:3]
            rand_name = ( '_' + result_str + '_')

            ob.name = name_spl + rand_name
            if ob.data:
                ob.data.name = ob.name
                
        random_name(6)

         
        bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)

        # Collection
        act_col = bpy.context.view_layer.active_layer_collection


        # Reference to "Object Axis Follow Curve"  Bool
        follow_axis_bool = context.scene.obj_axis_x_y_z_follow_curve


        def new_ob_main(range_num):

            for num in range(range_num):
                orig_ob = bpy.data.objects[ob.name]
                dupl_ob = bpy.data.objects.new(ob.name, ob.data)

                act_col.collection.objects.link(dupl_ob)
                dupl_ob.location = orig_ob.location
                
                dupl_ob.select_set(1)
                bpy.context.scene.tool_settings.transform_pivot_point = 'INDIVIDUAL_ORIGINS'
                

        new_ob_main(dup_obj2curve - 1)

        ## Make Single User
        bpy.ops.object.make_single_user(object=False, obdata=True, material=True, animation=True)

        # Name
        for o in bpy.context.selected_objects:
            if o.data:
                o.name = o.name 
                o.data.name = o.name 
           
        offset = 0
        for o in bpy.context.selected_objects:
            bpy.context.view_layer.objects.active = o
            o.constraints.new(type='FOLLOW_PATH')
            o.constraints[-1].name = 'Follow Path'
            o.constraints["Follow Path"].target = path_curve

            o.constraints.active.offset += offset

            offset += (path_anim_speed/ (dup_obj2curve))


            o.constraints["Follow Path"].use_curve_follow = True
            
            if follow_axis_bool:
                # Follow Axis
                dim = o.dimensions
                if dim.x > dim.y and dim.x > dim.z:
                    o.constraints["Follow Path"].up_axis = 'UP_Z'
                    o.constraints["Follow Path"].forward_axis = 'FORWARD_X' 
                                
                if dim.y > dim.x and dim.y > dim.z: 
                    o.constraints["Follow Path"].up_axis = 'UP_Z'
                    o.constraints["Follow Path"].forward_axis = 'FORWARD_Y' 
                        
                if dim.z > dim.x and dim.z > dim.y:                     
                    o.constraints["Follow Path"].up_axis = 'UP_X'
                    o.constraints["Follow Path"].forward_axis = 'FORWARD_Z' 
                        
                     

 
        bpy.ops.constraint.followpath_path_animate(constraint="Follow Path", owner='OBJECT')
          

        # Cursor back
        cursor.location = cursor_previous_location
           

        return {'FINISHED'}


# =====================================================
#             End  Obj2Curve
# =====================================================




# =====================================================
#             Curve Direction Switch
# =====================================================
   

# Switch Direction/Reverse Curve
class OBJECT_OT_switch_curve_direction_anbx(bpy.types.Operator):
    '''Switch Direction - Reverse Curve'''
    bl_idname = "object.switch_curve_direction_anbx"
    bl_label = "Curve - Switch Direction "
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and

                len(context.selected_objects) != 0
                and context.object.type == 'CURVE')


    def execute(self, context):

        if bpy.context.mode == 'OBJECT':
            bpy.ops.object.mode_set(mode='EDIT', toggle=False)  
        
        
        bpy.ops.curve.select_all(action='SELECT')
        bpy.ops.curve.switch_direction()

        bpy.ops.object.mode_set(mode='OBJECT', toggle=False)

        return {'FINISHED'}

# =====================================================
#            End Curve Direction Switch
# =====================================================
   




classes = [ OBJECT_OT_object_to_curve_anim_anbx,
            OBJECT_OT_switch_curve_direction_anbx,

            
          ]
 
 
 
def register():
    for cls in classes:
        bpy.utils.register_class(cls)
        

    bpy.types.Scene.dupl_object2curve_count = bpy.props.IntProperty(
        name="",
        default=3,
        description="Objects Count",
        min=0,
    )


    bpy.types.Scene.curve_frames_anim_speed = bpy.props.IntProperty(
        name="",
        default=30,
        description="Lower Value = Higher Speed",
        min=0,
    )



    bpy.types.Scene.obj_axis_x_y_z_follow_curve = bpy.props.BoolProperty(
        name="",
        description="Object Axis Follow Curve",
        default = False
    )



 
def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)

    del bpy.types.Scene.dupl_object2curve_count
    del bpy.types.Scene.curve_frames_anim_speed
    del bpy.types.Scene.obj_axis_x_y_z_follow_curve
 
if __name__ == "__main__":
    register() 