import bpy
from bpy.props import (BoolProperty, FloatProperty, IntProperty)




#==========================================================
#                Offset Shape Key Animation
#==========================================================



class OBJECT_OT_offset_shape_key_anbx(bpy.types.Operator):
    '''Offset Animated Shape Keys'''
    bl_idname = "object.offset_shape_key_anbx"
    bl_label = "Offset Shape Keys"
    bl_options = {'REGISTER', 'UNDO'}

    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) != 0)


    def execute(self, context):

        sel_ob = bpy.context.selected_objects
        act_ob = bpy.context.active_object

        # Ref 
        rev_bool = context.scene.revers_anim_bool_mdh
        
        frames_to_offset = context.scene.offset_frames_number_anbx
#-------------------------------------------

        # Shapekey Blocks Offset
        def offset_shape_key_blocks(): 
            sel_ob = bpy.context.selected_objects

            # Offset Shape_Key Blocks
            blocks_offset = 0
            if rev_bool:
                sel_ob = reversed(sel_ob) 
 
            for o in sel_ob:
                bpy.context.view_layer.objects.active = o
                bpy.ops.object.make_single_user(object=True, obdata=True, material=False, animation=False)
 
                # Offset Shape_Key Blocks
                c_blocks_fcurves = bpy.context.object.data.shape_keys.key_blocks.data.animation_data.action.fcurves
                 
                if rev_bool:
                    c_blocks_fcurves = reversed(c_blocks_fcurves) 
                for c_b in c_blocks_fcurves:
                    block_keyframe_points = c_b.keyframe_points 
         
 
                    # Offset Shape_Key Blocks
                    for kfb in block_keyframe_points:
                        kfb.co[0] += blocks_offset
                        kfb.handle_left[0] += blocks_offset
                        kfb.handle_right[0] += blocks_offset

                    blocks_offset += frames_to_offset

                    
#-------------------------------------------
        # Shapekey Offset
        def offset_shape_key():
            sel_ob = bpy.context.selected_objects
            offset = 0
            if rev_bool:
                sel_ob = reversed(sel_ob)
                    
            for o in sel_ob:
                bpy.context.view_layer.objects.active = o
                bpy.ops.object.make_single_user(object=True, obdata=True, material=False, animation=False)


                for c in bpy.context.object.data.shape_keys.animation_data.action.fcurves:
                    keyframe_points = c.keyframe_points


                    for kf in keyframe_points:
                        kf.co[0] += offset
                        kf.handle_left[0] += offset
                        kf.handle_right[0] += offset

                offset += frames_to_offset
                
#-------------------------------------------

        # Check if Object has Shape_Keys Animation

        if act_ob.type == 'MESH' and act_ob.data.shape_keys:
            sc_blocks = act_ob.data.shape_keys.key_blocks
            act_ob_sk = act_ob.data.shape_keys
            if act_ob_sk.animation_data:
                if act_ob_sk.animation_data.action:
                    if act_ob_sk.animation_data.action.fcurves:
                        if len(sc_blocks)== 2:
                            offset_shape_key()
                        if len(sc_blocks) > 2:
                            offset_shape_key_blocks()



        return {'FINISHED'}


#==========================================================
#             End Offset Shape Key Animation
#==========================================================


#==========================================================
#                Offset Animated Material
#==========================================================



class OBJECT_OT_offset_animated_material_anbx(bpy.types.Operator):
    '''Offset Animated Materials'''
    bl_idname = "object.offset_animated_material_anbx"
    bl_label = "Offset Materials"
    bl_options = {'REGISTER', 'UNDO'}

    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) != 0)


    def execute(self, context):

        sel_ob = bpy.context.selected_objects
        act_ob = bpy.context.active_object

        # Ref 
        rev_bool = context.scene.revers_anim_bool_mdh
        
        frames_to_offset = context.scene.offset_frames_number_anbx
#-------------------------------------------
#--------------------------------------------

        act_ob = bpy.context.active_object
        mat_object_types = ['CURVE', 'MESH', 'FONT', 'SURFACE']

        def offset_animated_mat_diff_color():
            sel_ob = bpy.context.selected_objects
            offset = 0
            if rev_bool:
                sel_ob = reversed(sel_ob)
                       
            for o in sel_ob:
                bpy.ops.object.make_single_user(object=False, obdata=False, material=True, animation=False)

                if o.type in mat_object_types:

                    for m in o.data.materials:
                        mat_name = m.name
                        if m.animation_data:
                            if m.animation_data.action:
                                if m.animation_data.action.fcurves:
                                    f_curve =  bpy.data.materials[mat_name].diffuse_color.data.animation_data.action.fcurves
                                    
                                    for f in f_curve:
                                        keyframe_points = f.keyframe_points

                                        for kf in keyframe_points:
                                            kf.co[0] += offset
                                            kf.handle_left[0] += offset
                                            kf.handle_right[0] += offset
                                            
                    offset += frames_to_offset



        def offset_animated_material():
            sel_ob = bpy.context.selected_objects
            offset = 0
            if rev_bool:
                sel_ob = reversed(sel_ob)
                    
            for o in sel_ob:
                bpy.ops.object.make_single_user(object=False, obdata=False, material=True, animation=False)
                
                if o.type in mat_object_types:
                        
                    for n in o.data.materials:
                        if n.node_tree:
                            if n.node_tree.animation_data:
                                if n.node_tree.animation_data.action:
                                    if n.node_tree.animation_data.action.fcurves:
                                    
                                        for fc in n.node_tree.animation_data.action.fcurves:
                                            keyframe_points = fc.keyframe_points

                                            for kf in keyframe_points:
                                                kf.co[0] += offset
                                                kf.handle_left[0] += offset
                                                kf.handle_right[0] += offset

                    offset += frames_to_offset


        # Set First Material Slot as Active
        for o in bpy.context.selected_objects:
            bpy.context.view_layer.objects.active = o
            bpy.context.object.active_material_index = 0

#--------------------------------------------

        # Check if Object has Material Node Animation

        if act_ob.type == 'MESH' and act_ob.data.materials:

            offset_animated_mat_diff_color()
            offset_animated_material()



        # Using with Text FX
        if act_ob.type == 'FONT' and act_ob.data.materials:

            offset_animated_mat_diff_color()
            offset_animated_material()


        return {'FINISHED'}


#==========================================================
#             End Offset Animated Material
#==========================================================



#==========================================================
#                Offset Animated Lights
#==========================================================


class OBJECT_OT_offset_animated_lights_anbx(bpy.types.Operator):
    '''Offset Animated Lights'''
    bl_idname = "object.offset_animated_lights_anbx"
    bl_label = "Offset Lights"
    bl_options = {'REGISTER', 'UNDO'}

    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) != 0)


    def execute(self, context):

        sel_ob = bpy.context.selected_objects
        act_ob = bpy.context.active_object

        # Ref 
        rev_bool = context.scene.revers_anim_bool_mdh
        
        frames_to_offset = context.scene.offset_frames_number_anbx
#-------------------------------------------

        def offset_light_param():
            sel_ob = bpy.context.selected_objects
            offset = 0
            if rev_bool:
                sel_ob = reversed(sel_ob)
                    
            for o in sel_ob:
                bpy.context.view_layer.objects.active = o

                if o.data.animation_data:
                    if o.data.animation_data.action:
                        if o.data.animation_data.action.fcurves:
                            for c in o.data.animation_data.action.fcurves:
                                keyframePoints = c.keyframe_points


                                for kf in keyframePoints:
                                    kf.co[0] += offset
                                    kf.handle_left[0] += offset
                                    kf.handle_right[0] += offset

                            offset += frames_to_offset


#--------------------------------------------
        # Check if Light has Animation

        if act_ob.type == 'LIGHT':
            offset_light_param()
 

        return {'FINISHED'}

#==========================================================
#             End Offset Animated Lights
#==========================================================

 
#==========================================================
#                Offset Animated Curve Bevel
#==========================================================


class OBJECT_OT_offset_animated_curve_bevel_anbx(bpy.types.Operator):
    '''Offset Animated Curve Bevel'''
    bl_idname = "object.offset_animated_curve_bevel_anbx"
    bl_label = "Offset Curve Bevel"
    bl_options = {'REGISTER', 'UNDO'}

    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) != 0)


    def execute(self, context):

        sel_ob = bpy.context.selected_objects
        act_ob = bpy.context.active_object

        # Ref 
        rev_bool = context.scene.revers_anim_bool_mdh
        
        frames_to_offset = context.scene.offset_frames_number_anbx
#-------------------------------------------


        # Location Rotation Scale Offset
        def offset_loc_rot_scale():
            sel_ob = bpy.context.selected_objects
            offset = 0
            if rev_bool:
                sel_ob = reversed(sel_ob)

            for o in sel_ob:
                anim_data = o.animation_data
                if anim_data:
                    action = anim_data.action
                    if action:   
                        fcurves = action.fcurves
                        if fcurves:
                            for c in fcurves:
                                keyframePoints = c.keyframe_points

                                for kf in keyframePoints:
                                    kf.co[0] += offset
                                    kf.handle_left[0] += offset
                                    kf.handle_right[0] += offset

                offset += frames_to_offset


#-------------------------------------------

        def curve_bevel_depth():
            sel_ob = bpy.context.selected_objects
            offset = 0
            if rev_bool:
                sel_ob = reversed(sel_ob)
                    
            for o in sel_ob:
                bpy.context.view_layer.objects.active = o
                o.data.name = o.name
                if o.type == 'CURVE':
                    curve_name = o.name 
                    curve = bpy.data.curves[curve_name]
                    for c in curve.animation_data.action.fcurves:
                        keyframe_points = c.keyframe_points

                        for kf in keyframe_points:
                            kf.co[0] += offset
                            kf.handle_left[0] += offset
                            kf.handle_right[0] += offset

                offset += frames_to_offset

#--------------------------------------------

        # Check if Object has Loc_Rot_Scale Animation
        anim_data = act_ob.animation_data

        if anim_data:
            if anim_data.action:
                if anim_data.action.fcurves:
                    offset_loc_rot_scale()


#--------------------------------------------
 
        # Check for Curves

        if act_ob.type == 'CURVE':
            try:
                curve_bevel_depth()
         
            except AttributeError :
                pass 
#--------------------------------------------
 

        return {'FINISHED'}

#==========================================================
#             End Offset Animated Curve Bevel
#==========================================================





#==========================================================
#                   Smart Offset
#==========================================================

class OBJECT_OT_offset_smart_anbx(bpy.types.Operator):
    '''Offset Animated :
- Location, Rotation, Scale.
- Light - Color / Intensity.
- Material - Diffuse / Emission Color.
- Curve Bevel.
- Shape Keys.'''

    bl_idname = "object.offset_smart_anbx"
    bl_label = "Offset Animation"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) != 0)



    def execute(self, context):

        sel_ob = bpy.context.selected_objects
        act_ob = bpy.context.active_object

        # Ref. to Props
        rev_bool = context.scene.revers_anim_bool_mdh
        
        frames_to_offset = context.scene.offset_frames_number_anbx
#-------------------------------------------


        # Shapekey Blocks Offset
        def offset_shape_key_blocks(): 
            sel_ob = bpy.context.selected_objects

            # Offset Shape_Key Blocks
            blocks_offset = 0
            if rev_bool:
                sel_ob = reversed(sel_ob) 
 
            for o in sel_ob:
                bpy.context.view_layer.objects.active = o
                bpy.ops.object.make_single_user(object=True, obdata=True, material=False, animation=False)
 
                # Offset Shape_Key Blocks
                c_blocks_fcurves = bpy.context.object.data.shape_keys.key_blocks.data.animation_data.action.fcurves
                 
                if rev_bool:
                    c_blocks_fcurves = reversed(c_blocks_fcurves) 
                for c_b in c_blocks_fcurves:
                    block_keyframe_points = c_b.keyframe_points 
         
 
                    # Offset Shape_Key Blocks
                    for kfb in block_keyframe_points:
                        kfb.co[0] += blocks_offset
                        kfb.handle_left[0] += blocks_offset
                        kfb.handle_right[0] += blocks_offset

                    blocks_offset += frames_to_offset

                    

#-------------------------------------------
        # Shapekey Offset
        def offset_shape_key():
            sel_ob = bpy.context.selected_objects
            offset = 0
            if rev_bool:
                sel_ob = reversed(sel_ob)
                    
            for o in sel_ob:
                bpy.context.view_layer.objects.active = o
                bpy.ops.object.make_single_user(object=True, obdata=True, material=False, animation=False)


                for c in bpy.context.object.data.shape_keys.animation_data.action.fcurves:
                    keyframe_points = c.keyframe_points


                    for kf in keyframe_points:
                        kf.co[0] += offset
                        kf.handle_left[0] += offset
                        kf.handle_right[0] += offset

                offset += frames_to_offset
                
#-------------------------------------------


        # Location Rotation Scale Offset
        def offset_loc_rot_scale():
            sel_ob = bpy.context.selected_objects
            offset = 0
            if rev_bool:
                sel_ob = reversed(sel_ob)

            for o in sel_ob:
                anim_data = o.animation_data
                if anim_data:
                    action = anim_data.action
                    if action:   
                        fcurves = action.fcurves
                        if fcurves:
                            for c in fcurves:
                                keyframePoints = c.keyframe_points

                                for kf in keyframePoints:
                                    kf.co[0] += offset
                                    kf.handle_left[0] += offset
                                    kf.handle_right[0] += offset

                offset += frames_to_offset


#--------------------------------------------

        act_ob = bpy.context.active_object
        mat_object_types = ['CURVE', 'MESH', 'FONT', 'SURFACE']

        def offset_animated_mat_diff_color():
            sel_ob = bpy.context.selected_objects
            offset = 0
            if rev_bool:
                sel_ob = reversed(sel_ob)
                       
            for o in sel_ob:
                bpy.ops.object.make_single_user(object=False, obdata=False, material=True, animation=False)

                if o.type in mat_object_types:

                    for m in o.data.materials:
                        mat_name = m.name
                        if m.animation_data:
                            if m.animation_data.action:
                                if m.animation_data.action.fcurves:
                                    f_curve =  bpy.data.materials[mat_name].diffuse_color.data.animation_data.action.fcurves
                                    
                                    for f in f_curve:
                                        keyframe_points = f.keyframe_points

                                        for kf in keyframe_points:
                                            kf.co[0] += offset
                                            kf.handle_left[0] += offset
                                            kf.handle_right[0] += offset
                                            
                    offset += frames_to_offset



        def offset_animated_material():
            sel_ob = bpy.context.selected_objects
            offset = 0
            if rev_bool:
                sel_ob = reversed(sel_ob)
                    
            for o in sel_ob:
                bpy.ops.object.make_single_user(object=False, obdata=False, material=True, animation=False)
                
                if o.type in mat_object_types:
                        
                    for n in o.data.materials:
                        if n.node_tree:
                            if n.node_tree.animation_data:
                                if n.node_tree.animation_data.action:
                                    if n.node_tree.animation_data.action.fcurves:
                                    
                                        for fc in n.node_tree.animation_data.action.fcurves:
                                            keyframe_points = fc.keyframe_points

                                            for kf in keyframe_points:
                                                kf.co[0] += offset
                                                kf.handle_left[0] += offset
                                                kf.handle_right[0] += offset

                    offset += frames_to_offset



        # Set First Material Slot as Active
        for o in bpy.context.selected_objects:
            bpy.context.view_layer.objects.active = o
            bpy.context.object.active_material_index = 0


#--------------------------------------------

        def offset_light_param():
            sel_ob = bpy.context.selected_objects
            offset = 0
            if rev_bool:
                sel_ob = reversed(sel_ob)
                    
            for o in sel_ob:
                bpy.context.view_layer.objects.active = o

                if o.data.animation_data:
                    if o.data.animation_data.action:
                        if o.data.animation_data.action.fcurves:
                            for c in o.data.animation_data.action.fcurves:
                                keyframePoints = c.keyframe_points


                                for kf in keyframePoints:
                                    kf.co[0] += offset
                                    kf.handle_left[0] += offset
                                    kf.handle_right[0] += offset

                            offset += frames_to_offset

#--------------------------------------------


        def curve_bevel_depth():
            sel_ob = bpy.context.selected_objects
            offset = 0
            if rev_bool:
                sel_ob = reversed(sel_ob)
                    
            for o in sel_ob:
                bpy.context.view_layer.objects.active = o
                o.data.name = o.name
                if o.type == 'CURVE':
                    curve_name = o.name 
                    curve = bpy.data.curves[curve_name]
                    for c in curve.animation_data.action.fcurves:
                        keyframe_points = c.keyframe_points

                        for kf in keyframe_points:
                            kf.co[0] += offset
                            kf.handle_left[0] += offset
                            kf.handle_right[0] += offset

                offset += frames_to_offset

#--------------------------------------------


        # Check if Object has Loc_Rot_Scale Animation
        anim_data = act_ob.animation_data

        if anim_data:
            if anim_data.action:
                if anim_data.action.fcurves:
                    offset_loc_rot_scale()


#--------------------------------------------
        # Check if Object has Shape_Keys Animation

        if act_ob.type == 'MESH' and act_ob.data.shape_keys:
            sc_blocks = act_ob.data.shape_keys.key_blocks
            act_ob_sk = act_ob.data.shape_keys
            if act_ob_sk.animation_data:
                if act_ob_sk.animation_data.action:
                    if act_ob_sk.animation_data.action.fcurves:
                        if len(sc_blocks)== 2:
                            offset_shape_key()
                        if len(sc_blocks) > 2:
                            offset_shape_key_blocks()



#--------------------------------------------
        # Check if Object has Material Node Animation

        if act_ob.type == 'MESH' and act_ob.data.materials:

            offset_animated_mat_diff_color()
            offset_animated_material()



        # Using with Text FX
        if act_ob.type == 'FONT' and act_ob.data.materials:

            offset_animated_mat_diff_color()
            offset_animated_material()

                        
#--------------------------------------------
        # Check if Light has Animation


        if act_ob.type == 'LIGHT':
            offset_light_param()
            offset_loc_rot_scale()


#--------------------------------------------
        # Check for Curves

        if act_ob.type == 'CURVE':
            try:
                curve_bevel_depth()
         
            except AttributeError :
                pass 
#--------------------------------------------

    


        return {'FINISHED'}




#==========================================================
#                End  Smart Offset
#==========================================================




classes = [OBJECT_OT_offset_shape_key_anbx,
           OBJECT_OT_offset_animated_material_anbx,
           OBJECT_OT_offset_animated_lights_anbx,
           OBJECT_OT_offset_animated_curve_bevel_anbx,
           OBJECT_OT_offset_smart_anbx,

           ]




def register():
    for cls in classes:
        bpy.utils.register_class(cls)


    bpy.types.Scene.revers_anim_bool_mdh = bpy.props.BoolProperty(
        name="",
        description="REVERSED OFFSET",
        default = False
    )

 
    bpy.types.Scene.offset_frames_number_anbx = bpy.props.IntProperty(
        name="Frames",
        default=1,
        description="Frames to Offset",
        min=1,
    )




def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)

    del bpy.types.Scene.revers_anim_bool_mdh
    del bpy.types.Scene.offset_frames_number_anbx



if __name__ == "__main__":
    register()




