import bpy
from bpy.props import (BoolProperty, FloatProperty, IntProperty)



#==========================================================
#                   Loc Rot Scale Offset
#==========================================================



class OBJECT_OT_offset_loc_rot_scale_anbx(bpy.types.Operator):
    '''Offset Animated - Location, Rotation, Scale'''
    bl_idname = "object.offset_loc_rot_scale_anbx"
    bl_label = "Offset Loc Rot Scale"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) != 0)



    def execute(self, context):

        sel_ob = bpy.context.selected_objects
        act_ob = bpy.context.active_object

        # Ref 
        rev_bool = context.scene.revers_anim_bool_mdh
        
        frames_to_offset = context.scene.offset_frames_number_anbx
#-------------------------------------------


        # Location Rotation Scale Offset
        def offset_loc_rot_scale():
            sel_ob = bpy.context.selected_objects
            offset = 0
            if rev_bool:
                sel_ob = reversed(sel_ob)

            for o in sel_ob:
                anim_data = o.animation_data
                if anim_data:
                    action = anim_data.action
                    if action:   
                        fcurves = action.fcurves
                        if fcurves:
                            for c in fcurves:
                                keyframePoints = c.keyframe_points

                                for kf in keyframePoints:
                                    kf.co[0] += offset
                                    kf.handle_left[0] += offset
                                    kf.handle_right[0] += offset

                offset += frames_to_offset


#--------------------------------------------

        # Check if Object has Anim Data
        anim_data = act_ob.animation_data

        if anim_data:
            if anim_data.action:
                if anim_data.action.fcurves:
                     offset_loc_rot_scale()


        return {'FINISHED'}


#==========================================================
#                  End Loc Rot Scale Offset
#==========================================================




#==========================================================
#                   Location Offset
#==========================================================



class OBJECT_OT_offset_location_anbx(bpy.types.Operator):
    '''Offset Animated - Location'''
    bl_idname = "object.offset_location_anbx"
    bl_label = "Offset Animated Location"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) != 0)



    def execute(self, context):


        sel_ob = bpy.context.selected_objects
        act_ob = bpy.context.active_object

        # Ref 
        rev_bool = context.scene.revers_anim_bool_mdh
        
        frames_to_offset = context.scene.offset_frames_number_anbx
#-------------------------------------------



        # Location Offset
        def offset_loc():
            sel_ob = bpy.context.selected_objects
            offset = 0
            if rev_bool:
                sel_ob = reversed(sel_ob)

            for o in sel_ob:
                anim_data = o.animation_data
                if anim_data:
                    action = anim_data.action
                    if action:
                        fcurves = action.fcurves
                        if fcurves:
                            for c in fcurves:
                                if c.data_path.endswith(('location')):
                                    keyframePoints = c.keyframe_points

                                    for kf in keyframePoints:
                                        kf.co[0] += offset
                                        kf.handle_left[0] += offset
                                        kf.handle_right[0] += offset

                offset += frames_to_offset

 
#--------------------------------------------


        # Check if Object  Anim Data
        anim_data = act_ob.animation_data

        if anim_data:
            if anim_data.action:
                if anim_data.action.fcurves:
                    offset_loc()


        return {'FINISHED'}


#==========================================================
#                  End Location Offset
#==========================================================



#==========================================================
#                   Rotation Offset
#==========================================================

class OBJECT_OT_offset_rotation_anbx(bpy.types.Operator):
    '''Offset Animated - Rotation'''
    bl_idname = "object.offset_rotation_anbx"
    bl_label = "Offset Animated Rotation"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) != 0)



    def execute(self, context):


        sel_ob = bpy.context.selected_objects
        act_ob = bpy.context.active_object

        # Ref 
        rev_bool = context.scene.revers_anim_bool_mdh
        
        frames_to_offset = context.scene.offset_frames_number_anbx
#-------------------------------------------



        # Rotation Offset
        def offset_rot():
            sel_ob = bpy.context.selected_objects
            offset = 0
            if rev_bool:
                sel_ob = reversed(sel_ob)

            for o in sel_ob:
                anim_data = o.animation_data
                if anim_data:
                    action = anim_data.action
                    if action:
                        fcurves = action.fcurves
                        if fcurves:
                            for c in fcurves:
                                if c.data_path.endswith(('rotation_euler')):
                                    keyframePoints = c.keyframe_points

                                    for kf in keyframePoints:
                                        kf.co[0] += offset
                                        kf.handle_left[0] += offset
                                        kf.handle_right[0] += offset

                offset += frames_to_offset

 
#--------------------------------------------


        # Check if Object  Anim Data
        anim_data = act_ob.animation_data

        if anim_data:
            if anim_data.action:
                if anim_data.action.fcurves:
                    offset_rot()


        return {'FINISHED'}


#==========================================================
#                  End Rotation Offset
#==========================================================





#==========================================================
#                   Scale Offset
#==========================================================

class OBJECT_OT_offset_scale_anbx(bpy.types.Operator):
    '''Offset Animated - Scale'''
    bl_idname = "object.offset_scale_anbx"
    bl_label = "Offset Animated Scale"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) != 0)



    def execute(self, context):


        sel_ob = bpy.context.selected_objects
        act_ob = bpy.context.active_object

        # Ref 
        rev_bool = context.scene.revers_anim_bool_mdh
        
        frames_to_offset = context.scene.offset_frames_number_anbx
#-------------------------------------------



        # Scale Offset
        def offset_scale():
            sel_ob = bpy.context.selected_objects
            offset = 0
            if rev_bool:
                sel_ob = reversed(sel_ob)

            for o in sel_ob:
                anim_data = o.animation_data
                if anim_data:
                    action = anim_data.action
                    if action:
                        fcurves = action.fcurves
                        if fcurves:
                            for c in fcurves:
                                if c.data_path.endswith(('scale')):
                                    keyframePoints = c.keyframe_points

                                    for kf in keyframePoints:
                                        kf.co[0] += offset
                                        kf.handle_left[0] += offset
                                        kf.handle_right[0] += offset

                offset += frames_to_offset

 
#--------------------------------------------


        # Check if Object  Anim Data
        anim_data = act_ob.animation_data

        if anim_data:
            if anim_data.action:
                if anim_data.action.fcurves:
                    offset_scale()


        return {'FINISHED'}


#==========================================================
#                  End Scale Offset
#==========================================================






classes = [ OBJECT_OT_offset_loc_rot_scale_anbx,
            OBJECT_OT_offset_location_anbx,
            OBJECT_OT_offset_rotation_anbx,
            OBJECT_OT_offset_scale_anbx,


          ]





def register():
    for cls in classes:
        bpy.utils.register_class(cls)




def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)



if __name__ == "__main__":
    register()



