import bpy
from bpy.props import (BoolProperty, FloatProperty, IntProperty)




#================ Offset Transformation Location  ==================
 
class OBJECT_OT_offset_transform_location_anbx(bpy.types.Operator):
    '''Offset Location 
-Selected Object(s)
-Alt Click = Insert Location Key(s)'''

    bl_idname = "object.offset_transform_location_anbx"
    bl_label = "Offset Location"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                len(context.selected_objects) > 1)


    def invoke(self, context, event):
  
        # Ref 
        x_axis = context.scene.offset_transform_axis_x_anbx        
        y_axis = context.scene.offset_transform_axis_y_anbx        
        z_axis = context.scene.offset_transform_axis_z_anbx        
        
        
        neg_axis = context.scene.offset_transform_neg_axis_anbx       
        
        offset_amount = context.scene.offset_amount_anbx
        rev_trans_offset = context.scene.revers_transform_offset_anbx
                
                
        def offset_selected_axis():
            sel_ob = bpy.context.selected_objects
            offset = 0

            if rev_trans_offset:
                sel_ob = reversed(sel_ob)
                
            for o in sel_ob:
 
                
                if x_axis:
                    o.location.x += offset
                    
                    
                if y_axis:
                    o.location.y += offset
                    
                    
                if z_axis:
                    o.location.z += offset
                                        
 
                
                if neg_axis:
                    offset -= offset_amount
                else:
                    offset += offset_amount
                    
                
                if event.alt:
                    o.keyframe_insert(data_path = 'location')
                    
                                     
        offset_selected_axis()
        

        return {'FINISHED'}

#================ Offset Transformation Location ==================







#================ Offset Transformation Rotation  ==================

class OBJECT_OT_offset_transform_rotation_anbx(bpy.types.Operator):
    '''Offset Rotation
-Selected Object(s)
-Alt Click = Insert Rotation Key(s)'''

    bl_idname = "object.offset_transform_rotation_anbx"
    bl_label = "Offset Rotation"
    bl_options = {'REGISTER', 'UNDO'}
    


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                len(context.selected_objects) > 1)


    def invoke(self, context, event):

        # Ref 
        x_axis = context.scene.offset_transform_axis_x_anbx        
        y_axis = context.scene.offset_transform_axis_y_anbx        
        z_axis = context.scene.offset_transform_axis_z_anbx        
        
        
        neg_axis = context.scene.offset_transform_neg_axis_anbx       
        
          
        offset_amount = context.scene.offset_amount_anbx
        rev_trans_offset = context.scene.revers_transform_offset_anbx



        def offset_selected_axis():
            sel_ob = bpy.context.selected_objects
            offset = 0

            if rev_trans_offset:
                sel_ob = reversed(sel_ob)

            for o in sel_ob:
 
                
                if x_axis:
                    o.rotation_euler.x += offset

                if y_axis:
                    o.rotation_euler.y += offset

                if z_axis:
                    o.rotation_euler.z += offset


                if neg_axis:
                    offset -= offset_amount
                else:
                    offset += offset_amount
                    
                

                if event.alt:
                    o.keyframe_insert(data_path = 'rotation_euler')


        offset_selected_axis()


        return {'FINISHED'}

#=============================== End Rotation ===============




#================ Offset Transformation Scale ==================

class OBJECT_OT_offset_transform_scale_anbx(bpy.types.Operator):
    '''Offset Scale
-Selected Object(s)
-Alt Click = Insert Scale Key(s)'''

    bl_idname = "object.offset_transform_scale_anbx"
    bl_label = "Offset Scale"
    bl_options = {'REGISTER', 'UNDO'}
    


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                len(context.selected_objects) > 1)


    def invoke(self, context, event):

        # Ref 
        x_axis = context.scene.offset_transform_axis_x_anbx        
        y_axis = context.scene.offset_transform_axis_y_anbx        
        z_axis = context.scene.offset_transform_axis_z_anbx        
        
        
        neg_axis = context.scene.offset_transform_neg_axis_anbx       
        
        offset_amount = context.scene.offset_amount_anbx
        rev_trans_offset = context.scene.revers_transform_offset_anbx
              

        def offset_selected_axis():
            sel_ob = bpy.context.selected_objects
            offset = 0

            if rev_trans_offset:
                sel_ob = reversed(sel_ob)

            for o in sel_ob:
 
                
                if x_axis:
                    o.scale.x += offset

                if y_axis:
                    o.scale.y += offset

                if z_axis:
                    o.scale.z += offset
                
                else:
                    if not x_axis and not y_axis and not z_axis:
                        o.scale.x += offset
                        o.scale.y += offset
                        o.scale.z += offset
                        

                if neg_axis:
                    offset -= offset_amount
                else:
                    offset += offset_amount
                    
                

                if event.alt:
                    o.keyframe_insert(data_path = 'scale')


        offset_selected_axis()


        return {'FINISHED'}

#=============================== End Scale ===============



 

classes = [ OBJECT_OT_offset_transform_location_anbx,
            OBJECT_OT_offset_transform_rotation_anbx,
            OBJECT_OT_offset_transform_scale_anbx,
 
            
          ]
 
 
 
def register():
    for cls in classes:
        bpy.utils.register_class(cls)
        



    bpy.types.Scene.offset_amount_anbx = bpy.props.FloatProperty(
        name="Offset Amount",
        default=0.01,
        description="Offset Transformation Amount - Selected Objects",
        min=0.0,
        step=1,

    )

    bpy.types.Scene.revers_transform_offset_anbx = bpy.props.BoolProperty(
        name="",
        description="REVERSED TRANSFORM OFFSET (Checkbox)",
        default = False
    )




 
    #-------------------Offset Transform Axis -------------------
    
    bpy.types.Scene.offset_transform_axis_x_anbx = bpy.props.BoolProperty(
        name="",
        description="X Axis (Checkbox)",
        default = False
    )

    bpy.types.Scene.offset_transform_axis_y_anbx = bpy.props.BoolProperty(
        name="",
        description="Y Axis (Checkbox)",
        default = False
    )

    bpy.types.Scene.offset_transform_axis_z_anbx = bpy.props.BoolProperty(
        name="",
        description="Z Axis (Checkbox)",
        default = False
    )


    #------------------- Negative Direction -------------------
    

    bpy.types.Scene.offset_transform_neg_axis_anbx = bpy.props.BoolProperty(
        name="",
        description="Negative Axis (Checkbox)",
        default = False
    )

    #------------------- Direction -------------------

 
def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)

    del bpy.types.Scene.offset_transform_axis_x_anbx
    del bpy.types.Scene.offset_transform_axis_y_anbx
    del bpy.types.Scene.offset_transform_axis_z_anbx

 
    del bpy.types.Scene.offset_transform_neg_axis_anbx

    del bpy.types.Scene.offset_amount_anbx
    del bpy.types.Scene.revers_transform_offset_anbx

    
 
 
if __name__ == "__main__":
    register()  
