import bpy
import bmesh
from mathutils import Matrix, Vector

from bpy.props import (BoolProperty, FloatProperty, IntProperty)


 
#=====================================================
#             Origin to Selected VEF
#=====================================================

class OBJECT_OT_origin_to_selected_anbx(bpy.types.Operator):
    '''Origin to Selected - Vertex, Edge, Face

- You may choose from the (Create Empty/ Parent Empty /Child Empty) options above'''
    
    bl_idname = "object.origin_to_selected_anbx"
    bl_label = "Origin to Selected"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'EDIT' and
                context.object.type == 'MESH')

    
    def invoke(self, context, event):

        # Refs
        parent_bool = context.scene.parent_empty_anbx
        child_bool = context.scene.child_empty_anbx
        create_empty = context.scene.just_create_empty_anbx


        context = bpy.context

        sel_ob = bpy.context.selected_objects
        cursor_location = bpy.context.scene.cursor.location


        def origin_to_sel_v_e_f():


            for o in bpy.context.objects_in_mode:
                bpy.context.view_layer.objects.active = o

                #------------------------------------------

                ob = context.edit_object
                mw = ob.matrix_world
                me = ob.data
                bm = bmesh.from_edit_mesh(me)

                selverts = [v.co for v in bm.verts if v.select]
                if len(selverts):
                    pt = sum(selverts, Vector()) / len(selverts)
                    bmesh.ops.translate(bm, vec=-pt, verts=bm.verts)
                    mw.translation = mw @ pt
                bmesh.update_edit_mesh(me)

                #------------------------------------------


        def just_create_empty():
            
            for o in bpy.context.selected_objects:
                bpy.context.view_layer.objects.active = o


                empty = bpy.data.objects.new("New_Empty_" + o.name, None )
                o.empty_display_type = 'PLAIN_AXES'

                o.users_collection[-1].objects.link(empty)

                empty.location = o.location
                
                display_size = o.dimensions.z
                if display_size < 1:
                    empty.empty_display_size = 1
                
                elif display_size == 1:
                    empty.empty_display_size = 1.2
                    
                elif display_size > 1:
                    empty.empty_display_size = 1 + (display_size - 1)
 
                empty.select_set(1)
                o.select_set(0)
         
                for o in bpy.context.selected_objects:
                    bpy.context.view_layer.objects.active = o  


 
        # Add Parent Empty to selected Vertex, Edge, Face
        def add_parent_empty_to_selected():

            for o in bpy.context.selected_objects:
                bpy.context.view_layer.objects.active = o


                empty = bpy.data.objects.new("Parent_Empty_" + o.name, None )
                o.empty_display_type = 'PLAIN_AXES'

                o.users_collection[-1].objects.link(empty)

                empty.location = o.location
                o.location = (0, 0, 0)

                # Size of Empty
                display_size = o.dimensions.z
                if display_size < 1:
                    empty.empty_display_size = 1

                elif display_size == 1:
                    empty.empty_display_size = 1.2

                elif display_size > 1:
                    empty.empty_display_size = 1 + (display_size - 1)

                o.parent = empty

                empty.select_set(1)
                o.select_set(0)
         
                for o in bpy.context.selected_objects:
                    bpy.context.view_layer.objects.active = o

                #------------------------------------------


        # Add Child Empty to selected Vertex, Edge, Face
        def add_child_empty_to_selected():

            for o in bpy.context.selected_objects:
                bpy.context.view_layer.objects.active = o


                empty = bpy.data.objects.new("Child_Empty_" + o.name, None )
                o.empty_display_type = 'PLAIN_AXES'

                o.users_collection[-1].objects.link(empty)

                empty.location = o.location
                empty.location = (0, 0, 0)

                # Size of Empty
                display_size = o.dimensions.z
                if display_size < 1:
                    empty.empty_display_size = 1

                elif display_size == 1:
                    empty.empty_display_size = 1.2

                elif display_size > 1:
                    empty.empty_display_size = 1 + (display_size - 1)


                empty.parent = o

                empty.select_set(1)
                o.select_set(0)
                for o in bpy.context.selected_objects:
                    bpy.context.view_layer.objects.active = o


                #------------------------------------------

        if event:
            origin_to_sel_v_e_f()

        if event and parent_bool and not child_bool and not create_empty:
            add_parent_empty_to_selected()


        if event and child_bool and not parent_bool and not create_empty:
            add_child_empty_to_selected()


        if create_empty:
            if event and not child_bool and not parent_bool:
                just_create_empty()


        bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
        bpy.context.scene.tool_settings.transform_pivot_point = 'INDIVIDUAL_ORIGINS'


        return {'FINISHED'}


#=====================================================
# End Origin to Selected VEF
#=====================================================




#=====================================================
#   Origin  
#=====================================================


class OBJECT_OT_origin_objects_anbx(bpy.types.Operator):
    '''Origin to Object(s) - MESH / FONT
    
- You may choose from the options above.
    
-Auto Convert Font to Mesh'''

    bl_idname = "object.origin_objects_anbx"
    bl_label = "Origin"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) != 0)


    def invoke(self, context, event):

        # Refs
        parent_bool = context.scene.parent_empty_anbx
        child_bool = context.scene.child_empty_anbx
        empty_to_center = context.scene.empty_to_center_of_sel_anbx
        create_empty = context.scene.just_create_empty_anbx
 
 
        o_t = context.scene.origin_to_top_anbx
        o_bo = context.scene.origin_to_bottom_anbx
        o_l = context.scene.origin_to_left_anbx
        o_r = context.scene.origin_to_right_anbx
        o_f = context.scene.origin_to_front_anbx
        o_ba = context.scene.origin_to_back_anbx

        
        context = bpy.context
        sel_ob = bpy.context.selected_objects
        cursor = bpy.context.scene.cursor
        cursor_location = bpy.context.scene.cursor.location
        cursor_previous_location = cursor.location.copy()


        #-------------------------------------

        # Text to Mesh  

        act_ob = bpy.context.active_object

        def text_to_mesh():
            bpy.ops.object.convert(target='MESH')
            bpy.ops.object.mode_set(mode='EDIT')  
            bpy.ops.mesh.select_all(action='SELECT')
            bpy.ops.mesh.remove_doubles()

            bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
            bpy.ops.mesh.separate(type='LOOSE')
            bpy.context.scene.tool_settings.transform_pivot_point = 'INDIVIDUAL_ORIGINS'

            bpy.ops.object.transform_apply(location=False, rotation=True, scale=False)
            bpy.ops.object.origin_set(type='ORIGIN_CENTER_OF_VOLUME', center='MEDIAN')

 
        # Add Edge_Split Modifier
        def font_split_modifier():
            for o in bpy.context.selected_objects:
                bpy.context.view_layer.objects.active = o
                mod = o.modifiers.get("Font_Split")
                if mod is None:
                    o.modifiers.new('Font_Split', 'EDGE_SPLIT')


        if act_ob.type == 'FONT':
            text_to_mesh()
            font_split_modifier() 


        #-------------------------------------
        
        def origin_to_center():
            for o in bpy.context.selected_objects:
                bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='MEDIAN')


        #-------------------------------------
            
        def origin_to_top(ob, matrix=Matrix()):
            me = ob.data
            mw = ob.matrix_world
            local_verts = [matrix @ Vector(v[:]) for v in ob.bound_box]
            o = sum(local_verts, Vector()) / 8
            o.z = max(v.z for v in local_verts)
            o = matrix.inverted() @ o
            me.transform(Matrix.Translation(-o))

            mw.translation = mw @ o

        #-------------------------------------
        
        def origin_to_bottom(ob, matrix=Matrix()):
            me = ob.data
            mw = ob.matrix_world
            local_verts = [matrix @ Vector(v[:]) for v in ob.bound_box]
            o = sum(local_verts, Vector()) / 8
            o.z = min(v.z for v in local_verts)
            o = matrix.inverted() @ o
            me.transform(Matrix.Translation(-o))

            mw.translation = mw @ o        
        
        #-------------------------------------        
  
        def origin_to_right(ob, matrix=Matrix()):
            me = ob.data
            mw = ob.matrix_world
            local_verts = [matrix @ Vector(v[:]) for v in ob.bound_box]
            o = sum(local_verts, Vector()) / 8
            o.x = max(v.x for v in local_verts)
            o = matrix.inverted() @ o
            me.transform(Matrix.Translation(-o))

            mw.translation = mw @ o  
  
        #-------------------------------------
          
        def origin_to_left(ob, matrix=Matrix()):
            me = ob.data
            mw = ob.matrix_world
            local_verts = [matrix @ Vector(v[:]) for v in ob.bound_box]
            o = sum(local_verts, Vector()) / 8
            o.x = min(v.x for v in local_verts)
            o = matrix.inverted() @ o
            me.transform(Matrix.Translation(-o))

            mw.translation = mw @ o        
        
        #-------------------------------------
          
        def origin_to_front(ob, matrix=Matrix()):
            me = ob.data
            mw = ob.matrix_world
            local_verts = [matrix @ Vector(v[:]) for v in ob.bound_box]
            o = sum(local_verts, Vector()) / 8
            o.y = min(v.y for v in local_verts)
            o = matrix.inverted() @ o
            me.transform(Matrix.Translation(-o))

            mw.translation = mw @ o        
        
        #-------------------------------------
          
        def origin_to_back(ob, matrix=Matrix()):
            me = ob.data
            mw = ob.matrix_world
            local_verts = [matrix @ Vector(v[:]) for v in ob.bound_box]
            o = sum(local_verts, Vector()) / 8
            o.y = max(v.y for v in local_verts)
            o = matrix.inverted() @ o
            me.transform(Matrix.Translation(-o))

            mw.translation = mw @ o
            
            
        #-------------------------------------

        
        def just_create_empty():
            
            for o in bpy.context.selected_objects:
                bpy.context.view_layer.objects.active = o


                empty = bpy.data.objects.new("New_Empty_" + o.name, None )
                o.empty_display_type = 'PLAIN_AXES'

                o.users_collection[-1].objects.link(empty)

                empty.location = o.location
                
                display_size = o.dimensions.z
                if display_size < 1:
                    empty.empty_display_size = 1
                
                elif display_size == 1:
                    empty.empty_display_size = 1.2
                    
                elif display_size > 1:
                    empty.empty_display_size = 1 + (display_size - 1)
 
                empty.select_set(1)
                o.select_set(0)

                bpy.context.view_layer.objects.active = empty       
        
        #-------------------------------------
            
 
        # Add Parent Empty to 1 selected object
        def add_parent_empty_one_selected():

            for o in bpy.context.selected_objects:    

                empty = bpy.data.objects.new("Parent_Empty_" + o.name, None )
                o.empty_display_type = 'PLAIN_AXES'

                o.users_collection[-1].objects.link(empty)
                
                empty.location = o.location
                o.location = (0, 0, 0)
                
                display_size = o.dimensions.z
                if display_size < 1:
                    empty.empty_display_size = 1
                
                elif display_size == 1:
                    empty.empty_display_size = 1.2
                    
                elif display_size > 1:
                    empty.empty_display_size = 1 + (display_size - 1)
                    
                o.parent = empty

                empty.select_set(1)
                o.select_set(0)

                bpy.context.view_layer.objects.active = empty


        # Add Child Empty to 1 selected object
        def add_child_empty_one_selected():

            for o in bpy.context.selected_objects:    

                empty = bpy.data.objects.new("Child_Empty_" + o.name, None )
                o.empty_display_type = 'PLAIN_AXES'

                o.users_collection[-1].objects.link(empty)
                
                empty.location = o.location
                empty.location = (0, 0, 0)
                
                display_size = o.dimensions.z
                if display_size < 1:
                    empty.empty_display_size = 1
                
                elif display_size == 1:
                    empty.empty_display_size = 1.2
                    
                elif display_size > 1:
                    empty.empty_display_size = 1 + (display_size - 1)
                    
                empty.parent = o

                empty.select_set(1)
                o.select_set(0)
 
                bpy.context.view_layer.objects.active = empty

 
        # Add Parent Empty Multiple Selected Object
        def add_parent_empty_multiple_selected():
            

            for o in bpy.context.selected_objects: 
                bpy.context.view_layer.objects.active = o  

                if empty_to_center:
                    bpy.ops.view3d.snap_cursor_to_selected()

                else:
                    bpy.context.scene.cursor.location = o.location

  
                empty = bpy.data.objects.new("Parent_Empty_" + o.name, None )
                o.empty_display_type = 'PLAIN_AXES'

                o.users_collection[-1].objects.link(empty)
                
                display_size = o.dimensions.z
                if display_size < 1:
                    empty.empty_display_size = 1
                
                elif display_size == 1:
                    empty.empty_display_size = 1.2
                    
                elif display_size > 1:
                    empty.empty_display_size = 1 + (display_size - 1)
                    
                
                if empty_to_center:
                    empty.location = cursor_location

                else:
                    empty.location = o.location
                    empty.location.z = cursor_location.z

                bpy.context.evaluated_depsgraph_get().update()
               
 
                o.parent = empty
                o.matrix_parent_inverse = empty.matrix_world.inverted()
 
                if not empty_to_center:
                    empty.select_set(1)
                    o.select_set(0)
                    for o in bpy.context.selected_objects:
                        bpy.context.view_layer.objects.active = o
 
 
        # Add Child Empty Multiple Selected Object
        def add_child_empty_multiple_selected():
            

            for o in bpy.context.selected_objects: 
                bpy.context.view_layer.objects.active = o  

                if empty_to_center:
                    bpy.ops.view3d.snap_cursor_to_selected()

                else:
                    bpy.context.scene.cursor.location = o.location

  
                empty = bpy.data.objects.new("Child_Empty_" + o.name, None )
                o.empty_display_type = 'PLAIN_AXES'

                o.users_collection[-1].objects.link(empty)
                
                display_size = o.dimensions.z
                if display_size < 1:
                    empty.empty_display_size = 1
                
                elif display_size == 1:
                    empty.empty_display_size = 1.2
                    
                elif display_size > 1:
                    empty.empty_display_size = 1 + (display_size - 1)
                    
                
                if empty_to_center:
                    empty.location = cursor_location

                else:
                    empty.location = o.location
                    empty.location.z = cursor_location.z

                bpy.context.evaluated_depsgraph_get().update()
               
 
                empty.parent = o
                empty.matrix_parent_inverse = o.matrix_world.inverted()

                if not empty_to_center:
                    empty.select_set(1)
                    o.select_set(0)
                    for o in bpy.context.selected_objects:
                        bpy.context.view_layer.objects.active = o
 


        #--------------------------------------------------------------------
        #--------------------------------------------------------------------
        #--------------------------------------------------------------------
             

        if event:
            if not o_t and not o_bo and not o_l and not o_r and not o_f and not o_ba:
                for o in bpy.context.selected_objects:
                    if o.type == 'MESH':
                        origin_to_center()

            if o_t and not o_bo and not o_l and not o_r and not o_f and not o_ba:     
                for o in bpy.context.selected_objects:
                    if o.type == 'MESH':
                        origin_to_top(o)     
     
            if o_bo and not o_t and not o_l and not o_r and not o_f and not o_ba:     
                for o in bpy.context.selected_objects:
                    if o.type == 'MESH':
                        origin_to_bottom(o)     
     
     
            if o_l and not o_bo and not o_t and not o_r and not o_f and not o_ba:     
                for o in bpy.context.selected_objects:
                    if o.type == 'MESH':
                        origin_to_left(o)       
     
     
            if o_r and not o_bo and not o_l and not o_t and not o_f and not o_ba:     
                for o in bpy.context.selected_objects:
                    if o.type == 'MESH':
                        origin_to_right(o)  
    
     
            if o_f and not o_bo and not o_l and not o_r and not o_t and not o_ba:     
                for o in bpy.context.selected_objects:
                    if o.type == 'MESH':
                        origin_to_front(o)

        
            if o_ba and not o_bo and not o_l and not o_r and not o_f and not o_t:     
                for o in bpy.context.selected_objects:
                    if o.type == 'MESH':
                        origin_to_back(o)          
       
       
       

        if event and parent_bool and not child_bool and not create_empty:

            if len(sel_ob) > 1:
                add_parent_empty_multiple_selected()
                bpy.context.scene.tool_settings.transform_pivot_point = 'INDIVIDUAL_ORIGINS'
                
            elif len(sel_ob) == 1:
                add_parent_empty_one_selected()
                 

        if event and child_bool and not parent_bool and not create_empty:
            if len(sel_ob) > 1:            
                add_child_empty_multiple_selected()
                bpy.context.scene.tool_settings.transform_pivot_point = 'INDIVIDUAL_ORIGINS'


            elif len(sel_ob) == 1:
                add_child_empty_one_selected() 
 

        if create_empty:
            if event and not child_bool and not parent_bool:
                just_create_empty()
                     
                     
        # Cursor back
        cursor.location = cursor_previous_location

        bpy.context.scene.tool_settings.transform_pivot_point = 'INDIVIDUAL_ORIGINS'


        return {'FINISHED'}
    
    
    
#-----------------------------func--------------------------


def reset_child_bool_fun_anbx(self, context):
    sc = bpy.context.scene
    sc.property_unset('parent_empty_anbx')
    sc.property_unset('just_create_empty_anbx')
    
    
def reset_parent_bool_fun_anbx(self, context):
    sc = bpy.context.scene
    sc.property_unset('child_empty_anbx')
    sc.property_unset('just_create_empty_anbx')
      
      
def reset_just_empty_bool_fun_anbx(self, context):
    sc = bpy.context.scene
    sc.property_unset('parent_empty_anbx')
    sc.property_unset('child_empty_anbx')
    sc.property_unset('empty_to_center_of_sel_anbx')
      
def reset_empty_to_center_of_sel_fun_anbx(self, context):
    sc = bpy.context.scene
    sc.property_unset('just_create_empty_anbx')
      


# top/bottom/r/l/front/back

def reset_origin_to_top_fun_anbx(self, context):
    sc = bpy.context.scene

    sc.property_unset('origin_to_bottom_anbx')
    sc.property_unset('origin_to_left_anbx')
    sc.property_unset('origin_to_right_anbx')
    sc.property_unset('origin_to_front_anbx')
    sc.property_unset('origin_to_back_anbx')


def reset_origin_to_bottom_fun_anbx(self, context):
    sc = bpy.context.scene

    sc.property_unset('origin_to_top_anbx')
    sc.property_unset('origin_to_left_anbx')
    sc.property_unset('origin_to_right_anbx')
    sc.property_unset('origin_to_front_anbx')
    sc.property_unset('origin_to_back_anbx')


def reset_origin_to_left_fun_anbx(self, context):
    sc = bpy.context.scene

    sc.property_unset('origin_to_top_anbx')
    sc.property_unset('origin_to_bottom_anbx')
    sc.property_unset('origin_to_right_anbx')
    sc.property_unset('origin_to_front_anbx')
    sc.property_unset('origin_to_back_anbx')


def reset_origin_to_right_fun_anbx(self, context):
    sc = bpy.context.scene

    sc.property_unset('origin_to_top_anbx')
    sc.property_unset('origin_to_bottom_anbx')
    sc.property_unset('origin_to_left_anbx')
    sc.property_unset('origin_to_front_anbx')
    sc.property_unset('origin_to_back_anbx')


def reset_origin_to_front_fun_anbx(self, context):
    sc = bpy.context.scene

    sc.property_unset('origin_to_top_anbx')
    sc.property_unset('origin_to_bottom_anbx')
    sc.property_unset('origin_to_left_anbx')
    sc.property_unset('origin_to_right_anbx')
    sc.property_unset('origin_to_back_anbx')


def reset_origin_to_back_fun_anbx(self, context):
    sc = bpy.context.scene

    sc.property_unset('origin_to_top_anbx')
    sc.property_unset('origin_to_bottom_anbx')
    sc.property_unset('origin_to_left_anbx')
    sc.property_unset('origin_to_right_anbx')
    sc.property_unset('origin_to_front_anbx')



#-----------------------------func--------------------------



classes = [ OBJECT_OT_origin_to_selected_anbx,
            OBJECT_OT_origin_objects_anbx,


          ]



def register():
    for cls in classes:
        bpy.utils.register_class(cls)



    bpy.types.Scene.just_create_empty_anbx = bpy.props.BoolProperty(
        name="",
        description="Just Create Empty (Checkbox)",
        default = False,
        update = reset_just_empty_bool_fun_anbx,
    )


    bpy.types.Scene.parent_empty_anbx = bpy.props.BoolProperty(
        name="",
        description="Parent Empty (Checkbox)",
        default = False,
        update = reset_parent_bool_fun_anbx,
    )

    bpy.types.Scene.child_empty_anbx = bpy.props.BoolProperty(
        name="",
        description="Child Empty (Checkbox)",
        default = False,
        update = reset_child_bool_fun_anbx,
    )

    bpy.types.Scene.empty_to_center_of_sel_anbx = bpy.props.BoolProperty(
        name="",
        description='''Additional Option (Checkbox): 
- Place Empty between selected Objects (Center of Selection)
- Only in Object Mode
- Font should be converted first to Mesh''',
        default = False,
        update = reset_empty_to_center_of_sel_fun_anbx,
    )



#------------------- Top, Bottom, Left, Right, Front, Back ----------------


    bpy.types.Scene.origin_to_top_anbx = bpy.props.BoolProperty(
        name="",
        description="Origin to Top (Checkbox)",
        default = False,
        update = reset_origin_to_top_fun_anbx,
    )


    bpy.types.Scene.origin_to_bottom_anbx = bpy.props.BoolProperty(
        name="",
        description="Origin to Bottom (Checkbox)",
        default = False,
        update = reset_origin_to_bottom_fun_anbx,
    )


    bpy.types.Scene.origin_to_left_anbx = bpy.props.BoolProperty(
        name="",
        description="Origin to Left (Checkbox)",
        default = False,
        update = reset_origin_to_left_fun_anbx,
    )


    bpy.types.Scene.origin_to_right_anbx = bpy.props.BoolProperty(
        name="",
        description="Origin to Right (Checkbox)",
        default = False,
        update = reset_origin_to_right_fun_anbx, 
    )


    bpy.types.Scene.origin_to_front_anbx = bpy.props.BoolProperty(
        name="",
        description="Origin to Front (Checkbox)",
        default = False,
        update = reset_origin_to_front_fun_anbx, 
    )


    bpy.types.Scene.origin_to_back_anbx = bpy.props.BoolProperty(
        name="",
        description="Origin to Back (Checkbox)",
        default = False,
        update = reset_origin_to_back_fun_anbx, 
    )


#------------------- Top, Bottom, Left, Right, Front, Back ----------------


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)


    del bpy.types.Scene.parent_empty_anbx 
    del bpy.types.Scene.child_empty_anbx
    del bpy.types.Scene.empty_to_center_of_sel_anbx


    del bpy.types.Scene.origin_to_top_anbx
    del bpy.types.Scene.origin_to_bottom_anbx
    del bpy.types.Scene.origin_to_left_anbx
    del bpy.types.Scene.origin_to_right_anbx
    del bpy.types.Scene.origin_to_front_anbx
    del bpy.types.Scene.origin_to_back_anbx 
     



if __name__ == "__main__":
    register()




