import bpy
import math
from bpy.props import (BoolProperty, FloatProperty, IntProperty)




class OBJECT_OT_overshoot_location_anbx(bpy.types.Operator):
    '''Overshoot Location
First select the direction - X, Y, or Z'''

    bl_idname = "object.overshoot_location_anbx"
    bl_label = "Location"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) != 0)


    def execute(self, context):

        # Ref

        # negative direction
        negative_direction = context.scene.overshoot_loc_negative_direction_anbx

        x_bool = context.scene.overshoot_loc_x_anbx
        y_bool = context.scene.overshoot_loc_y_anbx
        z_bool = context.scene.overshoot_loc_z_anbx


        overshoot_frames = context.scene.duration_frames_number_loc_anbx
        overshoot_amplitude_loc = context.scene.overshoot_loc_amplitude_anbx

        frequency_bool = context.scene.overshoot_loc_frequency_anbx
        decay_bool = context.scene.overshoot_loc_decay_anbx

# Amplitude, frequency, and decay

        frame = bpy.context.scene.frame_current

        ob = bpy.context.object
#        ob_loc = ob.location.copy()


        offset = 0
        sel_ob = bpy.context.selected_objects


        for i in range(overshoot_frames):

            bpy.context.scene.frame_set(frame)

            # Higher Value = higher Frequency
            freq = frequency_bool

            # Higher Value = faster and greater Decay
            decay = decay_bool

            amp = overshoot_amplitude_loc

            dur = 0.1
            w = freq*math.pi*2


            for o in sel_ob:
                if x_bool and not negative_direction:
                    o.location.x += amp*(math.cos((i-dur)*w)/math.exp(decay*(i-dur))/w)
                    o.keyframe_insert(data_path='location', index=0)

                if x_bool and negative_direction:
                    o.location.x -= amp*(math.cos((i-dur)*w)/math.exp(decay*(i-dur))/w)
                    o.keyframe_insert(data_path='location', index=0)



                if y_bool and not negative_direction:
                    o.location.y += amp*(math.cos((i-dur)*w)/math.exp(decay*(i-dur))/w)
                    o.keyframe_insert(data_path='location', index=1)

                if y_bool and negative_direction:
                    o.location.y -= amp*(math.cos((i-dur)*w)/math.exp(decay*(i-dur))/w)
                    o.keyframe_insert(data_path='location', index=1)



                if z_bool and not negative_direction:
                    o.location.z += amp*(math.cos((i-dur)*w)/math.exp(decay*(i-dur))/w)
                    o.keyframe_insert(data_path='location', index=2)

                if z_bool and negative_direction:
                    o.location.z -= amp*(math.cos((i-dur)*w)/math.exp(decay*(i-dur))/w)
                    o.keyframe_insert(data_path='location', index=2)


                else:
                    pass


            frame += 1


        bpy.context.scene.frame_current -= (overshoot_frames - 1)


        return {'FINISHED'}







classes = [ OBJECT_OT_overshoot_location_anbx ]






def register():
    for cls in classes:
        bpy.utils.register_class(cls)




#========================= Negative Direction ==========================

    bpy.types.Scene.overshoot_loc_negative_direction_anbx = bpy.props.BoolProperty(
        name="",
        description="Negative Direction ",
        default = False
    )


#========================= Negative Direction ==========================



#========================= Axis ==========================

    bpy.types.Scene.overshoot_loc_x_anbx = bpy.props.BoolProperty(
        name="",
        description="Overshoot X Location",
        default = False
    )

    bpy.types.Scene.overshoot_loc_y_anbx = bpy.props.BoolProperty(
        name="",
        description="Overshoot Y Location",
        default = False
    )


    bpy.types.Scene.overshoot_loc_z_anbx = bpy.props.BoolProperty(
        name="",
        description="Overshoot Z Location",
        default = False
    )

#========================= Axis ==========================



#========================= Frames ==========================

    bpy.types.Scene.duration_frames_number_loc_anbx = bpy.props.IntProperty(
        name="Frames - Loc",
        default=100,
        description="Number of Frames - Location",
        min=1,
    )
#========================= Frames ==========================


#========================= Location Amplitude ==========================

    bpy.types.Scene.overshoot_loc_amplitude_anbx = bpy.props.FloatProperty(
        name="Amplitude Location",
        default=0.2,
        description="Amplitude of Overshoot",
        step=1,
        precision=2,

    )



#========================= Location Frequency ==========================

    bpy.types.Scene.overshoot_loc_frequency_anbx = bpy.props.FloatProperty(
        name="Frequency Location",
        default=0.05,
        description="Higher Value = higher Frequency",
        step=1,
        precision=2,

    )


#========================= Location Decay ==========================

    bpy.types.Scene.overshoot_loc_decay_anbx = bpy.props.FloatProperty(
        name="Decay Location",
        default=0.1,
        description="Higher Value = faster and greater Decay",
        step=1,
        precision=2,

    )




def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)


    del bpy.types.Scene.overshoot_loc_amplitude_anbx
    del bpy.types.Scene.overshoot_loc_frequency_anbx
    del bpy.types.Scene.overshoot_loc_decay_anbx

    del bpy.types.Scene.duration_frames_number_loc_anbx

    del bpy.types.Scene.overshoot_loc_x_anbx
    del bpy.types.Scene.overshoot_loc_y_anbx
    del bpy.types.Scene.overshoot_loc_z_anbx

    del bpy.types.Scene.overshoot_loc_negative_direction_anbx


if __name__ == "__main__":
    register()




