import bpy
import math
from bpy.props import (BoolProperty, FloatProperty, IntProperty)





class OBJECT_OT_overshoot_rotation_anbx(bpy.types.Operator):
    '''Overshoot Rotation
First select the direction - X, Y, or Z'''

    bl_idname = "object.overshoot_rotation_anbx"
    bl_label = "Rotation"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) != 0)


    def execute(self, context):

        # Ref
        x_bool = context.scene.overshoot_rot_x_anbx
        y_bool = context.scene.overshoot_rot_y_anbx
        z_bool = context.scene.overshoot_rot_z_anbx
        cos_bool = context.scene.overshoot_cos_anbx
        overshoot_frames = context.scene.duration_frames_number_rot_anbx
        overshoot_amplitude = context.scene.overshoot_rot_amplitude_anbx

        frequency_bool = context.scene.overshoot_rot_frequency_anbx
        decay_bool = context.scene.overshoot_rot_decay_anbx



        frame = bpy.context.scene.frame_current

        ob = bpy.context.object
        ob_rot = ob.rotation_euler.copy()


        offset = 0
        sel_ob = bpy.context.selected_objects


        for i in range(overshoot_frames):

            time = i 

            bpy.context.scene.frame_set(frame)

            # Higher Value = higher Frequency
            freq = frequency_bool

            # Higher Value = faster and greater Decay
            decay = decay_bool

            amplitude = overshoot_amplitude


            math_var = math.sin

            if cos_bool:
                math_var = math.cos
                if x_bool:
                    amplitude = ob_rot[0]

                if y_bool:
                    amplitude = ob_rot[1]

                if z_bool:
                    amplitude = ob_rot[2]



            for o in sel_ob:
                if x_bool:
                    o.rotation_euler.x = amplitude * math_var(freq * time )/ math.exp(decay *time)
                    o.keyframe_insert(data_path='rotation_euler', index=0)


                if y_bool:
                    o.rotation_euler.y = amplitude * math_var(freq * time )/ math.exp(decay *time)
                    o.keyframe_insert(data_path='rotation_euler', index=1)

                if z_bool:
                    o.rotation_euler.z = amplitude * math_var(freq * time )/ math.exp(decay *time)
                    o.keyframe_insert(data_path='rotation_euler', index=2)

                else:
                    pass


            frame += 1


        bpy.context.scene.frame_current -= (overshoot_frames - 1)


        return {'FINISHED'}






classes = [ OBJECT_OT_overshoot_rotation_anbx]





def register():
    for cls in classes:
        bpy.utils.register_class(cls)




#========================= Axis ==========================

    bpy.types.Scene.overshoot_rot_x_anbx = bpy.props.BoolProperty(
        name="",
        description="Overshoot X Rotation",
        default = False
    )

    bpy.types.Scene.overshoot_rot_y_anbx = bpy.props.BoolProperty(
        name="",
        description="Overshoot Y Rotation",
        default = False
    )


    bpy.types.Scene.overshoot_rot_z_anbx = bpy.props.BoolProperty(
        name="",
        description="Overshoot Z Rotation",
        default = False
    )

#========================= Axis ==========================


#========================= Rotation Cosine ==========================

    bpy.types.Scene.overshoot_cos_anbx = bpy.props.BoolProperty(
        name="",
        description="Start at a Specified Position",
        default = False
    )
#========================= Rotation Cosine ==========================


#========================= Frames ==========================

    bpy.types.Scene.duration_frames_number_rot_anbx = bpy.props.IntProperty(
        name="Frames - Rot",
        default=200,
        description="Number of Frames - Rotation",
        min=1,
    )
#========================= Frames ==========================


#========================= Rotation Amplitude ==========================

    bpy.types.Scene.overshoot_rot_amplitude_anbx = bpy.props.FloatProperty(
        name="Amplitude Rotation",
        default=0.5,
        description="Amplitude of Overshoot",
        step=1,
        precision=2,

    )

#========================= Rotation Amplitude ==========================


#========================= Rotation Frequency ==========================

    bpy.types.Scene.overshoot_rot_frequency_anbx = bpy.props.FloatProperty(
        name="Frequency Rotation",
        default=0.2,
        description="Higher Value = higher Frequency",
        step=1,
        precision=2,

    )

#========================= Rotation Frequency ==========================


#========================= Rotation Decay ==========================

    bpy.types.Scene.overshoot_rot_decay_anbx = bpy.props.FloatProperty(
        name="Decay Rotation",
        default=0.03,
        description="Higher Value = faster and greater Decay",
        step=1,
        precision=2,

    )


#========================= Rotation Decay ==========================


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)

    del bpy.types.Scene.overshoot_cos_anbx
    del bpy.types.Scene.duration_frames_number_rot_anbx
    del bpy.types.Scene.overshoot_rot_amplitude_anbx
    del bpy.types.Scene.overshoot_rot_frequency_anbx
    del bpy.types.Scene.overshoot_rot_decay_anbx


    del bpy.types.Scene.overshoot_rot_x_anbx
    del bpy.types.Scene.overshoot_rot_y_anbx
    del bpy.types.Scene.overshoot_rot_z_anbx

if __name__ == "__main__":
    register()

