import bpy
import math
from bpy.props import (BoolProperty, FloatProperty, IntProperty)





class OBJECT_OT_overshoot_scale_anbx(bpy.types.Operator):
    '''Overshoot Scale'''

    bl_idname = "object.overshoot_scale_anbx"
    bl_label = "Scale"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) != 0)


    def execute(self, context):

        # Ref
        x_bool = context.scene.overshoot_scale_x_anbx
        y_bool = context.scene.overshoot_scale_y_anbx
        z_bool = context.scene.overshoot_scale_z_anbx

        overshoot_frames = context.scene.duration_frames_number_scale_anbx
        overshoot_amplitude_scale = context.scene.overshoot_scale_amplitude_anbx

        frequency_bool = context.scene.overshoot_scale_frequency_anbx
        decay_bool = context.scene.overshoot_scale_decay_anbx


        frame = bpy.context.scene.frame_current

        ob = bpy.context.object
        ob_scale = ob.scale.copy()


        offset = 0
        sel_ob = bpy.context.selected_objects


        for i in range(overshoot_frames):

            bpy.context.scene.frame_set(frame)

            # Higher Value = higher Frequency
            freq = frequency_bool


            # Higher Value = faster and greater Decay
            decay = decay_bool

            amp = overshoot_amplitude_scale


            dur = 0.1
            w = freq*math.pi*2


            for o in sel_ob:
                if x_bool:
                    o.scale.x += amp*(math.sin((i-dur)*w)/math.exp(decay*(i-dur))/w)
                    o.keyframe_insert(data_path='scale', index=0)


                if y_bool:
                    o.scale.y += amp*(math.sin((i-dur)*w)/math.exp(decay*(i-dur))/w)
                    o.keyframe_insert(data_path='scale', index=1)

                if z_bool:
                    o.scale.z += amp*(math.sin((i-dur)*w)/math.exp(decay*(i-dur))/w)
                    o.keyframe_insert(data_path='scale', index=2)

                else:
                    o.scale.x += amp*(math.sin((i-dur)*w)/math.exp(decay*(i-dur))/w)
                    o.scale.y += amp*(math.sin((i-dur)*w)/math.exp(decay*(i-dur))/w)
                    o.scale.z += amp*(math.sin((i-dur)*w)/math.exp(decay*(i-dur))/w)

                    o.keyframe_insert(data_path='scale')



            frame += 1


        bpy.context.scene.frame_current -= (overshoot_frames - 1)


        return {'FINISHED'}







classes = [ OBJECT_OT_overshoot_scale_anbx ]






def register():
    for cls in classes:
        bpy.utils.register_class(cls)




#========================= Axis ==========================

    bpy.types.Scene.overshoot_scale_x_anbx = bpy.props.BoolProperty(
        name="",
        description="Overshoot X Scale",
        default = False
    )

    bpy.types.Scene.overshoot_scale_y_anbx = bpy.props.BoolProperty(
        name="",
        description="Overshoot Y Scale",
        default = False
    )


    bpy.types.Scene.overshoot_scale_z_anbx = bpy.props.BoolProperty(
        name="",
        description="Overshoot Z Scale",
        default = False
    )

#========================= Axis ==========================



#========================= Frames ==========================

    bpy.types.Scene.duration_frames_number_scale_anbx = bpy.props.IntProperty(
        name="Frames - Scale",
        default=100,
        description="Number of Frames - Scale",
        min=1,
    )
#========================= Frames ==========================


#========================= Scale Amplitude ==========================

    bpy.types.Scene.overshoot_scale_amplitude_anbx = bpy.props.FloatProperty(
        name="Amplitude Scale",
        default=0.3,
        description="Amplitude of Overshoot",
        step=1,
        precision=2,

    )


#========================= Scale Frequency ==========================

    bpy.types.Scene.overshoot_scale_frequency_anbx = bpy.props.FloatProperty(
        name="Frequency Scale",
        default=0.08,
        description="Higher Value = higher Frequency",
        step=1,
        precision=2,

    )


#========================= Scale Decay ==========================

    bpy.types.Scene.overshoot_scale_decay_anbx = bpy.props.FloatProperty(
        name="Decay Scale",
        default=0.15,
        description="Higher Value = faster and greater Decay",
        step=1,
        precision=2,

    )




def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)


    del bpy.types.Scene.overshoot_scale_amplitude_anbx
    del bpy.types.Scene.overshoot_scale_frequency_anbx
    del bpy.types.Scene.overshoot_scale_decay_anbx

    del bpy.types.Scene.duration_frames_number_scale_anbx

    del bpy.types.Scene.overshoot_scale_x_anbx
    del bpy.types.Scene.overshoot_scale_y_anbx
    del bpy.types.Scene.overshoot_scale_z_anbx


if __name__ == "__main__":
    register()
