import bpy


                

# =====================================================
#              PANEL 001 Claybox
# =====================================================


class PANEL_PT_animbox_ax_001(bpy.types.Panel):
    bl_label = "Claybox"
    bl_idname = 'PANEL_PT_animbox_ax_001'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Animbox"
    bl_options = {'DEFAULT_CLOSED'}



    @classmethod
    def poll(cls, context): 
        bv = bpy.app.version_string
        return (bv[0] > '2')



    def draw(self, context):
        layout = self.layout

        mode = bpy.context.mode


        box = layout.box() 

        row = box.row()
        row.operator("object.create_eyelid_mesh_cbox", text='Eye + Eyelids', icon='CURSOR')   


        # Stay in Sculpt Mode
        box = layout.box()  
        row = box.row()   

        row.prop(bpy.context.scene, "stay_in_sculpt_mode_cx", text='Stay in Sculpt Mode')


        # Create and Mirror / Center on X Axis
        box = layout.box() 
        row = box.row(align=True)
        row.scale_x = 0.4
        row.label(text='Create and:')

        row.scale_x = 1.6
        row.prop(bpy.context.scene, "create_and_mirror_cx", text='', icon='MOD_MIRROR')
        row.prop(bpy.context.scene, "create_and_center_x_axis_cx", text='', icon='CENTER_ONLY')
        

        # Transform
        if mode == 'SCULPT':
        
            box = layout.box()  
            row = box.row(align=True)
            row.scale_x = 1.5
            row.scale_y = 1.2
            from bl_ui.space_toolsystem_common import ToolSelectPanelHelper
            row.operator(
                "wm.tool_set_by_id",
                text="",
                icon_value=ToolSelectPanelHelper._icon_value_from_icon_handle("ops.transform.translate"),
                ).name = 'builtin.move'
                
            row.operator(
                "wm.tool_set_by_id",
                text="",
                icon_value=ToolSelectPanelHelper._icon_value_from_icon_handle("ops.transform.rotate"),
                ).name = 'builtin.rotate'
                

            row.operator(
                "wm.tool_set_by_id",
                text="",
                icon_value=ToolSelectPanelHelper._icon_value_from_icon_handle("ops.transform.resize"),
                ).name = 'builtin.scale'
                
            row.operator(
                "wm.tool_set_by_id",
                text="",
                icon_value=ToolSelectPanelHelper._icon_value_from_icon_handle("ops.transform.transform"),
                ).name = 'builtin.transform'
                
            
            row.operator("object.tool_update_cbox", text='', icon='FILE_REFRESH')


        box = layout.box() 
        row = box.row()
        row.operator("object.claybox_info_cx", text='How to create', icon='INFO') 

        # Sphere
        row = box.row(align=True)
        row.scale_y = 1.2
        row.operator("object.create_sphere_low_res_cbox", text='x1', icon='SPHERE',)
        row.operator("object.create_sphere_mid_res_cbox", text='x2', icon='SPHERE',)
        row.operator("object.create_sphere_high_res_cbox", text='x3', icon='SPHERE',)
        
        # Round Cube
        row = box.row(align=True)
        row.operator("object.create_rcube_low_cbox", text='x1', icon='META_CUBE')
        row.operator("object.create_rcube_mid_cbox", text='x2', icon='META_CUBE')
        row.operator("object.create_rcube_high_cbox", text='x3', icon='META_CUBE')
        
        
        # Cylinder
        row = box.row(align=True)
        row.operator("object.create_cylinder_low_cbox", text='x1', icon='MESH_CYLINDER')
        row.operator("object.create_cylinder_mid_cbox", text='x2', icon='MESH_CYLINDER')
        row.operator("object.create_cylinder_high_cbox", text='x3', icon='MESH_CYLINDER')
        
        # Mid / Short Cylinder
        row = box.row(align=True)
        row.scale_y = 1.2
        row.operator("object.create_middle_cylinder_cbox", text='Middle', icon='MESH_CYLINDER')
        row.operator("object.create_short_cylinder_cbox", text='Short', icon='MESH_CYLINDER')



        # Capsule / Tapered Capsule
        row = box.row(align=True)
        row.operator("object.create_capsule_cbox", text='Capsule')
        row.operator("object.create_tapered_capsule_cbox", text='TCapsule')
        

        # Hemisphere / Cone
        row = box.row(align=True)
        row.operator("object.create_cone_cbox")
        row.operator("object.create_hsphere_cbox")


        # Spheroid oblate/prolate
        row = box.row(align=True)
        row.operator("object.create_oblate_spheroid_cbox", text='O Spheroid')
        row.operator("object.create_prolate_spheroid_cbox", text='P Spheroid')


        # Cuboid Prism / Triangular Prism
        row = box.row(align=True)
        row.operator("object.create_cuboid_prism_cbox", text='Cub. Prism')
        row.operator("object.create_triangular_prism_cbox", text='Tri. Prism')

        # Rectangular Quader / Quader
        row = box.row(align=True)
        row.operator("object.create_rect_quader_cbox", text='RQuader')
        row.operator("object.create_quader_cbox", text='Quader')


       

        # tools -------------------------------
        box = layout.box()

        box = layout.box()
        row = box.row(align=True)
        
        # Rotate on X / Y / Z Axis
        row.scale_y = 1.2
        row.operator("object.rotate_90_x_axis_cbox", text='90 X', icon='CON_ROTLIKE')    
        row.operator("object.rotate_90_y_axis_cbox", text='90 Y', icon='CON_ROTLIKE')    
        row.operator("object.rotate_90_z_axis_cbox", text='90 Z', icon='CON_ROTLIKE')    
        row = box.row(align=True)

        # Center on X Axis
        row.scale_y = 1.2
        row.operator("object.center_obj_cbox", text='Center X', icon='CENTER_ONLY')        
        row.scale_x = 1.5   
        row.operator("object.unlock_transform_cbox", text='', icon='UNLOCKED')        
        
        
        # Add Res
        box = layout.box()
        row = box.row()
        row.label(text='', icon='ADD')
        row.operator("object.add_subdiv_res_cbox", text='Add Res')
        
        # Mirror -----------------------------
        
        ob = bpy.context.object 
        if ob is not None:
        
            mirror_mods = [mod for mod in ob.modifiers if mod.type == 'MIRROR']
            if not mirror_mods:
     
                box = layout.box()  
                row = box.row()        
                row.label(text='', icon='MOD_MIRROR')
                row.scale_y = 1.2
                row.operator("object.mirror_mod_cbox", text='Mirror')  
                
            if ob.mode == 'OBJECT':     
                box = layout.box()      
                row = box.row()   
                row.label(text='', icon='SCULPTMODE_HLT')     
                row.scale_y = 1.2
                row.operator("object.apply_scale_back2sculpt_cbox", text='Sculpt Mode')        

            if ob.mode == 'SCULPT':     
                box = layout.box()      
                row = box.row()   
                row.label(text='', icon='OBJECT_DATAMODE')     
                row.scale_y = 1.2
                row.operator("object.apply_scale_back2sculpt_cbox", text='Object Mode')        

            if ob.mode != 'OBJECT' and ob.mode != 'SCULPT':
                box = layout.box()      
                row = box.row()   
                row.label(text='', icon='SCULPTMODE_HLT')     
                row.scale_y = 1.2
                row.operator("object.apply_scale_back2sculpt_cbox", text='Sculpt Mode')  



        row = layout.row(align=True)  
        row.operator("object.fill_obj_vertcolor_cbox", text='FillObject')


        bv = bpy.app.version_string
        

        if bv[0] == '3' and bv[2] == '1':
            row.prop(bpy.data.brushes["Draw"], "color", text='' )        
        

        if bv[0] == '3' and bv[2] != '1':
            row.prop(bpy.context.scene, "fill_object_color_cx" )
        

        if bv[0] >= '4':
            row.prop(bpy.context.scene, "fill_object_color_cx" )
        


        # material + vertex color
        row.scale_x = 0.4
        row.operator("object.create_vert_color_mat_cbox", text='M')



#        row = layout.row()     



         


# =====================================================
#              PANEL 001 End  
# =====================================================




# =====================================================
#              PANEL 01
# =====================================================




class PANEL_PT_animbox_ax_01(bpy.types.Panel):
    bl_label = "Origin - Empty - Lattice"
    bl_idname = 'PANEL_PT_animbox_ax_01'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Animbox"
    bl_options = {'DEFAULT_CLOSED'}

    def draw(self, context):
        layout = self.layout

        sc = bpy.context.scene


        text_to_center = 'Origin to Center'
        text_to_top = 'Origin to Top'
        text_to_bottom = 'Origin to Bottom'
        text_to_left = 'Origin to Left'
        text_to_right = 'Origin to Right'
        text_to_front = 'Origin to Front'
        text_to_back = 'Origin to Back'


        # Ref
        to_top_on = sc.origin_to_top_anbx
        to_bottom_on = sc.origin_to_bottom_anbx
        to_left_on = sc.origin_to_left_anbx
        to_right_on = sc.origin_to_right_anbx
        to_front_on = sc.origin_to_front_anbx
        to_back_on = sc.origin_to_back_anbx
 
        #--------------------------

        icon_top = 'ANCHOR_TOP'
        icon_bottom = 'ANCHOR_BOTTOM'
        icon_left = 'ANCHOR_LEFT'
        icon_right = 'ANCHOR_RIGHT'
        icon_front = 'LAYER_ACTIVE'
        icon_back = 'LAYER_USED'
        icon_main = 'PLAY'

        #--------------------------

        if to_top_on == True:
            text = text_to_top
            icon = icon_top


        elif to_bottom_on == True:
            text = text_to_bottom
            icon = icon_bottom


        elif to_left_on == True:
            text = text_to_left
            icon = icon_left


        elif to_right_on == True:
            text = text_to_right
            icon = icon_right


        elif to_front_on == True:
            text = text_to_front
            icon = icon_front


        elif to_back_on == True:
            text = text_to_back
            icon = icon_back


        else:
            text = text_to_center
            icon = icon_main



        #----------------------------------------------------------------
        # Drop to Floor       
        box = layout.box()  
        row = box.row() 
        row.scale_y = 1.2
        row.operator("object.drop_to_floor_anbx", icon='CON_FLOOR')       


        #----------------------------------------------------------------

        box = layout.box()  
        row = box.row()
        row.label(text="Origin")
        row = box.row()
        row.scale_y = 0.6
        row.label(text="Select Origin Placement", icon='CHECKMARK')
        row = box.row(align=True)
        row.scale_x = 1.5
        
        row.enabled = (context.object is not None and context.object.mode == 'OBJECT')
        row.prop(bpy.context.scene, "origin_to_top_anbx", icon='ANCHOR_TOP')
        row.prop(bpy.context.scene, "origin_to_bottom_anbx", icon='ANCHOR_BOTTOM')
        row.prop(bpy.context.scene, "origin_to_left_anbx", icon='ANCHOR_LEFT')
        row.prop(bpy.context.scene, "origin_to_right_anbx", icon='ANCHOR_RIGHT')
        row.prop(bpy.context.scene, "origin_to_front_anbx", icon='LAYER_ACTIVE')
        row.prop(bpy.context.scene, "origin_to_back_anbx", icon='LAYER_USED')        
        
        row = box.row()
        row.scale_y = 0.6
        row.label(text="Optional with Empty", icon='CHECKMARK')
        row = box.row(align=True)
        row.prop(bpy.context.scene, "just_create_empty_anbx", icon='EMPTY_AXIS')
        row.prop(bpy.context.scene, "parent_empty_anbx", text='Parent', icon='EMPTY_AXIS')
        row.prop(bpy.context.scene, "child_empty_anbx", text='Child', icon='EMPTY_AXIS')
        row.scale_x = 1
        row.prop(bpy.context.scene, "empty_to_center_of_sel_anbx", icon='PIVOT_BOUNDBOX')
 

        row = box.row(align=True)
        row.scale_y = 1.2

        row.operator("object.origin_objects_anbx", text=text, icon=icon)
 
        row = box.row(align=True)
        row.scale_y = 1.2
        row.operator("object.origin_to_selected_anbx", text='To Vertex / Edge / Face', icon='PLAY')
 


        box = layout.box()
        row = box.row()
        row.scale_y = 0.8
        row.label(text="Add Empty to Selected as")
        row = box.row(align=True)
        # row.scale_y = 1.2
        row.operator("object.add_parent_empty_2selected_anbx", text='Parent')


        row.operator("object.add_child_empty_2selected_anbx", text='Child')        
        
        row = box.row()
        # row.scale_y = 1.2
        row.operator("object.add_parent_empty_shared_anbx", text='Parent Shared')

        row = box.row()
        # row.scale_y = 1.2
        row.operator("object.add_parent_empty_shared_at_cursor_anbx", text='Parent Shared - at Cursor')



        layout = self.layout
        
        ob = bpy.context.object
        if ob is not None and ob.type == 'EMPTY':
            empty_name = ob.name

            row = layout.row()
            row.scale_y = 1.2 
            row.prop(bpy.data.objects[empty_name], "empty_display_type")              
            row = layout.row()   
            row.scale_y = 1.2 
            row.prop(bpy.data.objects[empty_name], "empty_display_size")  





        # Lattice ------------------------------------------------------------

        box = layout.box()
        row = box.row()
        row.scale_y = 0.6
        row.label(text="Lattice")         
        row = box.row()
        row.scale_y = 1.2
        row.operator("object.add_shared_lattice_to_sel_obj_anbx", 
                     text='Shared', icon='MOD_LATTICE')

        # row = box.row()
        row.operator("object.add_individual_lattice_to_sel_obj_anbx", 
                     text='Individual', icon='MOD_LATTICE')

        
        layout = self.layout
        ob = bpy.context.object
        if ob is not None and ob.type == 'LATTICE':
            lattice_name = ob.name
        
            row = layout.row()
            row.label(text='Lattice Resolution:')     
            row = layout.row(align=True)   
            row.scale_y = 1.2 
            row.prop(bpy.data.lattices[lattice_name], "points_u")  
            row.prop(bpy.data.lattices[lattice_name], "points_v")  
            row.prop(bpy.data.lattices[lattice_name], "points_w")  



        # Select by Name / Type
        row = layout.row()
        row.scale_x = 0.5
        row.label(icon='KEYTYPE_JITTER_VEC')
        row.scale_x = 1.5
        row.operator("object.select_by_name_type_anbx", icon='SORTALPHA')



# =====================================================
#          End  PANEL 01
# =====================================================




# =====================================================
#              PANEL 02
# =====================================================


class PANEL_PT_animbox_ax_02(bpy.types.Panel):
    bl_label = "Smooth Empty / Bone"
    bl_idname = 'PANEL_PT_animbox_ax_02'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Animbox"
    bl_options = {'DEFAULT_CLOSED'}

 

    def draw(self, context):
        layout = self.layout

        box = layout.box()  
 
        row = box.row() 
        row.scale_y = 1.2
        row.operator("object.create_smooth_bone_anbx")
        row = box.row() 
        row.prop(bpy.context.scene, "smooth_empty_bone_parent_to_act_bone_anbx")

        row = box.row() 
        row.scale_y = 1.2
        row.operator("object.create_smooth_empty_hook_anbx")
        row = box.row() 
        row.prop(bpy.context.scene, "smooth_empty_parent_to_parent_obj_anbx")

 
        layout = self.layout
        row = layout.row()         
 
        row.prop(bpy.context.scene, "vertex_group_smooth_repeat_anbx")
        row.scale_x = 2
        row.label(text="Smooth - Iterations")
        row = layout.row() 
        row.prop(bpy.context.scene, "vertex_group_levels_gain_anbx")
        row.scale_x = 2
        row.label(text="Levels - Gain")
    
        layout = self.layout
        
        ob = bpy.context.object
        if ob is not None and ob.type == 'EMPTY':
            empty_name = ob.name

            row = layout.row()   
            row.scale_y = 1.2 
            row.prop(bpy.data.objects[empty_name], "empty_display_size")  


 
# =====================================================
#          End  PANEL 02
# =====================================================


#============================ BONE TOOLS ====================================================
#============================ BONE TOOLS ====================================================
#============================ BONE TOOLS ====================================================


# Bone Tools
# =====================================================
#              PANEL 02_01
# =====================================================

class PANEL_PT_animbox_ax_02_01(bpy.types.Panel):
    bl_label = "Bone Tools"
    bl_idname = 'PANEL_PT_animbox_ax_02_01'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Animbox"
    bl_options = {'DEFAULT_CLOSED'}


    def draw(self, context):
        layout = self.layout
        




# =====================================================
#            End PANEL 02_01
# =====================================================



# Mesh to Bones

# =====================================================
#            End PANEL 02_01_01
# =====================================================

class PANEL_PT_animbox_ax_02_01_01(bpy.types.Panel):
    bl_label = "Mesh to Bone(s)"
    bl_idname = 'PANEL_PT_animbox_ax_02_01_01'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Animbox"
    bl_parent_id = 'PANEL_PT_animbox_ax_02_01'    
    bl_options = {'DEFAULT_CLOSED'}


    def draw(self, context):
        layout = self.layout

        icon_v = 'SEQUENCE_COLOR_07'
        # Check Blender Version
        blender_version = bpy.app.version[0]

        if blender_version == 2:
            icon_v = 'PLAY'

        if blender_version == 3:
            icon_v = 'SEQUENCE_COLOR_07'        
        
        
        box = layout.box() 
        row = box.row() 
        row.scale_y = 0.6
        row.label(text="  Add Mesh to Selected Bones") 
        box = layout.box() 
 
        row = box.row() 
        row.scale_y = 0.6
        row.label(text="  Select Mesh", icon='MESH_DATA')  

        row = box.row(align=True)
        row.scale_x = 2
        row.prop(bpy.context.scene, "mesh_box_to_bone_anbx", icon='MESH_CUBE') 
        row.prop(bpy.context.scene, "mesh_cylinder_to_bone_anbx", icon='MESH_CYLINDER') 
        row.prop(bpy.context.scene, "mesh_sphere_to_bone_anbx", icon='MESH_UVSPHERE') 
        row.prop(bpy.context.scene, "mesh_cone_to_bone_anbx", icon='CONE')  
        row = box.row() 
 
        row.prop(bpy.context.scene, "size_radius_of_mesh_to_bone_anbx")  
        
        row = box.row() 
        row = box.row() 
        row.scale_y = 0.6
        row.label(text='Select Placement on Bone', icon='CENTER_ONLY')
        row = box.row() 
        row.prop(bpy.context.scene, "align_mesh_to_bone_root_anbx", text='Head')  
        row.prop(bpy.context.scene, "align_mesh_to_bone_body_anbx", text='Body')  
        row.prop(bpy.context.scene, "align_mesh_to_bone_tip_anbx", text='Tail')  
        
        row = box.row() 
        row.scale_y = 1.2
        row.operator("object.create_mesh_on_sel_bones_anbx", icon='SEQUENCE_COLOR_07')

        row = box.row() 
        row.prop(bpy.context.scene, "mesh_parent_to_bone_anbx") 






# Select Bone(s)

# =====================================================
#            End PANEL 02_01_02
# =====================================================


class PANEL_PT_animbox_ax_02_01_02(bpy.types.Panel):
    bl_label = "Select"
    bl_idname = 'PANEL_PT_animbox_ax_02_01_02'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Animbox"
    bl_parent_id = 'PANEL_PT_animbox_ax_02_01'    
    bl_options = {'DEFAULT_CLOSED'}


    def draw(self, context):
        layout = self.layout

 
        # Select Bone(s) ------------------------------------------------------------


        box = layout.box() 
 
        row = box.row()
        row.scale_y = 0.6
        row.label(text='Select Bone(s)', icon='RESTRICT_SELECT_OFF')
        row = box.row(align=True)
        row.operator("object.select_similar_by_name_active_bone_anbx", text='Similar')
        row.operator("object.select_bone_children_anbx", text='Children')

        row = box.row(align=True)        
        row.operator("object.select_bone_head_anbx", text='Head')
        row.operator("object.select_bone_tail_anbx", text='Tail')





# Offset Bone Keyframes

# =====================================================
#            PANEL 02_01_03
# =====================================================

class PANEL_PT_animbox_ax_02_01_03(bpy.types.Panel):
    bl_label = "Offset Keyframes"
    bl_idname = 'PANEL_PT_animbox_ax_02_01_03'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Animbox"
    bl_parent_id = 'PANEL_PT_animbox_ax_02_01'    
    bl_options = {'DEFAULT_CLOSED'}


    def draw(self, context):
        layout = self.layout




        # Offset Bone Keyframes -----------------------------------------------------

        box = layout.box() 
        row = box.row()
        row.scale_y = 0.6
        row.label(text='Offset Bone Keyframes', icon='DECORATE_KEYFRAME')        
        row = box.row(align=True)
        row.scale_y = 1.2
        row.operator("object.offset_keyframes_similar_bones_anbx", text='Offset Similar', icon='AUTO')
        row = box.row()
        row.operator("object.offset_keyframes_selected_bones_anbx", text='Offset Selected', icon='RESTRICT_SELECT_ON')
        row = box.row()

        row.prop(bpy.context.scene, "offset_frames_number_bone_anbx")
        row.prop(bpy.context.scene, "revers_anim_bool_anbx")

        #---------------------------- Add Cycle Modifier ---------------------------------

        row = layout.row()        
        row.operator("object.add_cycles_modifier_to_sel_bones_anbx", text='Add Cycles Mod', icon='FCURVE')
           




# Cirlular Array - Bone 

# =====================================================
#            PANEL 02_01_04
# =====================================================


class PANEL_PT_animbox_ax_02_01_04(bpy.types.Panel):
    bl_label = "Cirlular Array"
    bl_idname = 'PANEL_PT_animbox_ax_02_01_04'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Animbox"
    bl_parent_id = 'PANEL_PT_animbox_ax_02_01'    
    bl_options = {'DEFAULT_CLOSED'}


    def draw(self, context):
        layout = self.layout

        # Cirlular Array - Bone

        box = layout.box() 

        row = box.row()
        row.label(text='Circular Array - Bone') 
        row = box.row()
        row.prop(bpy.context.scene, 'circular_array_bone_count_anbx', text='Number of Bones')            

        row = box.row()
        row.scale_y = 0.6
        row.label(text=" Selected:")
        row = box.row(align=True)
        row.scale_y = 1.2

        row.operator("object.circular_array_bone_x_axis_anbx", text='X Sel')
        row.operator("object.circular_array_bone_y_axis_anbx", text='Y Sel')
        row.operator("object.circular_array_bone_z_axis_anbx", text='Z Sel')

        # Auto Circular Array

        row = box.row()
        row.scale_y = 0.6
        row.label(text=" Auto:")
        row = box.row(align=True)
        row.scale_y = 1.2
        row.operator("object.auto_circular_array_bone_x_axis_anbx", text='X Auto')
        row.operator("object.auto_circular_array_bone_y_axis_anbx", text='Y Auto')
        row.operator("object.auto_circular_array_bone_z_axis_anbx", text='Z Auto')



        # MOVE IN OUT --------------------------------------------------------------------------
        row = box.row()
        row.scale_y = 0.6
        row.label(text='Move Selected Bone:') 
        row = box.row(align=True)
        row.operator("object.move_selected_bone_out_anbx", text='Out >')
        row.operator("object.move_selected_bone_in_anbx", text='< In')





# Stretchy Bendy Bones

# =====================================================
#            PANEL 02_01_04
# =====================================================


class PANEL_PT_animbox_ax_02_01_05(bpy.types.Panel):
    bl_label = "Stretchy BBones"
    bl_idname = 'PANEL_PT_animbox_ax_02_01_05'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Animbox"
    bl_parent_id = 'PANEL_PT_animbox_ax_02_01'    
    bl_options = {'DEFAULT_CLOSED'}


    def draw(self, context):
        layout = self.layout



        box = layout.box()  
        row = box.row()  
        
        row.scale_x = 1
        row.prop(bpy.context.scene, "bbone_count_anbx", text='Segments', )        
        row = box.row()
        row.scale_y = 1.2
        row.operator("object.bendy_bones_anbx", icon='BONE_DATA')





#============================ BONE TOOLS END====================================================
#============================ BONE TOOLS END====================================================
#============================ BONE TOOLS END====================================================




# =====================================================
#              PANEL 02.02
# =====================================================

# class PANEL_PT_animbox_ax_02_02(bpy.types.Panel):
#     bl_label = ""
#     bl_idname = 'PANEL_PT_animbox_ax_02_02'
#     bl_space_type = 'VIEW_3D'
#     bl_region_type = 'UI'
#     bl_category = "Animbox"
#     bl_options = {'DEFAULT_CLOSED'}


#     def draw(self, context):
#         layout = self.layout


#         box = layout.box()  
#         row = box.row()  
        



# =====================================================
#            End PANEL 02.02
# =====================================================




# =====================================================
#              PANEL 03
# =====================================================


class PANEL_PT_animbox_ax_03(bpy.types.Panel):
    bl_label = "Color - Lights"
    bl_idname = 'PANEL_PT_animbox_ax_03'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Animbox"
    bl_options = {'DEFAULT_CLOSED'}

    def draw(self, context):
        layout = self.layout


        mat_object_types = ['CURVE', 'MESH', 'FONT', 'SURFACE', 'META']


        # add Material - 'Principled BSDF'
        row = layout.row()
        row.label(text="", icon='MATERIAL')
        row.scale_y = 1.2
        row.scale_x = 2
        row.operator("object.set_default_material_anbx", text="+ Material")


        # Material Transparency - 'Principled BSDF'
        row = layout.row()   
        row.label(text="", icon='NODE_MATERIAL')

        row.enabled = (context.object is not None and
                        context.object.type in mat_object_types 
                        and len(context.selected_objects) != 0
                         and len(context.object.data.materials) != 0)
        row.scale_x = 2
        row.prop(bpy.context.scene, 'change_alpha_value_anbx')        

 
        # Diffuse / Emission Color Picker
        grid = layout.grid_flow(columns=2, align=True)
 
        grid.enabled = (context.object is not None and
                        context.object.type in mat_object_types 
                        and len(context.selected_objects) != 0
                         and len(context.object.data.materials) != 0)
        grid.prop(bpy.context.scene, "diffuse_color_changer_anbx") 
        grid.prop(bpy.context.scene, "emission_color_anbx") 


        grid = layout.grid_flow(columns=1, align=True)
        grid.enabled = (context.object is not None and
                        context.object.type in mat_object_types 
                        and len(context.selected_objects) != 0
                         and len(context.object.data.materials) != 0)

        # Emission Strength
        grid.prop(bpy.context.scene, 'emission_strength_anbx')


        box = layout.box()
        row = box.row()
        # row.scale_y = 0.5
        # row.label(text="Shift")
        row = box.row(align=True)

        row.enabled = (context.object is not None and
                        context.object.type in mat_object_types 
                        and len(context.selected_objects) != 0)
 

        # row.prop(bpy.context.scene, 'shift_color_factor')   
        # row = box.row()
        row.label(text="Shift ->")
        row.scale_x = 2
        row.operator("object.offset_colors_anbx", text='Color')
          
               
        row.operator("object.offset_colors_random_anbx", text='Random')
        row = box.row()
        row.prop(bpy.context.scene, 'shift_color_factor')
        row.prop(bpy.context.scene, 'revers_color_bool')  



        box = layout.box()
        row = box.row()
        row.scale_y = 0.5
        row.label(text="Material Color Random", icon='MATERIAL')
        row = box.row(align=True)
        row.operator("object.diffuse_color_random_anbx", text='Diffuse')

        row.operator("object.emission_color_random_anbx", text='Emission')



        box = layout.box()
        row = box.row()
        # row.scale_y = 0.5
        row.label( icon='LIGHT')
        # row = box.row()

        row.enabled = (context.object is not None 
                       and len(context.selected_objects) != 0
                       and context.object.type == 'LIGHT')

        row.prop(bpy.context.scene, "light_intensity_anbx" )
        row = box.row()
        
        row.enabled = (context.object is not None 
                       and len(context.selected_objects) != 0
                       and context.object.type == 'LIGHT')

        row.prop(bpy.context.scene, "light_color_anbx", text='' )


        # row = box.row()
        row.operator("object.light_color_random_anbx", text='Random')

        # Aditional with Keyframe(s)
        # box = layout.box() 
        row = layout.row() 
        row.label(text="Auto Keyframes", icon='HANDLETYPE_FREE_VEC')
        row.scale_x = 2
        row.prop(bpy.context.scene, "mat_light_add_keyframe_anbx", icon='KEY_HLT')



        # Select by Name / Type
        row = layout.row()
        row.scale_x = 0.5
        row.label(icon='KEYTYPE_JITTER_VEC')
        row.scale_x = 1.5
        row.operator("object.select_by_name_type_anbx", icon='SORTALPHA')



# =====================================================
#          End  PANEL 03
# =====================================================





# =====================================================
#              PANEL 04
# =====================================================


class PANEL_PT_animbox_ax_04(bpy.types.Panel):
    bl_label = "Motion Tools"
    bl_idname = 'PANEL_PT_animbox_ax_04'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Animbox"
    bl_options = {'DEFAULT_CLOSED'}

    def draw(self, context):
        layout = self.layout

 

 

# =====================================================
#              PANEL 03_01 Curve
# =====================================================


class PANEL_PT_animbox_ax_03_01(bpy.types.Panel):
    bl_label = "Curve"
    bl_idname = 'PANEL_PT_animbox_ax_03_01'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Animbox"
    bl_parent_id = 'PANEL_PT_animbox_ax_04'
    bl_options = {'DEFAULT_CLOSED'}
    bl_order = 1

    def draw(self, context):
        layout = self.layout



        box = layout.box()
        row = box.row()
        row.scale_y = 0.5
        row.label(text="Motion Presets", icon='PRESET')      
        row = box.row()
        row.scale_y = 1.2

        row.operator("object.circle_explode_anbx", icon='PROP_CON')
        row.prop(bpy.context.scene, 'align_to_view_bool_mdh')  
        row = box.row()
        row.scale_y = 1.2
        row.operator("object.star_burst_anbx", icon='SORTBYEXT')


        box = layout.box()
        row = box.row()
        row.scale_y = 0.5
        row.label(text="Animated Curve Bevel")         
        row = box.row()

        row.operator("object.anim_curve_bevel_closed_anbx")
        row = box.row()
        row.operator("object.anim_curve_bevel_open_anbx")


        box = layout.box()
        row = box.row()
        ob_bevel_depth = ('MESH', 'CURVE')
        row.enabled = (context.object is not None and
                       context.object.type in ob_bevel_depth)
        row.prop(bpy.context.scene, "change_bevel_depth_value_anbx")    
 

        # Flow Circle
        box = layout.box()
        row = box.row()
        row.scale_y = 1.2
        row.label(text="", icon='ANTIALIASED')  
        row.operator("object.trim_circle_anbx")
 


# =====================================================
#          End  PANEL 03_01 
# =====================================================


# =====================================================
#              PANEL 03_01_02 Shape Keys
# =====================================================


class PANEL_PT_animbox_ax_03_01_02(bpy.types.Panel):
    bl_label = "Shape Keys"
    bl_idname = 'PANEL_PT_animbox_ax_03_01_02'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Animbox"
    bl_parent_id = 'PANEL_PT_animbox_ax_04'
    bl_options = {'DEFAULT_CLOSED'}
    bl_order = 2

    def draw(self, context):
        layout = self.layout


        # Shape_Key Objects:
        shape_key_objects = ['MESH', 'CURVE', 'SURFACE', 'LATTICE']

 

        box = layout.box()
        row = box.row()
        row.scale_y = 0.5
        row.label(text="Shape Keys")
        row = box.row()
        row.operator("object.add_shape_key_to_sel_anbx", icon='ADD')

        row = box.row()

        row.enabled = (len(context.selected_objects) != 0 and
                 context.object.type in shape_key_objects and 
                 context.object.data.shape_keys is not None)  

        row.prop(bpy.context.scene, 'shape_key_value_active_anbx')
        row.operator("object.remove_shape_key_active_index_anbx", text='', icon='TRASH')

        row = box.row()

        row.enabled = (len(context.selected_objects) != 0 and
                 context.object.type in shape_key_objects and 
                 context.object.data.shape_keys is not None)  

        row.prop(bpy.context.scene, 'shape_key_value_all_anbx')
        row.operator("object.remove_shape_key_all_anbx", text='', icon='TRASH')



 
# =====================================================
#             End PANEL 03_01_02  
# =====================================================


# =====================================================
#              PANEL 03_01_03 Drivers
# =====================================================


class PANEL_PT_animbox_ax_03_01_03(bpy.types.Panel):
    bl_label = "Drivers"
    bl_idname = 'PANEL_PT_animbox_ax_03_01_03'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Animbox"
    bl_parent_id = 'PANEL_PT_animbox_ax_04'
    bl_options = {'DEFAULT_CLOSED'}
    bl_order = 3

    def draw(self, context):
        layout = self.layout



        box = layout.box()  
        row = box.row() 
        row.scale_y = 0.5
        row.label(text='Select Axis:')

        row = box.row(align=True) 
        row.scale_y = 1.2


        row.prop(bpy.context.scene, 'driver_axis_x_anbx', text='X')
        row.prop(bpy.context.scene, 'driver_axis_y_anbx', text='Y')
        row.prop(bpy.context.scene, 'driver_axis_z_anbx', text='Z')
    
        row = box.row() 
        row.label(text='Sine Wave / Offset:')
        row.prop(bpy.context.scene, 'driver_sine_wave_anbx', icon='FORCE_HARMONIC' )
        row.prop(bpy.context.scene, 'driver_offset_motion_anbx', icon='ONIONSKIN_ON' )
 
        

        box = layout.box()  
        row = box.row()
        row.scale_y = 0.6
        row.label(text="Rotate Around",)  
        row = box.row()
        row.scale_y = 1
        row.operator("object.driver_rotate_around_self_anbx", text='Origin')
        row.operator("object.driver_rotate_around_active_anbx", text='Active')
        row.operator("object.driver_rotate_around_cursor_anbx", text='Cursor')
        row.operator("object.remove_rotation_driver_anbx", text='', icon='TRASH')
 


        box = layout.box()  
        row = box.row()
        row.operator("object.driver_location_anbx")
        row.operator("object.remove_location_driver_anbx", text='', icon='TRASH')
        row = box.row()
        row.operator("object.driver_scale_anbx")
        row.operator("object.remove_scale_driver_anbx", text='', icon='TRASH')

        box = layout.box()
        row = box.row()
        row.prop(bpy.context.scene, 'driver_offset_motion_amplitude_anbx' )
        row = box.row()

        row.prop(bpy.context.scene, 'driver_speed_anbx')        
        
        row = box.row()
        
        row.prop(bpy.context.scene, 'driver_offset_speed_anbx' )
 
    

# =====================================================
#             End PANEL 03_01_03 
# =====================================================


# =====================================================
#              PANEL 03_01_04 F-Curve Modifiers
# =====================================================


# class PANEL_PT_animbox_ax_03_01_04(bpy.types.Panel):
#     bl_label = ""
#     bl_idname = 'PANEL_PT_animbox_ax_03_01_04'
#     bl_space_type = 'VIEW_3D'
#     bl_region_type = 'UI'
#     bl_category = "Animbox"
#     bl_parent_id = 'PANEL_PT_animbox_ax_03'
#     bl_options = {'DEFAULT_CLOSED'}

#     def draw(self, context):
#         layout = self.layout


# =====================================================
#             End PANEL 03_01_04 
# =====================================================



#========================= Cycle Modifier =============

# Panel "Cycle Modifier"

class PANEL_PT_animbox_ax_03_01_05(bpy.types.Panel):
    bl_label = "Cycle Modifier - F-Curve"
    bl_idname = 'PANEL_PT_animbox_ax_03_01_05'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Animbox"
    bl_parent_id = 'PANEL_PT_animbox_ax_04'
    bl_options = {'DEFAULT_CLOSED'}
    bl_order = 4

    def draw(self, context):
        layout = self.layout
        scene = context.scene


        box = layout.box()  
        row = box.row() 

        row.prop(bpy.context.scene, 'cycle_modifier_fcurve_location_anbx', text='Location')
        row = box.row()

        row.prop(bpy.context.scene, 'cycle_modifier_fcurve_rotation_anbx', text='Rotation' )
        row = box.row()

        row.prop(bpy.context.scene, 'cycle_modifier_fcurve_scale_anbx', text='Scale')
        row = box.row() 

        row.prop(bpy.context.scene, 'cycle_modifier_fcurve_lights_anbx',  text='Light Power' )
        row = box.row()

        row.prop(bpy.context.scene, 'cycle_modifier_fcurve_materials_anbx', text='Material')
        row = box.row()  

        row.prop(bpy.context.scene, 'cycle_modifier_fcurve_shape_keys_anbx', text='Shape Keys' )
        row = box.row()  

        row.prop(bpy.context.scene, 'cycle_modifier_fcurve_curve_bevel_anbx', text='Curve Bevel')
         
         
        box = layout.box()  
        row = box.row()          
        row.scale_y = 1.2
        row.operator("object.add_remove_cycle_modifier_anbx", text='Cycle', icon='ADD')
 
 

        #--------------------------------------

        # Pre_Post Infinity Cycle
        box = layout.box()
        row = box.row()
        row.enabled = (context.object is not None )
 
        row.scale_y = 1.2
        row.operator("object.add_remove_infinity_cycle_anbx", text='Cycle Smart', icon='FILE_REFRESH')
 
        #--------------------------------------
        



# Panel "Cycle Modifier"

# =====================================================
#             End PANEL 03_01_05
# =====================================================



 
# Panel Noise Modifier


class PANEL_PT_animbox_ax_03_01_06(bpy.types.Panel):
    bl_label = "Noise Modifier - F-Curve"
    bl_idname = 'PANEL_PT_animbox_ax_03_01_06'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Animbox"
    bl_parent_id = 'PANEL_PT_animbox_ax_04'
    bl_options = {'DEFAULT_CLOSED'}
    bl_order = 5

    def draw(self, context):
        layout = self.layout
        scene = context.scene


        box = layout.box()  
        row = box.row() 
        row.scale_y = 0.5
        row.label(text='Select Axis:')

        row = box.row(align=True) 
        row.scale_y = 1.2
        row.prop(bpy.context.scene, 'noise_modifier_fcurve_x_axis_anbx', text='X')
        row.prop(bpy.context.scene, 'noise_modifier_fcurve_y_axis_anbx', text='Y')
        row.prop(bpy.context.scene, 'noise_modifier_fcurve_z_axis_anbx', text='Z')
        
        row = box.row()
        row.scale_y = 0.5
        row.label(text='Select Properties:')
        row = box.row(align=True)  
        row.scale_y = 1.2
        row.prop(bpy.context.scene, 'noise_modifier_fcurve_location_anbx', text='Loc')
        row.prop(bpy.context.scene, 'noise_modifier_fcurve_rotation_anbx', text='Rot')
        row.prop(bpy.context.scene, 'noise_modifier_fcurve_scale_anbx', text='Scale')



        box = layout.box()  
        row = box.row() 
        row.label(text='Offset Option:')       
        row.prop(bpy.context.scene, 'noise_modifier_fcurve_offset_per_obj_anbx', icon='ONIONSKIN_ON' )
        row.prop(bpy.context.scene, 'noise_modifier_fcurve_offset_reversed_anbx', icon='LOOP_BACK' )


        box = layout.box()  
        row = box.row()
        row.label(text='Additional Properties:') 
        row.scale_y = 0.5
       
        row = box.row()    
        row.prop(bpy.context.scene, 'noise_modifier_fcurve_light_power_anbx', text='Light Power')
        row = box.row()        
        row.prop(bpy.context.scene, 'noise_modifier_fcurve_shape_key_value_anbx', text='Shape Key Value')
        row = box.row()             
        row.prop(bpy.context.scene, 'noise_modifier_fcurve_bevel_depth_anbx', text='Curve Bevel Depth')
        

        box = layout.box()  
        row = box.row(align=True)           
        row.scale_y = 1.5
        row.operator("object.add_noise_fcurve_modifier_anbx", text='ADD')
        row.operator("object.remove_noise_fcurve_modifier_anbx", text='REMOVE')
        row.operator("object.reset_settings_noise_fcurve_modifier_anbx", text='RESET')

        #------------------------------------------------------------------------------------


        box = layout.box()  
        row = box.row()         
        row.label(text="Scale")
        row.enabled = (context.object is not None)  
        row.scale_x = 2        
        row.prop(bpy.context.scene, 'noise_fcurve_mod_scale_anbx' )
        row = box.row() 
        row.label(text="Strength") 
        row.enabled = (context.object is not None) 
        row.scale_x = 2       
        row.prop(bpy.context.scene, 'noise_fcurve_mod_strength_anbx' )
        
     
        row = box.row() 
        row.label(text="Offset") 
        row.enabled = (context.object is not None) 
        row.scale_x = 2       
        row.prop(bpy.context.scene, 'noise_fcurve_mod_offset_anbx' )
        
     
        row = box.row() 
        row.label(text="Phase") 
        row.enabled = (context.object is not None) 
        row.scale_x = 2       
        row.prop(bpy.context.scene, 'noise_fcurve_mod_phase_anbx' )
               
        row = box.row() 
        row.label(text="Depth") 
        row.enabled = (context.object is not None) 
        row.scale_x = 2       
        row.prop(bpy.context.scene, 'noise_fcurve_mod_depth_anbx' )
                     
        row = layout.row() 
 
        row.label(text="Influence")
        row.enabled = (context.object is not None)         
 
        row.prop(bpy.context.scene, 'noise_use_influence_anbx' )
        row.prop(bpy.context.scene, 'noise_influence_anbx' )
 
 
# Panel Noise Modifier


# =====================================================
#             End PANEL 03_01_06
# =====================================================


# Panel "restrict_frame_range"

class PANEL_PT_animbox_ax_03_01_07(bpy.types.Panel):
    bl_label = ""
    bl_idname = 'PANEL_PT_animbox_ax_03_01_07'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Animbox"
    bl_parent_id = 'PANEL_PT_animbox_ax_03_01_06'
    bl_options = {'DEFAULT_CLOSED'}

    def draw_header(self,context):
        layout = self.layout
        layout.enabled = (context.object is not None) 

        self.layout.prop(bpy.context.scene, "noise_use_restricted_range_anbx", text="Restrict Frame Range")


    def draw(self, context):
        layout = self.layout
        scene = context.scene
        
        layout.enabled = (context.object is not None)         
        
        row = layout.row()
        row.label(text="Start" )
        row.scale_x = 2 
        row.enabled = scene.noise_use_restricted_range_anbx
        row.prop(bpy.context.scene, 'noise_restricted_frame_start_anbx' )
        
        row = layout.row()
        row.label(text="End" )
        row.scale_x = 2 
        row.enabled = scene.noise_use_restricted_range_anbx
        row.prop(bpy.context.scene, 'noise_restricted_frame_end_anbx' )
        
        row = layout.row()
        
        row = layout.row()
        row.label(text="Blend In" )
        row.scale_x = 2 
        row.enabled = scene.noise_use_restricted_range_anbx
        row.prop(bpy.context.scene, 'noise_restricted_blend_in_anbx' )
        

        row = layout.row()
        row.label(text="Out" )
        row.scale_x = 2 
        row.enabled = scene.noise_use_restricted_range_anbx
        row.prop(bpy.context.scene, 'noise_restricted_blend_out_anbx' )
        
        
# Panel "restrict_frame_range"        

# =====================================================
#             End PANEL 03_01_06
# =====================================================








# DUPLICATION DUPLICATION DUPLICATION DUPLICATION

# =====================================================
#              PANEL 03_02 Duplication
# =====================================================


class PANEL_PT_animbox_ax_03_02(bpy.types.Panel):
    bl_label = "Duplication"
    bl_idname = 'PANEL_PT_animbox_ax_03_02'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Animbox"
    bl_options = {'DEFAULT_CLOSED'}

    def draw(self, context):
        layout = self.layout


        # Copy Selected at 3D Cursor
        box = layout.box()
        row = box.row()
        row.scale_y = 1.2
        row.operator("object.copy_selected_at_cursor_anbx", icon='DUPLICATE')



# =====================================================
#              PANEL 05
# =====================================================


class PANEL_PT_animbox_ax_05(bpy.types.Panel):
    bl_label = "Obj2Curve"
    bl_idname = 'PANEL_PT_animbox_ax_05'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Animbox"
    bl_parent_id = 'PANEL_PT_animbox_ax_03_02'
    bl_options = {'DEFAULT_CLOSED'}

    def draw(self, context):
        layout = self.layout


        # Object to Curve

        row = layout.row()  
        row.scale_x = 1
        row.label(text='Objects',icon='FORCE_CHARGE')
        
        # Object Count
        row.scale_x = 0.8
        row.enabled = (len(context.selected_objects) != 0) 
        row.prop(bpy.context.scene, 'dupl_object2curve_count')   
        row = layout.row()
        
        # Speed
        row.scale_x = 1
        row.label(text='Curve Speed',icon='FORCE_CHARGE')

        row.scale_x = 0.8
        row.enabled = (len(context.selected_objects) != 0
                       and context.object.type == 'CURVE') 
        row.prop(bpy.context.scene, 'curve_frames_anim_speed')   
        row = layout.row()


        # Obj2Curve & Animate
        box = layout.box()
        row = box.row()
        row.scale_y = 1.25
        row.enabled = (len(context.selected_objects) > 1)
        row.operator("object.object_to_curve_anim_anbx", icon='HEART')

        row.scale_x = 1.15 
        row.prop(bpy.context.scene, 'obj_axis_x_y_z_follow_curve')   
    


        # Switch Direction
        box = layout.box()
        box.operator("object.switch_curve_direction_anbx", icon='CENTER_ONLY')
 
        
        # Select by Name / Type
        # row = box.row()
        row = layout.row()
        row.scale_x = 0.5
        row.label(icon='KEYTYPE_JITTER_VEC')
        row.scale_x = 1.5
        row.operator("object.select_by_name_type_anbx", icon='SORTALPHA')


# =====================================================
#          End  PANEL 05
# =====================================================




# =====================================================
#              PANEL 06
# =====================================================


class PANEL_PT_animbox_ax_06(bpy.types.Panel):
    bl_label = "Circular Array"
    bl_idname = 'PANEL_PT_animbox_ax_06'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Animbox"
    bl_parent_id = 'PANEL_PT_animbox_ax_03_02'
    bl_options = {'DEFAULT_CLOSED'}

    def draw(self, context):
        layout = self.layout

        ob_types = ['MESH', 'CURVE', 'SURFACE', 'META', 
                    'FONT',  'ARMATURE', 'LATTICE', ]
 

        box = layout.box()  
        row = box.row()
        row.scale_y = 0.6
        row.label(text="Select Axis", icon='CHECKMARK')
        row = box.row()
        row.prop(bpy.context.scene, "circular_array_x_bool_anbx", text='X')
        row.prop(bpy.context.scene, "circular_array_y_bool_anbx", text='Y')
        row.prop(bpy.context.scene, "circular_array_z_bool_anbx", text='Z')
        
        row = box.row()
        row.enabled = (len(context.selected_objects) != 0 and
                         context.object.type in ob_types)

        row.prop(bpy.context.scene, 'array_object_count_anbx', text='Number of Objects')        
        
        row = box.row()
        row.scale_y = 1.2
        row.operator("object.circular_array_around_self_origin_anbx")
        row = box.row()
        row.operator("object.circular_array_around_cursor_anbx")
        row = box.row()        
        row.operator("object.circular_array_around_active_anbx")

        row = layout.row()    
        
         
        #-----------------------------------  
        # Select by Name / Type
        row = layout.row()
        row.scale_x = 0.5
        row.label(icon='KEYTYPE_JITTER_VEC')
        row.scale_x = 1.5
        row.operator("object.select_by_name_type_anbx", icon='SORTALPHA')



# =====================================================
#          End  PANEL 06
# =====================================================






# =====================================================
#              PANEL 07
# =====================================================


class PANEL_PT_animbox_ax_07(bpy.types.Panel):
    bl_label = "Copy Active to Selected"
    bl_idname = 'PANEL_PT_animbox_ax_07'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Animbox"
    bl_parent_id = 'PANEL_PT_animbox_ax_03_02'
    bl_options = {'DEFAULT_CLOSED'}

    def draw(self, context):
        layout = self.layout


        grid = layout.grid_flow(columns=1, align=True, )
        grid.scale_y = 1
        grid.operator("object.copy_active_to_selected_center_anbx", text='To Origin', icon='ALIGN_MIDDLE')
        
        grid = layout.grid_flow(columns=2, align=True, )

        grid.operator("object.copy_active_to_selected_center_child_anbx", text='As Child', icon='ALIGN_MIDDLE')
        grid.operator("object.copy_active_to_selected_center_parent_anbx", text='As Parent', icon='ALIGN_MIDDLE')
        
        grid = layout.grid_flow(columns=1, align=True, )
        grid.scale_y = 1
        grid.operator("object.copy_active_to_sel_with_copyloc_constr_anbx", text='Add Copy Location Constraint')

 
        grid = layout.grid_flow(columns=1, align=True, )
        grid = layout.grid_flow(columns=1, align=True, )
        grid.scale_y = 1
  

        grid.operator("object.copy_active_to_selected_top_anbx", text='On Top', icon='ALIGN_TOP')

        grid = layout.grid_flow(columns=2, align=True, )

        grid.operator("object.copy_active_to_selected_top_child_anbx", text='As Child', icon='ALIGN_TOP')
        grid.operator("object.copy_active_to_selected_top_parent_anbx", text='As Parent', icon='ALIGN_TOP')

        grid = layout.grid_flow(columns=1, align=True, )
        grid = layout.grid_flow(columns=1, align=True, )
        grid.scale_y = 1
        grid.operator("object.copy_active_to_selected_bottom_anbx", text='To Bottom', icon='ALIGN_BOTTOM')


        grid = layout.grid_flow(columns=2, align=True, )
        grid.operator("object.copy_active_to_selected_bottom_child_anbx", text='As Child', icon='ALIGN_BOTTOM')
        grid.operator("object.copy_active_to_selected_bottom_parent_anbx", text='As Parent', icon='ALIGN_BOTTOM')


        grid = layout.grid_flow(columns=1, align=True, )
        grid = layout.grid_flow(columns=1, align=True, )
        grid.operator("object.copy_active_to_vertex_anbx", text='To Vertex / Vertices', icon='NORMALS_VERTEX')
        grid.operator("object.copy_active_to_faces_anbx", text='To Face(s)', icon='NORMALS_FACE')
        
        
 
        # Select by Name / Type
        row = layout.row()
        row.scale_x = 0.5
        row.label(icon='KEYTYPE_JITTER_VEC')
        row.scale_x = 1.5
        row.operator("object.select_by_name_type_anbx", icon='SORTALPHA')


 
# =====================================================
#          End  PANEL 07
# =====================================================





#  OFFSET OVERSHOOT OFFSET OVERSHOOT OFFSET OVERSHOOT


 
# =====================================================
#              PANEL 08
# =====================================================


class PANEL_PT_animbox_ax_08(bpy.types.Panel):
    bl_label = "Offset / Overshoot"
    bl_idname = 'PANEL_PT_animbox_ax_08'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Animbox"
    bl_options = {'DEFAULT_CLOSED'}

    def draw(self, context):
        layout = self.layout



# =====================================================
#              END PANEL 08
# =====================================================

# =====================================================
#              END PANEL 08_01
# =====================================================

class PANEL_PT_animbox_ax_08_01(bpy.types.Panel):
    bl_label = "Offset Transformation"
    bl_idname = 'PANEL_PT_animbox_ax_08_01'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Animbox"
    bl_parent_id = 'PANEL_PT_animbox_ax_08'
    bl_options = {'DEFAULT_CLOSED'}


    def draw(self, context):
        layout = self.layout

            
        row = layout.row() 
        row.scale_y = 1.2
        
        row.prop(bpy.context.scene, 'offset_amount_anbx')  
 
          
        row = layout.row(align=True)
        # grid = layout.grid_flow(columns=5, align=True)
 
 
        row.prop(bpy.context.scene, 'offset_transform_axis_x_anbx', text='X') 
        row.prop(bpy.context.scene, 'offset_transform_axis_y_anbx', text='Y') 
        row.prop(bpy.context.scene, 'offset_transform_axis_z_anbx', text='Z') 
        row.scale_x = 1.2
        row.prop(bpy.context.scene, 'offset_transform_neg_axis_anbx',   icon='REMOVE') 
        row.prop(bpy.context.scene, 'revers_transform_offset_anbx', icon='LOOP_BACK')
       
        grid = layout.grid_flow(columns=3, align=True)
   
        grid.scale_y = 1.3

        grid.operator('object.offset_transform_location_anbx', text='Loc')
        grid.operator('object.offset_transform_rotation_anbx', text='Rot')
        grid.operator('object.offset_transform_scale_anbx', text='Scale')
 

   
        # Select by Name / Type
        row = layout.row()
        row.scale_x = 0.5
        row.label(icon='KEYTYPE_JITTER_VEC')
        row.scale_x = 1.5
        row.operator("object.select_by_name_type_anbx", icon='SORTALPHA')


 
# =====================================================
#          End  PANEL 08_01
# =====================================================




# =====================================================
#              PANEL 09
# =====================================================


class PANEL_PT_animbox_ax_09(bpy.types.Panel):
    bl_label = "Offset Animation"
    bl_idname = 'PANEL_PT_animbox_ax_09'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Animbox"
    bl_parent_id = 'PANEL_PT_animbox_ax_08'
    bl_options = {'DEFAULT_CLOSED'}
 

    def draw(self, context):
        layout = self.layout


        row = layout.row() 

        row.enabled = (context.object is not None)
        row.prop(bpy.context.scene, 'offset_frames_number_anbx', text='Frames to Offset')
        # Reverse 
        row.prop(bpy.context.scene, 'revers_anim_bool_mdh')
        row = layout.row()
        row.scale_y = 1.2
        row.operator("object.offset_smart_anbx", text='Offset Smart', icon='ONIONSKIN_ON')

        # Frames Number
        row = layout.row()
        row = layout.row()
        row.scale_y = 1.2

        row.operator("object.offset_loc_rot_scale_anbx", text='Location Rotation Scale', icon='ONIONSKIN_ON')
        # Offset Separate
        row = layout.row(align=True)
        row.scale_y = 1.2
        row.operator("object.offset_location_anbx", text='Location')
        row.operator("object.offset_rotation_anbx", text='Rotation')
        row.operator("object.offset_scale_anbx", text='Scale')
        
        row = layout.row()
        row = layout.row(align=True)
        row.scale_y = 1.2
        row.operator("object.offset_shape_key_anbx", text='Shape Keys')
        row.operator("object.offset_animated_curve_bevel_anbx", text='Curve Bevel')
        row = layout.row(align=True)
        row.scale_y = 1.2
        row.operator("object.offset_animated_material_anbx", text='Materials')
        row.operator("object.offset_animated_lights_anbx", text='Lights')

 




        #--------------------------------------

        
        # Select by Name / Type
        
        row = layout.row()
        row.scale_x = 0.5
        row.label(icon='KEYTYPE_JITTER_VEC')
        row.scale_x = 1.5
        row.operator("object.select_by_name_type_anbx", icon='SORTALPHA')



# =====================================================
#            End  PANEL 09
# =====================================================






# =====================================================
#              PANEL 10
# =====================================================


class PANEL_PT_animbox_ax_10(bpy.types.Panel):
    bl_label = "Overshoot Animation"
    bl_idname = 'PANEL_PT_animbox_ax_10'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Animbox"
    bl_parent_id = 'PANEL_PT_animbox_ax_08'
    bl_options = {'DEFAULT_CLOSED'}
 

    def draw(self, context):
        layout = self.layout

    
        # Location

        box = layout.box()  
        row = box.row()
        row.label(text="Overshoot LOCATION", icon='PHYSICS')  
        row = box.row()        
        row.label(text="", icon='DECORATE_KEYFRAME')
        row.prop(bpy.context.scene, 'duration_frames_number_loc_anbx')
   
        row = box.row()
        row.prop(bpy.context.scene, 'overshoot_loc_amplitude_anbx')
        row = box.row()
        row.prop(bpy.context.scene, 'overshoot_loc_frequency_anbx')
        row = box.row()
        row.prop(bpy.context.scene, 'overshoot_loc_decay_anbx')
        row = box.row()
        row.prop(bpy.context.scene, 'overshoot_loc_x_anbx', text='', icon='EVENT_X' )
        row.prop(bpy.context.scene, 'overshoot_loc_y_anbx', text='', icon='EVENT_Y' )
        row.prop(bpy.context.scene, 'overshoot_loc_z_anbx', text='', icon='EVENT_Z' )

        row.scale_x = 1.2
        
        # row = box.row()        
        row.prop(bpy.context.scene, 'overshoot_loc_negative_direction_anbx', text='Direction', icon='REMOVE' )
   


        row = box.row()
        row.scale_y = 1.2
        row.operator("object.overshoot_location_anbx", text='Bake Location', icon='PLAY')
        row = layout.row()         
        


        # Rotation

        box = layout.box()  
        row = box.row()
        row.label(text="Overshoot ROTATION", icon='PHYSICS')  
        row = box.row()
        row.label(text="", icon='DECORATE_KEYFRAME')
        row.prop(bpy.context.scene, 'duration_frames_number_rot_anbx')
        row = box.row()
        row.prop(bpy.context.scene, 'overshoot_rot_amplitude_anbx')
        row = box.row()
        row.prop(bpy.context.scene, 'overshoot_rot_frequency_anbx')
        row = box.row()
        row.prop(bpy.context.scene, 'overshoot_rot_decay_anbx')
        row = box.row()
        row.prop(bpy.context.scene, 'overshoot_rot_x_anbx', icon='EVENT_X' )
        row.prop(bpy.context.scene, 'overshoot_rot_y_anbx', icon='EVENT_Y' )
        row.prop(bpy.context.scene, 'overshoot_rot_z_anbx', icon='EVENT_Z' )
        row.prop(bpy.context.scene, 'overshoot_cos_anbx', text='Cos', icon='DRIVER_ROTATIONAL_DIFFERENCE' )
        row = box.row()
        row.scale_y = 1.2
        row.operator("object.overshoot_rotation_anbx", text='Bake Rotation', icon='PLAY')
    

        # Scale

        box = layout.box()  
        row = box.row()
        row.label(text="Overshoot SCALE", icon='PHYSICS')  
        row = box.row()
                
        row.label(text="", icon='DECORATE_KEYFRAME')
        row.prop(bpy.context.scene, 'duration_frames_number_scale_anbx')
   
        row = box.row()
        row.prop(bpy.context.scene, 'overshoot_scale_amplitude_anbx')
        row = box.row()
        row.prop(bpy.context.scene, 'overshoot_scale_frequency_anbx')
        row = box.row()
        row.prop(bpy.context.scene, 'overshoot_scale_decay_anbx')
        row = box.row()
        row.prop(bpy.context.scene, 'overshoot_scale_x_anbx', text='S', icon='EVENT_X' )
        row.prop(bpy.context.scene, 'overshoot_scale_y_anbx', text='S', icon='EVENT_Y' )
        row.prop(bpy.context.scene, 'overshoot_scale_z_anbx', text='S', icon='EVENT_Z' )
           
        row = box.row()
        row.operator("object.overshoot_scale_anbx", text='Bake Scale', icon='PLAY')

        


 
# =====================================================
#          End  PANEL 10
# =====================================================







# =====================================================
#              PANEL 11
# =====================================================


class PANEL_PT_animbox_ax_11(bpy.types.Panel):
    bl_label = "Selection / Visibility"
    bl_idname = 'PANEL_PT_animbox_ax_11'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Animbox"
    bl_options = {'DEFAULT_CLOSED'}
 

    def draw(self, context):
        layout = self.layout


        # Parent Row

        box = layout.box()  
        row = box.row()      
        row.scale_x = 0.8
        row.label(icon='COMMUNITY')
        row.scale_x = 1
        row.operator("object.set_parent_sequence_row_anbx")


        row.enabled = (len(context.selected_objects) > 1)                
        row.prop(bpy.context.scene, 'revers_parent_row_mdh')   

 
        box = layout.box() 
 
        row = box.row()   
        row.scale_x = 0.5
        row.label(icon='KEYTYPE_JITTER_VEC')
        row.scale_x = 1.5
        row.operator("object.select_by_name_type_anbx", icon='SORTALPHA')

        row = box.row() 
        row.scale_x = 0.5
        row.label(icon='KEYTYPE_BREAKDOWN_VEC')  
        row.scale_x = 1.5
        row.operator("object.set_unique_name_anbx", icon='SORTALPHA')

        # Select Every Nth...

        row = box.row()   
        row.operator("object.select_by_name_type_every_nth_anbx")
        
        row.scale_x = 0.4
        row.enabled = (len(context.selected_objects) != 0)                
        row.prop(bpy.context.scene, 'select_by_name_type_every_nth_anbx')


        # Deselect Every Nth...
 
        row = box.row()  

        row.operator("object.deselect_by_name_type_every_nth_anbx")

        row.scale_x = 0.4
        row.enabled = (len(context.selected_objects) > 1)                
        row.prop(bpy.context.scene, 'deselect_by_name_type_every_nth_anbx')  
          


        row = box.row()    
 
        # Invert Selected by Name / Type
 
        row.scale_x = 0.5
        row.label(icon='ARROW_LEFTRIGHT')
        row.scale_x = 1.5
        row.operator("object.invert_sel_by_name_type_anbx")


        # Visibility

        box = layout.box()  
        
        box = layout.box()  
        row = box.row()         

        # row.scale_y = 1.2
        row.operator("object.hide_before_current_frame_anbx", text='Hide before Current Frame', icon='HIDE_ON')
        row = box.row()
        # row.scale_y = 1.2
        row.operator("object.hide_after_current_frame_anbx", text='Hide after Current Frame', icon='HIDE_ON')

        row = box.row(align=True)   

        row.operator("object.hide_mesh_obj_anbx", text='Hide', icon='HIDE_ON')
        row.operator("object.show_mesh_obj_anbx", text='Show', icon='HIDE_OFF')
        


# =====================================================
#          End  PANEL 11
# =====================================================


# parent 'PANEL_PT_animbox_ax_08' for 'PANEL_PT_animbox_ax_07_01' not found


classes = [ PANEL_PT_animbox_ax_001,
            PANEL_PT_animbox_ax_01,
            PANEL_PT_animbox_ax_02,

            # Bone Tools
            PANEL_PT_animbox_ax_02_01,
            PANEL_PT_animbox_ax_02_01_01,
            PANEL_PT_animbox_ax_02_01_02,
            PANEL_PT_animbox_ax_02_01_03,
            PANEL_PT_animbox_ax_02_01_04,
            PANEL_PT_animbox_ax_02_01_05,

      
            # PANEL_PT_animbox_ax_02_02,
            PANEL_PT_animbox_ax_03,
            PANEL_PT_animbox_ax_04,
            PANEL_PT_animbox_ax_03_01,
            PANEL_PT_animbox_ax_03_01_02,
            PANEL_PT_animbox_ax_03_01_03,
            # PANEL_PT_animbox_ax_03_01_04,
            PANEL_PT_animbox_ax_03_01_05,
            PANEL_PT_animbox_ax_03_01_06,
            PANEL_PT_animbox_ax_03_01_07,


            PANEL_PT_animbox_ax_03_02,
            # PANEL_PT_animbox_ax_04,
            PANEL_PT_animbox_ax_05,
            PANEL_PT_animbox_ax_06,
            PANEL_PT_animbox_ax_07,
            PANEL_PT_animbox_ax_08,
            PANEL_PT_animbox_ax_08_01,
            PANEL_PT_animbox_ax_09,
            PANEL_PT_animbox_ax_10,
            PANEL_PT_animbox_ax_11,
          ]




def register():
    for cls in classes:
        bpy.utils.register_class(cls)




def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)



if __name__ == "__main__":
    register()



