import bpy
import random
import string
from bpy.props import (BoolProperty, IntProperty)


class OBJECT_OT_select_by_name_type_anbx(bpy.types.Operator):
    '''Select Duplicated Objects by Name - Type

-Alt Click = Insert Keys (Loc / Rot / Scale)'''

    bl_idname = "object.select_by_name_type_anbx"
    bl_label = "Select by Name-Type"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT')


    def invoke(self, context, event):



        cur_frame = bpy.context.scene.frame_current

        def insert_keys(frame):
            sel_ob = bpy.context.selected_objects
            for o in sel_ob:
                o.keyframe_insert(data_path = 'location', frame=frame)
                o.keyframe_insert(data_path = 'scale', frame=frame)
                o.keyframe_insert(data_path = 'rotation_euler', frame=frame)


        # Select Objects by Name / Type
        def select_by_name_type():

            act_name = bpy.context.view_layer.objects.active.name
            act_type = bpy.context.view_layer.objects.active.type

         
            name = act_name.split('.')
            name_spl = name[0]

            for obj in bpy.context.scene.objects:
                if obj.name.startswith(name_spl) and obj.type == act_type:
                    obj.select_set(1)

        if event:
            select_by_name_type()


            sel_ob = bpy.context.selected_objects
            act_ob = bpy.context.view_layer.objects

            first_ob = sel_ob[0]
            last_ob = sel_ob[-1]


            # Active Object = Last in Selected
            for o in bpy.context.selected_objects:
                if act_ob.active != first_ob and act_ob.active != last_ob:
                    act_ob.active = last_ob

                if act_ob.active == first_ob:
                    act_ob.active = last_ob

                if act_ob.active == last_ob:
                    act_ob.active = last_ob

        if event.alt:
            insert_keys(cur_frame)




        return {'FINISHED'}


#--------------------------------------------------------------------




#-------------------------- Select by Name/Type Every Nth... ----------


class OBJECT_OT_select_by_name_type_every_nth_anbx(bpy.types.Operator):
    '''Select every Nth Object by Name/Type'''

    bl_idname = "object.select_by_name_type_every_nth_anbx"
    bl_label = "Select Every Nth"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) != 0 )
 


    def execute(self, context):

        select_bool = context.scene.select_by_name_type_every_nth_anbx

 
        act_name = bpy.context.view_layer.objects.active.name
        act_type = bpy.context.view_layer.objects.active.type

         
        name = act_name.split('.')
        name_spl = name[0]

        num = 0

        for o in bpy.context.scene.objects:
            if o.name.startswith(name_spl) and o.type == act_type:
                o.select_set(1)
                
         

        sel_ob = bpy.context.selected_objects

        for i in range(len(sel_ob)):
         
            s_num = i

        for o in sel_ob: 
            if num <= s_num:
                sel_ob[num].select_set(0)
         
            num += select_bool


            
        def invert_selected_by_name_type():

            act_name = bpy.context.view_layer.objects.active.name
            act_type = bpy.context.view_layer.objects.active.type

         
            name = act_name.split('.')
            name_spl = name[0]

            for obj in bpy.context.scene.objects:
                if obj.name.startswith(name_spl) and obj.type == act_type:
                    if obj.select_get() == True:
                        obj.select_set(0)
                    else:
                        obj.select_set(1)
                        
                                     
        invert_selected_by_name_type()   



        return {'FINISHED'}


#--------------------------------------------------------------------



#-------------------------- Deselect by Name/Type Every Nth... ----------


class OBJECT_OT_deselect_by_name_type_every_nth_anbx(bpy.types.Operator):
    '''Deselect every Nth Object by Name/Type'''

    bl_idname = "object.deselect_by_name_type_every_nth_anbx"
    bl_label = "Deselect Every Nth"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) > 1 )
 


    def execute(self, context):

        deselect_bool = context.scene.deselect_by_name_type_every_nth_anbx

        sel_ob = bpy.context.selected_objects
         
        def deselect_every():
            num = 0
            for i in range(len(sel_ob)):
                s_num = i

            for o in sel_ob:
                if num <= s_num:
                    sel_ob[num-1].select_set(0)
         
                num += deselect_bool
                
                
        deselect_every()



        return {'FINISHED'}


#--------------------------------------------------------------------



#-------------------------- Invert Selection by Name/Type ------------


class OBJECT_OT_invert_sel_by_name_type_anbx(bpy.types.Operator):
    '''Invert Selection by Name/Type'''

    bl_idname = "object.invert_sel_by_name_type_anbx"
    bl_label = "Invert Selection"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT' and
                len(context.selected_objects) != 0 )
 


    def execute(self, context):

        def invert_selected_by_name_type():

            act_name = bpy.context.view_layer.objects.active.name
            act_type = bpy.context.view_layer.objects.active.type

         
            name = act_name.split('.')
            name_spl = name[0]

            for obj in bpy.context.scene.objects:
                if obj.name.startswith(name_spl) and obj.type == act_type:
                    if obj.select_get() == True:
                        obj.select_set(0)
                    else:
                        obj.select_set(1)
                        
                                     
        invert_selected_by_name_type()    


        return {'FINISHED'}


#--------------------------------------------------------------------




#---------------------Set Unique Name(s)-----------------------------

class OBJECT_OT_set_unique_name_anbx(bpy.types.Operator):
    '''Set Unique Name(s) -> Selected Object(s)'''

    bl_idname = "object.set_unique_name_anbx"
    bl_label = "Set Unique Name(s)"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT')


    def execute(self, context):
         
        def random_name(length):

            source = string.ascii_lowercase + string.digits
            result_str = ''.join((random.choice(source) for i in range(length)))

            for o in bpy.context.selected_objects:
             
                name_spl = o.name[:3]
                rand_name = ( '_' + result_str + '_')

                o.name = name_spl + rand_name
                
        random_name(6)        
                
         
        for o in bpy.context.selected_objects:
            if o.data:
                o.data.name = o.name
         


        return {'FINISHED'}


#---------------------Set Unique Name(s)-----------------------------


 


#--------------------- Parent Row -----------------------------


class OBJECT_OT_set_parent_sequence_row_anbx(bpy.types.Operator):
    '''Quick "Sequence" Parent -> Selected Object(s)'''

    bl_idname = "object.set_parent_sequence_row_anbx"
    bl_label = "Parent Row"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
                context.object.mode == 'OBJECT'
                and len(context.selected_objects) > 1)


    def execute(self, context):

        # Reference to Reverse Bool
        rev_parent_row_bool = context.scene.revers_parent_row_mdh
        

        sel_ob = bpy.context.selected_objects
         
        def parent_row():
            num = 0
            for i in range(len(sel_ob)):
                s_num = i

            for o in sel_ob:
                if num < s_num:
                    sel_ob[num+1].parent = sel_ob[num]
                    sel_ob[num+1].matrix_parent_inverse = sel_ob[num].matrix_world.inverted()
             
             
                num += 1


        def parent_row_reversed():
         
            num = len(sel_ob)
            for i in range(len(sel_ob)):
                s_num = i

            #for o in reversed(sel_ob):
            for o in sel_ob:
                if num <= s_num:
                    sel_ob[num-1].parent = sel_ob[num]
                    sel_ob[num-1].matrix_parent_inverse = sel_ob[num].matrix_world.inverted()
             
             
                num -= 1


        if rev_parent_row_bool:
            parent_row_reversed()



        else:
            parent_row()



        bpy.context.scene.tool_settings.transform_pivot_point = 'INDIVIDUAL_ORIGINS'


        return {'FINISHED'}



#--------------------- Parent Row -----------------------------



classes = [OBJECT_OT_select_by_name_type_anbx,
           OBJECT_OT_select_by_name_type_every_nth_anbx,
           OBJECT_OT_deselect_by_name_type_every_nth_anbx,
           OBJECT_OT_invert_sel_by_name_type_anbx,
           OBJECT_OT_set_unique_name_anbx,
           OBJECT_OT_set_parent_sequence_row_anbx,
          ]




def register():
    for cls in classes:
        bpy.utils.register_class(cls)



    bpy.types.Scene.revers_parent_row_mdh = bpy.props.BoolProperty(
        name="",
        description="REVERSED Parent Row",
        default = False
    )







    bpy.types.Scene.select_by_name_type_every_nth_anbx = bpy.props.IntProperty(
        name="",
        default=2,
        description="Select Every Nth",
        min=2,
       
    )



    bpy.types.Scene.deselect_by_name_type_every_nth_anbx = bpy.props.IntProperty(
        name="",
        default=2,
        description="Deselect Every Nth",
        min=2,
       
    )




def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)

    del bpy.types.Scene.revers_parent_row_mdh
    del bpy.types.Scene.select_by_name_type_every_nth_anbx
    del bpy.types.Scene.deselect_by_name_type_every_nth_anbx



if __name__ == "__main__":
    register()



