import bpy



#-----------------------------------------------------------------------
#--------------------Hide before Current Frame--------------------------
#-----------------------------------------------------------------------


class OBJECT_OT_hide_before_current_frame_anbx(bpy.types.Operator):
    '''Add Mask modifier to Mesh object(s) and Hide before Current Frame'''
    bl_idname = "object.hide_before_current_frame_anbx"
    bl_label = "Hide before Current Frame"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
            context.mode == 'OBJECT' and
            context.object.type == 'MESH' and
            len(context.selected_objects) != 0)


    def execute(self, context):
  
        sel_ob = bpy.context.selected_objects
    
        cur_frame = bpy.context.scene.frame_current
        
        def get_mask_mod():
            ob = bpy.context.object
            ob_mod = ob.modifiers
            
            if ob_mod:
                for m in ob_mod:
                    if m.type == 'MASK': 
                        return True           
                        
        
        def add_mask_mod():
            
            for o in sel_ob:
                bpy.context.view_layer.objects.active = o            
            
                mask_mod = get_mask_mod()
                if not mask_mod:
                    o.modifiers.new('Mask', type='MASK')
                    o.modifiers["Mask"].invert_vertex_group = True
        
        add_mask_mod()        
                    
        
        def hide_frame_before():
                        
            ob = bpy.context.object

            ob.modifiers["Mask"].invert_vertex_group = True
            ob.modifiers["Mask"].keyframe_insert(data_path='invert_vertex_group')


            prev_frame = cur_frame - 1
            bpy.context.scene.frame_set(prev_frame)

            ob.modifiers["Mask"].invert_vertex_group = False
            ob.modifiers["Mask"].keyframe_insert(data_path='invert_vertex_group')


            bpy.context.scene.frame_set(cur_frame)
                             
        
        for o in sel_ob:
            bpy.context.view_layer.objects.active = o
            hide_frame_before()
 


        return {'FINISHED'}

#-----------------------------------------------------------------------
#--------------------Hide before Current Frame End----------------------
#-----------------------------------------------------------------------



#-----------------------------------------------------------------------
#-----------------------Hide after Current Frame -----------------------
#-----------------------------------------------------------------------


class OBJECT_OT_hide_after_current_frame_anbx(bpy.types.Operator):
    '''Add Mask modifier to Mesh object(s) and Hide after Current Frame'''
    bl_idname = "object.hide_after_current_frame_anbx"
    bl_label = "Hide after Current Frame"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
            context.mode == 'OBJECT' and
            context.object.type == 'MESH' and
            len(context.selected_objects) != 0)


    def execute(self, context):
  
        sel_ob = bpy.context.selected_objects
    
        cur_frame = bpy.context.scene.frame_current
        
        def get_mask_mod():
            ob = bpy.context.object
            ob_mod = ob.modifiers
            
            if ob_mod:
                for m in ob_mod:
                    if m.type == 'MASK': 
                        return True           
                        
        
        def add_mask_mod():
            
            for o in sel_ob:
                bpy.context.view_layer.objects.active = o            
            
                mask_mod = get_mask_mod()
                if not mask_mod:
                    o.modifiers.new('Mask', type='MASK')
                    o.modifiers["Mask"].invert_vertex_group = True
        
        add_mask_mod()         
                    
        
        
        def hide_frame_after():
                        
            ob = bpy.context.object

            bpy.data.objects[ob.name].modifiers["Mask"].invert_vertex_group = True
            ob.modifiers["Mask"].keyframe_insert(data_path='invert_vertex_group')


            prev_frame = cur_frame + 1
            bpy.context.scene.frame_set(prev_frame)

            ob.modifiers["Mask"].invert_vertex_group = False
            ob.modifiers["Mask"].keyframe_insert(data_path='invert_vertex_group')


            bpy.context.scene.frame_set(cur_frame)
                             

 
        for o in sel_ob:
            bpy.context.view_layer.objects.active = o
            hide_frame_after()


        return {'FINISHED'}

#-----------------------------------------------------------------------
#------------------- Hide after Current Frame End ----------------------
#-----------------------------------------------------------------------



#-----------------------------------------------------------------------
#------------------------ Hide Mesh Objects ----------------------------
#-----------------------------------------------------------------------


class OBJECT_OT_hide_mesh_obj_anbx(bpy.types.Operator):
    '''Hide mesh object(s) by adding Mask modifier
Alt + Click = insert visibility keyframe(s)'''
    bl_idname = "object.hide_mesh_obj_anbx"
    bl_label = "Hide"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
            context.mode == 'OBJECT' and
            context.object.type == 'MESH' and
            len(context.selected_objects) != 0)


    def invoke(self, context, event):
  
        sel_ob = bpy.context.selected_objects
    
        cur_frame = bpy.context.scene.frame_current
        
        def get_mask_mod():
            ob = bpy.context.object
            ob_mod = ob.modifiers
            
            if ob_mod:
                for m in ob_mod:
                    if m.type == 'MASK': 
                        return True           
                        
        
        def add_mask_mod():
            
            for o in sel_ob:
                bpy.context.view_layer.objects.active = o            
            
                mask_mod = get_mask_mod()
                if not mask_mod:
                    o.modifiers.new('Mask', type='MASK')
                    o.modifiers["Mask"].invert_vertex_group = True
        
        add_mask_mod()         
                    
        
        def hide_frame_after():
                        
            ob = bpy.context.object
            
            if event:

                bpy.data.objects[ob.name].modifiers["Mask"].invert_vertex_group = False
                
            if event.alt:
            
                ob.modifiers["Mask"].keyframe_insert(data_path='invert_vertex_group')


                             
        for o in sel_ob:
            bpy.context.view_layer.objects.active = o
            hide_frame_after()


        return {'FINISHED'}

#-----------------------------------------------------------------------
#------------------------ Hide Mesh Objects End ------------------------
#-----------------------------------------------------------------------


#-----------------------------------------------------------------------
#------------------------ Show Mesh Objects ----------------------------
#-----------------------------------------------------------------------


class OBJECT_OT_show_mesh_obj_anbx(bpy.types.Operator):
    '''Show mesh object(s) by adding Mask modifier
Alt + Click = insert visibility keyframe(s)'''
    bl_idname = "object.show_mesh_obj_anbx"
    bl_label = "Show"
    bl_options = {'REGISTER', 'UNDO'}


    @classmethod
    def poll(cls, context):
        return (context.object is not None and
            context.mode == 'OBJECT' and
            context.object.type == 'MESH' and
            len(context.selected_objects) != 0)


    def invoke(self, context, event):
  

        sel_ob = bpy.context.selected_objects
    
        cur_frame = bpy.context.scene.frame_current
        
        def get_mask_mod():
            ob = bpy.context.object
            ob_mod = ob.modifiers
            
            if ob_mod:
                for m in ob_mod:
                    if m.type == 'MASK': 
                        return True           
                        
        
        def add_mask_mod():
            
            for o in sel_ob:
                bpy.context.view_layer.objects.active = o            
            
                mask_mod = get_mask_mod()
                if not mask_mod:
                    o.modifiers.new('Mask', type='MASK')
                    o.modifiers["Mask"].invert_vertex_group = True
        
        add_mask_mod()         
                    
        
    
        def hide_frame_after():
                        
            ob = bpy.context.object
            
            if event:

                bpy.data.objects[ob.name].modifiers["Mask"].invert_vertex_group = True
                
            if event.alt:
            
                ob.modifiers["Mask"].keyframe_insert(data_path='invert_vertex_group')


                             
        for o in sel_ob:
            bpy.context.view_layer.objects.active = o
            hide_frame_after()


        return {'FINISHED'}

#-----------------------------------------------------------------------
#------------------------ Show Mesh Objects End ------------------------
#-----------------------------------------------------------------------




        

classes = [ 
            OBJECT_OT_hide_before_current_frame_anbx,
            OBJECT_OT_hide_after_current_frame_anbx,
            OBJECT_OT_hide_mesh_obj_anbx,
            OBJECT_OT_show_mesh_obj_anbx,

          ]
 
 
 
def register():
    for cls in classes:
        bpy.utils.register_class(cls)
        

 
def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)

 
 
 
if __name__ == "__main__":
    register() 

